@@ -1,31 +1,48 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:BSD$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** BSD License Usage
+** Alternatively, you may use this file under the terms of the BSD license
+** as follows:
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** "Redistribution and use in source and binary forms, with or without
+** modification, are permitted provided that the following conditions are
+** met:
+**   * Redistributions of source code must retain the above copyright
+**     notice, this list of conditions and the following disclaimer.
+**   * Redistributions in binary form must reproduce the above copyright
+**     notice, this list of conditions and the following disclaimer in
+**     the documentation and/or other materials provided with the
+**     distribution.
+**   * Neither the name of The Qt Company Ltd nor the names of its
+**     contributors may be used to endorse or promote products derived
+**     from this software without specific prior written permission.
+**
+**
+** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 **
 ** $QT_END_LICENSE$
 **
@@ -44,45 +61,17 @@ class ColorSwatch : public QDockWidget
 {
     Q_OBJECT
 
-    QAction *closableAction;
-    QAction *movableAction;
-    QAction *floatableAction;
-    QAction *floatingAction;
-    QAction *verticalTitleBarAction;
-
-    QActionGroup *allowedAreasActions;
-    QAction *allowLeftAction;
-    QAction *allowRightAction;
-    QAction *allowTopAction;
-    QAction *allowBottomAction;
-
-    QActionGroup *areaActions;
-    QAction *leftAction;
-    QAction *rightAction;
-    QAction *topAction;
-    QAction *bottomAction;
-
-    QAction *changeSizeHintsAction;
-
-    QMenu *tabMenu;
-    QMenu *splitHMenu;
-    QMenu *splitVMenu;
-
-    QAction *windowModifiedAction;
-
 public:
-    explicit ColorSwatch(const QString &colorName, QWidget *parent = 0, Qt::WindowFlags flags = 0);
+    explicit ColorSwatch(const QString &colorName, QMainWindow *parent = Q_NULLPTR, Qt::WindowFlags flags = 0);
 
-    QMenu *menu;
     void setCustomSizeHint(const QSize &size);
+    QMenu *colorSwatchMenu() const { return menu; }
 
 protected:
-    virtual void contextMenuEvent(QContextMenuEvent *event) Q_DECL_OVERRIDE;
-    virtual void resizeEvent(QResizeEvent *e) Q_DECL_OVERRIDE;
-
-private:
-    void allow(Qt::DockWidgetArea area, bool allow);
-    void place(Qt::DockWidgetArea area, bool place);
+#ifndef QT_NO_CONTEXTMENU
+    void contextMenuEvent(QContextMenuEvent *event) Q_DECL_OVERRIDE;
+#endif // QT_NO_CONTEXTMENU
+    void resizeEvent(QResizeEvent *e) Q_DECL_OVERRIDE;
 
 private slots:
     void changeClosable(bool on);
@@ -104,25 +93,57 @@ private slots:
 
     void splitInto(QAction *action);
     void tabInto(QAction *action);
+
+private:
+    void allow(Qt::DockWidgetArea area, bool allow);
+    void place(Qt::DockWidgetArea area, bool place);
+
+    QAction *closableAction;
+    QAction *movableAction;
+    QAction *floatableAction;
+    QAction *floatingAction;
+    QAction *verticalTitleBarAction;
+
+    QActionGroup *allowedAreasActions;
+    QAction *allowLeftAction;
+    QAction *allowRightAction;
+    QAction *allowTopAction;
+    QAction *allowBottomAction;
+
+    QActionGroup *areaActions;
+    QAction *leftAction;
+    QAction *rightAction;
+    QAction *topAction;
+    QAction *bottomAction;
+
+    QMenu *tabMenu;
+    QMenu *splitHMenu;
+    QMenu *splitVMenu;
+    QMenu *menu;
+
+    QMainWindow *mainWindow;
 };
 
 class BlueTitleBar : public QWidget
 {
     Q_OBJECT
 public:
-    BlueTitleBar(QWidget *parent = 0);
+    explicit BlueTitleBar(QWidget *parent = Q_NULLPTR);
 
     QSize sizeHint() const Q_DECL_OVERRIDE { return minimumSizeHint(); }
     QSize minimumSizeHint() const Q_DECL_OVERRIDE;
+
 protected:
     void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
     void mouseReleaseEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
+
 public slots:
     void updateMask();
 
 private:
-    QPixmap leftPm, centerPm, rightPm;
+    const QPixmap leftPm;
+    const QPixmap centerPm;
+    const QPixmap rightPm;
 };
 
-
-#endif
+#endif // COLORSWATCH_H
