@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,17 +50,14 @@ QString project_builtin_regx() //calculate the builtin regular expression..
     return ret;
 }
 
-ProjectGenerator::ProjectGenerator() : MakefileGenerator(), init_flag(false)
+ProjectGenerator::ProjectGenerator() : MakefileGenerator()
 {
 }
 
 void
 ProjectGenerator::init()
 {
-    if(init_flag)
-        return;
     int file_count = 0;
-    init_flag = true;
     verifyCompilers();
 
     project->loadSpec();
@@ -169,7 +161,7 @@ ProjectGenerator::init()
                             QString nd = newdir;
                             if(nd == ".")
                                 nd = "";
-                            else if(!nd.isEmpty() && !nd.endsWith(QString(QChar(QDir::separator()))))
+                            else if (!nd.isEmpty() && !nd.endsWith(QDir::separator()))
                                 nd += QDir::separator();
                             nd += profiles[i];
                             fileFixify(nd);
@@ -311,7 +303,7 @@ ProjectGenerator::init()
         for (ProStringList::ConstIterator it2 = tmp.begin(); it2 != tmp.end(); ++it2) {
             ProStringList &inputs = project->values((*it2).toKey());
             for (ProStringList::Iterator input = inputs.begin(); input != inputs.end(); ++input) {
-                QString path = replaceExtraCompilerVariables(tmp_out, (*input).toQString(), QString());
+                QString path = replaceExtraCompilerVariables(tmp_out, (*input).toQString(), QString(), NoShell);
                 path = fixPathToQmake(path).section('/', -1);
                 for(int i = 0; i < var_out.size(); ++i) {
                     ProString v = var_out.at(i);
@@ -382,7 +374,7 @@ ProjectGenerator::addConfig(const QString &cfg, bool add)
 bool
 ProjectGenerator::addFile(QString file)
 {
-    file = fileFixify(file, qmake_getpwd());
+    file = fileFixify(file, FileFixifyToIndir);
     QString dir;
     int s = file.lastIndexOf(Option::dir_sep);
     if(s != -1)
@@ -491,7 +483,7 @@ ProjectGenerator::fixPathToQmake(const QString &file)
 {
     QString ret = file;
     if(Option::dir_sep != QLatin1String("/"))
-        ret = ret.replace(Option::dir_sep, QLatin1String("/"));
+        ret.replace(Option::dir_sep, QLatin1String("/"));
     return ret;
 }
 
