@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -178,18 +173,15 @@ QStringList CeSdkHandler::getMsBuildToolPaths() const
 QStringList CeSdkHandler::filterMsBuildToolPaths(const QStringList &paths) const
 {
     QStringList result;
-    foreach (const QString &path, paths) {
+    for (const QString &path : paths) {
         QDir dirVC110(path);
         if (path.endsWith(QLatin1String("bin")))
             dirVC110.cdUp();
         QDir dirVC120 = dirVC110;
-        QDir dirVC140 = dirVC110;
         if (dirVC110.cd(QStringLiteral("Microsoft.Cpp\\v4.0\\V110\\Platforms")))
             result << dirVC110.absolutePath();
         if (dirVC120.cd(QStringLiteral("Microsoft.Cpp\\v4.0\\V120\\Platforms")))
             result << dirVC120.absolutePath();
-        if (dirVC140.cd(QStringLiteral("Microsoft.Cpp\\v4.0\\V140\\Platforms")))
-            result << dirVC140.absolutePath();
     }
     return result;
 }
@@ -199,9 +191,9 @@ bool CeSdkHandler::retrieveEnvironment(const QStringList &relativePaths,
                                        CeSdkInfo *info)
 {
     bool result = false;
-    foreach (const QString &path, toolPaths) {
+    for (const QString &path : toolPaths) {
         const QDir dir(path);
-        foreach (const QString &filePath, relativePaths) {
+        for (const QString &filePath : relativePaths) {
             QFile file(dir.absoluteFilePath(filePath));
             if (file.exists())
                 result = parseMsBuildFile(&file, info) || result;
@@ -217,7 +209,7 @@ void CeSdkHandler::retrieveWEC2013SDKs()
     const QStringList filteredToolPaths = filterMsBuildToolPaths(toolPaths);
     QSettings settings("HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows CE Tools\\SDKs", QSettings::NativeFormat);
     const QStringList keys = settings.allKeys();
-    foreach (const QString &key, keys) {
+    for (const QString &key : keys) {
         if (key.contains(QLatin1String("SDKInformation")) || key.contains(QLatin1Char('.'))) {
             QFile sdkPropertyFile(settings.value(key).toString());
             if (!sdkPropertyFile.exists())
@@ -293,8 +285,6 @@ void CeSdkHandler::retrieveWEC2013SDKs()
                                 filteredToolPaths, &currentSdk);
             retrieveEnvironment(currentProperty.properties.value(QLatin1String("MSBuild Files120")).value.split(';'),
                                 filteredToolPaths, &currentSdk);
-            retrieveEnvironment(currentProperty.properties.value(QLatin1String("MSBuild Files140")).value.split(';'),
-                                filteredToolPaths, &currentSdk);
             if (!currentSdk.m_include.isEmpty())
                 m_list.append(currentSdk);
         }
