@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -181,7 +176,7 @@ const char _Optimization[]                      = "Optimization";
 const char _OptimizeReferences[]                = "OptimizeReferences";
 const char _OutputDirectory[]                   = "OutputDirectory";
 const char _OutputFile[]                        = "OutputFile";
-const char _PlatformToolSet[]                   = "PlatformToolset";
+const char _PlatformToolSet[]                   = "PlatformToolSet";
 const char _PrecompiledHeader[]                 = "PrecompiledHeader";
 const char _PrecompiledHeaderFile[]             = "PrecompiledHeaderFile";
 const char _PrecompiledHeaderOutputFile[]       = "PrecompiledHeaderOutputFile";
@@ -861,7 +856,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
             << tag("AppxManifest")
             << attrTag("Include", manifest)
             << closetag();
-        foreach (const QString &icon, icons) {
+        for (const QString &icon : qAsConst(icons)) {
             xml << tag("Image")
                 << attrTag("Include", icon)
                 << closetag();
@@ -1163,21 +1158,6 @@ static inline QString toString(subSystemOption option)
     return QString();
 }
 
-static inline QString toString(triState genDebugInfo, linkerDebugOption option)
-{
-    switch (genDebugInfo) {
-    case unset:
-        break;
-    case _False:
-        return "false";
-    case _True:
-        if (option == linkerDebugOptionFastLink)
-            return "DebugFastLink";
-        return "true";
-    }
-    return QString();
-}
-
 static inline QString toString(machineTypeOption option)
 {
     switch (option) {
@@ -1556,7 +1536,7 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCLinkerTool &tool)
             << attrTagS(_EntryPointSymbol, tool.EntryPointSymbol)
             << attrTagX(_ForceSymbolReferences, tool.ForceSymbolReferences, ";")
             << attrTagS(_FunctionOrder, tool.FunctionOrder)
-            << attrTagS(_GenerateDebugInformation, toString(tool.GenerateDebugInformation, tool.DebugInfoOption))
+            << attrTagT(_GenerateDebugInformation, tool.GenerateDebugInformation)
             << attrTagT(_GenerateManifest, tool.GenerateManifest)
             << attrTagT(_GenerateWindowsMetadata, tool.GenerateWindowsMetadata)
             << attrTagS(_WindowsMetadataFile, tool.GenerateWindowsMetadata == _True ? tool.WindowsMetadataFile : QString())
