@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,8 +52,7 @@ enum DotNET {
     NET2010 = 0xa0,
     NET2012 = 0xb0,
     NET2013 = 0xc0,
-    NET2015 = 0xd0,
-    NET2017 = 0xe0
+    NET2015 = 0xd0
 };
 
 /*
@@ -283,10 +277,6 @@ enum inlineExpansionOption {
     expandAnySuitable,
     expandDefault // Not useful number, but stops the output
 };
-enum linkerDebugOption {
-    linkerDebugOptionNone,
-    linkerDebugOptionFastLink
-};
 enum linkIncrementalType {
     linkIncrementalDefault,
     linkIncrementalNo,
@@ -487,7 +477,7 @@ class VCCLCompilerTool : public VCToolBase
 public:
     // Functions
     VCCLCompilerTool();
-    virtual ~VCCLCompilerTool(){}
+
     bool parseOption(const char* option);
 
     // Variables
@@ -584,7 +574,7 @@ class VCLinkerTool : public VCToolBase
 public:
     // Functions
     VCLinkerTool();
-    virtual ~VCLinkerTool(){}
+
     bool parseOption(const char* option);
 
     // Variables
@@ -600,7 +590,6 @@ public:
     QStringList             ForceSymbolReferences;
     QString                 FunctionOrder;
     triState                GenerateDebugInformation;
-    linkerDebugOption       DebugInfoOption;
     triState                GenerateMapFile;
     qlonglong               HeapCommitSize;
     qlonglong               HeapReserveSize;
@@ -679,7 +668,7 @@ class VCManifestTool : public VCToolBase
 {
 public:
     VCManifestTool();
-    ~VCManifestTool();
+
     bool parseOption(const char* option);
 
     triState                EmbedManifest;
@@ -690,7 +679,7 @@ class VCMIDLTool : public VCToolBase
 public:
     // Functions
     VCMIDLTool();
-    virtual ~VCMIDLTool(){}
+
     bool parseOption(const char* option);
 
     // Variables
@@ -744,7 +733,7 @@ class VCLibrarianTool : public VCToolBase
 public:
     // Functions
     VCLibrarianTool();
-    virtual ~VCLibrarianTool(){}
+
     bool parseOption(const char*){ return false; }
 
     // Variables
@@ -765,7 +754,7 @@ class VCCustomBuildTool : public VCToolBase
 public:
     // Functions
     VCCustomBuildTool();
-    virtual ~VCCustomBuildTool(){}
+
     bool parseOption(const char*){ return false; }
 
     // Variables
@@ -784,7 +773,7 @@ class VCResourceCompilerTool : public VCToolBase
 public:
     // Functions
     VCResourceCompilerTool();
-    virtual ~VCResourceCompilerTool(){}
+
     bool parseOption(const char*){ return false; }
 
     // Variables
@@ -805,7 +794,6 @@ class VCDeploymentTool
 public:
     // Functions
     VCDeploymentTool();
-    virtual ~VCDeploymentTool() {}
 
     // Variables
     QString                 DeploymentTag;
@@ -819,7 +807,7 @@ class VCEventTool : public VCToolBase
 protected:
     // Functions
     VCEventTool(const QString &eventName);
-    virtual ~VCEventTool(){}
+
     bool parseOption(const char*){ return false; }
 
 public:
@@ -836,28 +824,24 @@ class VCPostBuildEventTool : public VCEventTool
 {
 public:
     VCPostBuildEventTool();
-    ~VCPostBuildEventTool(){}
 };
 
 class VCPreBuildEventTool : public VCEventTool
 {
 public:
     VCPreBuildEventTool();
-    ~VCPreBuildEventTool(){}
 };
 
 class VCPreLinkEventTool : public VCEventTool
 {
 public:
     VCPreLinkEventTool();
-    ~VCPreLinkEventTool(){}
 };
 
 class VCWinDeployQtTool : public VCToolBase
 {
 public:
     VCWinDeployQtTool() {}
-    ~VCWinDeployQtTool() {}
 
 protected:
     bool parseOption(const char *) { return false; }
