@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -170,9 +165,6 @@ Win32MakefileGenerator::findLibraries(bool linkPrl, bool mergeLflags)
 
 void Win32MakefileGenerator::processVars()
 {
-    if (project->first("TEMPLATE").endsWith("aux"))
-        return;
-
     project->values("QMAKE_ORIG_TARGET") = project->values("TARGET");
     if (project->isEmpty("QMAKE_PROJECT_NAME"))
         project->values("QMAKE_PROJECT_NAME") = project->values("QMAKE_ORIG_TARGET");
@@ -271,7 +263,9 @@ void Win32MakefileGenerator::processRcFileVar()
         QString versionString = vers.join('.');
 
         QStringList rcIcons;
-        foreach (const ProString &icon, project->values("RC_ICONS"))
+        const auto icons = project->values("RC_ICONS");
+        rcIcons.reserve(icons.size());
+        for (const ProString &icon : icons)
             rcIcons.append(fileFixify(icon.toQString(), FileFixifyAbsolute));
 
         QString companyName;
@@ -578,18 +572,16 @@ void Win32MakefileGenerator::writeStandardParts(QTextStream &t)
     t << "####### Build rules\n\n";
     writeBuildRulesPart(t);
 
-    if (project->first("TEMPLATE") != "aux") {
-        if (project->isActiveConfig("shared") && !project->values("DLLDESTDIR").isEmpty()) {
-            const ProStringList &dlldirs = project->values("DLLDESTDIR");
-            for (ProStringList::ConstIterator dlldir = dlldirs.begin(); dlldir != dlldirs.end(); ++dlldir) {
-                t << "\t-$(COPY_FILE) $(DESTDIR_TARGET) "
-                  << escapeFilePath(Option::fixPathToTargetOS((*dlldir).toQString(), false)) << endl;
-            }
+    if(project->isActiveConfig("shared") && !project->values("DLLDESTDIR").isEmpty()) {
+        const ProStringList &dlldirs = project->values("DLLDESTDIR");
+        for (ProStringList::ConstIterator dlldir = dlldirs.begin(); dlldir != dlldirs.end(); ++dlldir) {
+            t << "\t-$(COPY_FILE) $(DESTDIR_TARGET) "
+              << escapeFilePath(Option::fixPathToTargetOS((*dlldir).toQString(), false)) << endl;
         }
-        t << endl;
-
-        writeRcFilePart(t);
     }
+    t << endl;
+
+    writeRcFilePart(t);
 
     writeMakeQmake(t);
 
@@ -613,10 +605,8 @@ void Win32MakefileGenerator::writeStandardParts(QTextStream &t)
         const ProStringList &quc = project->values("QMAKE_EXTRA_COMPILERS");
         for (ProStringList::ConstIterator it = quc.begin(); it != quc.end(); ++it) {
             const ProStringList &inputs = project->values(ProKey(*it + ".input"));
-            for (ProStringList::ConstIterator input = inputs.begin(); input != inputs.end(); ++input) {
-                const ProStringList &val = project->values((*input).toKey());
-                t << escapeFilePaths(val).join(' ') << ' ';
-            }
+            for (ProStringList::ConstIterator input = inputs.begin(); input != inputs.end(); ++input)
+                t << escapeFilePath(*input) << ' ';
         }
     }
     t << endl << endl;
