@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,7 +32,7 @@
 #include <qregexp.h>
 #include <qhash.h>
 #include <qdebug.h>
-#include <qsettings.h>
+#include <qlibraryinfo.h>
 #include <stdlib.h>
 #include <stdarg.h>
 
@@ -67,6 +54,8 @@ QStringList Option::h_ext;
 QString Option::cpp_moc_ext;
 QStringList Option::cpp_ext;
 QStringList Option::c_ext;
+QString Option::objc_ext;
+QString Option::objcpp_ext;
 QString Option::obj_ext;
 QString Option::lex_ext;
 QString Option::yacc_ext;
@@ -178,6 +167,7 @@ bool usage(const char *a0)
             "  -set <prop> <value> Set persistent property\n"
             "  -unset <prop>  Unset persistent property\n"
             "  -query <prop>  Query persistent property. Show all if <prop> is empty.\n"
+            "  -qtconf file   Use file instead of looking for qt.conf\n"
             "  -cache file    Use file as cache           [makefile mode only]\n"
             "  -spec spec     Use spec as QMAKESPEC       [makefile mode only]\n"
             "  -nocache       Don't use a cache file      [makefile mode only]\n"
@@ -223,7 +213,7 @@ Option::parseCommandLine(QStringList &args, QMakeCmdLineParserState &state)
                             QMAKE_VERSION_STR, QT_VERSION_STR,
                             QLibraryInfo::location(QLibraryInfo::LibrariesPath).toLatin1().constData());
 #ifdef QMAKE_OPENSOURCE_VERSION
-                    fprintf(stdout, "QMake is Open Source software from Digia Plc and/or its subsidiary(-ies).\n");
+                    fprintf(stdout, "QMake is Open Source software from The Qt Company Ltd and/or its subsidiary(-ies).\n");
 #endif
                     return Option::QMAKE_CMDLINE_BAIL;
                 } else if (arg == "-h" || arg == "-help" || arg == "--help") {
@@ -401,6 +391,7 @@ Option::init(int argc, char **argv)
     }
     if(argc && argv) {
         QStringList args;
+        args.reserve(argc - 1);
         for (int i = 1; i < argc; i++)
             args << QString::fromLocal8Bit(argv[i]);
 
@@ -473,6 +464,8 @@ bool Option::postProcessProject(QMakeProject *project)
     Option::cpp_ext = project->values("QMAKE_EXT_CPP").toQStringList();
     Option::h_ext = project->values("QMAKE_EXT_H").toQStringList();
     Option::c_ext = project->values("QMAKE_EXT_C").toQStringList();
+    Option::objc_ext = project->first("QMAKE_EXT_OBJC").toQString();
+    Option::objcpp_ext = project->first("QMAKE_EXT_OBJCXX").toQString();
     Option::res_ext = project->first("QMAKE_EXT_RES").toQString();
     Option::pkgcfg_ext = project->first("QMAKE_EXT_PKGCONFIG").toQString();
     Option::libtool_ext = project->first("QMAKE_EXT_LIBTOOL").toQString();
@@ -538,15 +531,15 @@ Option::fixString(QString string, uchar flags)
 
     //fix separators
     if (flags & Option::FixPathToNormalSeparators) {
-        string = string.replace('\\', '/');
+        string.replace('\\', '/');
     } else if (flags & Option::FixPathToLocalSeparators) {
 #if defined(Q_OS_WIN32)
-        string = string.replace('/', '\\');
+        string.replace('/', '\\');
 #else
-        string = string.replace('\\', '/');
+        string.replace('\\', '/');
 #endif
     } else if(flags & Option::FixPathToTargetSeparators) {
-        string = string.replace('/', Option::dir_sep).replace('\\', Option::dir_sep);
+        string.replace('/', Option::dir_sep).replace('\\', Option::dir_sep);
     }
 
     if ((string.startsWith("\"") && string.endsWith("\"")) ||
@@ -605,8 +598,9 @@ void EvalHandler::message(int type, const QString &msg, const QString &fileName,
         fprintf(stderr, "%s%s\n", qPrintable(pfx), qPrintable(msg));
 }
 
-void EvalHandler::fileMessage(const QString &msg)
+void EvalHandler::fileMessage(int type, const QString &msg)
 {
+    Q_UNUSED(type)
     fprintf(stderr, "%s\n", qPrintable(msg));
 }
 
@@ -644,8 +638,10 @@ qmakeAddCacheClear(qmakeCacheClearFunc func, void **data)
     cache_items.append(new QMakeCacheClearItem(func, data));
 }
 
-QString qt_libraryInfoFile()
+QString qmake_libraryInfoFile()
 {
+    if (!Option::globals->qtconf.isEmpty())
+        return Option::globals->qtconf;
     if (!Option::globals->qmake_abslocation.isEmpty())
         return QDir(QFileInfo(Option::globals->qmake_abslocation).absolutePath()).filePath("qt.conf");
     return QString();
