@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -78,7 +73,7 @@ static ProStringList prepareBuiltinArgs(const QList<ProStringList> &args)
 {
     ProStringList ret;
     ret.reserve(args.size());
-    foreach (const ProStringList &arg, args)
+    for (const ProStringList &arg : args)
         ret << arg.join(' ');
     return ret;
 }
@@ -110,9 +105,8 @@ QStringList QMakeProject::expand(const ProKey &func, const QList<ProStringList>
     QHash<ProKey, ProFunctionDef>::ConstIterator it =
             m_functionDefs.replaceFunctions.constFind(func);
     if (it != m_functionDefs.replaceFunctions.constEnd()) {
-        QMakeProject::VisitReturn vr;
-        ProStringList ret = evaluateFunction(*it, args, &vr);
-        if (vr == QMakeProject::ReturnError)
+        ProStringList ret;
+        if (evaluateFunction(*it, args, &ret) == QMakeProject::ReturnError)
             exit(3);
         return ret.toQStringList();
     }
@@ -130,7 +124,9 @@ ProString QMakeProject::expand(const QString &expr, const QString &where, int li
         m_current.pro = pro;
         m_current.line = 0;
         const ushort *tokPtr = pro->tokPtr();
-        ProStringList result = expandVariableReferences(tokPtr, 1, true);
+        ProStringList result;
+        if (expandVariableReferences(tokPtr, 1, &result, true) == ReturnError)
+            exit(3);
         if (!result.isEmpty())
             ret = result.at(0);
     }
@@ -151,13 +147,13 @@ void QMakeProject::dump() const
          it != m_valuemapStack.first().end(); ++it) {
         if (!it.key().startsWith('.')) {
             QString str = it.key() + " =";
-            foreach (const ProString &v, it.value())
+            for (const ProString &v : it.value())
                 str += ' ' + formatValue(v);
             out << str;
         }
     }
     out.sort();
-    foreach (const QString &v, out)
+    for (const QString &v : qAsConst(out))
         puts(qPrintable(v));
 }
 
