@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,6 +36,7 @@ QT_BEGIN_NAMESPACE
 class QMakeProject : private QMakeEvaluator
 {
     QString m_projectFile;
+    QString m_projectDir;
 
 public:
     QMakeProject();
@@ -49,6 +45,7 @@ public:
     bool read(const QString &project, LoadFlags what = LoadAll);
 
     QString projectFile() const { return m_projectFile; }
+    QString projectDir() const { return m_projectDir; }
     QString sourceRoot() const { return m_sourceRoot.isEmpty() ? m_buildRoot : m_sourceRoot; }
     QString buildRoot() const { return m_buildRoot; }
     QString confFile() const { return m_conffile; }
@@ -58,7 +55,7 @@ public:
     ProString expand(const QString &v, const QString &file, int line);
     QStringList expand(const ProKey &func, const QList<ProStringList> &args);
     bool test(const QString &v, const QString &file, int line)
-        { m_current.clear(); return evaluateConditional(v, file, line); }
+        { m_current.clear(); return evaluateConditional(v, file, line) == ReturnTrue; }
     bool test(const ProKey &func, const QList<ProStringList> &args);
 
     bool isSet(const ProKey &v) const { return m_valuemapStack.first().contains(v); }
@@ -78,6 +75,7 @@ public:
     using QMakeEvaluator::evaluateFeatureFile;
     using QMakeEvaluator::evaluateConfigFeatures;
     using QMakeEvaluator::evaluateExpression;
+    using QMakeEvaluator::propertyValue;
     using QMakeEvaluator::values;
     using QMakeEvaluator::first;
     using QMakeEvaluator::isActiveConfig;
