@@ -94,7 +94,7 @@ static inline void positionCluster(HB_ShaperItem *item, int gfrom,  int glast)
                 offsetBase = ((size * 10) - markTotalHeight) / 2; // Use offset that just fits
         }
 
-    bool rightToLeft = item->item.bidiLevel % 2;
+    const bool rightToLeft = (item->item.bidiLevel % 2) != 0;
 
     int i;
     unsigned char lastCmb = 0;
@@ -156,14 +156,12 @@ static inline void positionCluster(HB_ShaperItem *item, int gfrom,  int glast)
                 // ### wrong in rtl context!
         case HB_Combining_BelowLeft:
             p.y += offset;
-            // fall through
         case HB_Combining_BelowLeftAttached:
             p.x += attachmentRect.x - markMetrics.x;
             p.y += (attachmentRect.y + attachmentRect.height) - markMetrics.y;
             break;
         case HB_Combining_Below:
             p.y += offset;
-            // fall through
         case HB_Combining_BelowAttached:
             p.x += attachmentRect.x - markMetrics.x;
             p.y += (attachmentRect.y + attachmentRect.height) - markMetrics.y;
@@ -172,33 +170,28 @@ static inline void positionCluster(HB_ShaperItem *item, int gfrom,  int glast)
             break;
         case HB_Combining_BelowRight:
             p.y += offset;
-            // fall through
         case HB_Combining_BelowRightAttached:
             p.x += attachmentRect.x + attachmentRect.width - markMetrics.width - markMetrics.x;
             p.y += attachmentRect.y + attachmentRect.height - markMetrics.y;
             break;
         case HB_Combining_Left:
             p.x -= offset;
-            // fall through
         case HB_Combining_LeftAttached:
             break;
         case HB_Combining_Right:
             p.x += offset;
-            // fall through
         case HB_Combining_RightAttached:
             break;
         case HB_Combining_DoubleAbove:
             // ### wrong in RTL context!
         case HB_Combining_AboveLeft:
             p.y -= offset;
-            // fall through
         case HB_Combining_AboveLeftAttached:
             p.x += attachmentRect.x - markMetrics.x;
             p.y += attachmentRect.y - markMetrics.y - markMetrics.height;
             break;
         case HB_Combining_Above:
             p.y -= offset;
-            // fall through
         case HB_Combining_AboveAttached:
             p.x += attachmentRect.x - markMetrics.x;
             p.y += attachmentRect.y - markMetrics.y - markMetrics.height;
@@ -207,7 +200,6 @@ static inline void positionCluster(HB_ShaperItem *item, int gfrom,  int glast)
             break;
         case HB_Combining_AboveRight:
             p.y -= offset;
-            // fall through
         case HB_Combining_AboveRightAttached:
             p.x += attachmentRect.x + attachmentRect.width - markMetrics.x - markMetrics.width;
             p.y += attachmentRect.y - markMetrics.y - markMetrics.height;
@@ -289,7 +281,7 @@ void HB_HeuristicSetGlyphAttributes(HB_ShaperItem *item)
 
     // first char in a run is never (treated as) a mark
     int cStart = 0;
-    const bool symbolFont = item->face->isSymbolFont;
+    const bool symbolFont = item->face->isSymbolFont != 0;
     attributes[0].mark = false;
     attributes[0].clusterStart = true;
     attributes[0].dontPrint = (!symbolFont && uc[0] == 0x00ad) || HB_IsControlChar(uc[0]);
