@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -229,6 +235,8 @@
     Calls \a function once for each item in \a sequence. The \a function is
     passed a reference to the item, so that any modifications done to the item
     will appear in \a sequence.
+
+    \sa {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -237,6 +245,8 @@
     Calls \a function once for each item from \a begin to \a end. The
     \a function is passed a reference to the item, so that any modifications
     done to the item will appear in the sequence which the iterators belong to.
+
+    \sa {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -245,6 +255,8 @@
     Calls \a function once for each item in \a sequence and returns a future
     with each mapped item as a result. You can use QFuture::const_iterator or
     QFutureIterator to iterate through the results.
+
+    \sa {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -253,6 +265,8 @@
     Calls \a function once for each item from \a begin to \a end and returns a
     future with each mapped item as a result. You can use
     QFuture::const_iterator or QFutureIterator to iterate through the results.
+
+    \sa {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -266,6 +280,8 @@
     Note that while \a mapFunction is called concurrently, only one thread at a
     time will call \a reduceFunction. The order in which \a reduceFunction is
     called is determined by \a reduceOptions.
+
+    \sa {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -281,6 +297,8 @@
     \a reduceFunction is called is undefined.
 
     \note QtConcurrent::OrderedReduce results in the ordered reduction.
+
+    \sa {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -292,7 +310,7 @@
 
   \note This function will block until all items in the sequence have been processed.
 
-  \sa map()
+  \sa map(), {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -305,7 +323,7 @@
   \note This function will block until the iterator reaches the end of the
   sequence being processed.
 
-  \sa map()
+  \sa map(), {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -316,7 +334,7 @@
 
   \note This function will block until all items in the sequence have been processed.
 
-  \sa mapped()
+  \sa mapped(), {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -333,7 +351,7 @@
   \note This function will block until the iterator reaches the end of the
   sequence being processed.
 
-  \sa mapped()
+  \sa mapped(), {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -348,7 +366,7 @@
 
   \note This function will block until all items in the sequence have been processed.
 
-  \sa mapped()
+  \sa mapped(), {Concurrent Map and Map-Reduce}
 */
 
 /*!
@@ -364,5 +382,5 @@
   \note This function will block until the iterator reaches the end of the
   sequence being processed.
 
-  \sa blockingMappedReduced()
+  \sa blockingMappedReduced(), {Concurrent Map and Map-Reduce}
 */
