@@ -1,33 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Copyright (C) 2014 Olivier Goffart <ogoffart@woboq.com>
-** Copyright (C) 2014 Intel Corporation.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -48,6 +54,7 @@
 #include "qthread.h"
 #include "private/qloggingregistry_p.h"
 #include "private/qcoreapplication_p.h"
+#include "private/qsimd_p.h"
 #endif
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
@@ -75,17 +82,13 @@
 # include "private/qcore_unix_p.h"
 #endif
 
-#ifndef __has_include
-#  define __has_include(x) 0
-#endif
-
 #ifndef QT_BOOTSTRAPPED
 #if !defined QT_NO_REGULAREXPRESSION
 #  ifdef __UCLIBC__
 #    if __UCLIBC_HAS_BACKTRACE__
 #      define QLOGGING_HAVE_BACKTRACE
 #    endif
-#  elif (defined(__GLIBC__) && defined(__GLIBCXX__)) || (__has_include(<cxxabi.h>) && __has_include(<execinfo.h>))
+#  elif (defined(__GLIBC__) && defined(__GLIBCXX__)) || (QT_HAS_INCLUDE(<cxxabi.h>) && QT_HAS_INCLUDE(<execinfo.h>))
 #    define QLOGGING_HAVE_BACKTRACE
 #  endif
 #endif
@@ -94,7 +97,7 @@
 extern char *__progname;
 #endif
 
-#if defined(Q_OS_LINUX) && (defined(__GLIBC__) || __has_include(<sys/syscall.h>))
+#if defined(Q_OS_LINUX) && (defined(__GLIBC__) || QT_HAS_INCLUDE(<sys/syscall.h>))
 #  include <sys/syscall.h>
 
 # if defined(Q_OS_ANDROID) && !defined(SYS_gettid)
@@ -1120,7 +1123,7 @@ void QMessagePattern::setPattern(const QString &pattern)
                 if (m.hasMatch()) {
                     int depth = m.capturedRef(1).toInt();
                     if (depth <= 0)
-                        error += QStringLiteral("QT_MESSAGE_PATTERN: %{backtrace} depth must be a number greater than 0\n");
+                        error += QLatin1String("QT_MESSAGE_PATTERN: %{backtrace} depth must be a number greater than 0\n");
                     else
                         backtraceDepth = depth;
                 }
@@ -1132,7 +1135,7 @@ void QMessagePattern::setPattern(const QString &pattern)
                 backtraceParams.backtraceSeparator = backtraceSeparator;
                 backtraceArgs.append(backtraceParams);
 #else
-                error += QStringLiteral("QT_MESSAGE_PATTERN: %{backtrace} is not supported by this Qt build\n");
+                error += QLatin1String("QT_MESSAGE_PATTERN: %{backtrace} is not supported by this Qt build\n");
                 tokens[i] = "";
 #endif
             }
@@ -1154,7 +1157,7 @@ void QMessagePattern::setPattern(const QString &pattern)
             else if (lexeme == QLatin1String(endifTokenC)) {
                 tokens[i] = endifTokenC;
                 if (!inIf && !nestedIfError)
-                    error += QStringLiteral("QT_MESSAGE_PATTERN: %{endif} without an %{if-*}\n");
+                    error += QLatin1String("QT_MESSAGE_PATTERN: %{endif} without an %{if-*}\n");
                 inIf = false;
             } else {
                 tokens[i] = emptyTokenC;
@@ -1170,9 +1173,9 @@ void QMessagePattern::setPattern(const QString &pattern)
         }
     }
     if (nestedIfError)
-        error += QStringLiteral("QT_MESSAGE_PATTERN: %{if-*} cannot be nested\n");
+        error += QLatin1String("QT_MESSAGE_PATTERN: %{if-*} cannot be nested\n");
     else if (inIf)
-        error += QStringLiteral("QT_MESSAGE_PATTERN: missing %{endif}\n");
+        error += QLatin1String("QT_MESSAGE_PATTERN: missing %{endif}\n");
     if (!error.isEmpty()) {
 #if defined(Q_OS_WINCE) || defined(Q_OS_WINRT)
         OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
@@ -1192,6 +1195,90 @@ void QMessagePattern::setPattern(const QString &pattern)
     memcpy(literals, literalsVar.constData(), literalsVar.size() * sizeof(const char*));
 }
 
+#if defined(QLOGGING_HAVE_BACKTRACE) && !defined(QT_BOOTSTRAPPED)
+// make sure the function has "Message" in the name so the function is removed
+
+#if ((defined(Q_CC_GNU) && defined(QT_COMPILER_SUPPORTS_SIMD_ALWAYS)) || QT_HAS_ATTRIBUTE(optimize)) \
+    && !defined(Q_CC_INTEL)
+// force skipping the frame pointer, to save the backtrace() function some work
+__attribute__((optimize("omit-frame-pointer")))
+#endif
+static QStringList backtraceFramesForLogMessage(int frameCount)
+{
+    QStringList result;
+    if (frameCount == 0)
+        return result;
+
+    // The results of backtrace_symbols looks like this:
+    //    /lib/libc.so.6(__libc_start_main+0xf3) [0x4a937413]
+    // The offset and function name are optional.
+    // This regexp tries to extract the library name (without the path) and the function name.
+    // This code is protected by QMessagePattern::mutex so it is thread safe on all compilers
+    static QRegularExpression rx(QStringLiteral("^(?:[^(]*/)?([^(/]+)\\(([^+]*)(?:[\\+[a-f0-9x]*)?\\) \\[[a-f0-9x]*\\]$"),
+                                 QRegularExpression::OptimizeOnFirstUsageOption);
+
+    QVarLengthArray<void*, 32> buffer(7 + frameCount);
+    int n = backtrace(buffer.data(), buffer.size());
+    if (n > 0) {
+        int numberPrinted = 0;
+        for (int i = 0; i < n && numberPrinted < frameCount; ++i) {
+            QScopedPointer<char*, QScopedPointerPodDeleter> strings(backtrace_symbols(buffer.data() + i, 1));
+            QString trace = QString::fromLatin1(strings.data()[0]);
+            QRegularExpressionMatch m = rx.match(trace);
+            if (m.hasMatch()) {
+                QString library = m.captured(1);
+                QString function = m.captured(2);
+
+                // skip the trace from QtCore that are because of the qDebug itself
+                if (!numberPrinted && library.contains(QLatin1String("Qt5Core"))
+                        && (function.isEmpty() || function.contains(QLatin1String("Message"), Qt::CaseInsensitive)
+                            || function.contains(QLatin1String("QDebug")))) {
+                    continue;
+                }
+
+                if (function.startsWith(QLatin1String("_Z"))) {
+                    QScopedPointer<char, QScopedPointerPodDeleter> demangled(
+                                abi::__cxa_demangle(function.toUtf8(), 0, 0, 0));
+                    if (demangled)
+                        function = QString::fromUtf8(qCleanupFuncinfo(demangled.data()));
+                }
+
+                if (function.isEmpty()) {
+                    result.append(QLatin1Char('?') + library + QLatin1Char('?'));
+                } else {
+                    result.append(function);
+                }
+            } else {
+                if (numberPrinted == 0) {
+                    // innermost, unknown frames are usually the logging framework itself
+                    continue;
+                }
+                result.append(QStringLiteral("???"));
+            }
+            numberPrinted++;
+        }
+    }
+    return result;
+}
+
+static QString formatBacktraceForLogMessage(const QMessagePattern::BacktraceParams backtraceParams,
+                                            const char *function)
+{
+    QString backtraceSeparator = backtraceParams.backtraceSeparator;
+    int backtraceDepth = backtraceParams.backtraceDepth;
+
+    QStringList frames = backtraceFramesForLogMessage(backtraceDepth);
+    if (frames.isEmpty())
+        return QString();
+
+    // if the first frame is unknown, replace it with the context function
+    if (function && frames.at(0).startsWith(QLatin1Char('?')))
+        frames[0] = QString::fromUtf8(qCleanupFuncinfo(function));
+
+    return frames.join(backtraceSeparator);
+}
+#endif // QLOGGING_HAVE_BACKTRACE && !QT_BOOTSTRAPPED
+
 #if defined(QT_USE_SLOG2)
 #ifndef QT_LOG_CODE
 #define QT_LOG_CODE 9000
@@ -1335,62 +1422,8 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
 #ifdef QLOGGING_HAVE_BACKTRACE
         } else if (token == backtraceTokenC) {
             QMessagePattern::BacktraceParams backtraceParams = pattern->backtraceArgs.at(backtraceArgsIdx);
-            QString backtraceSeparator = backtraceParams.backtraceSeparator;
-            int backtraceDepth = backtraceParams.backtraceDepth;
             backtraceArgsIdx++;
-            QVarLengthArray<void*, 32> buffer(7 + backtraceDepth);
-            int n = backtrace(buffer.data(), buffer.size());
-            if (n > 0) {
-                int numberPrinted = 0;
-                for (int i = 0; i < n && numberPrinted < backtraceDepth; ++i) {
-                    QScopedPointer<char*, QScopedPointerPodDeleter> strings(backtrace_symbols(buffer.data() + i, 1));
-                    QString trace = QString::fromLatin1(strings.data()[0]);
-                    // The results of backtrace_symbols looks like this:
-                    //    /lib/libc.so.6(__libc_start_main+0xf3) [0x4a937413]
-                    // The offset and function name are optional.
-                    // This regexp tries to extract the librry name (without the path) and the function name.
-                    // This code is protected by QMessagePattern::mutex so it is thread safe on all compilers
-                    static QRegularExpression rx(QStringLiteral("^(?:[^(]*/)?([^(/]+)\\(([^+]*)(?:[\\+[a-f0-9x]*)?\\) \\[[a-f0-9x]*\\]$"),
-                                                 QRegularExpression::OptimizeOnFirstUsageOption);
-
-                    QRegularExpressionMatch m = rx.match(trace);
-                    if (m.hasMatch()) {
-                        // skip the trace from QtCore that are because of the qDebug itself
-                        QString library = m.captured(1);
-                        QString function = m.captured(2);
-                        if (!numberPrinted && library.contains(QLatin1String("Qt5Core"))
-                            && (function.isEmpty() || function.contains(QLatin1String("Message"), Qt::CaseInsensitive)
-                                || function.contains(QLatin1String("QDebug")))) {
-                            continue;
-                        }
-
-                        if (function.startsWith(QLatin1String("_Z"))) {
-                            QScopedPointer<char, QScopedPointerPodDeleter> demangled(
-                                abi::__cxa_demangle(function.toUtf8(), 0, 0, 0));
-                            if (demangled)
-                                function = QString::fromUtf8(qCleanupFuncinfo(demangled.data()));
-                        }
-
-                        if (numberPrinted > 0)
-                            message.append(backtraceSeparator);
-
-                        if (function.isEmpty()) {
-                            if (numberPrinted == 0 && context.function)
-                                message += QString::fromUtf8(qCleanupFuncinfo(context.function));
-                            else
-                                message += QLatin1Char('?') + library + QLatin1Char('?');
-                        } else {
-                            message += function;
-                        }
-
-                    } else {
-                        if (numberPrinted == 0)
-                            continue;
-                        message += backtraceSeparator + QLatin1String("???");
-                    }
-                    numberPrinted++;
-                }
-            }
+            message.append(formatBacktraceForLogMessage(backtraceParams, context.function));
 #endif
         } else if (token == timeTokenC) {
             QString timeFormat = pattern->timeArgs.at(timeArgsIdx);
@@ -1748,7 +1781,7 @@ void qErrnoWarning(int code, const char *msg, ...)
 
     \snippet code/src_corelib_global_qglobal.cpp 23
 
-    \sa QtMessageHandler, QtMsgType, qDebug(), qInfo(), qWarning(), qCritical(), qFatal(),
+    \sa QtMessageHandler, QtMsgType, qDebug(), qWarning(), qCritical(), qFatal(),
     {Debugging Techniques}
 */
 
