@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,10 +47,11 @@
 /*
    The operating system, must be one of: (Q_OS_x)
 
-     DARWIN   - Any Darwin system
-     MAC      - OS X and iOS
-     OSX      - OS X
+     DARWIN   - Any Darwin system (OS X, iOS, watchOS, tvOS)
+     MACOS    - macOS
      IOS      - iOS
+     WATCHOS  - watchOS
+     TVOS     - tvOS
      MSDOS    - MS-DOS and Windows
      OS2      - OS/2
      OS2EMX   - XFree86 on OS/2 (not PM)
@@ -60,6 +67,7 @@
      NETBSD   - NetBSD
      OPENBSD  - OpenBSD
      BSDI     - BSD/OS
+     INTERIX  - Interix
      IRIX     - SGI Irix
      OSF      - HP Tru64 UNIX
      SCO      - SCO OpenServer 5
@@ -80,21 +88,39 @@
    The following operating systems have variants:
      LINUX    - both Q_OS_LINUX and Q_OS_ANDROID are defined when building for Android
               - only Q_OS_LINUX is defined if building for other Linux systems
-     QNX      - both Q_OS_QNX and Q_OS_BLACKBERRY are defined when building for Blackberry 10
-              - only Q_OS_QNX is defined if building for other QNX targets
      FREEBSD  - Q_OS_FREEBSD is defined only when building for FreeBSD with a BSD userland
               - Q_OS_FREEBSD_KERNEL is always defined on FreeBSD, even if the userland is from GNU
 */
 
 #if defined(__APPLE__) && (defined(__GNUC__) || defined(__xlC__) || defined(__xlc__))
-#  define Q_OS_DARWIN
-#  define Q_OS_BSD4
-#  ifdef __LP64__
-#    define Q_OS_DARWIN64
+#  include <TargetConditionals.h>
+#  if defined(TARGET_OS_MAC) && TARGET_OS_MAC
+#    define Q_OS_DARWIN
+#    define Q_OS_BSD4
+#    ifdef __LP64__
+#      define Q_OS_DARWIN64
+#    else
+#      define Q_OS_DARWIN32
+#    endif
+#    if defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE
+#      if defined(TARGET_OS_TV) && TARGET_OS_TV
+#        define Q_OS_TVOS
+#      elif defined(TARGET_OS_WATCH) && TARGET_OS_WATCH
+#        define Q_OS_WATCHOS
+#      else
+#        // TARGET_OS_IOS is only available in newer SDKs,
+#        // so assume any other iOS-based platform is iOS for now
+#        define Q_OS_IOS
+#      endif
+#    else
+#      // there is no "real" OS X define (rdar://22640089),
+#      // assume any non iOS-based platform is OS X for now
+#      define Q_OS_MACOS
+#    endif
 #  else
-#    define Q_OS_DARWIN32
+#    error "Qt has not been ported to this Apple platform - see http://www.qt.io/developers"
 #  endif
-#elif defined(ANDROID)
+#elif defined(__ANDROID__) || defined(ANDROID)
 #  define Q_OS_ANDROID
 #  define Q_OS_LINUX
 #elif defined(__CYGWIN__)
@@ -106,10 +132,13 @@
 #  if defined(WINCE) || defined(_WIN32_WCE)
 #    define Q_OS_WINCE
 #  elif defined(WINAPI_FAMILY)
+#    ifndef WINAPI_FAMILY_PC_APP
+#      define WINAPI_FAMILY_PC_APP WINAPI_FAMILY_APP
+#    endif
 #    if defined(WINAPI_FAMILY_PHONE_APP) && WINAPI_FAMILY==WINAPI_FAMILY_PHONE_APP
 #      define Q_OS_WINPHONE
 #      define Q_OS_WINRT
-#    elif WINAPI_FAMILY==WINAPI_FAMILY_APP
+#    elif WINAPI_FAMILY==WINAPI_FAMILY_PC_APP
 #      define Q_OS_WINRT
 #    else
 #      define Q_OS_WIN32
@@ -144,6 +173,9 @@
 #elif defined(__bsdi__)
 #  define Q_OS_BSDI
 #  define Q_OS_BSD4
+#elif defined(__INTERIX)
+#  define Q_OS_INTERIX
+#  define Q_OS_BSD4
 #elif defined(__sgi)
 #  define Q_OS_IRIX
 #elif defined(__osf__)
@@ -181,33 +213,32 @@
 #  define Q_OS_WIN
 #endif
 
-#if defined(Q_OS_DARWIN)
-#  define Q_OS_MAC
-#  if defined(Q_OS_DARWIN64)
-#     define Q_OS_MAC64
-#  elif defined(Q_OS_DARWIN32)
-#     define Q_OS_MAC32
-#  endif
-#  include <TargetConditionals.h>
-#  if defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE
-#     define Q_OS_IOS
-#  elif defined(TARGET_OS_MAC) && TARGET_OS_MAC
-#     define Q_OS_OSX
-#     define Q_OS_MACX // compatibility synonym
-#  endif
-#endif
-
 #if defined(Q_OS_WIN)
 #  undef Q_OS_UNIX
 #elif !defined(Q_OS_UNIX)
 #  define Q_OS_UNIX
 #endif
 
+// Compatibility synonyms
+#ifdef Q_OS_DARWIN
+#define Q_OS_MAC
+#endif
+#ifdef Q_OS_DARWIN32
+#define Q_OS_MAC32
+#endif
+#ifdef Q_OS_DARWIN64
+#define Q_OS_MAC64
+#endif
+#ifdef Q_OS_MACOS
+#define Q_OS_MACX
+#define Q_OS_OSX
+#endif
+
 #ifdef Q_OS_DARWIN
 #  include <Availability.h>
 #  include <AvailabilityMacros.h>
 #
-#  ifdef Q_OS_OSX
+#  ifdef Q_OS_MACOS
 #    if !defined(__MAC_OS_X_VERSION_MIN_REQUIRED) || __MAC_OS_X_VERSION_MIN_REQUIRED < __MAC_10_6
 #       undef __MAC_OS_X_VERSION_MIN_REQUIRED
 #       define __MAC_OS_X_VERSION_MIN_REQUIRED __MAC_10_6
@@ -236,6 +267,9 @@
 #  if !defined(__MAC_10_11)
 #       define __MAC_10_11 101100
 #  endif
+#  if !defined(__MAC_10_12)
+#       define __MAC_10_12 101200
+#  endif
 #  if !defined(MAC_OS_X_VERSION_10_7)
 #       define MAC_OS_X_VERSION_10_7 1070
 #  endif
@@ -251,6 +285,9 @@
 #  if !defined(MAC_OS_X_VERSION_10_11)
 #       define MAC_OS_X_VERSION_10_11 101100
 #  endif
+#  if !defined(MAC_OS_X_VERSION_10_12)
+#       define MAC_OS_X_VERSION_10_12 101200
+#  endif
 #
 #  if !defined(__IPHONE_4_3)
 #       define __IPHONE_4_3 40300
@@ -291,6 +328,18 @@
 #  if !defined(__IPHONE_9_0)
 #       define __IPHONE_9_0 90000
 #  endif
+#  if !defined(__IPHONE_9_1)
+#       define __IPHONE_9_1 90100
+#  endif
+#  if !defined(__IPHONE_9_2)
+#       define __IPHONE_9_2 90200
+#  endif
+#  if !defined(__IPHONE_9_3)
+#       define __IPHONE_9_3 90300
+#  endif
+#  if !defined(__IPHONE_10_0)
+#       define __IPHONE_10_0 100000
+#  endif
 #endif
 
 #ifdef __LSB_VERSION__
