@@ -1,33 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2017 Intel Corporation.
-** Copyright (C) 2015 The Qt Company Ltd.
+** Copyright (C) 2016 The Qt Company Ltd.
 ** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -60,13 +65,6 @@
 #include <MobileCoreServices/MobileCoreServices.h>
 #endif
 
-#if defined(Q_OS_DARWIN)
-// We cannot include <Foundation/Foundation.h> (it's an Objective-C header), but
-// we need these declarations:
-Q_FORWARD_DECLARE_OBJC_CLASS(NSString);
-extern "C" NSString *NSTemporaryDirectory();
-#endif
-
 QT_BEGIN_NAMESPACE
 
 #if defined(Q_OS_DARWIN)
@@ -121,7 +119,7 @@ static bool isPackage(const QFileSystemMetaData &data, const QFileSystemEntry &e
         // Find if an application other than Finder claims to know how to handle the package
         QCFType<CFURLRef> application;
         LSGetApplicationForURL(url,
-                               kLSRolesEditor|kLSRolesViewer|kLSRolesViewer,
+                               kLSRolesEditor|kLSRolesViewer,
                                NULL,
                                &application);
 
@@ -284,7 +282,7 @@ QFileSystemEntry QFileSystemEngine::canonicalName(const QFileSystemEntry &entry,
     if (ret) {
         data.knownFlagsMask |= QFileSystemMetaData::ExistsAttribute;
         data.entryFlags |= QFileSystemMetaData::ExistsAttribute;
-        QString canonicalPath = QDir::cleanPath(QFile::decodeName(ret));
+        QString canonicalPath = QDir::cleanPath(QString::fromLocal8Bit(ret));
         free(ret);
         return QFileSystemEntry(canonicalPath);
     } else if (errno == ENOENT) { // file doesn't exist
@@ -353,17 +351,17 @@ QString QFileSystemEngine::resolveUserName(uint userId)
     QVarLengthArray<char, 1024> buf(size_max);
 #endif
 
-    struct passwd *pw = 0;
 #if !defined(Q_OS_INTEGRITY)
+    struct passwd *pw = 0;
 #if !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && !defined(Q_OS_OPENBSD) && !defined(Q_OS_VXWORKS)
     struct passwd entry;
     getpwuid_r(userId, &entry, buf.data(), buf.size(), &pw);
 #else
     pw = getpwuid(userId);
-#endif
 #endif
     if (pw)
         return QFile::decodeName(QByteArray(pw->pw_name));
+#endif
     return QString();
 }
 
@@ -377,8 +375,8 @@ QString QFileSystemEngine::resolveGroupName(uint groupId)
     QVarLengthArray<char, 1024> buf(size_max);
 #endif
 
-    struct group *gr = 0;
 #if !defined(Q_OS_INTEGRITY)
+    struct group *gr = 0;
 #if !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && !defined(Q_OS_OPENBSD) && !defined(Q_OS_VXWORKS)
     size_max = sysconf(_SC_GETGR_R_SIZE_MAX);
     if (size_max == -1)
@@ -397,10 +395,10 @@ QString QFileSystemEngine::resolveGroupName(uint groupId)
     }
 #else
     gr = getgrgid(groupId);
-#endif
 #endif
     if (gr)
         return QFile::decodeName(QByteArray(gr->gr_name));
+#endif
     return QString();
 }
 
@@ -550,76 +548,45 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     return data.hasFlags(what);
 }
 
-static bool pathIsDir(const QByteArray &nativeName)
-{
-    // helper function to check if a given path is a directory, since mkdir can
-    // fail if the dir already exists (it may have been created by another
-    // thread or another process)
-    QT_STATBUF st;
-    return QT_STAT(nativeName.constData(), &st) == 0 && (st.st_mode & S_IFMT) == S_IFDIR;
-};
-
-// Note: if \a shouldMkdirFirst is false, we assume the caller did try to mkdir
-// before calling this function.
-static bool createDirectoryWithParents(const QByteArray &nativeName, bool shouldMkdirFirst = true)
-{
-    if (shouldMkdirFirst && QT_MKDIR(nativeName, 0777) == 0)
-        return true;
-    if (errno == EEXIST)
-        return pathIsDir(nativeName);
-    if (errno != ENOENT)
-        return false;
-
-    // mkdir failed because the parent dir doesn't exist, so try to create it
-    int slash = nativeName.lastIndexOf('/');
-    if (slash < 1)
-        return false;
-
-    QByteArray parentNativeName = nativeName.left(slash);
-    if (!createDirectoryWithParents(parentNativeName))
-        return false;
-
-    // try again
-    if (QT_MKDIR(nativeName, 0777) == 0)
-        return true;
-    return errno == EEXIST && pathIsDir(nativeName);
-}
-
 //static
 bool QFileSystemEngine::createDirectory(const QFileSystemEntry &entry, bool createParents)
 {
     QString dirName = entry.filePath();
-
-    // Darwin doesn't support trailing /'s, so remove for everyone
-    while (dirName.size() > 1 && dirName.endsWith(QLatin1Char('/')))
-        dirName.chop(1);
-
-    // try to mkdir this directory
-    QByteArray nativeName = QFile::encodeName(dirName);
-    if (QT_MKDIR(nativeName, 0777) == 0)
+    if (createParents) {
+        dirName = QDir::cleanPath(dirName);
+        for (int oldslash = -1, slash=0; slash != -1; oldslash = slash) {
+            slash = dirName.indexOf(QDir::separator(), oldslash+1);
+            if (slash == -1) {
+                if (oldslash == dirName.length())
+                    break;
+                slash = dirName.length();
+            }
+            if (slash) {
+                const QByteArray chunk = QFile::encodeName(dirName.left(slash));
+                if (QT_MKDIR(chunk.constData(), 0777) != 0) {
+                    if (errno == EEXIST
+#if defined(Q_OS_QNX)
+                        // On QNX the QNet (VFS paths of other hosts mounted under a directory
+                        // such as /net) mountpoint returns ENOENT, despite existing. stat()
+                        // on the QNet mountpoint returns successfully and reports S_IFDIR.
+                        || errno == ENOENT
+#endif
+                    ) {
+                        QT_STATBUF st;
+                        if (QT_STAT(chunk.constData(), &st) == 0 && (st.st_mode & S_IFMT) == S_IFDIR)
+                            continue;
+                    }
+                    return false;
+                }
+            }
+        }
         return true;
-    if (!createParents)
-        return false;
-
-    // we need the cleaned path in order to create the parents
-    // and we save errno just in case encodeName needs to load codecs
-    int savedErrno = errno;
-    bool pathChanged;
-    {
-        QString cleanName = QDir::cleanPath(dirName);
-
-        // Check if the cleaned name is the same or not. If we were given a
-        // path with resolvable "../" sections, cleanPath will remove them, but
-        // this may change the target dir if one of those segments was a
-        // symlink. This operation depends on cleanPath's optimization of
-        // returning the original string if it didn't modify anything.
-        pathChanged = !dirName.isSharedWith(cleanName);
-        if (pathChanged)
-            nativeName = QFile::encodeName(cleanName);
     }
-
-    errno = savedErrno;
-    return createDirectoryWithParents(nativeName, pathChanged);
+#if defined(Q_OS_DARWIN)  // Mac X doesn't support trailing /'s
+    if (dirName.endsWith(QLatin1Char('/')))
+        dirName.chop(1);
+#endif
+    return (QT_MKDIR(QFile::encodeName(dirName).constData(), 0777) == 0);
 }
 
 //static
@@ -733,29 +700,10 @@ QString QFileSystemEngine::tempPath()
 {
 #ifdef QT_UNIX_TEMP_PATH_OVERRIDE
     return QLatin1String(QT_UNIX_TEMP_PATH_OVERRIDE);
-#elif defined(Q_OS_BLACKBERRY)
-    QString temp = QFile::decodeName(qgetenv("TEMP"));
-    if (temp.isEmpty())
-        temp = QFile::decodeName(qgetenv("TMPDIR"));
-
-    if (temp.isEmpty()) {
-        qWarning("Neither the TEMP nor the TMPDIR environment variable is set, falling back to /var/tmp.");
-        temp = QLatin1String("/var/tmp");
-    }
-    return QDir::cleanPath(temp);
 #else
     QString temp = QFile::decodeName(qgetenv("TMPDIR"));
-    if (temp.isEmpty()) {
-#if defined(Q_OS_DARWIN) && !defined(QT_BOOTSTRAPPED)
-        if (NSString *nsPath = NSTemporaryDirectory()) {
-            temp = QString::fromCFString((CFStringRef)nsPath);
-        } else {
-#else
-        {
-#endif
-            temp = QLatin1String("/tmp");
-        }
-    }
+    if (temp.isEmpty())
+        temp = QLatin1String("/tmp");
     return QDir::cleanPath(temp);
 #endif
 }
