@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -61,8 +59,7 @@ static bool isUncRoot(const QString &server)
     if (idx == -1 || idx + 1 == localPath.length())
         return true;
 
-    localPath = localPath.right(localPath.length() - idx - 1).trimmed();
-    return localPath.isEmpty();
+    return localPath.rightRef(localPath.length() - idx - 1).trimmed().isEmpty();
 }
 
 static inline QString fixIfRelativeUncPath(const QString &path)
@@ -169,6 +166,18 @@ void QFileSystemEntry::resolveNativeFilePath() const
         m_nativeFilePath = QDir::toNativeSeparators(m_filePath);
 #else
         m_nativeFilePath = QFile::encodeName(QDir::toNativeSeparators(m_filePath));
+#endif
+#ifdef Q_OS_WINRT
+        while (m_nativeFilePath.startsWith(QLatin1Char('\\')))
+            m_nativeFilePath.remove(0,1);
+        if (m_nativeFilePath.isEmpty())
+            m_nativeFilePath.append(QLatin1Char('.'));
+        // WinRT/MSVC2015 allows a maximum of 256 characters for a filepath
+        // unless //?/ is prepended which extends the rule to have a maximum
+        // of 256 characters in the filename plus the preprending path
+#if _MSC_VER >= 1900
+        m_nativeFilePath.prepend("\\\\?\\");
+#endif
 #endif
     }
 }
@@ -189,7 +198,7 @@ QString QFileSystemEntry::path() const
     if (m_lastSeparator == -1) {
 #if defined(Q_OS_WIN)
         if (m_filePath.length() >= 2 && m_filePath.at(1) == QLatin1Char(':'))
-            return QFSFileEngine::currentPath(m_filePath.left(2));
+            return m_filePath.left(2);
 #endif
         return QString(QLatin1Char('.'));
     }
@@ -257,17 +266,21 @@ QString QFileSystemEntry::completeSuffix() const
 bool QFileSystemEntry::isRelative() const
 {
     resolveFilePath();
-    return (m_filePath.isEmpty() || (!m_filePath.isEmpty() && (m_filePath[0].unicode() != '/')
-        && (!(m_filePath.length() >= 2 && m_filePath[1].unicode() == ':'))));
+    return (m_filePath.isEmpty()
+            || (m_filePath.at(0).unicode() != '/'
+                && !(m_filePath.length() >= 2 && m_filePath.at(1).unicode() == ':')));
 }
 
 bool QFileSystemEntry::isAbsolute() const
 {
     resolveFilePath();
-    return (!m_filePath.isEmpty() && ((m_filePath.length() >= 3
-                                       && (m_filePath[0].isLetter() && m_filePath[1].unicode() == ':' && m_filePath[2].unicode() == '/'))
-                                      || (m_filePath.length() >= 2 && (m_filePath.at(0) == QLatin1Char('/') && m_filePath.at(1) == QLatin1Char('/')))
-                                      ));
+    return ((m_filePath.length() >= 3
+             && m_filePath.at(0).isLetter()
+             && m_filePath.at(1).unicode() == ':'
+             && m_filePath.at(2).unicode() == '/')
+         || (m_filePath.length() >= 2
+             && m_filePath.at(0) == QLatin1Char('/')
+             && m_filePath.at(1) == QLatin1Char('/')));
 }
 #else
 bool QFileSystemEntry::isRelative() const
@@ -278,7 +291,7 @@ bool QFileSystemEntry::isRelative() const
 bool QFileSystemEntry::isAbsolute() const
 {
     resolveFilePath();
-    return (!m_filePath.isEmpty() && (m_filePath[0].unicode() == '/'));
+    return (!m_filePath.isEmpty() && (m_filePath.at(0).unicode() == '/'));
 }
 #endif
 
@@ -286,9 +299,13 @@ bool QFileSystemEntry::isAbsolute() const
 bool QFileSystemEntry::isDriveRoot() const
 {
     resolveFilePath();
+#ifndef Q_OS_WINRT
     return (m_filePath.length() == 3
            && m_filePath.at(0).isLetter() && m_filePath.at(1) == QLatin1Char(':')
            && m_filePath.at(2) == QLatin1Char('/'));
+#else // !Q_OS_WINRT
+    return m_filePath == QDir::rootPath();
+#endif // !Q_OS_WINRT
 }
 #endif
 
@@ -317,13 +334,7 @@ void QFileSystemEntry::findLastSeparator() const
 {
     if (m_lastSeparator == -2) {
         resolveFilePath();
-        m_lastSeparator = -1;
-        for (int i = m_filePath.size() - 1; i >= 0; --i) {
-            if (m_filePath[i].unicode() == '/') {
-                m_lastSeparator = i;
-                break;
-            }
-        }
+        m_lastSeparator = m_filePath.lastIndexOf(QLatin1Char('/'));
     }
 }
 
@@ -345,10 +356,10 @@ void QFileSystemEntry::findFileNameSeparators() const
 
         int i = m_filePath.size() - 1;
         for (; i >= stop; --i) {
-            if (m_filePath[i].unicode() == '.') {
+            if (m_filePath.at(i).unicode() == '.') {
                 firstDotInFileName = lastDotInFileName = i;
                 break;
-            } else if (m_filePath[i].unicode() == '/') {
+            } else if (m_filePath.at(i).unicode() == '/') {
                 lastSeparator = i;
                 break;
             }
@@ -356,9 +367,9 @@ void QFileSystemEntry::findFileNameSeparators() const
 
         if (lastSeparator != i) {
             for (--i; i >= stop; --i) {
-                if (m_filePath[i].unicode() == '.')
+                if (m_filePath.at(i).unicode() == '.')
                     firstDotInFileName = i;
-                else if (m_filePath[i].unicode() == '/') {
+                else if (m_filePath.at(i).unicode() == '/') {
                     lastSeparator = i;
                     break;
                 }
@@ -381,7 +392,7 @@ bool QFileSystemEntry::isClean() const
     int dots = 0;
     bool dotok = true; // checking for ".." or "." starts to relative paths
     bool slashok = true;
-    for (QString::const_iterator iter = m_filePath.constBegin(); iter != m_filePath.constEnd(); iter++) {
+    for (QString::const_iterator iter = m_filePath.constBegin(); iter != m_filePath.constEnd(); ++iter) {
         if (*iter == QLatin1Char('/')) {
             if (dots == 1 || dots == 2)
                 return false; // path contains "./" or "../"
