@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -81,7 +87,7 @@ bool QFseventsFileSystemWatcherEngine::checkDir(DirsByName::iterator &it)
     if (res == -1) {
         needsRestart |= derefPath(info.watchedPath);
         emit emitDirectoryChanged(info.origPath, true);
-        it = watchingState.watchedDirectories.erase(it);
+        it = watchedDirectories.erase(it);
     } else if (st.st_ctimespec != info.ctime || st.st_mode != info.mode) {
         info.ctime = st.st_ctimespec;
         info.mode = st.st_mode;
@@ -132,8 +138,7 @@ bool QFseventsFileSystemWatcherEngine::rescanDirs(const QString &path)
 {
     bool needsRestart = false;
 
-    for (DirsByName::iterator it = watchingState.watchedDirectories.begin();
-            it != watchingState.watchedDirectories.end(); ) {
+    for (DirsByName::iterator it = watchedDirectories.begin(); it != watchedDirectories.end(); ) {
         if (it.key().startsWith(path))
             needsRestart |= checkDir(it);
         else
@@ -172,12 +177,11 @@ bool QFseventsFileSystemWatcherEngine::rescanFiles(const QString &path)
 {
     bool needsRestart = false;
 
-    for (FilesByPath::iterator i = watchingState.watchedFiles.begin();
-            i != watchingState.watchedFiles.end(); ) {
+    for (FilesByPath::iterator i = watchedFiles.begin(); i != watchedFiles.end(); ) {
         if (i.key().startsWith(path)) {
             needsRestart |= rescanFiles(i.value());
             if (i.value().isEmpty()) {
-                i = watchingState.watchedFiles.erase(i);
+                i = watchedFiles.erase(i);
                 continue;
             }
         }
@@ -228,8 +232,8 @@ void QFseventsFileSystemWatcherEngine::processEvent(ConstFSEventStreamRef stream
 
         if (eFlags & kFSEventStreamEventFlagRootChanged) {
             // re-check everything:
-            DirsByName::iterator dirIt = watchingState.watchedDirectories.find(path);
-            if (dirIt != watchingState.watchedDirectories.end())
+            DirsByName::iterator dirIt = watchedDirectories.find(path);
+            if (dirIt != watchedDirectories.end())
                 needsRestart |= checkDir(dirIt);
             needsRestart |= rescanFiles(path);
             continue;
@@ -239,13 +243,13 @@ void QFseventsFileSystemWatcherEngine::processEvent(ConstFSEventStreamRef stream
             needsRestart |= rescanDirs(path);
 
         // check watched directories:
-        DirsByName::iterator dirIt = watchingState.watchedDirectories.find(path);
-        if (dirIt != watchingState.watchedDirectories.end())
+        DirsByName::iterator dirIt = watchedDirectories.find(path);
+        if (dirIt != watchedDirectories.end())
             needsRestart |= checkDir(dirIt);
 
         // check watched files:
-        FilesByPath::iterator pIt = watchingState.watchedFiles.find(path);
-        if (pIt != watchingState.watchedFiles.end())
+        FilesByPath::iterator pIt = watchedFiles.find(path);
+        if (pIt != watchedFiles.end())
             needsRestart |= rescanFiles(pIt.value());
     }
 
@@ -278,11 +282,12 @@ void QFseventsFileSystemWatcherEngine::doEmitDirectoryChanged(const QString &pat
     emit directoryChanged(path, removed);
 }
 
-bool QFseventsFileSystemWatcherEngine::restartStream()
+void QFseventsFileSystemWatcherEngine::restartStream()
 {
+    QMacAutoReleasePool pool;
     QMutexLocker locker(&lock);
     stopStream();
-    return startStream();
+    startStream();
 }
 
 QFseventsFileSystemWatcherEngine *QFseventsFileSystemWatcherEngine::create(QObject *parent)
@@ -312,7 +317,6 @@ QFseventsFileSystemWatcherEngine::~QFseventsFileSystemWatcherEngine()
 {
     QMacAutoReleasePool pool;
 
-    // Stop the stream in case we have to wait for the lock below to be acquired.
     if (stream)
         FSEventStreamStop(stream);
 
@@ -336,14 +340,12 @@ QStringList QFseventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
 
     QMutexLocker locker(&lock);
 
-    bool wasRunning = stream != Q_NULLPTR;
     bool needsRestart = false;
 
-    WatchingState oldState = watchingState;
     QStringList p = paths;
     QMutableListIterator<QString> it(p);
     while (it.hasNext()) {
-        QString origPath = it.next().normalized(QString::NormalizationForm_C);
+        QString origPath = it.next();
         QString realPath = origPath;
         if (realPath.endsWith(QDir::separator()))
             realPath = realPath.mid(0, realPath.size() - 1);
@@ -360,7 +362,7 @@ QStringList QFseventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
 
         const bool isDir = S_ISDIR(st.st_mode);
         if (isDir) {
-            if (watchingState.watchedDirectories.contains(realPath))
+            if (watchedDirectories.contains(realPath))
                 continue;
             directories->append(origPath);
             watchedPath = realPath;
@@ -375,18 +377,17 @@ QStringList QFseventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
             parentPath = watchedPath;
         }
 
-        for (PathRefCounts::const_iterator i = watchingState.watchedPaths.begin(),
-                ei = watchingState.watchedPaths.end(); i != ei; ++i) {
+        for (PathRefCounts::const_iterator i = watchedPaths.begin(), ei = watchedPaths.end(); i != ei; ++i) {
             if (watchedPath.startsWith(i.key())) {
                 watchedPath = i.key();
                 break;
             }
         }
 
-        PathRefCounts::iterator it = watchingState.watchedPaths.find(watchedPath);
-        if (it == watchingState.watchedPaths.end()) {
+        PathRefCounts::iterator it = watchedPaths.find(watchedPath);
+        if (it == watchedPaths.end()) {
             needsRestart = true;
-            watchingState.watchedPaths.insert(watchedPath, 1);
+            watchedPaths.insert(watchedPath, 1);
             DEBUG("Adding '%s' to watchedPaths", qPrintable(watchedPath));
         } else {
             ++it.value();
@@ -397,25 +398,18 @@ QStringList QFseventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
             DirInfo dirInfo;
             dirInfo.dirInfo = info;
             dirInfo.entries = scanForDirEntries(realPath);
-            watchingState.watchedDirectories.insert(realPath, dirInfo);
+            watchedDirectories.insert(realPath, dirInfo);
             DEBUG("-- Also adding '%s' to watchedDirectories", qPrintable(realPath));
         } else {
-            watchingState.watchedFiles[parentPath].insert(realPath, info);
+            watchedFiles[parentPath].insert(realPath, info);
             DEBUG("-- Also adding '%s' to watchedFiles", qPrintable(realPath));
         }
     }
 
     if (needsRestart) {
         stopStream();
-        if (!startStream()) {
-            // ok, something went wrong, let's try to restore the previous state
-            watchingState = qMove(oldState);
-            // and because we don't know which path caused the issue (if any), fail on all of them
+        if (!startStream())
             p = paths;
-
-            if (wasRunning)
-                startStream();
-        }
     }
 
     return p;
@@ -431,7 +425,6 @@ QStringList QFseventsFileSystemWatcherEngine::removePaths(const QStringList &pat
 
     bool needsRestart = false;
 
-    WatchingState oldState = watchingState;
     QStringList p = paths;
     QMutableListIterator<QString> it(p);
     while (it.hasNext()) {
@@ -444,10 +437,10 @@ QStringList QFseventsFileSystemWatcherEngine::removePaths(const QStringList &pat
         realPath = fi.canonicalFilePath();
 
         if (fi.isDir()) {
-            DirsByName::iterator dirIt = watchingState.watchedDirectories.find(realPath);
-            if (dirIt != watchingState.watchedDirectories.end()) {
+            DirsByName::iterator dirIt = watchedDirectories.find(realPath);
+            if (dirIt != watchedDirectories.end()) {
                 needsRestart |= derefPath(dirIt->dirInfo.watchedPath);
-                watchingState.watchedDirectories.erase(dirIt);
+                watchedDirectories.erase(dirIt);
                 directories->removeAll(origPath);
                 it.remove();
                 DEBUG("Removed directory '%s'", qPrintable(realPath));
@@ -455,15 +448,15 @@ QStringList QFseventsFileSystemWatcherEngine::removePaths(const QStringList &pat
         } else {
             QFileInfo fi(realPath);
             QString parentPath = fi.path();
-            FilesByPath::iterator pIt = watchingState.watchedFiles.find(parentPath);
-            if (pIt != watchingState.watchedFiles.end()) {
+            FilesByPath::iterator pIt = watchedFiles.find(parentPath);
+            if (pIt != watchedFiles.end()) {
                 InfoByName &filesInDir = pIt.value();
                 InfoByName::iterator fIt = filesInDir.find(realPath);
                 if (fIt != filesInDir.end()) {
                     needsRestart |= derefPath(fIt->watchedPath);
                     filesInDir.erase(fIt);
                     if (filesInDir.isEmpty())
-                        watchingState.watchedFiles.erase(pIt);
+                        watchedFiles.erase(pIt);
                     files->removeAll(origPath);
                     it.remove();
                     DEBUG("Removed file '%s'", qPrintable(realPath));
@@ -474,33 +467,26 @@ QStringList QFseventsFileSystemWatcherEngine::removePaths(const QStringList &pat
 
     locker.unlock();
 
-    if (needsRestart) {
-        if (!restartStream()) {
-            watchingState = qMove(oldState);
-            startStream();
-        }
-    }
+    if (needsRestart)
+        restartStream();
 
     return p;
 }
 
-// Returns false if FSEventStream* calls failed for some mysterious reason, true if things got a
-// thumbs-up.
 bool QFseventsFileSystemWatcherEngine::startStream()
 {
     Q_ASSERT(stream == 0);
-    if (stream) // Ok, this really shouldn't happen, esp. not after the assert. But let's be nice in release mode and still handle it.
+    QMacAutoReleasePool pool;
+    if (stream) // This shouldn't happen, but let's be nice and handle it.
         stopStream();
 
-    QMacAutoReleasePool pool;
+    if (watchedPaths.isEmpty())
+        return false;
 
-    if (watchingState.watchedPaths.isEmpty())
-        return true; // we succeeded in doing nothing
+    DEBUG() << "Starting stream with paths" << watchedPaths.keys();
 
-    DEBUG() << "Starting stream with paths" << watchingState.watchedPaths.keys();
-
-    NSMutableArray *pathsToWatch = [NSMutableArray arrayWithCapacity:watchingState.watchedPaths.size()];
-    for (PathRefCounts::const_iterator i = watchingState.watchedPaths.begin(), ei = watchingState.watchedPaths.end(); i != ei; ++i)
+    NSMutableArray *pathsToWatch = [NSMutableArray arrayWithCapacity:watchedPaths.size()];
+    for (PathRefCounts::const_iterator i = watchedPaths.begin(), ei = watchedPaths.end(); i != ei; ++i)
         [pathsToWatch addObject:i.key().toNSString()];
 
     struct FSEventStreamContext callBackInfo = {
@@ -524,7 +510,7 @@ bool QFseventsFileSystemWatcherEngine::startStream()
                                  latency,
                                  FSEventStreamCreateFlags(0));
 
-    if (!stream) { // nope, no way to know what went wrong, so just fail
+    if (!stream) {
         DEBUG() << "Failed to create stream!";
         return false;
     }
@@ -534,7 +520,7 @@ bool QFseventsFileSystemWatcherEngine::startStream()
     if (FSEventStreamStart(stream)) {
         DEBUG() << "Stream started successfully with sinceWhen =" << lastReceivedEvent;
         return true;
-    } else { // again, no way to know what went wrong, so just clean up and fail
+    } else {
         DEBUG() << "Stream failed to start!";
         FSEventStreamInvalidate(stream);
         FSEventStreamRelease(stream);
@@ -545,7 +531,6 @@ bool QFseventsFileSystemWatcherEngine::startStream()
 
 void QFseventsFileSystemWatcherEngine::stopStream(bool isStopped)
 {
-    QMacAutoReleasePool pool;
     if (stream) {
         if (!isStopped)
             FSEventStreamStop(stream);
@@ -575,9 +560,9 @@ QFseventsFileSystemWatcherEngine::InfoByName QFseventsFileSystemWatcherEngine::s
 
 bool QFseventsFileSystemWatcherEngine::derefPath(const QString &watchedPath)
 {
-    PathRefCounts::iterator it = watchingState.watchedPaths.find(watchedPath);
-    if (it != watchingState.watchedPaths.end() && --it.value() < 1) {
-        watchingState.watchedPaths.erase(it);
+    PathRefCounts::iterator it = watchedPaths.find(watchedPath);
+    if (it != watchedPaths.end() && --it.value() < 1) {
+        watchedPaths.erase(it);
         DEBUG("Removing '%s' from watchedPaths.", qPrintable(watchedPath));
         return true;
     }
