@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -62,11 +68,11 @@ QWindowsFileSystemWatcherEngine::Handle::Handle()
 
 QWindowsFileSystemWatcherEngine::~QWindowsFileSystemWatcherEngine()
 {
-    foreach(QWindowsFileSystemWatcherEngineThread *thread, threads) {
+    for (auto *thread : qAsConst(threads))
         thread->stop();
+    for (auto *thread : qAsConst(threads))
         thread->wait();
-        delete thread;
-    }
+    qDeleteAll(threads);
 }
 
 QStringList QWindowsFileSystemWatcherEngine::addPaths(const QStringList &paths,
@@ -164,8 +170,8 @@ QStringList QWindowsFileSystemWatcherEngine::addPaths(const QStringList &paths,
 
             // now look for a thread to insert
             bool found = false;
-            foreach(QWindowsFileSystemWatcherEngineThread *thread, threads) {
-                QMutexLocker(&(thread->mutex));
+            for (QWindowsFileSystemWatcherEngineThread *thread : qAsConst(threads)) {
+                QMutexLocker locker(&(thread->mutex));
                 if (thread->handles.count() < MAXIMUM_WAIT_OBJECTS) {
                     DEBUG() << "Added handle" << handle.handle << "for" << absolutePath << "to watch" << fileInfo.absoluteFilePath()
                             << "to existing thread " << thread;
@@ -186,7 +192,7 @@ QStringList QWindowsFileSystemWatcherEngine::addPaths(const QStringList &paths,
             }
             if (!found) {
                 QWindowsFileSystemWatcherEngineThread *thread = new QWindowsFileSystemWatcherEngineThread();
-                DEBUG() << "  ###Creating new thread" << thread << "(" << (threads.count()+1) << "threads)";
+                DEBUG() << "  ###Creating new thread" << thread << '(' << (threads.count()+1) << "threads)";
                 thread->handles.append(handle.handle);
                 thread->handleForDir.insert(QFileSystemWatcherPathKey(absolutePath), handle);
 
@@ -311,7 +317,7 @@ QWindowsFileSystemWatcherEngineThread::~QWindowsFileSystemWatcherEngineThread()
     CloseHandle(handles.at(0));
     handles[0] = INVALID_HANDLE_VALUE;
 
-    foreach (HANDLE h, handles) {
+    for (HANDLE h : qAsConst(handles)) {
         if (h == INVALID_HANDLE_VALUE)
             continue;
         FindCloseChangeNotification(h);
@@ -323,7 +329,7 @@ static inline QString msgFindNextFailed(const QWindowsFileSystemWatcherEngineThr
     QString result;
     QTextStream str(&result);
     str << "QFileSystemWatcher: FindNextChangeNotification failed for";
-    foreach (const QWindowsFileSystemWatcherEngine::PathInfo &pathInfo, pathInfos)
+    for (const QWindowsFileSystemWatcherEngine::PathInfo &pathInfo : pathInfos)
         str << " \"" << QDir::toNativeSeparators(pathInfo.absolutePath) << '"';
     str << ' ';
     return result;
