@@ -1,31 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -724,29 +731,33 @@ qint64 QFSFileEnginePrivate::writeFdFh(const char *data, qint64 len)
 
     qint64 writtenBytes = 0;
 
-    if (fh) {
-        // Buffered stdlib mode.
+    if (len) { // avoid passing nullptr to fwrite() or QT_WRITE() (UB)
 
-        size_t result;
-        do {
-            result = fwrite(data + writtenBytes, 1, size_t(len - writtenBytes), fh);
-            writtenBytes += result;
-        } while (result == 0 ? errno == EINTR : writtenBytes < len);
+        if (fh) {
+            // Buffered stdlib mode.
 
-    } else if (fd != -1) {
-        // Unbuffered stdio mode.
+            size_t result;
+            do {
+                result = fwrite(data + writtenBytes, 1, size_t(len - writtenBytes), fh);
+                writtenBytes += result;
+            } while (result == 0 ? errno == EINTR : writtenBytes < len);
+
+        } else if (fd != -1) {
+            // Unbuffered stdio mode.
+
+            SignedIOType result;
+            do {
+                // calculate the chunk size
+                // on Windows or 32-bit no-largefile Unix, we'll need to read in chunks
+                // we limit to the size of the signed type, otherwise we could get a negative number as a result
+                quint64 wantedBytes = quint64(len) - quint64(writtenBytes);
+                UnsignedIOType chunkSize = std::numeric_limits<SignedIOType>::max();
+                if (chunkSize > wantedBytes)
+                    chunkSize = wantedBytes;
+                result = QT_WRITE(fd, data + writtenBytes, chunkSize);
+            } while (result > 0 && (writtenBytes += result) < len);
+        }
 
-        SignedIOType result;
-        do {
-            // calculate the chunk size
-            // on Windows or 32-bit no-largefile Unix, we'll need to read in chunks
-            // we limit to the size of the signed type, otherwise we could get a negative number as a result
-            quint64 wantedBytes = quint64(len) - quint64(writtenBytes);
-            UnsignedIOType chunkSize = std::numeric_limits<SignedIOType>::max();
-            if (chunkSize > wantedBytes)
-                chunkSize = wantedBytes;
-            result = QT_WRITE(fd, data + writtenBytes, chunkSize);
-        } while (result > 0 && (writtenBytes += result) < len);
     }
 
     if (len &&  writtenBytes == 0) {
@@ -875,9 +886,8 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
 
 /*! \fn QFileInfoList QFSFileEngine::drives()
   For Windows, returns the list of drives in the file system as a list
-  of QFileInfo objects. On Unix and Windows CE, only the
-  root path is returned.  On Windows, this function returns all drives
-  (A:\, C:\, D:\, etc.).
+  of QFileInfo objects. On Unix, only the root path is returned.
+  On Windows, this function returns all drives (A:\, C:\, D:\, and so on).
 
   For Unix, the list contains just the root path "/".
 */
