@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,7 +41,6 @@
 #include "qresource_p.h"
 #include "qresource_iterator_p.h"
 #include "qset.h"
-#include "qhash.h"
 #include "qmutex.h"
 #include "qdebug.h"
 #include "qlocale.h"
@@ -205,7 +202,7 @@ Q_GLOBAL_STATIC(QStringList, resourceSearchPaths)
     path requested in setFileName().
 
     The unregisterResource() function removes a reference to a particular
-    file. If there are QResources that currently reference resources related
+    file. If there are QResource objects that currently reference resources related
     to the unregistered file, they will continue to be valid but the resource
     file itself will be removed from the resource roots, and thus no further
     QResource can be created pointing into this resource data. The resource
@@ -799,6 +796,7 @@ QStringList QResourceRoot::children(int node) const
         offset += 4;
         const int child_off = (tree[offset+0] << 24) + (tree[offset+1] << 16) +
                               (tree[offset+2] << 8) + (tree[offset+3] << 0);
+        ret.reserve(child_count);
         for(int i = child_off; i < child_off+child_count; ++i)
             ret << name(i);
     }
@@ -808,8 +806,8 @@ bool QResourceRoot::mappingRootSubdir(const QString &path, QString *match) const
 {
     const QString root = mappingRoot();
     if(!root.isEmpty()) {
-        const QStringList root_segments = root.split(QLatin1Char('/'), QString::SkipEmptyParts),
-                          path_segments = path.split(QLatin1Char('/'), QString::SkipEmptyParts);
+        const QVector<QStringRef> root_segments = root.splitRef(QLatin1Char('/'), QString::SkipEmptyParts),
+                                  path_segments = path.splitRef(QLatin1Char('/'), QString::SkipEmptyParts);
         if(path_segments.size() <= root_segments.size()) {
             int matched = 0;
             for(int i = 0; i < path_segments.size(); ++i) {
@@ -819,7 +817,7 @@ bool QResourceRoot::mappingRootSubdir(const QString &path, QString *match) const
             }
             if(matched == path_segments.size()) {
                 if(match && root_segments.size() > matched)
-                    *match = root_segments.at(matched);
+                    *match = root_segments.at(matched).toString();
                 return true;
             }
         }
@@ -881,10 +879,16 @@ public:
     inline QDynamicBufferResourceRoot(const QString &_root) : root(_root), buffer(0) { }
     inline ~QDynamicBufferResourceRoot() { }
     inline const uchar *mappingBuffer() const { return buffer; }
-    virtual QString mappingRoot() const { return root; }
-    virtual ResourceRootType type() const { return Resource_Buffer; }
+    virtual QString mappingRoot() const Q_DECL_OVERRIDE { return root; }
+    virtual ResourceRootType type() const Q_DECL_OVERRIDE { return Resource_Buffer; }
+
+    // size == -1 means "unknown"
+    bool registerSelf(const uchar *b, int size)
+    {
+        // 5 int "pointers"
+        if (size >= 0 && size < 20)
+            return false;
 
-    bool registerSelf(const uchar *b) {
         //setup the data now
         int offset = 0;
 
@@ -911,6 +915,10 @@ public:
                                 (b[offset+2] << 8) + (b[offset+3] << 0);
         offset += 4;
 
+        // Some sanity checking for sizes. This is _not_ a security measure.
+        if (size >= 0 && (tree_offset >= size || data_offset >= size || name_offset >= size))
+            return false;
+
         if(version == 0x01) {
             buffer = b;
             setSource(b+tree_offset, b+name_offset, b+data_offset);
@@ -954,11 +962,11 @@ public:
         } else
 #endif
         {
-            delete [] (uchar *)mappingBuffer();
+            delete [] mappingBuffer();
         }
     }
     QString mappingFile() const { return fileName; }
-    virtual ResourceRootType type() const { return Resource_File; }
+    virtual ResourceRootType type() const Q_DECL_OVERRIDE { return Resource_File; }
 
     bool registerSelf(const QString &f) {
         bool fromMM = false;
@@ -1017,7 +1025,7 @@ public:
             }
             fromMM = false;
         }
-        if(data && QDynamicBufferResourceRoot::registerSelf(data)) {
+        if (data && QDynamicBufferResourceRoot::registerSelf(data, data_len)) {
             if(fromMM) {
                 unmapPointer = data;
                 unmapLength = data_len;
@@ -1092,8 +1100,8 @@ QResource::unregisterResource(const QString &rccFilename, const QString &resourc
     for(int i = 0; i < list->size(); ++i) {
         QResourceRoot *res = list->at(i);
         if(res->type() == QResourceRoot::Resource_File) {
-	    QDynamicFileResourceRoot *root = reinterpret_cast<QDynamicFileResourceRoot*>(res);
-	    if(root->mappingFile() == rccFilename && root->mappingRoot() == r) {
+            QDynamicFileResourceRoot *root = reinterpret_cast<QDynamicFileResourceRoot*>(res);
+            if (root->mappingFile() == rccFilename && root->mappingRoot() == r) {
                 resourceList()->removeAt(i);
                 if(!root->ref.deref()) {
                     delete root;
@@ -1101,7 +1109,7 @@ QResource::unregisterResource(const QString &rccFilename, const QString &resourc
                 }
                 return false;
             }
-	}
+        }
     }
     return false;
 }
@@ -1132,7 +1140,7 @@ QResource::registerResource(const uchar *rccData, const QString &resourceRoot)
     }
 
     QDynamicBufferResourceRoot *root = new QDynamicBufferResourceRoot(r);
-    if(root->registerSelf(rccData)) {
+    if (root->registerSelf(rccData, -1)) {
         root->ref.ref();
         QMutexLocker lock(resourceMutex());
         resourceList()->append(root);
@@ -1163,20 +1171,21 @@ QResource::unregisterResource(const uchar *rccData, const QString &resourceRoot)
     for(int i = 0; i < list->size(); ++i) {
         QResourceRoot *res = list->at(i);
         if(res->type() == QResourceRoot::Resource_Buffer) {
-	    QDynamicBufferResourceRoot *root = reinterpret_cast<QDynamicBufferResourceRoot*>(res);
-	    if(root->mappingBuffer() == rccData && root->mappingRoot() == r) {
+            QDynamicBufferResourceRoot *root = reinterpret_cast<QDynamicBufferResourceRoot*>(res);
+            if (root->mappingBuffer() == rccData && root->mappingRoot() == r) {
                 resourceList()->removeAt(i);
                 if(!root->ref.deref()) {
                     delete root;
                     return true;
                 }
-		return false;
+                return false;
             }
-	}
+        }
     }
     return false;
 }
 
+#if !defined(QT_BOOTSTRAPPED)
 //resource engine
 class QResourceFileEnginePrivate : public QAbstractFileEnginePrivate
 {
@@ -1185,9 +1194,10 @@ protected:
 private:
     uchar *map(qint64 offset, qint64 size, QFile::MemoryMapFlags flags);
     bool unmap(uchar *ptr);
+    void uncompress() const;
     qint64 offset;
     QResource resource;
-    QByteArray uncompressed;
+    mutable QByteArray uncompressed;
 protected:
     QResourceFileEnginePrivate() : offset(0) { }
 };
@@ -1222,13 +1232,6 @@ QResourceFileEngine::QResourceFileEngine(const QString &file) :
 {
     Q_D(QResourceFileEngine);
     d->resource.setFileName(file);
-    if(d->resource.isCompressed() && d->resource.size()) {
-#ifndef QT_NO_COMPRESS
-        d->uncompressed = qUncompress(d->resource.data(), d->resource.size());
-#else
-        Q_ASSERT(!"QResourceFileEngine::open: Qt built without support for compression");
-#endif
-    }
 }
 
 QResourceFileEngine::~QResourceFileEngine()
@@ -1250,8 +1253,11 @@ bool QResourceFileEngine::open(QIODevice::OpenMode flags)
     }
     if(flags & QIODevice::WriteOnly)
         return false;
-    if(!d->resource.isValid())
-       return false;
+    d->uncompress();
+    if (!d->resource.isValid()) {
+        d->errorString = qt_error_string(ENOENT);
+        return false;
+    }
     return true;
 }
 
@@ -1313,8 +1319,10 @@ qint64 QResourceFileEngine::size() const
     Q_D(const QResourceFileEngine);
     if(!d->resource.isValid())
         return 0;
-    if(d->resource.isCompressed())
+    if (d->resource.isCompressed()) {
+        d->uncompress();
         return d->uncompressed.size();
+    }
     return d->resource.size();
 }
 
@@ -1381,13 +1389,13 @@ QString QResourceFileEngine::fileName(FileName file) const
 {
     Q_D(const QResourceFileEngine);
     if(file == BaseName) {
-	int slash = d->resource.fileName().lastIndexOf(QLatin1Char('/'));
-	if (slash == -1)
-	    return d->resource.fileName();
-	return d->resource.fileName().mid(slash + 1);
+        int slash = d->resource.fileName().lastIndexOf(QLatin1Char('/'));
+        if (slash == -1)
+            return d->resource.fileName();
+        return d->resource.fileName().mid(slash + 1);
     } else if(file == PathName || file == AbsolutePathName) {
         const QString path = (file == AbsolutePathName) ? d->resource.absoluteFilePath() : d->resource.fileName();
-	const int slash = path.lastIndexOf(QLatin1Char('/'));
+        const int slash = path.lastIndexOf(QLatin1Char('/'));
         if (slash == -1)
             return QLatin1String(":");
         else if (slash <= 1)
@@ -1448,13 +1456,13 @@ bool QResourceFileEngine::extension(Extension extension, const ExtensionOption *
 {
     Q_D(QResourceFileEngine);
     if (extension == MapExtension) {
-        const MapExtensionOption *options = (MapExtensionOption*)(option);
+        const MapExtensionOption *options = (const MapExtensionOption*)(option);
         MapExtensionReturn *returnValue = static_cast<MapExtensionReturn*>(output);
         returnValue->address = d->map(options->offset, options->size, options->flags);
         return (returnValue->address != 0);
     }
     if (extension == UnMapExtension) {
-        UnMapExtensionOption *options = (UnMapExtensionOption*)option;
+        const UnMapExtensionOption *options = (const UnMapExtensionOption*)option;
         return d->unmap(options->address);
     }
     return false;
@@ -1483,4 +1491,17 @@ bool QResourceFileEnginePrivate::unmap(uchar *ptr)
     return true;
 }
 
+void QResourceFileEnginePrivate::uncompress() const
+{
+    if (resource.isCompressed() && uncompressed.isEmpty() && resource.size()) {
+#ifndef QT_NO_COMPRESS
+        uncompressed = qUncompress(resource.data(), resource.size());
+#else
+        Q_ASSERT(!"QResourceFileEngine::open: Qt built without support for compression");
+#endif
+    }
+}
+
+#endif // !defined(QT_BOOTSTRAPPED)
+
 QT_END_NAMESPACE
