@@ -1,31 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -38,16 +45,25 @@
 #include "qdiriterator.h"
 #include "qplatformdefs.h"
 #include <QDebug>
+#include <QPair>
 
 #if defined(QT_BUILD_CORE_LIB)
 #include "qcoreapplication.h"
 #endif
 
+#if !defined(Q_OS_QNX) && !defined(Q_OS_WIN) && !defined(Q_OS_ANDROID) && !defined(Q_OS_INTEGRITY)
+#  define USE_SYSTEM_MKDTEMP
+#endif
+
 #include <stdlib.h> // mkdtemp
-#if defined(Q_OS_QNX) || defined(Q_OS_WIN) || defined(Q_OS_ANDROID)
+#ifndef USE_SYSTEM_MKDTEMP
 #include <private/qfilesystemengine_p.h>
 #endif
 
+#if !defined(Q_OS_WIN)
+#include <errno.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 //************* QTemporaryDirPrivate
@@ -59,7 +75,7 @@ public:
 
     void create(const QString &templateName);
 
-    QString path;
+    QString pathOrError;
     bool autoRemove;
     bool success;
 };
@@ -86,8 +102,7 @@ static QString defaultTemplateName()
     return QDir::tempPath() + QLatin1Char('/') + baseName + QLatin1String("-XXXXXX");
 }
 
-#if defined(Q_OS_QNX ) || defined(Q_OS_WIN) || defined(Q_OS_ANDROID)
-
+#ifndef USE_SYSTEM_MKDTEMP
 static int nextRand(int &v)
 {
     int r = v % 62;
@@ -97,31 +112,28 @@ static int nextRand(int &v)
     return r;
 }
 
-static char *q_mkdtemp(char *templateName)
+QPair<QString, bool> q_mkdtemp(QString templateName)
 {
+    Q_ASSERT(templateName.endsWith(QLatin1String("XXXXXX")));
+
     static const char letters[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 
-    const size_t length = strlen(templateName);
+    const int length = templateName.size();
 
-    char *XXXXXX = templateName + length - 6;
-
-    if ((length < 6u) || strncmp(XXXXXX, "XXXXXX", 6))
-        return 0;
+    QChar *XXXXXX = templateName.data() + length - 6;
 
     for (int i = 0; i < 256; ++i) {
         int v = qrand();
 
         /* Fill in the random bits.  */
-        XXXXXX[0] = letters[nextRand(v)];
-        XXXXXX[1] = letters[nextRand(v)];
-        XXXXXX[2] = letters[nextRand(v)];
-        XXXXXX[3] = letters[nextRand(v)];
-        XXXXXX[4] = letters[nextRand(v)];
-        XXXXXX[5] = letters[v % 62];
+        XXXXXX[0] = QLatin1Char(letters[nextRand(v)]);
+        XXXXXX[1] = QLatin1Char(letters[nextRand(v)]);
+        XXXXXX[2] = QLatin1Char(letters[nextRand(v)]);
+        XXXXXX[3] = QLatin1Char(letters[nextRand(v)]);
+        XXXXXX[4] = QLatin1Char(letters[nextRand(v)]);
+        XXXXXX[5] = QLatin1Char(letters[v % 62]);
 
-        QString templateNameStr = QFile::decodeName(templateName);
-
-        QFileSystemEntry fileSystemEntry(templateNameStr);
+        QFileSystemEntry fileSystemEntry(templateName);
         if (QFileSystemEngine::createDirectory(fileSystemEntry, false)) {
             QSystemError error;
             QFileSystemEngine::setPermissions(fileSystemEntry,
@@ -130,33 +142,49 @@ static char *q_mkdtemp(char *templateName)
                                               QFile::ExeOwner, error);
             if (error.error() != 0) {
                 if (!QFileSystemEngine::removeDirectory(fileSystemEntry, false))
-                    qWarning() << "Unable to remove unused directory" << templateNameStr;
+                    qWarning() << "Unable to remove unused directory" << templateName;
                 continue;
             }
-            return templateName;
+            return qMakePair(templateName, true);
         }
+#  ifdef Q_OS_WIN
+        const int exists = ERROR_ALREADY_EXISTS;
+        int code = GetLastError();
+#  else
+        const int exists = EEXIST;
+        int code = errno;
+#  endif
+        if (code != exists)
+            return qMakePair(qt_error_string(code), false);
     }
-    return 0;
+    return qMakePair(qt_error_string(), false);
 }
 
-#else // defined(Q_OS_QNX ) || defined(Q_OS_WIN) || defined(Q_OS_ANDROID)
+#else // !USE_SYSTEM_MKDTEMP
 
-static char *q_mkdtemp(char *templateName)
+QPair<QString, bool> q_mkdtemp(char *templateName)
 {
-   return mkdtemp(templateName);
+    bool ok = (mkdtemp(templateName) != 0);
+    return qMakePair(ok ? QFile::decodeName(templateName) : qt_error_string(), ok);
 }
 
-#endif
+#endif // USE_SYSTEM_MKDTEMP
 
 void QTemporaryDirPrivate::create(const QString &templateName)
 {
+#ifndef USE_SYSTEM_MKDTEMP
+    QString buffer = templateName;
+    if (!buffer.endsWith(QLatin1String("XXXXXX")))
+        buffer += QLatin1String("XXXXXX");
+    const QPair<QString, bool> result = q_mkdtemp(buffer);
+#else // !USE_SYSTEM_MKDTEMP
     QByteArray buffer = QFile::encodeName(templateName);
     if (!buffer.endsWith("XXXXXX"))
         buffer += "XXXXXX";
-    if (q_mkdtemp(buffer.data())) { // modifies buffer
-        success = true;
-        path = QFile::decodeName(buffer.constData());
-    }
+    QPair<QString, bool> result = q_mkdtemp(buffer.data()); // modifies buffer
+#endif // USE_SYSTEM_MKDTEMP
+    pathOrError = result.first;
+    success = result.second;
 }
 
 //************* QTemporaryDir
@@ -255,13 +283,25 @@ bool QTemporaryDir::isValid() const
     return d_ptr->success;
 }
 
+/*!
+   \since 5.6
+
+   If isValid() returns \c false, this function returns the error string that
+   explains why the creation of the temporary directory failed. Otherwise, this
+   function return an empty string.
+*/
+QString QTemporaryDir::errorString() const
+{
+    return d_ptr->success ? QString() : d_ptr->pathOrError;
+}
+
 /*!
    Returns the path to the temporary directory.
    Empty if the QTemporaryDir could not be created.
 */
 QString QTemporaryDir::path() const
 {
-    return d_ptr->path;
+    return d_ptr->success ? d_ptr->pathOrError : QString();
 }
 
 /*!
