@@ -1,31 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -860,6 +867,21 @@ void QTextStreamPrivate::write(QLatin1String data)
     }
 }
 
+/*!
+    \internal
+*/
+void QTextStreamPrivate::writePadding(int len)
+{
+    if (string) {
+        // ### What about seek()??
+        string->resize(string->size() + len, params.padChar);
+    } else {
+        writeBuffer.resize(writeBuffer.size() + len, params.padChar);
+        if (writeBuffer.size() > QTEXTSTREAM_BUFFERSIZE)
+            flushWriteBuffer();
+    }
+}
+
 /*!
     \internal
 */
@@ -917,31 +939,24 @@ QTextStreamPrivate::PaddingResult QTextStreamPrivate::padding(int len) const
 {
     Q_ASSERT(params.fieldWidth > len); // calling padding() when no padding is needed is an error
 
-    // Do NOT break NRVO in this function or kittens will die!
-
-    PaddingResult result;
+    int left = 0, right = 0;
 
     const int padSize = params.fieldWidth - len;
 
-    result.padding.resize(padSize);
-    std::fill_n(result.padding.begin(), padSize, params.padChar);
-
     switch (params.fieldAlignment) {
     case QTextStream::AlignLeft:
-        result.left  = 0;
-        result.right = padSize;
+        right = padSize;
         break;
     case QTextStream::AlignRight:
     case QTextStream::AlignAccountingStyle:
-        result.left  = padSize;
-        result.right = 0;
+        left  = padSize;
         break;
     case QTextStream::AlignCenter:
-        result.left  = padSize/2;
-        result.right = padSize - padSize/2;
+        left  = padSize/2;
+        right = padSize - padSize/2;
         break;
     }
-
+    const PaddingResult result = { left, right };
     return result;
 }
 
@@ -966,9 +981,9 @@ void QTextStreamPrivate::putString(const QChar *data, int len, bool number)
             }
         }
 
-        write(pad.padding.constData(), pad.left);
+        writePadding(pad.left);
         write(data, len);
-        write(pad.padding.constData(), pad.right);
+        writePadding(pad.right);
     } else {
         write(data, len);
     }
@@ -994,9 +1009,9 @@ void QTextStreamPrivate::putString(QLatin1String data, bool number)
             }
         }
 
-        write(pad.padding.constData(), pad.left);
+        writePadding(pad.left);
         write(data);
-        write(pad.padding.constData(), pad.right);
+        writePadding(pad.right);
     } else {
         write(data);
     }
@@ -2546,6 +2561,8 @@ QTextStream &QTextStream::operator<<(double f)
         flags |= QLocaleData::Alternate;
     if (locale() != QLocale::c() && !(locale().numberOptions() & QLocale::OmitGroupSeparator))
         flags |= QLocaleData::ThousandsGroup;
+    if (!(locale().numberOptions() & QLocale::OmitLeadingZeroInExponent))
+        flags |= QLocaleData::ZeroPadExponent;
 
     const QLocaleData *dd = d->locale.d->m_data;
     QString num = dd->doubleToString(f, d->params.realNumberPrecision, form, -1, flags);
@@ -3114,10 +3131,7 @@ void QTextStream::setGenerateByteOrderMark(bool generate)
 {
     Q_D(QTextStream);
     if (d->writeBuffer.isEmpty()) {
-        if (generate)
-            d->writeConverterState.flags &= ~QTextCodec::IgnoreHeader;
-        else
-            d->writeConverterState.flags |= QTextCodec::IgnoreHeader;
+        d->writeConverterState.flags.setFlag(QTextCodec::IgnoreHeader, !generate);
     }
 }
 
