@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -298,6 +304,7 @@ QVariantHash QJsonObject::toVariantHash() const
 {
     QVariantHash hash;
     if (o) {
+        hash.reserve(o->length);
         for (uint i = 0; i < o->length; ++i) {
             QJsonPrivate::Entry *e = o->entryAt(i);
             hash.insert(e->key(), QJsonValue(d, o, e->value).toVariant());
@@ -367,6 +374,22 @@ QJsonValue QJsonObject::value(const QString &key) const
     return QJsonValue(d, o, o->entryAt(i)->value);
 }
 
+/*!
+    \overload
+    \since 5.7
+*/
+QJsonValue QJsonObject::value(QLatin1String key) const
+{
+    if (!d)
+        return QJsonValue(QJsonValue::Undefined);
+
+    bool keyExists;
+    int i = o->indexOf(key, &keyExists);
+    if (!keyExists)
+        return QJsonValue(QJsonValue::Undefined);
+    return QJsonValue(d, o, o->entryAt(i)->value);
+}
+
 /*!
     Returns a QJsonValue representing the value for the key \a key.
 
@@ -381,6 +404,13 @@ QJsonValue QJsonObject::operator [](const QString &key) const
     return value(key);
 }
 
+/*!
+    \fn QJsonValue QJsonObject::operator [](QLatin1String key) const
+
+    \overload
+    \since 5.7
+*/
+
 /*!
     Returns a reference to the value for \a key.
 
@@ -404,6 +434,16 @@ QJsonValueRef QJsonObject::operator [](const QString &key)
     return QJsonValueRef(this, index);
 }
 
+/*!
+    \overload
+    \since 5.7
+*/
+QJsonValueRef QJsonObject::operator [](QLatin1String key)
+{
+    // ### optimize me
+    return operator[](QString(key));
+}
+
 /*!
     Inserts a new item with the key \a key and a value of \a value.
 
@@ -528,6 +568,20 @@ bool QJsonObject::contains(const QString &key) const
     return keyExists;
 }
 
+/*!
+    \overload
+    \since 5.7
+*/
+bool QJsonObject::contains(QLatin1String key) const
+{
+    if (!o)
+        return false;
+
+    bool keyExists;
+    o->indexOf(key, &keyExists);
+    return keyExists;
+}
+
 /*!
     Returns \c true if \a other is equal to this object.
  */
@@ -602,11 +656,31 @@ QJsonObject::iterator QJsonObject::find(const QString &key)
     return iterator(this, index);
 }
 
+/*!
+    \overload
+    \since 5.7
+*/
+QJsonObject::iterator QJsonObject::find(QLatin1String key)
+{
+    bool keyExists = false;
+    int index = o ? o->indexOf(key, &keyExists) : 0;
+    if (!keyExists)
+        return end();
+    detach2();
+    return iterator(this, index);
+}
+
 /*! \fn QJsonObject::const_iterator QJsonObject::find(const QString &key) const
 
     \overload
 */
 
+/*! \fn QJsonObject::const_iterator QJsonObject::find(QLatin1String key) const
+
+    \overload
+    \since 5.7
+*/
+
 /*!
     Returns a const iterator pointing to the item with key \a key in the
     map.
@@ -623,6 +697,19 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     return const_iterator(this, index);
 }
 
+/*!
+    \overload
+    \since 5.7
+*/
+QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
+{
+    bool keyExists = false;
+    int index = o ? o->indexOf(key, &keyExists) : 0;
+    if (!keyExists)
+        return end();
+    return const_iterator(this, index);
+}
+
 /*! \fn int QJsonObject::count() const
 
     \overload
