@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -36,7 +42,6 @@
 #include <QtCore/QCoreApplication>
 #include <QtCore/QThread>
 #include <QtCore/QHash>
-#include <QtCore/QMutex>
 #include <QtCore/qfunctions_winrt.h>
 #include <private/qabstracteventdispatcher_p.h>
 #include <private/qcoreapplication_p.h>
@@ -101,7 +106,6 @@ public:
 private:
     QHash<int, QObject *> timerIdToObject;
     QVector<WinRTTimerInfo> timerInfos;
-    mutable QMutex timerInfoLock;
     QHash<HANDLE, int> timerHandleToId;
     QHash<int, HANDLE> timerIdToHandle;
     QHash<int, HANDLE> timerIdToCancelHandle;
@@ -115,10 +119,9 @@ private:
             timerIdToHandle.insert(id, handle);
             timerIdToCancelHandle.insert(id, cancelHandle);
         }
-
+        timerIdToObject.insert(id, obj);
         const quint64 targetTime = qt_msectime() + interval;
         const WinRTTimerInfo info(id, interval, type, obj, targetTime);
-        QMutexLocker locker(&timerInfoLock);
         if (id >= timerInfos.size())
             timerInfos.resize(id + 1);
         timerInfos[id] = info;
@@ -127,7 +130,6 @@ private:
 
     bool removeTimer(int id)
     {
-        QMutexLocker locker(&timerInfoLock);
         if (id >= timerInfos.size())
             return false;
 
@@ -251,18 +253,14 @@ bool QEventDispatcherWinRT::processEvents(QEventLoop::ProcessEventsFlags flags)
             if (timerId == INTERRUPT_HANDLE)
                 break;
 
-            {
-                QMutexLocker locker(&d->timerInfoLock);
+            WinRTTimerInfo &info = d->timerInfos[timerId];
+            Q_ASSERT(info.timerId != INVALID_TIMER_ID);
 
-                WinRTTimerInfo &info = d->timerInfos[timerId];
-                Q_ASSERT(info.timerId != INVALID_TIMER_ID);
+            QCoreApplication::postEvent(this, new QTimerEvent(timerId));
 
-                QCoreApplication::postEvent(this, new QTimerEvent(timerId));
-
-                // Update timer's targetTime
-                const quint64 targetTime = qt_msectime() + info.interval;
-                info.targetTime = targetTime;
-            }
+            // Update timer's targetTime
+            const quint64 targetTime = qt_msectime() + info.interval;
+            info.targetTime = targetTime;
             waitResult = WaitForMultipleObjectsEx(timerHandles.count(), timerHandles.constData(), FALSE, 0, TRUE);
         }
         emit awake();
@@ -411,7 +409,8 @@ bool QEventDispatcherWinRT::unregisterTimers(QObject *object)
     }
 
     Q_D(QEventDispatcherWinRT);
-    foreach (int id, d->timerIdToObject.keys()) {
+    const auto timerIds = d->timerIdToObject.keys(); // ### FIXME: iterate over hash directly? But unregisterTimer() modifies the hash!
+    for (int id : timerIds) {
         if (d->timerIdToObject.value(id) == object)
             unregisterTimer(id);
     }
@@ -429,9 +428,8 @@ QList<QAbstractEventDispatcher::TimerInfo> QEventDispatcherWinRT::registeredTime
     }
 
     Q_D(const QEventDispatcherWinRT);
-    QMutexLocker locker(&d->timerInfoLock);
     QList<TimerInfo> timerInfos;
-    foreach (const WinRTTimerInfo &info, d->timerInfos) {
+    for (const WinRTTimerInfo &info : d->timerInfos) {
         if (info.object == object && info.timerId != INVALID_TIMER_ID)
             timerInfos.append(info);
     }
@@ -461,7 +459,6 @@ int QEventDispatcherWinRT::remainingTime(int timerId)
     }
 
     Q_D(QEventDispatcherWinRT);
-    QMutexLocker locker(&d->timerInfoLock);
     const WinRTTimerInfo timerInfo = d->timerInfos.at(timerId);
     if (timerInfo.timerId == INVALID_TIMER_ID) {
 #ifndef QT_NO_DEBUG
@@ -510,9 +507,6 @@ bool QEventDispatcherWinRT::event(QEvent *e)
     case QEvent::Timer: {
         QTimerEvent *timerEvent = static_cast<QTimerEvent *>(e);
         const int id = timerEvent->timerId();
-
-        QMutexLocker locker(&d->timerInfoLock);
-
         Q_ASSERT(id < d->timerInfos.size());
         WinRTTimerInfo &info = d->timerInfos[id];
         Q_ASSERT(info.timerId != INVALID_TIMER_ID);
@@ -521,18 +515,15 @@ bool QEventDispatcherWinRT::event(QEvent *e)
             break;
         info.inEvent = true;
 
-        QObject *timerObj = d->timerIdToObject.value(id);
-        locker.unlock();
-
         QTimerEvent te(id);
-        QCoreApplication::sendEvent(timerObj, &te);
+        QCoreApplication::sendEvent(d->timerIdToObject.value(id), &te);
 
-        locker.relock();
+        // The timer might have been removed in the meanwhile
+        if (id >= d->timerInfos.size())
+            break;
 
-        // The timer might have been removed in the meanwhile. If the timer was
-        // the last one in the list, id is bigger than the list's size.
-        // Otherwise, the id will just be set to INVALID_TIMER_ID.
-        if (id >= d->timerInfos.size() || info.timerId == INVALID_TIMER_ID)
+        info = d->timerInfos[id];
+        if (info.timerId == INVALID_TIMER_ID)
             break;
 
         if (info.interval == 0 && info.inEvent) {
