@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -82,7 +88,14 @@ public:
             m_pattern = m_pattern.toLower();
         }
     }
-    ~QMimeGlobPattern() {}
+
+    void swap(QMimeGlobPattern &other) Q_DECL_NOTHROW
+    {
+        qSwap(m_pattern,         other.m_pattern);
+        qSwap(m_mimeType,        other.m_mimeType);
+        qSwap(m_weight,          other.m_weight);
+        qSwap(m_caseSensitivity, other.m_caseSensitivity);
+    }
 
     bool matchFileName(const QString &filename) const;
 
@@ -97,6 +110,7 @@ private:
     int m_weight;
     Qt::CaseSensitivity m_caseSensitivity;
 };
+Q_DECLARE_SHARED(QMimeGlobPattern)
 
 class QMimeGlobPatternList : public QList<QMimeGlobPattern>
 {
@@ -116,11 +130,10 @@ public:
      */
     void removeMimeType(const QString &mimeType)
     {
-        QMutableListIterator<QMimeGlobPattern> it(*this);
-        while (it.hasNext()) {
-            if (it.next().mimeType() == mimeType)
-                it.remove();
-        }
+        auto isMimeTypeEqual = [&mimeType](const QMimeGlobPattern &pattern) {
+            return pattern.mimeType() == mimeType;
+        };
+        erase(std::remove_if(begin(), end(), isMimeTypeEqual), end());
     }
 
     void match(QMimeGlobMatchResult &result, const QString &fileName) const;
