@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -154,25 +160,36 @@ QMimeTypeParserBase::ParseState QMimeTypeParserBase::nextState(ParseState curren
 }
 
 // Parse int number from an (attribute) string
-bool QMimeTypeParserBase::parseNumber(const QString &n, int *target, QString *errorMessage)
+bool QMimeTypeParserBase::parseNumber(const QStringRef &n, int *target, QString *errorMessage)
 {
     bool ok;
     *target = n.toInt(&ok);
-    if (!ok) {
-        *errorMessage = QString::fromLatin1("Not a number '%1'.").arg(n);
+    if (Q_UNLIKELY(!ok)) {
+        *errorMessage = QLatin1String("Not a number '") + n + QLatin1String("'.");
         return false;
     }
     return true;
 }
 
 #ifndef QT_NO_XMLSTREAMREADER
-static QMimeMagicRule *createMagicMatchRule(const QXmlStreamAttributes &atts, QString *errorMessage)
+struct CreateMagicMatchRuleResult {
+    QString errorMessage; // must be first
+    QMimeMagicRule rule;
+
+    CreateMagicMatchRuleResult(const QStringRef &type, const QStringRef &value, const QStringRef &offsets, const QStringRef &mask)
+        : errorMessage(), rule(type.toString(), value.toUtf8(), offsets.toString(), mask.toLatin1(), &errorMessage)
+    {
+
+    }
+};
+
+static CreateMagicMatchRuleResult createMagicMatchRule(const QXmlStreamAttributes &atts)
 {
-    const QString type = atts.value(QLatin1String(matchTypeAttributeC)).toString();
-    const QString value = atts.value(QLatin1String(matchValueAttributeC)).toString();
-    const QString offsets = atts.value(QLatin1String(matchOffsetAttributeC)).toString();
-    const QString mask = atts.value(QLatin1String(matchMaskAttributeC)).toString();
-    return new QMimeMagicRule(type, value.toUtf8(), offsets, mask.toLatin1(), errorMessage);
+    const QStringRef type = atts.value(QLatin1String(matchTypeAttributeC));
+    const QStringRef value = atts.value(QLatin1String(matchValueAttributeC));
+    const QStringRef offsets = atts.value(QLatin1String(matchOffsetAttributeC));
+    const QStringRef mask = atts.value(QLatin1String(matchMaskAttributeC));
+    return CreateMagicMatchRuleResult(type, value, offsets, mask);
 }
 #endif
 
@@ -189,17 +206,16 @@ bool QMimeTypeParserBase::parse(QIODevice *dev, const QString &fileName, QString
     QList<QMimeMagicRule> rules; // toplevel rules
     QXmlStreamReader reader(dev);
     ParseState ps = ParseBeginning;
-    QXmlStreamAttributes atts;
     while (!reader.atEnd()) {
         switch (reader.readNext()) {
-        case QXmlStreamReader::StartElement:
+        case QXmlStreamReader::StartElement: {
             ps = nextState(ps, reader.name());
-            atts = reader.attributes();
+            const QXmlStreamAttributes atts = reader.attributes();
             switch (ps) {
             case ParseMimeType: { // start parsing a MIME type name
                 const QString name = atts.value(QLatin1String(mimeTypeAttributeC)).toString();
                 if (name.isEmpty()) {
-                    reader.raiseError(QString::fromLatin1("Missing '%1'-attribute").arg(QString::fromLatin1(mimeTypeAttributeC)));
+                    reader.raiseError(QStringLiteral("Missing 'type'-attribute"));
                 } else {
                     data.name = name;
                 }
@@ -213,8 +229,8 @@ bool QMimeTypeParserBase::parse(QIODevice *dev, const QString &fileName, QString
                 break;
             case ParseGlobPattern: {
                 const QString pattern = atts.value(QLatin1String(patternAttributeC)).toString();
-                unsigned weight = atts.value(QLatin1String(weightAttributeC)).toString().toInt();
-                const bool caseSensitive = atts.value(QLatin1String(caseSensitiveAttributeC)).toString() == QLatin1String("true");
+                unsigned weight = atts.value(QLatin1String(weightAttributeC)).toInt();
+                const bool caseSensitive = atts.value(QLatin1String(caseSensitiveAttributeC)) == QLatin1String("true");
 
                 if (weight == 0)
                     weight = QMimeGlobPattern::DefaultWeight;
@@ -249,7 +265,7 @@ bool QMimeTypeParserBase::parse(QIODevice *dev, const QString &fileName, QString
                 break;
             case ParseMagic: {
                 priority = 50;
-                const QString priorityS = atts.value(QLatin1String(priorityAttributeC)).toString();
+                const QStringRef priorityS = atts.value(QLatin1String(priorityAttributeC));
                 if (!priorityS.isEmpty()) {
                     if (!parseNumber(priorityS, &priority, errorMessage))
                         return false;
@@ -260,28 +276,27 @@ bool QMimeTypeParserBase::parse(QIODevice *dev, const QString &fileName, QString
             }
                 break;
             case ParseMagicMatchRule: {
-                QString magicErrorMessage;
-                QMimeMagicRule *rule = createMagicMatchRule(atts, &magicErrorMessage);
-                if (!rule->isValid())
-                    qWarning("QMimeDatabase: Error parsing %s\n%s", qPrintable(fileName), qPrintable(magicErrorMessage));
+                auto result = createMagicMatchRule(atts);
+                if (Q_UNLIKELY(!result.rule.isValid()))
+                    qWarning("QMimeDatabase: Error parsing %ls\n%ls",
+                             qUtf16Printable(fileName), qUtf16Printable(result.errorMessage));
                 QList<QMimeMagicRule> *ruleList;
                 if (currentRules.isEmpty())
                     ruleList = &rules;
                 else // nest this rule into the proper parent
                     ruleList = &currentRules.top()->m_subMatches;
-                ruleList->append(*rule);
+                ruleList->append(std::move(result.rule));
                 //qDebug() << " MATCH added. Stack size was" << currentRules.size();
                 currentRules.push(&ruleList->last());
-                delete rule;
                 break;
             }
             case ParseError:
-                reader.raiseError(QString::fromLatin1("Unexpected element <%1>").
-                                  arg(reader.name().toString()));
+                reader.raiseError(QLatin1String("Unexpected element <") + reader.name() + QLatin1Char('>'));
                 break;
             default:
                 break;
             }
+        }
             break;
         // continue switch QXmlStreamReader::Token...
         case QXmlStreamReader::EndElement: // Finished element
@@ -310,9 +325,13 @@ bool QMimeTypeParserBase::parse(QIODevice *dev, const QString &fileName, QString
         }
     }
 
-    if (reader.hasError()) {
-        if (errorMessage)
-            *errorMessage = QString::fromLatin1("An error has been encountered at line %1 of %2: %3:").arg(reader.lineNumber()).arg(fileName, reader.errorString());
+    if (Q_UNLIKELY(reader.hasError())) {
+        if (errorMessage) {
+            *errorMessage = QString::asprintf("An error has been encountered at line %lld of %ls: %ls:",
+                                              reader.lineNumber(),
+                                              qUtf16Printable(fileName),
+                                              qUtf16Printable(reader.errorString()));
+        }
         return false;
     }
 
