@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,10 +47,10 @@
 #include "qdebug.h"
 #include "qdir.h"
 
-#ifndef QT_NO_LIBRARY
-
 QT_BEGIN_NAMESPACE
 
+#ifndef QT_NO_LIBRARY
+
 /*!
     \class QPluginLoader
     \inmodule QtCore
@@ -148,7 +154,6 @@ QPluginLoader::QPluginLoader(const QString &fileName, QObject *parent)
     : QObject(parent), d(0), did_load(false)
 {
     setFileName(fileName);
-    setLoadHints(QLibrary::PreventUnloadHint);
 }
 
 /*!
@@ -301,9 +306,9 @@ static QString locatePlugin(const QString& fileName)
         paths.prepend(QStringLiteral(".")); // search in current dir first
     }
 
-    foreach (const QString &path, paths) {
-        foreach (const QString &prefix, prefixes) {
-            foreach (const QString &suffix, suffixes) {
+    for (const QString &path : qAsConst(paths)) {
+        for (const QString &prefix : qAsConst(prefixes)) {
+            for (const QString &suffix : qAsConst(suffixes)) {
                 const QString fn = path + QLatin1Char('/') + basePath + prefix + baseName + suffix;
                 if (debug)
                     qDebug() << "Trying..." << fn;
@@ -343,7 +348,7 @@ static QString locatePlugin(const QString& fileName)
 void QPluginLoader::setFileName(const QString &fileName)
 {
 #if defined(QT_SHARED)
-    QLibrary::LoadHints lh = QLibrary::PreventUnloadHint;
+    QLibrary::LoadHints lh;
     if (d) {
         lh = d->loadHints();
         d->release();
@@ -383,16 +388,13 @@ QString QPluginLoader::errorString() const
     return (!d || d->errorString.isEmpty()) ? tr("Unknown error") : d->errorString;
 }
 
-typedef QVector<QStaticPlugin> StaticPluginList;
-Q_GLOBAL_STATIC(StaticPluginList, staticPluginList)
-
 /*! \since 4.4
 
     \property QPluginLoader::loadHints
     \brief Give the load() function some hints on how it should behave.
 
     You can give hints on how the symbols in the plugin are
-    resolved. By default since Qt 5.7, QLibrary::PreventUnloadHint is set.
+    resolved. By default, none of the hints are set.
 
     See the documentation of QLibrary::loadHints for a complete
     description of how this property works.
@@ -414,6 +416,11 @@ QLibrary::LoadHints QPluginLoader::loadHints() const
     return d ? d->loadHints() : QLibrary::LoadHints();
 }
 
+#endif // QT_NO_LIBRARY
+
+typedef QVector<QStaticPlugin> StaticPluginList;
+Q_GLOBAL_STATIC(StaticPluginList, staticPluginList)
+
 /*!
     \relates QPluginLoader
     \since 5.0
@@ -466,9 +473,8 @@ QVector<QStaticPlugin> QPluginLoader::staticPlugins()
 */
 QJsonObject QStaticPlugin::metaData() const
 {
-    return QLibraryPrivate::fromRawMetaData(rawMetaData()).object();
+    return qJsonFromRawLibraryMetaData(rawMetaData()).object();
 }
 
 QT_END_NAMESPACE
 
-#endif // QT_NO_LIBRARY
