@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -116,11 +114,6 @@ QSignalTransitionPrivate::QSignalTransitionPrivate()
     signalIndex = -1;
 }
 
-QSignalTransitionPrivate *QSignalTransitionPrivate::get(QSignalTransition *q)
-{
-    return q->d_func();
-}
-
 void QSignalTransitionPrivate::unregister()
 {
     Q_Q(QSignalTransition);
@@ -158,6 +151,18 @@ QSignalTransition::QSignalTransition(const QObject *sender, const char *signal,
     d->maybeRegister();
 }
 
+/*!
+  \fn QSignalTransition::QSignalTransition(const QObject *sender,
+                       PointerToMemberFunction signal, QState *sourceState)
+  \since 5.7
+  \overload
+
+  Constructs a new signal transition associated with the given \a signal of
+  the given \a sender object and with the given \a sourceState.
+  This constructor is enabled if the compiler supports delegating constructors,
+  as indicated by the presence of the macro Q_COMPILER_DELEGATING_CONSTRUCTORS.
+*/
+
 /*!
   Destroys this signal transition.
 */
@@ -185,6 +190,7 @@ void QSignalTransition::setSenderObject(const QObject *sender)
     d->unregister();
     d->sender = sender;
     d->maybeRegister();
+    emit senderObjectChanged(QPrivateSignal());
 }
 
 /*!
@@ -207,14 +213,15 @@ void QSignalTransition::setSignal(const QByteArray &signal)
     d->unregister();
     d->signal = signal;
     d->maybeRegister();
+    emit signalChanged(QPrivateSignal());
 }
 
 /*!
   \reimp
 
-  The default implementation returns true if the \a event is a
+  The default implementation returns \c true if the \a event is a
   QStateMachine::SignalEvent object and the event's sender and signal index
-  match this transition, and returns false otherwise.
+  match this transition, and returns \c false otherwise.
 */
 bool QSignalTransition::eventTest(QEvent *event)
 {
@@ -245,6 +252,24 @@ bool QSignalTransition::event(QEvent *e)
     return QAbstractTransition::event(e);
 }
 
+/*!
+  \fn QSignalTransition::senderObjectChanged()
+  \since 5.4
+
+  This signal is emitted when the senderObject property is changed.
+
+  \sa QSignalTransition::senderObject
+*/
+
+/*!
+  \fn QSignalTransition::signalChanged()
+  \since 5.4
+
+  This signal is emitted when the signal property is changed.
+
+  \sa QSignalTransition::signal
+*/
+
 void QSignalTransitionPrivate::callOnTransition(QEvent *e)
 {
     Q_Q(QSignalTransition);
@@ -260,6 +285,7 @@ void QSignalTransitionPrivate::callOnTransition(QEvent *e)
     }
 }
 
+
 QT_END_NAMESPACE
 
 #endif //QT_NO_STATEMACHINE
