@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -62,6 +60,7 @@ class Q_CORE_EXPORT QStateMachine : public QState
     Q_OBJECT
     Q_PROPERTY(QString errorString READ errorString)
     Q_PROPERTY(QState::RestorePolicy globalRestorePolicy READ globalRestorePolicy WRITE setGlobalRestorePolicy)
+    Q_PROPERTY(bool running READ isRunning WRITE setRunning NOTIFY runningChanged)
 #ifndef QT_NO_ANIMATION
     Q_PROPERTY(bool animated READ isAnimated WRITE setAnimated)
 #endif
@@ -111,8 +110,8 @@ public:
         NoCommonAncestorForTransitionError
     };
 
-    explicit QStateMachine(QObject *parent = 0);
-    explicit QStateMachine(QState::ChildMode childMode, QObject *parent = 0);
+    explicit QStateMachine(QObject *parent = Q_NULLPTR);
+    explicit QStateMachine(QState::ChildMode childMode, QObject *parent = Q_NULLPTR);
     ~QStateMachine();
 
     void addState(QAbstractState *state);
@@ -143,28 +142,23 @@ public:
     QSet<QAbstractState*> configuration() const;
 
 #ifndef QT_NO_STATEMACHINE_EVENTFILTER
-    bool eventFilter(QObject *watched, QEvent *event);
+    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
 #endif
 
 public Q_SLOTS:
     void start();
     void stop();
+    void setRunning(bool running);
 
 Q_SIGNALS:
-    void started(
-#if !defined(Q_QDOC)
-      QPrivateSignal
-#endif
-    );
-    void stopped(
-#if !defined(Q_QDOC)
-      QPrivateSignal
-#endif
-    );
+    void started(QPrivateSignal);
+    void stopped(QPrivateSignal);
+    void runningChanged(bool running);
+
 
 protected:
-    void onEntry(QEvent *event);
-    void onExit(QEvent *event);
+    void onEntry(QEvent *event) Q_DECL_OVERRIDE;
+    void onExit(QEvent *event) Q_DECL_OVERRIDE;
 
     virtual void beginSelectTransitions(QEvent *event);
     virtual void endSelectTransitions(QEvent *event);
@@ -172,7 +166,7 @@ protected:
     virtual void beginMicrostep(QEvent *event);
     virtual void endMicrostep(QEvent *event);
 
-    bool event(QEvent *e);
+    bool event(QEvent *e) Q_DECL_OVERRIDE;
 
 protected:
     QStateMachine(QStateMachinePrivate &dd, QObject *parent);
