@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -77,33 +83,13 @@ QFutureInterfaceBase::~QFutureInterfaceBase()
         delete d;
 }
 
-static inline int switch_on(QAtomicInt &a, int which)
-{
-    return a.fetchAndOrRelaxed(which) | which;
-}
-
-static inline int switch_off(QAtomicInt &a, int which)
-{
-    return a.fetchAndAndRelaxed(~which) & ~which;
-}
-
-static inline int switch_from_to(QAtomicInt &a, int from, int to)
-{
-    int newValue;
-    int expected = a.load();
-    do {
-        newValue = (expected & ~from) | to;
-    } while (!a.testAndSetRelaxed(expected, newValue, expected));
-    return newValue;
-}
-
 void QFutureInterfaceBase::cancel()
 {
     QMutexLocker locker(&d->m_mutex);
-    if (d->state.load() & Canceled)
+    if (d->state & Canceled)
         return;
 
-    switch_from_to(d->state, Paused, Canceled);
+    d->state = State((d->state & ~Paused) | Canceled);
     d->waitCondition.wakeAll();
     d->pausedWaitCondition.wakeAll();
     d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Canceled));
@@ -113,10 +99,10 @@ void QFutureInterfaceBase::setPaused(bool paused)
 {
     QMutexLocker locker(&d->m_mutex);
     if (paused) {
-        switch_on(d->state, Paused);
+        d->state = State(d->state | Paused);
         d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Paused));
     } else {
-        switch_off(d->state, Paused);
+        d->state = State(d->state & ~Paused);
         d->pausedWaitCondition.wakeAll();
         d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Resumed));
     }
@@ -125,24 +111,29 @@ void QFutureInterfaceBase::setPaused(bool paused)
 void QFutureInterfaceBase::togglePaused()
 {
     QMutexLocker locker(&d->m_mutex);
-    if (d->state.load() & Paused) {
-        switch_off(d->state, Paused);
+    if (d->state & Paused) {
+        d->state = State(d->state & ~Paused);
         d->pausedWaitCondition.wakeAll();
         d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Resumed));
     } else {
-        switch_on(d->state, Paused);
+        d->state = State(d->state | Paused);
         d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Paused));
     }
 }
 
 void QFutureInterfaceBase::setThrottled(bool enable)
 {
+    // bail out if we are not changing the state
+    if ((enable && (d->state & Throttled)) || (!enable && !(d->state & Throttled)))
+        return;
+
+    // lock and change the state
     QMutexLocker lock(&d->m_mutex);
     if (enable) {
-        switch_on(d->state, Throttled);
+        d->state  = State(d->state | Throttled);
     } else {
-        switch_off(d->state, Throttled);
-        if (!(d->state.load() & Paused))
+        d->state  = State(d->state & ~Throttled);
+        if (!(d->state & Paused))
             d->pausedWaitCondition.wakeAll();
     }
 }
@@ -193,15 +184,11 @@ bool QFutureInterfaceBase::waitForNextResult()
 void QFutureInterfaceBase::waitForResume()
 {
     // return early if possible to avoid taking the mutex lock.
-    {
-        const int state = d->state.load();
-        if (!(state & Paused) || (state & Canceled))
-            return;
-    }
+    if ((d->state & Paused) == false || (d->state & Canceled))
+        return;
 
     QMutexLocker lock(&d->m_mutex);
-    const int state = d->state.load();
-    if (!(state & Paused) || (state & Canceled))
+    if ((d->state & Paused) == false || (d->state & Canceled))
         return;
 
     // decrease active thread count since this thread will wait.
@@ -249,7 +236,7 @@ bool QFutureInterfaceBase::isProgressUpdateNeeded() const
 void QFutureInterfaceBase::reportStarted()
 {
     QMutexLocker locker(&d->m_mutex);
-    if (d->state.load() & (Started|Canceled|Finished))
+    if ((d->state & Started) || (d->state & Canceled) || (d->state & Finished))
         return;
 
     d->setState(State(Started | Running));
@@ -265,11 +252,11 @@ void QFutureInterfaceBase::reportCanceled()
 void QFutureInterfaceBase::reportException(const QException &exception)
 {
     QMutexLocker locker(&d->m_mutex);
-    if (d->state.load() & (Canceled|Finished))
+    if ((d->state & Canceled) || (d->state & Finished))
         return;
 
     d->m_exceptionStore.setException(exception);
-    switch_on(d->state, Canceled);
+    d->state = State(d->state | Canceled);
     d->waitCondition.wakeAll();
     d->pausedWaitCondition.wakeAll();
     d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Canceled));
@@ -279,8 +266,8 @@ void QFutureInterfaceBase::reportException(const QException &exception)
 void QFutureInterfaceBase::reportFinished()
 {
     QMutexLocker locker(&d->m_mutex);
-    if (!isFinished()) {
-        switch_from_to(d->state, Running, Finished);
+    if (!(d->state & Finished)) {
+        d->state = State((d->state & ~Running) | Finished);
         d->waitCondition.wakeAll();
         d->sendCallOut(QFutureCallOutEvent(QFutureCallOutEvent::Finished));
     }
@@ -300,7 +287,7 @@ int QFutureInterfaceBase::expectedResultCount()
 
 bool QFutureInterfaceBase::queryState(State state) const
 {
-    return d->state.load() & state;
+    return (d->state & state);
 }
 
 void QFutureInterfaceBase::waitForResult(int resultIndex)
@@ -308,7 +295,7 @@ void QFutureInterfaceBase::waitForResult(int resultIndex)
     d->m_exceptionStore.throwPossibleException();
 
     QMutexLocker lock(&d->m_mutex);
-    if (!isRunning())
+    if (!(d->state & Running))
         return;
     lock.unlock();
 
@@ -318,9 +305,11 @@ void QFutureInterfaceBase::waitForResult(int resultIndex)
 
     lock.relock();
 
-    const int waitIndex = (resultIndex == -1) ? INT_MAX : resultIndex;
-    while (isRunning() && !d->internal_isResultReadyAt(waitIndex))
-        d->waitCondition.wait(&d->m_mutex);
+    if (d->state & Running) {
+        const int waitIndex = (resultIndex == -1) ? INT_MAX : resultIndex;
+        while ((d->state & Running) && d->internal_isResultReadyAt(waitIndex) == false)
+            d->waitCondition.wait(&d->m_mutex);
+    }
 
     d->m_exceptionStore.throwPossibleException();
 }
@@ -328,7 +317,7 @@ void QFutureInterfaceBase::waitForResult(int resultIndex)
 void QFutureInterfaceBase::waitForFinished()
 {
     QMutexLocker lock(&d->m_mutex);
-    const bool alreadyFinished = !isRunning();
+    const bool alreadyFinished = !(d->state & Running);
     lock.unlock();
 
     if (!alreadyFinished) {
@@ -336,7 +325,7 @@ void QFutureInterfaceBase::waitForFinished()
 
         lock.relock();
 
-        while (isRunning())
+        while (d->state & Running)
             d->waitCondition.wait(&d->m_mutex);
     }
 
@@ -345,7 +334,7 @@ void QFutureInterfaceBase::waitForFinished()
 
 void QFutureInterfaceBase::reportResultsReady(int beginIndex, int endIndex)
 {
-    if (beginIndex == endIndex || (d->state.load() & (Canceled|Finished)))
+    if ((d->state & Canceled) || (d->state & Finished) || beginIndex == endIndex)
         return;
 
     d->waitCondition.wakeAll();
@@ -407,7 +396,7 @@ void QFutureInterfaceBase::setProgressValueAndText(int progressValue,
     if (d->m_progressValue >= progressValue)
         return;
 
-    if (d->state.load() & (Canceled|Finished))
+    if ((d->state & Canceled) || (d->state & Finished))
         return;
 
     if (d->internal_updateProgress(progressValue, progressText)) {
@@ -479,10 +468,10 @@ bool QFutureInterfaceBasePrivate::internal_waitForNextResult()
     if (m_results.hasNextResult())
         return true;
 
-    while ((state.load() & QFutureInterfaceBase::Running) && m_results.hasNextResult() == false)
+    while ((state & QFutureInterfaceBase::Running) && m_results.hasNextResult() == false)
         waitCondition.wait(&m_mutex);
 
-    return !(state.load() & QFutureInterfaceBase::Canceled) && m_results.hasNextResult();
+    return (!(state & QFutureInterfaceBase::Canceled) && m_results.hasNextResult());
 }
 
 bool QFutureInterfaceBasePrivate::internal_updateProgress(int progress,
@@ -505,16 +494,16 @@ bool QFutureInterfaceBasePrivate::internal_updateProgress(int progress,
 void QFutureInterfaceBasePrivate::internal_setThrottled(bool enable)
 {
     // bail out if we are not changing the state
-    if ((enable && (state.load() & QFutureInterfaceBase::Throttled))
-        || (!enable && !(state.load() & QFutureInterfaceBase::Throttled)))
+    if ((enable && (state & QFutureInterfaceBase::Throttled))
+        || (!enable && !(state & QFutureInterfaceBase::Throttled)))
         return;
 
     // change the state
     if (enable) {
-        switch_on(state, QFutureInterfaceBase::Throttled);
+        state  = QFutureInterfaceBase::State(state | QFutureInterfaceBase::Throttled);
     } else {
-        switch_off(state, QFutureInterfaceBase::Throttled);
-        if (!(state.load() & QFutureInterfaceBase::Paused))
+        state  = QFutureInterfaceBase::State(state & ~QFutureInterfaceBase::Throttled);
+        if (!(state & QFutureInterfaceBase::Paused))
             pausedWaitCondition.wakeAll();
     }
 }
@@ -549,7 +538,7 @@ void QFutureInterfaceBasePrivate::connectOutputInterface(QFutureCallOutInterface
 {
     QMutexLocker locker(&m_mutex);
 
-    if (state.load() & QFutureInterfaceBase::Started) {
+    if (state & QFutureInterfaceBase::Started) {
         interface->postCallOutEvent(QFutureCallOutEvent(QFutureCallOutEvent::Started));
         interface->postCallOutEvent(QFutureCallOutEvent(QFutureCallOutEvent::ProgressRange,
                                                         m_progressMinimum,
@@ -569,13 +558,13 @@ void QFutureInterfaceBasePrivate::connectOutputInterface(QFutureCallOutInterface
         it.batchedAdvance();
     }
 
-    if (state.load() & QFutureInterfaceBase::Paused)
+    if (state & QFutureInterfaceBase::Paused)
         interface->postCallOutEvent(QFutureCallOutEvent(QFutureCallOutEvent::Paused));
 
-    if (state.load() & QFutureInterfaceBase::Canceled)
+    if (state & QFutureInterfaceBase::Canceled)
         interface->postCallOutEvent(QFutureCallOutEvent(QFutureCallOutEvent::Canceled));
 
-    if (state.load() & QFutureInterfaceBase::Finished)
+    if (state & QFutureInterfaceBase::Finished)
         interface->postCallOutEvent(QFutureCallOutEvent(QFutureCallOutEvent::Finished));
 
     outputConnections.append(interface);
@@ -594,7 +583,7 @@ void QFutureInterfaceBasePrivate::disconnectOutputInterface(QFutureCallOutInterf
 
 void QFutureInterfaceBasePrivate::setState(QFutureInterfaceBase::State newState)
 {
-    state.store(newState);
+    state = newState;
 }
 
 QT_END_NAMESPACE
