@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -33,6 +39,7 @@
 
 #include "qstringbuilder.h"
 #include <QtCore/qtextcodec.h>
+#include <private/qutfcodec_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -43,7 +50,7 @@ QT_BEGIN_NAMESPACE
     \reentrant
     \since 4.6
 
-    \brief The QStringBuilder class is a template class that provides a facility to build up QStrings and QByteArrays from smaller chunks.
+    \brief The QStringBuilder class is a template class that provides a facility to build up QStrings from smaller chunks.
 
     \ingroup tools
     \ingroup shared
@@ -51,34 +58,22 @@ QT_BEGIN_NAMESPACE
 
 
     To build a QString by multiple concatenations, QString::operator+()
-    is typically used. This causes \e{n - 1} allocations when building
-    a string from \e{n} chunks. The same is true for QByteArray.
+    is typically used. This causes \e{n - 1} reallocations when building
+    a string from \e{n} chunks.
 
     QStringBuilder uses expression templates to collect the individual
     chunks, compute the total size, allocate the required amount of
-    memory for the final string object, and copy the chunks into the
+    memory for the final QString object, and copy the chunks into the
     allocated memory.
 
     The QStringBuilder class is not to be used explicitly in user
     code.  Instances of the class are created as return values of the
-    operator%() function, acting on objects of the following types:
-
-    For building QStrings:
-
-    \li QString, QStringRef,
-    \li QChar, QCharRef, QLatin1Char,
-    \li QLatin1String,
-    \li QByteArray, \c char, \c{const char[]}.
-
-    The types in the last list point are only available when
-    QT_NO_CAST_FROM_ASCII is not defined.
-
-    For building QByteArrays:
-
-    \li QByteArray, \c char, \c{const char[]}.
+    operator%() function, acting on objects of type QString,
+    QLatin1String, QStringRef, QChar, QCharRef,
+    QLatin1Char, and \c char.
 
     Concatenating strings with operator%() generally yields better
-    performance than using \c QString::operator+() on the same chunks
+    performance then using \c QString::operator+() on the same chunks
     if there are three or more of them, and performs equally well in other
     cases.
 
@@ -113,29 +108,14 @@ QT_BEGIN_NAMESPACE
 /*!
     \internal
  */
-void QAbstractConcatenable::convertFromAscii(const char *a, int len, QChar *&out)
+void QAbstractConcatenable::convertFromAscii(const char *a, int len, QChar *&out) Q_DECL_NOTHROW
 {
-    if (len == -1) {
+    if (Q_UNLIKELY(len == -1)) {
         if (!a)
             return;
-        while (*a && uchar(*a) < 0x80U)
-            *out++ = QLatin1Char(*a++);
-        if (!*a)
-            return;
-    } else {
-        int i;
-        for (i = 0; i < len && uchar(a[i]) < 0x80U; ++i)
-            *out++ = QLatin1Char(a[i]);
-        if (i == len)
-            return;
-        a += i;
-        len -= i;
+        len = int(strlen(a));
     }
-
-    // we need to complement with UTF-8 appending
-    QString tmp = QString::fromUtf8(a, len);
-    memcpy(out, reinterpret_cast<const char *>(tmp.constData()), sizeof(QChar) * tmp.size());
-    out += tmp.size();
+    out = QUtf8::convertToUnicode(out, a, len);
 }
 
 QT_END_NAMESPACE
