@@ -1,32 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
+** Copyright (C) 2016 The Qt Company Ltd.
 ** Copyright (C) 2015 Olivier Goffart <ogoffart@woboq.com>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,10 +47,13 @@
 #include "private/qiconloader_p.h"
 #include "qpainter.h"
 #include "qfileinfo.h"
+#include <qmimedatabase.h>
+#include <qmimetype.h>
 #include "qpixmapcache.h"
 #include "qvariant.h"
 #include "qcache.h"
 #include "qdebug.h"
+#include "qdir.h"
 #include "qpalette.h"
 #include "qmath.h"
 
@@ -277,7 +286,7 @@ QPixmap QPixmapIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::St
     if (pm.isNull()) {
         int idx = pixmaps.count();
         while (--idx >= 0) {
-            if (pe == &pixmaps[idx]) {
+            if (pe == &pixmaps.at(idx)) {
                 pixmaps.remove(idx);
                 break;
             }
@@ -294,7 +303,7 @@ QPixmap QPixmapIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::St
 
     QString key = QLatin1String("qt_")
                   % HexString<quint64>(pm.cacheKey())
-                  % HexString<uint>(pe ? pe->mode : QIcon::Normal)
+                  % HexString<uint>(pe->mode)
                   % HexString<quint64>(QGuiApplication::palette().cacheKey())
                   % HexString<uint>(actualSize.width())
                   % HexString<uint>(actualSize.height());
@@ -436,7 +445,7 @@ void QPixmapIconEngine::addFile(const QString &fileName, const QSize &size, QIco
             }
         }
     }
-    foreach (const QImage &i, icoImages)
+    for (const QImage &i : qAsConst(icoImages))
         pixmaps += QPixmapIconEngineEntry(abs, i, mode, state);
     if (icoImages.isEmpty() && !ignoreSize) // Add placeholder with the filename and empty pixmap for the size.
         pixmaps += QPixmapIconEngineEntry(abs, size, mode, state);
@@ -523,7 +532,6 @@ void QPixmapIconEngine::virtual_hook(int id, void *data)
     }
 }
 
-#ifndef QT_NO_LIBRARY
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
     (QIconEngineFactoryInterface_iid, QLatin1String("/iconengines"), Qt::CaseInsensitive))
 
@@ -531,8 +539,6 @@ QFactoryLoader *qt_iconEngineFactoryLoader()
 {
     return loader();
 }
-#endif
-
 
 
 /*!
@@ -923,7 +929,7 @@ void QIcon::paint(QPainter *painter, const QRect &rect, Qt::Alignment alignment,
 */
 bool QIcon::isNull() const
 {
-    return !d;
+    return !d || d->engine->isNull();
 }
 
 /*!\internal
@@ -938,7 +944,12 @@ bool QIcon::isDetached() const
 void QIcon::detach()
 {
     if (d) {
-        if (d->ref.load() != 1) {
+        if (d->engine->isNull()) {
+            if (!d->ref.deref())
+                delete d;
+            d = 0;
+            return;
+        } else if (d->ref.load() != 1) {
             QIconPrivate *x = new QIconPrivate;
             x->engine = d->engine->clone();
             if (!d->ref.deref())
@@ -962,15 +973,26 @@ void QIcon::addPixmap(const QPixmap &pixmap, Mode mode, State state)
 {
     if (pixmap.isNull())
         return;
+    detach();
     if (!d) {
         d = new QIconPrivate;
         d->engine = new QPixmapIconEngine;
-    } else {
-        detach();
     }
     d->engine->addPixmap(pixmap, mode, state);
 }
 
+static QIconEngine *iconEngineFromSuffix(const QString &fileName, const QString &suffix)
+{
+    if (!suffix.isEmpty()) {
+        const int index = loader()->indexOf(suffix);
+        if (index != -1) {
+            if (QIconEnginePlugin *factory = qobject_cast<QIconEnginePlugin*>(loader()->instance(index))) {
+                return factory->create(fileName);
+            }
+        }
+    }
+    return nullptr;
+}
 
 /*!  Adds an image from the file with the given \a fileName to the
      icon, as a specialization for \a size, \a mode and \a state. The
@@ -1006,30 +1028,17 @@ void QIcon::addFile(const QString &fileName, const QSize &size, Mode mode, State
 {
     if (fileName.isEmpty())
         return;
+    detach();
     if (!d) {
-#ifndef QT_NO_LIBRARY
+
         QFileInfo info(fileName);
-        QString suffix = info.suffix();
-        if (!suffix.isEmpty()) {
-            // first try version 2 engines..
-            const int index = loader()->indexOf(suffix);
-            if (index != -1) {
-                if (QIconEnginePlugin *factory = qobject_cast<QIconEnginePlugin*>(loader()->instance(index))) {
-                    if (QIconEngine *engine = factory->create(fileName)) {
-                        d = new QIconPrivate;
-                        d->engine = engine;
-                    }
-                }
-            }
-        }
-#endif
-        // ...then fall back to the default engine
-        if (!d) {
-            d = new QIconPrivate;
-            d->engine = new QPixmapIconEngine;
-        }
-    } else {
-        detach();
+        QIconEngine *engine = iconEngineFromSuffix(fileName, info.suffix());
+#ifndef QT_NO_MIMETYPE
+        if (!engine)
+            engine = iconEngineFromSuffix(fileName, QMimeDatabase().mimeTypeForFile(info).preferredSuffix());
+#endif // !QT_NO_MIMETYPE
+        d = new QIconPrivate;
+        d->engine = engine ? engine : new QPixmapIconEngine;
     }
 
     d->engine->addFile(fileName, size, mode, state);
@@ -1139,8 +1148,7 @@ QString QIcon::themeName()
     \since 4.6
 
     Returns the QIcon corresponding to \a name in the current
-    icon theme. If no such icon is found in the current theme
-    \a fallback is returned instead.
+    icon theme.
 
     The latest version of the freedesktop icon specification and naming
     specification can be obtained here:
@@ -1154,24 +1162,25 @@ QString QIcon::themeName()
 
     \snippet code/src_gui_image_qicon.cpp 3
 
-    Or if you want to provide a guaranteed fallback for platforms that
-    do not support theme icons, you can use the second argument:
-
-    \snippet code/src_gui_image_qicon.cpp 4
-
     \note By default, only X11 will support themed icons. In order to
     use themed icons on Mac and Windows, you will have to bundle a
     compliant theme in one of your themeSearchPaths() and set the
     appropriate themeName().
 
+    \note Qt will make use of GTK's icon-theme.cache if present to speed up
+    the lookup. These caches can be generated using gtk-update-icon-cache:
+    \l{https://developer.gnome.org/gtk3/stable/gtk-update-icon-cache.html}.
+
     \sa themeName(), setThemeName(), themeSearchPaths()
 */
-QIcon QIcon::fromTheme(const QString &name, const QIcon &fallback)
+QIcon QIcon::fromTheme(const QString &name)
 {
     QIcon icon;
 
     if (qtIconCache()->contains(name)) {
         icon = *qtIconCache()->object(name);
+    } else if (QDir::isAbsolutePath(name)) {
+        return QIcon(name);
     } else {
         QPlatformTheme * const platformTheme = QGuiApplicationPrivate::platformTheme();
         bool hasUserTheme = QIconLoader::instance()->hasUserTheme();
@@ -1182,7 +1191,26 @@ QIcon QIcon::fromTheme(const QString &name, const QIcon &fallback)
         qtIconCache()->insert(name, cachedIcon);
     }
 
-    if (qApp && icon.availableSizes().isEmpty())
+    return icon;
+}
+
+/*!
+    \overload
+
+    Returns the QIcon corresponding to \a name in the current
+    icon theme. If no such icon is found in the current theme
+    \a fallback is returned instead.
+
+    If you want to provide a guaranteed fallback for platforms that
+    do not support theme icons, you can use the second argument:
+
+    \snippet code/src_gui_image_qicon.cpp 4
+*/
+QIcon QIcon::fromTheme(const QString &name, const QIcon &fallback)
+{
+    QIcon icon = fromTheme(name);
+
+    if (icon.isNull() || icon.availableSizes().isEmpty())
         return fallback;
 
     return icon;
@@ -1206,8 +1234,8 @@ bool QIcon::hasThemeIcon(const QString &name)
 /*!
     \since 5.6
 
-    Indicate that this icon is a mask image, and hence can potentially
-    be modified based on where it's displayed.
+    Indicate that this icon is a mask image(boolean \a isMask), and hence can
+    potentially be modified based on where it's displayed.
     \sa isMask()
 */
 void QIcon::setIsMask(bool isMask)
@@ -1307,7 +1335,6 @@ QDataStream &operator>>(QDataStream &s, QIcon &icon)
             QIconEngine *engine = new QIconLoaderEngine();
             icon.d->engine = engine;
             engine->read(s);
-#ifndef QT_NO_LIBRARY
         } else {
             const int index = loader()->indexOf(key);
             if (index != -1) {
@@ -1319,7 +1346,6 @@ QDataStream &operator>>(QDataStream &s, QIcon &icon)
                     } // factory
                 } // instance
             } // index
-#endif
         }
     } else if (s.version() == QDataStream::Qt_4_2) {
         icon = QIcon();
