@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -76,7 +82,7 @@ struct QIconDirInfo
     short maxSize;
     short minSize;
     short threshold;
-    Type type : 4;
+    Type type;
 };
 Q_DECLARE_TYPEINFO(QIconDirInfo, Q_MOVABLE_TYPE);
 
@@ -106,6 +112,12 @@ struct PixmapEntry : public QIconLoaderEngineEntry
 
 typedef QList<QIconLoaderEngineEntry*> QThemeIconEntries;
 
+struct QThemeIconInfo
+{
+    QThemeIconEntries entries;
+    QString iconName;
+};
+
 class QIconLoaderEngine : public QIconEngine
 {
 public:
@@ -126,13 +138,15 @@ private:
     void virtual_hook(int id, void *data);
     QIconLoaderEngineEntry *entryForSize(const QSize &size);
     QIconLoaderEngine(const QIconLoaderEngine &other);
-    QThemeIconEntries m_entries;
+    QThemeIconInfo m_info;
     QString m_iconName;
     uint m_key;
 
     friend class QIconLoader;
 };
 
+class QIconCacheGtkReader;
+
 class QIconTheme
 {
 public:
@@ -140,21 +154,22 @@ public:
     QIconTheme() : m_valid(false) {}
     QStringList parents() { return m_parents; }
     QVector<QIconDirInfo> keyList() { return m_keyList; }
-    QString contentDir() { return m_contentDir; }
+    QStringList contentDirs() { return m_contentDirs; }
     bool isValid() { return m_valid; }
-
 private:
-    QString m_contentDir;
+    QStringList m_contentDirs;
     QVector<QIconDirInfo> m_keyList;
     QStringList m_parents;
     bool m_valid;
+public:
+    QVector<QSharedPointer<QIconCacheGtkReader>> m_gtkCaches;
 };
 
 class Q_GUI_EXPORT QIconLoader
 {
 public:
     QIconLoader();
-    QThemeIconEntries loadIcon(const QString &iconName) const;
+    QThemeIconInfo loadIcon(const QString &iconName) const;
     uint themeKey() const { return m_themeKey; }
 
     QString themeName() const { return m_userTheme.isEmpty() ? m_systemTheme : m_userTheme; }
@@ -167,11 +182,12 @@ public:
     void updateSystemTheme();
     void invalidateKey() { m_themeKey++; }
     void ensureInitialized();
+    bool hasUserTheme() const { return !m_userTheme.isEmpty(); }
 
 private:
-    QThemeIconEntries findIconHelper(const QString &themeName,
-                                     const QString &iconName,
-                                     QStringList &visited) const;
+    QThemeIconInfo findIconHelper(const QString &themeName,
+                                  const QString &iconName,
+                                  QStringList &visited) const;
     uint m_themeKey;
     bool m_supportsSvg;
     bool m_initialized;
