@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -64,7 +70,7 @@ QT_BEGIN_NAMESPACE
 
 static bool qt_pixmap_thread_test()
 {
-    if (!QCoreApplication::instance()) {
+    if (Q_UNLIKELY(!QCoreApplication::instance())) {
         qFatal("QPixmap: Must construct a QGuiApplication before a QPixmap");
         return false;
     }
@@ -762,37 +768,39 @@ QBitmap QPixmap::createMaskFromColor(const QColor &maskColor, Qt::MaskMode mode)
 
 bool QPixmap::load(const QString &fileName, const char *format, Qt::ImageConversionFlags flags)
 {
-    if (!fileName.isEmpty()) {
-
-        QFileInfo info(fileName);
-        // Note: If no extension is provided, we try to match the
-        // file against known plugin extensions
-        if (info.completeSuffix().isEmpty() || info.exists()) {
-
-            QString key = QLatin1String("qt_pixmap")
-                    % info.absoluteFilePath()
-                    % HexString<uint>(info.lastModified().toTime_t())
-                    % HexString<quint64>(info.size())
-                    % HexString<uint>(data ? data->pixelType() : QPlatformPixmap::PixmapType);
-
-            if (QPixmapCache::find(key, this))
-                return true;
-
-            data = QPlatformPixmap::create(0, 0, data ? data->pixelType() : QPlatformPixmap::PixmapType);
-
-            if (data->fromFile(fileName, format, flags)) {
-                QPixmapCache::insert(key, *this);
-                return true;
-            }
-        }
+    if (fileName.isEmpty()) {
+        data.reset();
+        return false;
     }
 
-    if (!isNull()) {
-        if (isQBitmap())
-            *this = QBitmap();
-        else
-            data.reset();
+    detach();
+
+    QFileInfo info(fileName);
+    QString key = QLatin1String("qt_pixmap")
+                  % info.absoluteFilePath()
+                  % HexString<uint>(info.lastModified().toTime_t())
+                  % HexString<quint64>(info.size())
+                  % HexString<uint>(data ? data->pixelType() : QPlatformPixmap::PixmapType);
+
+    // Note: If no extension is provided, we try to match the
+    // file against known plugin extensions
+    if (!info.completeSuffix().isEmpty() && !info.exists()) {
+        data.reset();
+        return false;
     }
+
+    if (QPixmapCache::find(key, this))
+        return true;
+
+    if (!data)
+        data = QPlatformPixmap::create(0, 0, QPlatformPixmap::PixmapType);
+
+    if (data->fromFile(fileName, format, flags)) {
+        QPixmapCache::insert(key, *this);
+        return true;
+    }
+
+    data.reset();
     return false;
 }
 
