@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -118,18 +124,17 @@ static bool read_xbm_body(QIODevice *device, int w, int h, QImage *outImage)
 
     qint64 readBytes = 0;
 
-    char *p;
-
     // scan for database
-    do {
+    for (;;) {
         if ((readBytes = device->readLine(buf, buflen)) <= 0) {
             // end of file
             return false;
         }
 
         buf[readBytes] = '\0';
-        p = strstr(buf, "0x");
-    } while (!p);
+        if (QByteArray::fromRawData(buf, readBytes).contains("0x"))
+            break;
+    }
 
     if (outImage->size() != QSize(w, h) || outImage->format() != QImage::Format_MonoLSB) {
         *outImage = QImage(w, h, QImage::Format_MonoLSB);
@@ -143,6 +148,7 @@ static bool read_xbm_body(QIODevice *device, int w, int h, QImage *outImage)
 
     int           x = 0, y = 0;
     uchar *b = outImage->scanLine(0);
+    char  *p = buf + QByteArray::fromRawData(buf, readBytes).indexOf("0x");
     w = (w+7)/8;                                // byte width
 
     while (y < h) {                                // for all encoded bytes...
@@ -157,8 +163,7 @@ static bool read_xbm_body(QIODevice *device, int w, int h, QImage *outImage)
         } else {                                // read another line
             if ((readBytes = device->readLine(buf,buflen)) <= 0)        // EOF ==> truncated image
                 break;
-            buf[readBytes] = '\0';
-            p = strstr(buf, "0x");
+            p = buf + QByteArray::fromRawData(buf, readBytes).indexOf("0x");
         }
     }
 
