@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -33,10 +39,10 @@
 
 #include "qplatformdialoghelper.h"
 
+#include <QtCore/QCoreApplication>
 #include <QtCore/QVariant>
 #include <QtCore/QSharedData>
 #include <QtCore/QSettings>
-#include <QtCore/QHash>
 #include <QtCore/QUrl>
 #include <QtGui/QColor>
 
@@ -120,6 +126,8 @@ static const int buttonRoleLayouts[2][5][14] =
 
 QPlatformDialogHelper::QPlatformDialogHelper()
 {
+    qRegisterMetaType<StandardButton>();
+    qRegisterMetaType<ButtonRole>();
 }
 
 QPlatformDialogHelper::~QPlatformDialogHelper()
@@ -250,7 +258,7 @@ void QColorDialogStaticData::readSettings()
 #ifndef QT_NO_SETTINGS
     const QSettings settings(QSettings::UserScope, QStringLiteral("QtProject"));
     for (int i = 0; i < int(CustomColorCount); ++i) {
-        const QVariant v = settings.value(QStringLiteral("Qt/customColors/") + QString::number(i));
+        const QVariant v = settings.value(QLatin1String("Qt/customColors/") + QString::number(i));
         if (v.isValid())
             customRgb[i] = v.toUInt();
     }
@@ -263,7 +271,7 @@ void QColorDialogStaticData::writeSettings() const
     if (!customSet) {
         QSettings settings(QSettings::UserScope, QStringLiteral("QtProject"));
         for (int i = 0; i < int(CustomColorCount); ++i)
-            settings.setValue(QStringLiteral("Qt/customColors/") + QString::number(i), customRgb[i]);
+            settings.setValue(QLatin1String("Qt/customColors/") + QString::number(i), customRgb[i]);
     }
 #endif
 }
@@ -404,7 +412,8 @@ public:
         viewMode(QFileDialogOptions::Detail),
         fileMode(QFileDialogOptions::AnyFile),
         acceptMode(QFileDialogOptions::AcceptOpen),
-        filters(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::AllDirs)
+        filters(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::AllDirs),
+        useDefaultNameFilters(true)
     {}
 
     QFileDialogOptions::FileDialogOptions options;
@@ -416,6 +425,7 @@ public:
     QString labels[QFileDialogOptions::DialogLabelCount];
     QDir::Filters filters;
     QList<QUrl> sidebarUrls;
+    bool useDefaultNameFilters;
     QStringList nameFilters;
     QStringList mimeTypeFilters;
     QString defaultSuffix;
@@ -423,6 +433,7 @@ public:
     QUrl initialDirectory;
     QString initiallySelectedNameFilter;
     QList<QUrl> initiallySelectedFiles;
+    QStringList supportedSchemes;
 };
 
 QFileDialogOptions::QFileDialogOptions() : d(new QFileDialogOptionsPrivate)
@@ -526,14 +537,48 @@ QList<QUrl> QFileDialogOptions::sidebarUrls() const
     return d->sidebarUrls;
 }
 
+/*!
+    \since 5.7
+    \internal
+    The bool property useDefaultNameFilters indicates that no name filters have been
+    set or that they are equivalent to \gui{All Files (*)}. If it is true, the
+    platform can choose to hide the filter combo box.
+
+    \sa defaultNameFilterString().
+*/
+bool QFileDialogOptions::useDefaultNameFilters() const
+{
+    return d->useDefaultNameFilters;
+}
+
+void QFileDialogOptions::setUseDefaultNameFilters(bool dnf)
+{
+    d->useDefaultNameFilters = dnf;
+}
+
 void QFileDialogOptions::setNameFilters(const QStringList &filters)
 {
+    d->useDefaultNameFilters = filters.size() == 1
+        && filters.first() == QFileDialogOptions::defaultNameFilterString();
     d->nameFilters = filters;
 }
 
 QStringList QFileDialogOptions::nameFilters() const
 {
-    return d->nameFilters;
+    return d->useDefaultNameFilters ?
+        QStringList(QFileDialogOptions::defaultNameFilterString()) : d->nameFilters;
+}
+
+/*!
+    \since 5.6
+    \internal
+    \return The translated default name filter string (\gui{All Files (*)}).
+    \sa defaultNameFilters(), nameFilters()
+*/
+
+QString QFileDialogOptions::defaultNameFilterString()
+{
+    return QCoreApplication::translate("QFileDialog", "All Files (*)");
 }
 
 void QFileDialogOptions::setMimeTypeFilters(const QStringList &filters)
@@ -614,6 +659,18 @@ void QFileDialogOptions::setInitiallySelectedFiles(const QList<QUrl> &files)
     d->initiallySelectedFiles = files;
 }
 
+// Schemes supported by the application
+void QFileDialogOptions::setSupportedSchemes(const QStringList &schemes)
+{
+    d->supportedSchemes = schemes;
+}
+
+QStringList QFileDialogOptions::supportedSchemes() const
+{
+    return d->supportedSchemes;
+}
+
+// Return true if the URL is supported by the filedialog implementation *and* by the application.
 bool QPlatformFileDialogHelper::isSupportedUrl(const QUrl &url) const
 {
     return url.isLocalFile();
@@ -639,12 +696,13 @@ void QPlatformFileDialogHelper::setOptions(const QSharedPointer<QFileDialogOptio
 }
 
 const char *QPlatformFileDialogHelper::filterRegExp =
-"^(.*)\\(([a-zA-Z0-9_.*? +;#\\-\\[\\]@\\{\\}/!<>\\$%&=^~:\\|]*)\\)$";
+"^(.*)\\(([a-zA-Z0-9_.,*? +;#\\-\\[\\]@\\{\\}/!<>\\$%&=^~:\\|]*)\\)$";
 
 // Makes a list of filters from a normal filter string "Image Files (*.png *.jpg)"
 QStringList QPlatformFileDialogHelper::cleanFilterList(const QString &filter)
 {
     QRegExp regexp(QString::fromLatin1(filterRegExp));
+    Q_ASSERT(regexp.isValid());
     QString f = filter;
     int i = regexp.indexIn(f);
     if (i >= 0)
