@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -103,11 +101,19 @@ public:
         KeyboardScheme,
         UiEffects,
         SpellCheckUnderlineStyle,
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+        TabFocusBehavior,
+#else
         TabAllWidgets,
+        TabFocusBehavior = TabAllWidgets,
+#endif
         IconPixmapSizes,
         PasswordMaskCharacter,
         DialogSnapToDefaultButton,
-        ContextMenuOnMouseRelease
+        ContextMenuOnMouseRelease,
+        MousePressAndHoldInterval,
+        MouseDoubleClickDistance,
+        WheelScrollLines
     };
 
     enum DialogType {
@@ -128,6 +134,7 @@ public:
         ComboBoxPalette,
         ItemViewPalette,
         MessageBoxLabelPelette,
+        MessageBoxLabelPalette = MessageBoxLabelPelette,
         TabBarPalette,
         LabelPalette,
         GroupBoxPalette,
@@ -163,6 +170,9 @@ public:
         SmallFont,
         MiniFont,
         FixedFont,
+        GroupBoxTitleFont,
+        TabButtonFont,
+        EditorFont,
         NFonts
     };
 
@@ -237,6 +247,7 @@ public:
         MediaSeekBackward,
         MediaVolume,
         MediaVolumeMuted,
+        LineEditClearButton,
         // do not add any values below/greater than this
         CustomBase = 0xf0000000
     };
@@ -273,6 +284,7 @@ public:
     virtual QPlatformMenuItem* createPlatformMenuItem() const;
     virtual QPlatformMenu* createPlatformMenu() const;
     virtual QPlatformMenuBar* createPlatformMenuBar() const;
+    virtual void showPlatformMenuBar() {}
 
     virtual bool usePlatformNativeDialog(DialogType type) const;
     virtual QPlatformDialogHelper *createPlatformDialogHelper(DialogType type) const;
@@ -295,7 +307,11 @@ public:
 
     virtual QList<QKeySequence> keyBindings(QKeySequence::StandardKey key) const;
 
+    virtual QString standardButtonText(int button) const;
+
     static QVariant defaultThemeHint(ThemeHint hint);
+    static QString defaultStandardButtonText(int button);
+    static QString removeMnemonics(const QString &original);
 
 protected:
     explicit QPlatformTheme(QPlatformThemePrivate *priv);
