@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -491,10 +497,12 @@ QPlatformScreen *QPlatformWindow::screenForGeometry(const QRect &newGeometry) co
         center = mapToGlobal(center - newGeometry.topLeft());
 
     if (!parent() && currentScreen && !currentScreen->geometry().contains(center)) {
-        Q_FOREACH (QPlatformScreen* screen, currentScreen->virtualSiblings()) {
-            if (screen->geometry().contains(center))
+        const auto screens = currentScreen->virtualSiblings();
+        for (QPlatformScreen *screen : screens) {
+            const QRect screenGeometry = screen->geometry();
+            if (screenGeometry.contains(center))
                 return screen;
-            if (screen->geometry().intersects(newGeometry))
+            if (screenGeometry.intersects(newGeometry))
                 fallback = screen;
         }
     }
@@ -546,13 +554,14 @@ static inline const QScreen *effectiveScreen(const QWindow *window)
     const QScreen *screen = window->screen();
     if (!screen)
         return QGuiApplication::primaryScreen();
-    const QList<QScreen *> siblings = screen->virtualSiblings();
 #ifndef QT_NO_CURSOR
+    const QList<QScreen *> siblings = screen->virtualSiblings();
     if (siblings.size() > 1) {
         const QPoint referencePoint = window->transientParent() ? window->transientParent()->geometry().center() : QCursor::pos();
-        foreach (const QScreen *sibling, siblings)
+        for (const QScreen *sibling : siblings) {
             if (sibling->geometry().contains(referencePoint))
                 return sibling;
+        }
     }
 #endif
     return screen;
@@ -572,6 +581,20 @@ void QPlatformWindow::invalidateSurface()
 {
 }
 
+static QSize fixInitialSize(QSize size, const QWindow *w,
+                            int defaultWidth, int defaultHeight)
+{
+    if (size.width() == 0) {
+        const int minWidth = w->minimumWidth();
+        size.setWidth(minWidth > 0 ? minWidth : defaultWidth);
+    }
+    if (size.height() == 0) {
+        const int minHeight = w->minimumHeight();
+        size.setHeight(minHeight > 0 ? minHeight : defaultHeight);
+    }
+    return size;
+}
+
 /*!
     Helper function to get initial geometry on windowing systems which do not
     do smart positioning and also do not provide a means of centering a
@@ -584,19 +607,18 @@ void QPlatformWindow::invalidateSurface()
 QRect QPlatformWindow::initialGeometry(const QWindow *w,
     const QRect &initialGeometry, int defaultWidth, int defaultHeight)
 {
+    if (!w->isTopLevel()) {
+        const qreal factor = QHighDpiScaling::factor(w);
+        const QSize size = fixInitialSize(QHighDpi::fromNative(initialGeometry.size(), factor),
+                                          w, defaultWidth, defaultHeight);
+        return QRect(initialGeometry.topLeft(), QHighDpi::toNative(size, factor));
+    }
     const QScreen *screen = effectiveScreen(w);
     if (!screen)
         return initialGeometry;
     QRect rect(QHighDpi::fromNativePixels(initialGeometry, w));
-    if (rect.width() == 0) {
-        const int minWidth = w->minimumWidth();
-        rect.setWidth(minWidth > 0 ? minWidth : defaultWidth);
-    }
-    if (rect.height() == 0) {
-        const int minHeight = w->minimumHeight();
-        rect.setHeight(minHeight > 0 ? minHeight : defaultHeight);
-    }
-    if (w->isTopLevel() && qt_window_private(const_cast<QWindow*>(w))->positionAutomatic
+    rect.setSize(fixInitialSize(rect.size(), w, defaultWidth, defaultHeight));
+    if (qt_window_private(const_cast<QWindow*>(w))->positionAutomatic
             && w->type() != Qt::Popup) {
         const QRect availableGeometry = screen->availableGeometry();
         // Center unless the geometry ( + unknown window frame) is too large for the screen).
