@@ -1,31 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -311,90 +318,6 @@ void QT_FASTCALL comp_func_solid_SourceOver_sse2(uint *destPixels, int length, u
     }
 }
 
-#ifndef QDRAWHELPER_AVX
-CompositionFunctionSolid qt_functionForModeSolid_SSE2[numCompositionFunctions] = {
-    comp_func_solid_SourceOver_sse2,
-    comp_func_solid_DestinationOver,
-    comp_func_solid_Clear,
-    comp_func_solid_Source,
-    comp_func_solid_Destination,
-    comp_func_solid_SourceIn,
-    comp_func_solid_DestinationIn,
-    comp_func_solid_SourceOut,
-    comp_func_solid_DestinationOut,
-    comp_func_solid_SourceAtop,
-    comp_func_solid_DestinationAtop,
-    comp_func_solid_XOR,
-    comp_func_solid_Plus,
-    comp_func_solid_Multiply,
-    comp_func_solid_Screen,
-    comp_func_solid_Overlay,
-    comp_func_solid_Darken,
-    comp_func_solid_Lighten,
-    comp_func_solid_ColorDodge,
-    comp_func_solid_ColorBurn,
-    comp_func_solid_HardLight,
-    comp_func_solid_SoftLight,
-    comp_func_solid_Difference,
-    comp_func_solid_Exclusion,
-    rasterop_solid_SourceOrDestination,
-    rasterop_solid_SourceAndDestination,
-    rasterop_solid_SourceXorDestination,
-    rasterop_solid_NotSourceAndNotDestination,
-    rasterop_solid_NotSourceOrNotDestination,
-    rasterop_solid_NotSourceXorDestination,
-    rasterop_solid_NotSource,
-    rasterop_solid_NotSourceAndDestination,
-    rasterop_solid_SourceAndNotDestination,
-    rasterop_solid_NotSourceOrDestination,
-    rasterop_solid_SourceOrNotDestination,
-    rasterop_solid_ClearDestination,
-    rasterop_solid_SetDestination,
-    rasterop_solid_NotDestination
-};
-
-CompositionFunction qt_functionForMode_SSE2[numCompositionFunctions] = {
-    comp_func_SourceOver_sse2,
-    comp_func_DestinationOver,
-    comp_func_Clear,
-    comp_func_Source_sse2,
-    comp_func_Destination,
-    comp_func_SourceIn,
-    comp_func_DestinationIn,
-    comp_func_SourceOut,
-    comp_func_DestinationOut,
-    comp_func_SourceAtop,
-    comp_func_DestinationAtop,
-    comp_func_XOR,
-    comp_func_Plus_sse2,
-    comp_func_Multiply,
-    comp_func_Screen,
-    comp_func_Overlay,
-    comp_func_Darken,
-    comp_func_Lighten,
-    comp_func_ColorDodge,
-    comp_func_ColorBurn,
-    comp_func_HardLight,
-    comp_func_SoftLight,
-    comp_func_Difference,
-    comp_func_Exclusion,
-    rasterop_SourceOrDestination,
-    rasterop_SourceAndDestination,
-    rasterop_SourceXorDestination,
-    rasterop_NotSourceAndNotDestination,
-    rasterop_NotSourceOrNotDestination,
-    rasterop_NotSourceXorDestination,
-    rasterop_NotSource,
-    rasterop_NotSourceAndDestination,
-    rasterop_SourceAndNotDestination,
-    rasterop_NotSourceOrDestination,
-    rasterop_SourceOrNotDestination,
-    rasterop_ClearDestination,
-    rasterop_SetDestination,
-    rasterop_NotDestination
-};
-#endif
-
 void qt_memfill16(quint16 *dest, quint16 value, int count)
 {
     if (count < 3) {
@@ -417,7 +340,7 @@ void qt_memfill16(quint16 *dest, quint16 value, int count)
         dest[count - 1] = value;
 }
 
-void qt_bitmapblit32_sse2(QRasterBuffer *rasterBuffer, int x, int y,
+void qt_bitmapblit32_sse2_base(QRasterBuffer *rasterBuffer, int x, int y,
                           quint32 color,
                           const uchar *src, int width, int height, int stride)
 {
@@ -468,18 +391,25 @@ void qt_bitmapblit32_sse2(QRasterBuffer *rasterBuffer, int x, int y,
     }
 }
 
+void qt_bitmapblit32_sse2(QRasterBuffer *rasterBuffer, int x, int y,
+                          const QRgba64 &color,
+                          const uchar *src, int width, int height, int stride)
+{
+    qt_bitmapblit32_sse2_base(rasterBuffer, x, y, color.toArgb32(), src, width, height, stride);
+}
+
 void qt_bitmapblit8888_sse2(QRasterBuffer *rasterBuffer, int x, int y,
-                            quint32 color,
+                            const QRgba64 &color,
                             const uchar *src, int width, int height, int stride)
 {
-    qt_bitmapblit32_sse2(rasterBuffer, x, y, ARGB2RGBA(color), src, width, height, stride);
+    qt_bitmapblit32_sse2_base(rasterBuffer, x, y, ARGB2RGBA(color.toArgb32()), src, width, height, stride);
 }
 
 void qt_bitmapblit16_sse2(QRasterBuffer *rasterBuffer, int x, int y,
-                          quint32 color,
+                          const QRgba64 &color,
                           const uchar *src, int width, int height, int stride)
 {
-    const quint16 c = qConvertRgb32To16(color);
+    const quint16 c = qConvertRgb32To16(color.toArgb32());
     quint16 *dest = reinterpret_cast<quint16*>(rasterBuffer->scanLine(y)) + x;
     const int destStride = rasterBuffer->bytesPerLine() / sizeof(quint16);
 
@@ -543,7 +473,7 @@ public:
 const uint * QT_FASTCALL qt_fetch_radial_gradient_sse2(uint *buffer, const Operator *op, const QSpanData *data,
                                                        int y, int x, int length)
 {
-    return qt_fetch_radial_gradient_template<QRadialFetchSimd<QSimdSse2> >(buffer, op, data, y, x, length);
+    return qt_fetch_radial_gradient_template<QRadialFetchSimd<QSimdSse2>,uint>(buffer, op, data, y, x, length);
 }
 
 void qt_scale_image_argb32_on_argb32_sse2(uchar *destPixels, int dbpl,
