@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -132,6 +130,19 @@ public:
         return checkStateAgainstMask(capabillitiesState, opacityPixmapMask);
     }
 
+    bool canBlitterDrawCachedGlyphs(const QTransform &transform, QFontEngine::GlyphFormat requestedGlyphFormat, bool complexClip) const
+    {
+        if (transform.type() > QTransform::TxScale)
+            return false;
+        if (!(m_capabilities & QBlittable::DrawScaledCachedGlyphsCapability))
+            return false;
+        if (requestedGlyphFormat == QFontEngine::Format_ARGB && !(m_capabilities & QBlittable::SubPixelGlyphsCapability))
+            return false;
+        if (complexClip && !(m_capabilities & QBlittable::ComplexClipCapability))
+            return false;
+        return true;
+    }
+
     inline void updateState(uint mask, bool on) {
         updateStateBits(&capabillitiesState, mask, on);
     }
@@ -185,7 +196,7 @@ private:
     }
 
     void setSourcePixmapMask() {
-        updateStateBits(&drawPixmapMask, STATE_XFORM_SCALE, true);
+        updateStateBits(&drawPixmapMask, STATE_XFORM_SCALE, false);
         updateStateBits(&drawPixmapMask, STATE_XFORM_COMPLEX, false);
 
         updateStateBits(&drawPixmapMask, STATE_BRUSH_PATTERN, true);
@@ -207,7 +218,7 @@ private:
 
     void setSourceOverScaledPixmapMask() {
         setSourceOverPixmapMask();
-        updateStateBits(&drawRectMask, STATE_XFORM_SCALE, true);
+        updateStateBits(&drawPixmapMask, STATE_XFORM_SCALE, true);
     }
 
     void setOpacityPixmapMask() {
@@ -526,7 +537,7 @@ void QBlitterPaintEngine::fill(const QVectorPath &path, const QBrush &brush)
 {
     Q_D(QBlitterPaintEngine);
     if (path.shape() == QVectorPath::RectangleHint) {
-        QRectF rect(((QPointF *) path.points())[0], ((QPointF *) path.points())[2]);
+        QRectF rect(((const QPointF *) path.points())[0], ((const QPointF *) path.points())[2]);
         fillRect(rect, brush);
     } else {
         d->lock();
@@ -597,12 +608,11 @@ void QBlitterPaintEngine::fillRect(const QRectF &rect, const QBrush &brush)
                     d->pmData->blittable()->drawPixmap(targetRect, pm, srcRect);
                 }
             } else if (clipData->hasRegionClip) {
-                QVector<QRect> clipRects = clipData->clipRegion.rects();
                 QRect unclippedTargetRect(x, y, blitWidth, blitHeight);
-                QRegion intersectedRects = clipData->clipRegion.intersected(unclippedTargetRect);
-
-                for (int i = 0; i < intersectedRects.rects().size(); ++i) {
-                    QRect targetRect = intersectedRects.rects().at(i);
+                const QVector<QRect> intersectedRects = clipData->clipRegion.intersected(unclippedTargetRect).rects();
+                const int intersectedSize = intersectedRects.size();
+                for (int i = 0; i < intersectedSize; ++i) {
+                    const QRect &targetRect = intersectedRects.at(i);
                     if (!targetRect.isValid() || targetRect.isEmpty())
                         continue;
                     int tmpSrcX = srcX + (targetRect.x() - x);
@@ -798,6 +808,28 @@ void QBlitterPaintEngine::drawStaticTextItem(QStaticTextItem *sti)
 #endif
 }
 
+bool QBlitterPaintEngine::drawCachedGlyphs(int numGlyphs, const glyph_t *glyphs, const QFixedPoint *positions, QFontEngine *fontEngine)
+{
+    Q_D(QBlitterPaintEngine);
+    QFontEngine::GlyphFormat glyphFormat = d->glyphCacheFormat;
+    if (fontEngine->glyphFormat != QFontEngine::Format_None)
+        glyphFormat = fontEngine->glyphFormat;
+
+    const QClipData *clipData = d->clip();
+    const bool complexClip = clipData && !clipData->hasRectClip;
+
+    const QPainterState *s = state();
+    if (d->caps.canBlitterDrawCachedGlyphs(s->transform(), glyphFormat, complexClip)) {
+        d->unlock();
+        const bool result = d->pmData->blittable()->drawCachedGlyphs(s, glyphFormat, numGlyphs, glyphs, positions, fontEngine);
+        // Lock again as the raster paint engine might draw decorations now.
+        d->lock();
+        return result;
+    } else {
+        return QRasterPaintEngine::drawCachedGlyphs(numGlyphs, glyphs, positions, fontEngine);
+    }
+}
+
 QT_END_NAMESPACE
 #endif //QT_NO_BLITTABLE
 
