@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -44,6 +50,7 @@
 #include <qpa/qplatformfontdatabase.h>
 
 #include <QtCore/qendian.h>
+#include <QtCore/qfile.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -310,6 +317,13 @@ bool QRawFont::operator==(const QRawFont &other) const
 /*!
    Returns the ascent of this QRawFont in pixel units.
 
+   The ascent of a font is the distance from the baseline to the
+   highest position characters extend to. In practice, some font
+   designers break this rule, e.g. when they put more than one accent
+   on top of a character, or to accommodate an unusual character in
+   an exotic language, so it is possible (though rare) that this
+   value will be too small.
+
    \sa QFontMetricsF::ascent()
 */
 qreal QRawFont::ascent() const
@@ -320,6 +334,11 @@ qreal QRawFont::ascent() const
 /*!
    Returns the descent of this QRawFont in pixel units.
 
+   The descent is the distance from the base line to the lowest point
+   characters extend to. In practice, some font designers break this rule,
+   e.g. to accommodate an unusual character in an exotic language, so
+   it is possible (though rare) that this value will be too small.
+
    \sa QFontMetricsF::descent()
 */
 qreal QRawFont::descent() const
@@ -330,6 +349,8 @@ qreal QRawFont::descent() const
 /*!
    Returns the xHeight of this QRawFont in pixel units.
 
+   This is often but not always the same as the height of the character 'x'.
+
    \sa QFontMetricsF::xHeight()
 */
 qreal QRawFont::xHeight() const
@@ -340,6 +361,8 @@ qreal QRawFont::xHeight() const
 /*!
    Returns the leading of this QRawFont in pixel units.
 
+   This is the natural inter-line spacing.
+
    \sa QFontMetricsF::leading()
 */
 qreal QRawFont::leading() const
@@ -686,13 +709,27 @@ extern int qt_script_for_writing_system(QFontDatabase::WritingSystem writingSyst
 QRawFont QRawFont::fromFont(const QFont &font, QFontDatabase::WritingSystem writingSystem)
 {
     QRawFont rawFont;
-    QFontPrivate *font_d = QFontPrivate::get(font);
+    const QFontPrivate *font_d = QFontPrivate::get(font);
     int script = qt_script_for_writing_system(writingSystem);
     QFontEngine *fe = font_d->engineForScript(script);
 
     if (fe != 0 && fe->type() == QFontEngine::Multi) {
         QFontEngineMulti *multiEngine = static_cast<QFontEngineMulti *>(fe);
         fe = multiEngine->engine(0);
+
+        if (script > QChar::Script_Latin) {
+            // keep in sync with QFontEngineMulti::loadEngine()
+            QFontDef request(multiEngine->fontDef);
+            request.styleStrategy |= QFont::NoFontMerging;
+
+            if (QFontEngine *engine = QFontDatabase::findFont(request, script)) {
+                if (request.weight > QFont::Normal)
+                    engine->fontDef.weight = request.weight;
+                if (request.style > QFont::StyleNormal)
+                    engine->fontDef.style = request.style;
+                fe = engine;
+            }
+        }
         Q_ASSERT(fe);
     }
 
