@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -133,14 +131,12 @@ static bool isValidBlockSeparator(QChar ch)
         || ch == QTextEndOfFrame;
 }
 
-#if !defined(QT_NO_DEBUG) || defined(QT_FORCE_ASSERTS)
 static bool noBlockInString(const QString &str)
 {
     return !str.contains(QChar::ParagraphSeparator)
         && !str.contains(QTextBeginningOfFrame)
         && !str.contains(QTextEndOfFrame);
 }
-#endif
 
 bool QTextUndoCommand::tryMerge(const QTextUndoCommand &other)
 {
@@ -239,7 +235,7 @@ void QTextDocumentPrivate::clear()
 {
     Q_Q(QTextDocument);
 
-    foreach (QTextCursorPrivate *curs, cursors) {
+    for (QTextCursorPrivate *curs : qAsConst(cursors)) {
         curs->setPosition(0);
         curs->currentCharFormat = -1;
         curs->anchor = 0;
@@ -291,7 +287,7 @@ void QTextDocumentPrivate::clear()
 
 QTextDocumentPrivate::~QTextDocumentPrivate()
 {
-    foreach (QTextCursorPrivate *curs, cursors)
+    for (QTextCursorPrivate *curs : qAsConst(cursors))
         curs->priv = 0;
     cursors.clear();
     undoState = 0;
@@ -678,7 +674,7 @@ void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operati
     blockCursorAdjustment = true;
     move(pos, -1, length, op);
     blockCursorAdjustment = false;
-    foreach (QTextCursorPrivate *curs, cursors) {
+    for (QTextCursorPrivate *curs : qAsConst(cursors)) {
         if (curs->adjustPosition(pos, -length, op) == QTextCursorPrivate::CursorMoved) {
             curs->changed = true;
         }
@@ -1010,9 +1006,9 @@ int QTextDocumentPrivate::undoRedo(bool undo)
         bool inBlock = (
                 undoState > 0
                 && undoState < undoStack.size()
-                && undoStack[undoState].block_part
-                && undoStack[undoState-1].block_part
-                && !undoStack[undoState-1].block_end
+                && undoStack.at(undoState).block_part
+                && undoStack.at(undoState - 1).block_part
+                && !undoStack.at(undoState - 1).block_end
                 );
         if (!inBlock)
             break;
@@ -1078,12 +1074,13 @@ void QTextDocumentPrivate::appendUndoItem(const QTextUndoCommand &c)
 
 
     if (!undoStack.isEmpty() && modified) {
-        QTextUndoCommand &last = undoStack[undoState - 1];
+        const int lastIdx = undoState - 1;
+        const QTextUndoCommand &last = undoStack.at(lastIdx);
 
         if ( (last.block_part && c.block_part && !last.block_end) // part of the same block => can merge
             || (!c.block_part && !last.block_part)) {  // two single undo items => can merge
 
-            if (last.tryMerge(c))
+            if (undoStack[lastIdx].tryMerge(c))
                 return;
         }
     }
@@ -1105,7 +1102,7 @@ void QTextDocumentPrivate::clearUndoRedoStacks(QTextDocument::Stacks stacksToCle
     bool redoCommandsAvailable = undoState != undoStack.size();
     if (stacksToClear == QTextDocument::UndoStack && undoCommandsAvailable) {
         for (int i = 0; i < undoState; ++i) {
-            QTextUndoCommand c = undoStack[undoState];
+            QTextUndoCommand c = undoStack.at(undoState);
             if (c.command & QTextUndoCommand::Custom)
                 delete c.custom;
         }
@@ -1117,7 +1114,7 @@ void QTextDocumentPrivate::clearUndoRedoStacks(QTextDocument::Stacks stacksToCle
     } else if (stacksToClear == QTextDocument::RedoStack
                && redoCommandsAvailable) {
         for (int i = undoState; i < undoStack.size(); ++i) {
-            QTextUndoCommand c = undoStack[i];
+            QTextUndoCommand c = undoStack.at(i);
             if (c.command & QTextUndoCommand::Custom)
                 delete c.custom;
         }
@@ -1127,12 +1124,12 @@ void QTextDocumentPrivate::clearUndoRedoStacks(QTextDocument::Stacks stacksToCle
     } else if (stacksToClear == QTextDocument::UndoAndRedoStacks
                && !undoStack.isEmpty()) {
         for (int i = 0; i < undoStack.size(); ++i) {
-            QTextUndoCommand c = undoStack[i];
+            QTextUndoCommand c = undoStack.at(i);
             if (c.command & QTextUndoCommand::Custom)
                 delete c.custom;
         }
         undoState = 0;
-        undoStack.resize(0);
+        undoStack.clear();
         if (emitSignals && undoCommandsAvailable)
             emitUndoAvailable(false);
         if (emitSignals && redoCommandsAvailable)
@@ -1190,8 +1187,8 @@ void QTextDocumentPrivate::endEditBlock()
         return;
 
     if (undoEnabled && undoState > 0) {
-        const bool wasBlocking = !undoStack[undoState - 1].block_end;
-        if (undoStack[undoState - 1].block_part) {
+        const bool wasBlocking = !undoStack.at(undoState - 1).block_end;
+        if (undoStack.at(undoState - 1).block_part) {
             undoStack[undoState - 1].block_end = true;
             if (wasBlocking)
                 emit document()->undoCommandAdded();
@@ -1236,13 +1233,13 @@ void QTextDocumentPrivate::finishEdit()
     }
 
     QList<QTextCursor> changedCursors;
-    foreach (QTextCursorPrivate *curs, cursors) {
+    for (QTextCursorPrivate *curs : qAsConst(cursors)) {
         if (curs->changed) {
             curs->changed = false;
             changedCursors.append(QTextCursor(curs));
         }
     }
-    foreach (const QTextCursor &cursor, changedCursors)
+    for (const QTextCursor &cursor : qAsConst(changedCursors))
         emit q->cursorPositionChanged(cursor);
 
     contentsChanged();
@@ -1288,7 +1285,7 @@ void QTextDocumentPrivate::adjustDocumentChangesAndCursors(int from, int addedOr
     if (blockCursorAdjustment)  {
         ; // postpone, will be called again from QTextDocumentPrivate::remove()
     } else {
-        foreach (QTextCursorPrivate *curs, cursors) {
+        for (QTextCursorPrivate *curs : qAsConst(cursors)) {
             if (curs->adjustPosition(from, addedOrRemoved, op) == QTextCursorPrivate::CursorMoved) {
                 curs->changed = true;
             }
@@ -1735,7 +1732,7 @@ bool QTextDocumentPrivate::ensureMaximumBlockCount()
 void QTextDocumentPrivate::aboutToRemoveCell(int from, int to)
 {
     Q_ASSERT(from <= to);
-    foreach (QTextCursorPrivate *curs, cursors)
+    for (QTextCursorPrivate *curs : qAsConst(cursors))
         curs->aboutToRemoveCell(from, to);
 }
 
