@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -96,7 +94,7 @@ public:
         if (contentStream)
             contentStream->close();
     }
-    virtual void addFile(const QString &, const QString &, const QByteArray &)
+    virtual void addFile(const QString &, const QString &, const QByteArray &) Q_DECL_OVERRIDE
     {
         // we ignore this...
     }
@@ -139,7 +137,7 @@ public:
         zip.close();
     }
 
-    virtual void addFile(const QString &fileName, const QString &mimeType, const QByteArray &bytes)
+    virtual void addFile(const QString &fileName, const QString &mimeType, const QByteArray &bytes) Q_DECL_OVERRIDE
     {
         zip.addFile(fileName, bytes);
         addFile(fileName, mimeType);
@@ -286,7 +284,13 @@ void QTextOdfWriter::writeBlock(QXmlStreamWriter &writer, const QTextBlock &bloc
     writer.writeStartElement(textNS, QString::fromLatin1("p"));
     writer.writeAttribute(textNS, QString::fromLatin1("style-name"), QString::fromLatin1("p%1")
         .arg(block.blockFormatIndex()));
-    for (QTextBlock::Iterator frag= block.begin(); !frag.atEnd(); frag++) {
+    for (QTextBlock::Iterator frag = block.begin(); !frag.atEnd(); ++frag) {
+        bool isHyperlink = frag.fragment().charFormat().hasProperty(QTextFormat::AnchorHref);
+        if (isHyperlink) {
+            QString value = frag.fragment().charFormat().property(QTextFormat::AnchorHref).toString();
+            writer.writeStartElement(textNS, QString::fromLatin1("a"));
+            writer.writeAttribute(xlinkNS, QString::fromLatin1("href"), value);
+        }
         writer.writeCharacters(QString()); // Trick to make sure that the span gets no linefeed in front of it.
         writer.writeStartElement(textNS, QString::fromLatin1("span"));
 
@@ -343,6 +347,9 @@ void QTextOdfWriter::writeBlock(QXmlStreamWriter &writer, const QTextBlock &bloc
 
         writer.writeCharacters(fragmentText.mid(exportedIndex));
         writer.writeEndElement(); // span
+        writer.writeCharacters(QString()); // Trick to make sure that the span gets no linefeed behind it.
+        if (isHyperlink)
+            writer.writeEndElement(); // a
     }
     writer.writeCharacters(QString()); // Trick to make sure that the span gets no linefeed behind it.
     writer.writeEndElement(); // p
@@ -404,7 +411,7 @@ void QTextOdfWriter::writeInlineCharacter(QXmlStreamWriter &writer, const QTextF
     writer.writeEndElement(); // frame
 }
 
-void QTextOdfWriter::writeFormats(QXmlStreamWriter &writer, QSet<int> formats) const
+void QTextOdfWriter::writeFormats(QXmlStreamWriter &writer, const QSet<int> &formats) const
 {
     writer.writeStartElement(officeNS, QString::fromLatin1("automatic-styles"));
     QVector<QTextFormat> allStyles = m_document->allFormats();
@@ -677,13 +684,13 @@ void QTextOdfWriter::writeFrameFormat(QXmlStreamWriter &writer, QTextFrameFormat
     writer.writeEndElement(); // style
 
 // TODO consider putting the following properties in a qt-namespace.
-// Position   position () const 
-// qreal   border () const 
-// QBrush   borderBrush () const 
-// BorderStyle   borderStyle () const 
-// qreal   padding () const 
-// QTextLength   width () const 
-// QTextLength   height () const 
+// Position   position () const
+// qreal   border () const
+// QBrush   borderBrush () const
+// BorderStyle   borderStyle () const
+// qreal   padding () const
+// QTextLength   width () const
+// QTextLength   height () const
 // PageBreakFlags   pageBreakPolicy () const
 }
 
@@ -762,7 +769,7 @@ bool QTextOdfWriter::writeAll()
         m_strategy = new QXmlStreamStrategy(m_device);
 
     if (!m_device->isWritable() && ! m_device->open(QIODevice::WriteOnly)) {
-        qWarning() << "QTextOdfWriter::writeAll: the device can not be opened for writing";
+        qWarning("QTextOdfWriter::writeAll: the device can not be opened for writing");
         return false;
     }
     QXmlStreamWriter writer(m_strategy->contentStream);
@@ -805,10 +812,10 @@ bool QTextOdfWriter::writeAll()
     }
 
     // add objects for lists, frames and tables
-    QVector<QTextFormat> allFormats = m_document->allFormats();
-    QList<int> copy = formats.toList();
-    for (QList<int>::Iterator iter = copy.begin(); iter != copy.end(); ++iter) {
-        QTextObject *object = m_document->objectForFormat(allFormats[*iter]);
+    const QVector<QTextFormat> allFormats = m_document->allFormats();
+    const QList<int> copy = formats.toList();
+    for (auto index : copy) {
+        QTextObject *object = m_document->objectForFormat(allFormats[index]);
         if (object)
             formats << object->formatIndex();
     }
