@@ -1,37 +1,44 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
-#include "qnetworkinterface_win_p.h"
+#define WIN32_LEAN_AND_MEAN 1
 
 #include "qnetworkinterface.h"
 #include "qnetworkinterface_p.h"
@@ -41,16 +48,29 @@
 #include <qhostinfo.h>
 #include <qhash.h>
 #include <qurl.h>
-#include <private/qsystemlibrary_p.h>
+
+// Since we need to include winsock2.h, we need to define WIN32_LEAN_AND_MEAN
+// (above) so windows.h won't include winsock.h.
+// In addition, we need to include winsock2.h before iphlpapi.h and we need
+// to include ws2ipdef.h to work around an MinGW-w64 bug
+// (http://sourceforge.net/p/mingw-w64/mailman/message/32935366/)
+#include <winsock2.h>
+#include <ws2ipdef.h>
+#include <iphlpapi.h>
+#include <ws2tcpip.h>
+
+#include <qt_windows.h>
 
 QT_BEGIN_NAMESPACE
 
-typedef DWORD (WINAPI *PtrGetAdaptersInfo)(PIP_ADAPTER_INFO, PULONG);
-static PtrGetAdaptersInfo ptrGetAdaptersInfo = 0;
-typedef ULONG (WINAPI *PtrGetAdaptersAddresses)(ULONG, ULONG, PVOID, PIP_ADAPTER_ADDRESSES, PULONG);
-static PtrGetAdaptersAddresses ptrGetAdaptersAddresses = 0;
-typedef DWORD (WINAPI *PtrGetNetworkParams)(PFIXED_INFO, PULONG);
-static PtrGetNetworkParams ptrGetNetworkParams = 0;
+typedef NETIO_STATUS (WINAPI *PtrConvertInterfaceIndexToLuid)(NET_IFINDEX, PNET_LUID);
+typedef NETIO_STATUS (WINAPI *PtrConvertInterfaceLuidToName)(const NET_LUID *, PWSTR, SIZE_T);
+typedef NETIO_STATUS (WINAPI *PtrConvertInterfaceLuidToIndex)(const NET_LUID *, PNET_IFINDEX);
+typedef NETIO_STATUS (WINAPI *PtrConvertInterfaceNameToLuid)(const WCHAR *, PNET_LUID);
+static PtrConvertInterfaceIndexToLuid ptrConvertInterfaceIndexToLuid = 0;
+static PtrConvertInterfaceLuidToName ptrConvertInterfaceLuidToName = 0;
+static PtrConvertInterfaceLuidToIndex ptrConvertInterfaceLuidToIndex = 0;
+static PtrConvertInterfaceNameToLuid ptrConvertInterfaceNameToLuid = 0;
 
 static void resolveLibs()
 {
@@ -58,21 +78,23 @@ static void resolveLibs()
     static bool done = false;
 
     if (!done) {
-        done = true;
-
-        HINSTANCE iphlpapiHnd = QSystemLibrary::load(L"iphlpapi");
-        if (iphlpapiHnd == NULL)
-            return;
+        HINSTANCE iphlpapiHnd = GetModuleHandle(L"iphlpapi");
+        Q_ASSERT(iphlpapiHnd);
 
 #if defined(Q_OS_WINCE)
-        ptrGetAdaptersInfo = (PtrGetAdaptersInfo)GetProcAddress(iphlpapiHnd, L"GetAdaptersInfo");
-        ptrGetAdaptersAddresses = (PtrGetAdaptersAddresses)GetProcAddress(iphlpapiHnd, L"GetAdaptersAddresses");
-        ptrGetNetworkParams = (PtrGetNetworkParams)GetProcAddress(iphlpapiHnd, L"GetNetworkParams");
+        // since Windows Embedded Compact 7
+        ptrConvertInterfaceIndexToLuid = (PtrConvertInterfaceIndexToLuid)GetProcAddress(iphlpapiHnd, L"ConvertInterfaceIndexToLuid");
+        ptrConvertInterfaceLuidToName = (PtrConvertInterfaceLuidToName)GetProcAddress(iphlpapiHnd, L"ConvertInterfaceLuidToNameW");
+        ptrConvertInterfaceLuidToIndex = (PtrConvertInterfaceLuidToIndex)GetProcAddress(iphlpapiHnd, L"ConvertInterfaceLuidToIndex");
+        ptrConvertInterfaceNameToLuid = (PtrConvertInterfaceNameToLuid)GetProcAddress(iphlpapiHnd, L"ConvertInterfaceNameToLuidW");
 #else
-        ptrGetAdaptersInfo = (PtrGetAdaptersInfo)GetProcAddress(iphlpapiHnd, "GetAdaptersInfo");
-        ptrGetAdaptersAddresses = (PtrGetAdaptersAddresses)GetProcAddress(iphlpapiHnd, "GetAdaptersAddresses");
-        ptrGetNetworkParams = (PtrGetNetworkParams)GetProcAddress(iphlpapiHnd, "GetNetworkParams");
+        // since Windows Vista
+        ptrConvertInterfaceIndexToLuid = (PtrConvertInterfaceIndexToLuid)GetProcAddress(iphlpapiHnd, "ConvertInterfaceIndexToLuid");
+        ptrConvertInterfaceLuidToName = (PtrConvertInterfaceLuidToName)GetProcAddress(iphlpapiHnd, "ConvertInterfaceLuidToNameW");
+        ptrConvertInterfaceLuidToIndex = (PtrConvertInterfaceLuidToIndex)GetProcAddress(iphlpapiHnd, "ConvertInterfaceLuidToIndex");
+        ptrConvertInterfaceNameToLuid = (PtrConvertInterfaceNameToLuid)GetProcAddress(iphlpapiHnd, "ConvertInterfaceNameToLuidW");
 #endif
+        done = true;
     }
 }
 
@@ -85,16 +107,45 @@ static QHostAddress addressFromSockaddr(sockaddr *sa)
     if (sa->sa_family == AF_INET)
         address.setAddress(htonl(((sockaddr_in *)sa)->sin_addr.s_addr));
     else if (sa->sa_family == AF_INET6) {
-        address.setAddress(((qt_sockaddr_in6 *)sa)->sin6_addr.qt_s6_addr);
-        int scope = ((qt_sockaddr_in6 *)sa)->sin6_scope_id;
+        address.setAddress(((sockaddr_in6 *)sa)->sin6_addr.s6_addr);
+        int scope = ((sockaddr_in6 *)sa)->sin6_scope_id;
         if (scope)
-            address.setScopeId(QString::number(scope));
+            address.setScopeId(QNetworkInterfaceManager::interfaceNameFromIndex(scope));
     } else
         qWarning("Got unknown socket family %d", sa->sa_family);
     return address;
 
 }
 
+uint QNetworkInterfaceManager::interfaceIndexFromName(const QString &name)
+{
+    resolveLibs();
+    if (!ptrConvertInterfaceNameToLuid || !ptrConvertInterfaceLuidToIndex)
+        return 0;
+
+    NET_IFINDEX id;
+    NET_LUID luid;
+    if (ptrConvertInterfaceNameToLuid(reinterpret_cast<const wchar_t *>(name.constData()), &luid) == NO_ERROR
+            && ptrConvertInterfaceLuidToIndex(&luid, &id) == NO_ERROR)
+        return uint(id);
+    return 0;
+}
+
+QString QNetworkInterfaceManager::interfaceNameFromIndex(uint index)
+{
+    resolveLibs();
+    if (ptrConvertInterfaceIndexToLuid && ptrConvertInterfaceLuidToName) {
+        NET_LUID luid;
+        if (ptrConvertInterfaceIndexToLuid(index, &luid) == NO_ERROR) {
+            WCHAR buf[IF_MAX_STRING_SIZE + 1];
+            if (ptrConvertInterfaceLuidToName(&luid, buf, sizeof(buf)/sizeof(buf[0])) == NO_ERROR)
+                return QString::fromWCharArray(buf);
+        }
+    }
+
+    return QString::number(index);
+}
+
 static QHash<QHostAddress, QHostAddress> ipv4Netmasks()
 {
     //Retrieve all the IPV4 addresses & netmasks
@@ -103,14 +154,14 @@ static QHash<QHostAddress, QHostAddress> ipv4Netmasks()
     ULONG bufSize = sizeof staticBuf;
     QHash<QHostAddress, QHostAddress> ipv4netmasks;
 
-    DWORD retval = ptrGetAdaptersInfo(pAdapter, &bufSize);
+    DWORD retval = GetAdaptersInfo(pAdapter, &bufSize);
     if (retval == ERROR_BUFFER_OVERFLOW) {
         // need more memory
         pAdapter = (IP_ADAPTER_INFO *)malloc(bufSize);
         if (!pAdapter)
             return ipv4netmasks;
         // try again
-        if (ptrGetAdaptersInfo(pAdapter, &bufSize) != ERROR_SUCCESS) {
+        if (GetAdaptersInfo(pAdapter, &bufSize) != ERROR_SUCCESS) {
             free(pAdapter);
             return ipv4netmasks;
         }
@@ -145,14 +196,14 @@ static QList<QNetworkInterfacePrivate *> interfaceListingWinXP()
     ULONG flags = GAA_FLAG_INCLUDE_PREFIX |
                   GAA_FLAG_SKIP_DNS_SERVER |
                   GAA_FLAG_SKIP_MULTICAST;
-    ULONG retval = ptrGetAdaptersAddresses(AF_UNSPEC, flags, NULL, pAdapter, &bufSize);
+    ULONG retval = GetAdaptersAddresses(AF_UNSPEC, flags, NULL, pAdapter, &bufSize);
     if (retval == ERROR_BUFFER_OVERFLOW) {
         // need more memory
         pAdapter = (IP_ADAPTER_ADDRESSES *)malloc(bufSize);
         if (!pAdapter)
             return interfaces;
         // try again
-        if (ptrGetAdaptersAddresses(AF_UNSPEC, flags, NULL, pAdapter, &bufSize) != ERROR_SUCCESS) {
+        if (GetAdaptersAddresses(AF_UNSPEC, flags, NULL, pAdapter, &bufSize) != ERROR_SUCCESS) {
             free(pAdapter);
             return interfaces;
         }
@@ -180,7 +231,16 @@ static QList<QNetworkInterfacePrivate *> interfaceListingWinXP()
         if (ptr->IfType == IF_TYPE_PPP)
             iface->flags |= QNetworkInterface::IsPointToPoint;
 
-        iface->name = QString::fromLocal8Bit(ptr->AdapterName);
+        if (ptrConvertInterfaceLuidToName && ptr->Length >= offsetof(IP_ADAPTER_ADDRESSES, Luid)) {
+            // use ConvertInterfaceLuidToName because that returns a friendlier name, though not
+            // as friendly as FriendlyName below
+            WCHAR buf[IF_MAX_STRING_SIZE + 1];
+            if (ptrConvertInterfaceLuidToName(&ptr->Luid, buf, sizeof(buf)/sizeof(buf[0])) == NO_ERROR)
+                iface->name = QString::fromWCharArray(buf);
+        }
+        if (iface->name.isEmpty())
+            iface->name = QString::fromLocal8Bit(ptr->AdapterName);
+
         iface->friendlyName = QString::fromWCharArray(ptr->FriendlyName);
         if (ptr->PhysicalAddressLength)
             iface->hardwareAddress = iface->makeHwAddress(ptr->PhysicalAddressLength,
@@ -221,92 +281,25 @@ static QList<QNetworkInterfacePrivate *> interfaceListingWinXP()
     return interfaces;
 }
 
-static QList<QNetworkInterfacePrivate *> interfaceListingWin2k()
-{
-    QList<QNetworkInterfacePrivate *> interfaces;
-    IP_ADAPTER_INFO staticBuf[2]; // 2 is arbitrary
-    PIP_ADAPTER_INFO pAdapter = staticBuf;
-    ULONG bufSize = sizeof staticBuf;
-
-    DWORD retval = ptrGetAdaptersInfo(pAdapter, &bufSize);
-    if (retval == ERROR_BUFFER_OVERFLOW) {
-        // need more memory
-        pAdapter = (IP_ADAPTER_INFO *)malloc(bufSize);
-        if (!pAdapter)
-            return interfaces;
-        // try again
-        if (ptrGetAdaptersInfo(pAdapter, &bufSize) != ERROR_SUCCESS) {
-            free(pAdapter);
-            return interfaces;
-        }
-    } else if (retval != ERROR_SUCCESS) {
-        // error
-        return interfaces;
-    }
-
-    // iterate over the list and add the entries to our listing
-    for (PIP_ADAPTER_INFO ptr = pAdapter; ptr; ptr = ptr->Next) {
-        QNetworkInterfacePrivate *iface = new QNetworkInterfacePrivate;
-        interfaces << iface;
-
-        iface->index = ptr->Index;
-        iface->flags = QNetworkInterface::IsUp | QNetworkInterface::IsRunning;
-        if (ptr->Type == MIB_IF_TYPE_PPP)
-            iface->flags |= QNetworkInterface::IsPointToPoint;
-        else
-            iface->flags |= QNetworkInterface::CanBroadcast;
-        iface->name = QString::fromLocal8Bit(ptr->AdapterName);
-        iface->hardwareAddress = QNetworkInterfacePrivate::makeHwAddress(ptr->AddressLength,
-                                                                         ptr->Address);
-
-        for (PIP_ADDR_STRING addr = &ptr->IpAddressList; addr; addr = addr->Next) {
-            QNetworkAddressEntry entry;
-            entry.setIp(QHostAddress(QLatin1String(addr->IpAddress.String)));
-            entry.setNetmask(QHostAddress(QLatin1String(addr->IpMask.String)));
-            // broadcast address is set on postProcess()
-
-            iface->addressEntries << entry;
-        }
-    }
-
-    if (pAdapter != staticBuf)
-        free(pAdapter);
-
-    return interfaces;
-}
-
-static QList<QNetworkInterfacePrivate *> interfaceListing()
-{
-    resolveLibs();
-    if (ptrGetAdaptersAddresses != NULL)
-        return interfaceListingWinXP();
-    else if (ptrGetAdaptersInfo != NULL)
-        return interfaceListingWin2k();
-
-    // failed
-    return QList<QNetworkInterfacePrivate *>();
-}
-
 QList<QNetworkInterfacePrivate *> QNetworkInterfaceManager::scan()
 {
-    return interfaceListing();
+    resolveLibs();
+    return interfaceListingWinXP();
 }
 
 QString QHostInfo::localDomainName()
 {
     resolveLibs();
-    if (ptrGetNetworkParams == NULL)
-        return QString();       // couldn't resolve
 
     FIXED_INFO info, *pinfo;
     ULONG bufSize = sizeof info;
     pinfo = &info;
-    if (ptrGetNetworkParams(pinfo, &bufSize) == ERROR_BUFFER_OVERFLOW) {
+    if (GetNetworkParams(pinfo, &bufSize) == ERROR_BUFFER_OVERFLOW) {
         pinfo = (FIXED_INFO *)malloc(bufSize);
         if (!pinfo)
             return QString();
         // try again
-        if (ptrGetNetworkParams(pinfo, &bufSize) != ERROR_SUCCESS) {
+        if (GetNetworkParams(pinfo, &bufSize) != ERROR_SUCCESS) {
             free(pinfo);
             return QString();   // error
         }
