@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -218,7 +224,6 @@
 #include "private/qhttpsocketengine_p.h"
 #include "qauthenticator.h"
 #include "qdebug.h"
-#include "qhash.h"
 #include "qmutex.h"
 #include "qstringlist.h"
 #include "qurl.h"
@@ -292,7 +297,7 @@ public:
 
     QNetworkProxy applicationProxy()
     {
-        return proxyForQuery(QNetworkProxyQuery()).first();
+        return proxyForQuery(QNetworkProxyQuery()).constFirst();
     }
 
     QList<QNetworkProxy> proxyForQuery(const QNetworkProxyQuery &query);
@@ -1523,7 +1528,7 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
     those settings are not found, this function will attempt to obtain
     Internet Explorer's settings and use them.
 
-    On MacOS X, this function will obtain the proxy settings using the
+    On \macos, this function will obtain the proxy settings using the
     SystemConfiguration framework from Apple. It will apply the FTP,
     HTTP and HTTPS proxy configurations for queries that contain the
     protocol tag "ftp", "http" and "https", respectively. If the SOCKS
@@ -1531,12 +1536,6 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
     SOCKS server for all queries. If SOCKS isn't enabled, it will use
     the HTTPS proxy for all TcpSocket and UrlRequest queries.
 
-    On BlackBerry, this function obtains proxy settings for the default
-    configuration using system configuration. The type will be set based on
-    protocol tag "http", "https", "ftp", respectively. By default, it
-    assumes http type. Proxy username and password are also set during
-    the query using system configuration.
-
     On other systems, this function will pick up proxy settings from
     the "http_proxy" environment variable. This variable must be a URL
     using one of the following schemes: "http", "socks5" or "socks5h".
@@ -1548,15 +1547,11 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
     listed here.
 
     \list
-    \li On MacOS X, this function will ignore the Proxy Auto Configuration
+    \li On \macos, this function will ignore the Proxy Auto Configuration
     settings, since it cannot execute the associated ECMAScript code.
 
     \li On Windows platforms, this function may take several seconds to
     execute depending on the configuration of the user's system.
-
-    \li On BlackBerry, only UrlRequest and TcpSocket queries are supported. SOCKS is
-    not supported. The proxy credentials are only retrieved for the
-    default configuration.
     \endlist
 */
 
@@ -1606,7 +1601,7 @@ QDebug operator<<(QDebug debug, const QNetworkProxy &proxy)
         debug << "Unknown proxy " << int(type);
         break;
     }
-    debug << "\"" << proxy.hostName() << ":" << proxy.port() << "\" ";
+    debug << '"' << proxy.hostName() << ':' << proxy.port() << "\" ";
     QNetworkProxy::Capabilities caps = proxy.capabilities();
     QStringList scaps;
     if (caps & QNetworkProxy::TunnelingCapability)
@@ -1619,7 +1614,7 @@ QDebug operator<<(QDebug debug, const QNetworkProxy &proxy)
         scaps << QStringLiteral("Caching");
     if (caps & QNetworkProxy::HostNameLookupCapability)
         scaps << QStringLiteral("NameLookup");
-    debug << "[" << scaps.join(QLatin1Char(' ')) << "]";
+    debug << '[' << scaps.join(QLatin1Char(' ')) << ']';
     return debug;
 }
 #endif
