@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -188,14 +194,16 @@ static bool isBypassed(const QString &host, const QStringList &bypassList)
         return true;
 
     // does it match the list of exclusions?
-    foreach (const QString &entry, bypassList) {
+    for (const QString &entry : bypassList) {
         if (entry == QLatin1String("<local>")) {
             if (isSimple)
                 return true;
             if (isIpAddress) {
                 //exclude all local subnets
-                foreach (const QNetworkInterface &iface, QNetworkInterface::allInterfaces()) {
-                    foreach (const QNetworkAddressEntry netaddr, iface.addressEntries()) {
+                const auto ifaces = QNetworkInterface::allInterfaces();
+                for (const QNetworkInterface &iface : ifaces) {
+                    const auto netaddrs = iface.addressEntries();
+                    for (const QNetworkAddressEntry &netaddr : netaddrs) {
                         if (ipAddress.isInSubnet(netaddr.ip(), netaddr.prefixLength())) {
                             return true;
                         }
@@ -235,7 +243,7 @@ static QList<QNetworkProxy> filterProxyListByCapabilities(const QList<QNetworkPr
         break;
     }
     QList<QNetworkProxy> result;
-    foreach (const QNetworkProxy& proxy, proxyList) {
+    for (const QNetworkProxy &proxy : proxyList) {
         if (proxy.capabilities() & requiredCaps)
             result.append(proxy);
     }
@@ -245,7 +253,7 @@ static QList<QNetworkProxy> filterProxyListByCapabilities(const QList<QNetworkPr
 static QList<QNetworkProxy> removeDuplicateProxies(const QList<QNetworkProxy> &proxyList)
 {
     QList<QNetworkProxy> result;
-     foreach (QNetworkProxy proxy, proxyList) {
+    for (const QNetworkProxy &proxy : proxyList) {
          bool append = true;
          for (int i=0; i < result.count(); i++) {
              if (proxy.hostName() == result.at(i).hostName()
@@ -277,7 +285,7 @@ static QList<QNetworkProxy> parseServerList(const QNetworkProxyQuery &query, con
     QHash<QString, QNetworkProxy> taggedProxies;
     const QString requiredTag = query.protocolTag();
     bool checkTags = !requiredTag.isEmpty() && query.queryType() != QNetworkProxyQuery::TcpServer; //windows tags are only for clients
-    foreach (const QString &entry, proxyList) {
+    for (const QString &entry : proxyList) {
         int server = 0;
 
         QNetworkProxy::ProxyType proxyType = QNetworkProxy::HttpProxy;
@@ -315,7 +323,7 @@ static QList<QNetworkProxy> parseServerList(const QNetworkProxyQuery &query, con
         pos = entry.indexOf(QLatin1Char(':'), server);
         if (pos != -1) {
             bool ok;
-            uint value = entry.mid(pos + 1).toUInt(&ok);
+            uint value = entry.midRef(pos + 1).toUInt(&ok);
             if (!ok || value > 65535)
                 continue;       // invalid port number
 
@@ -326,7 +334,7 @@ static QList<QNetworkProxy> parseServerList(const QNetworkProxyQuery &query, con
 
         result << QNetworkProxy(proxyType, entry.mid(server, pos - server), port);
         if (!protocolTag.isEmpty())
-            taggedProxies.insert(protocolTag.toString(), result.last());
+            taggedProxies.insert(protocolTag.toString(), result.constLast());
     }
 
     if (checkTags && taggedProxies.contains(requiredTag)) {
@@ -386,9 +394,9 @@ public:
     }
 
     void clear() {
-        foreach (HANDLE event, m_watchEvents)
+        for (HANDLE event : qAsConst(m_watchEvents))
             CloseHandle(event);
-        foreach (HKEY key, m_registryHandles)
+        for (HKEY key : qAsConst(m_registryHandles))
             RegCloseKey(key);
 
         m_watchEvents.clear();
