@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -182,17 +188,26 @@ bool QHttpSocketEngine::connectToHostByName(const QString &hostname, quint16 por
 
 bool QHttpSocketEngine::bind(const QHostAddress &, quint16)
 {
+    qWarning("Operation is not supported");
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Unsupported socket operation"));
     return false;
 }
 
 bool QHttpSocketEngine::listen()
 {
+    qWarning("Operation is not supported");
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Unsupported socket operation"));
     return false;
 }
 
 int QHttpSocketEngine::accept()
 {
-    return 0;
+    qWarning("Operation is not supported");
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Unsupported socket operation"));
+    return -1;
 }
 
 void QHttpSocketEngine::close()
@@ -245,16 +260,18 @@ qint64 QHttpSocketEngine::write(const char *data, qint64 len)
 bool QHttpSocketEngine::joinMulticastGroup(const QHostAddress &,
                                            const QNetworkInterface &)
 {
+    qWarning("Operation is not supported");
     setError(QAbstractSocket::UnsupportedSocketOperationError,
-             QLatin1String("Operation on socket is not supported"));
+             QLatin1String("Unsupported socket operation"));
     return false;
 }
 
 bool QHttpSocketEngine::leaveMulticastGroup(const QHostAddress &,
                                             const QNetworkInterface &)
 {
+    qWarning("Operation is not supported");
     setError(QAbstractSocket::UnsupportedSocketOperationError,
-             QLatin1String("Operation on socket is not supported"));
+             QLatin1String("Unsupported socket operation"));
     return false;
 }
 
@@ -265,32 +282,39 @@ QNetworkInterface QHttpSocketEngine::multicastInterface() const
 
 bool QHttpSocketEngine::setMulticastInterface(const QNetworkInterface &)
 {
+    qWarning("Operation is not supported");
     setError(QAbstractSocket::UnsupportedSocketOperationError,
-             QLatin1String("Operation on socket is not supported"));
+             QLatin1String("Unsupported socket operation"));
     return false;
 }
 #endif // QT_NO_NETWORKINTERFACE
 
-qint64 QHttpSocketEngine::readDatagram(char *, qint64, QHostAddress *,
-                                       quint16 *)
+qint64 QHttpSocketEngine::readDatagram(char *, qint64, QIpPacketHeader *, PacketHeaderOptions)
 {
-    return 0;
+    qWarning("Operation is not supported");
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Unsupported socket operation"));
+    return -1;
 }
 
-qint64 QHttpSocketEngine::writeDatagram(const char *, qint64, const QHostAddress &,
-                                        quint16)
+qint64 QHttpSocketEngine::writeDatagram(const char *, qint64, const QIpPacketHeader &)
 {
-    return 0;
+    qWarning("Operation is not supported");
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Unsupported socket operation"));
+    return -1;
 }
 
 bool QHttpSocketEngine::hasPendingDatagrams() const
 {
+    qWarning("Operation is not supported");
     return false;
 }
 
 qint64 QHttpSocketEngine::pendingDatagramSize() const
 {
-    return 0;
+    qWarning("Operation is not supported");
+    return -1;
 }
 #endif // QT_NO_UDPSOCKET
 
@@ -441,8 +465,11 @@ void QHttpSocketEngine::setReadNotificationEnabled(bool enable)
     d->readNotificationEnabled = enable;
     if (enable) {
         // Enabling read notification can trigger a notification.
-        if (bytesAvailable())
+        if (bytesAvailable()) {
             slotSocketReadNotification();
+        } else if (d->socket && d->socket->state() == QAbstractSocket::UnconnectedState) {
+            emitReadNotification();
+        }
     }
 }
 
@@ -483,16 +510,16 @@ void QHttpSocketEngine::slotSocketConnected()
                              QUrl::toAce(d->peerName);
     QByteArray path = peerAddress + ':' + QByteArray::number(d->peerPort);
     QByteArray data = method;
-    data += " ";
+    data += ' ';
     data += path;
     data += " HTTP/1.1\r\n";
     data += "Proxy-Connection: keep-alive\r\n";
     data += "Host: " + peerAddress + "\r\n";
     if (!d->proxy.hasRawHeader("User-Agent"))
         data += "User-Agent: Mozilla/5.0\r\n";
-    foreach (const QByteArray &header, d->proxy.rawHeaderList()) {
+    const auto headers = d->proxy.rawHeaderList();
+    for (const QByteArray &header : headers)
         data += header + ": " + d->proxy.rawHeader(header) + "\r\n";
-    }
     QAuthenticatorPrivate *priv = QAuthenticatorPrivate::getPrivate(d->authenticator);
     //qDebug() << "slotSocketConnected: priv=" << priv << (priv ? (int)priv->method : -1);
     if (priv && priv->method != QAuthenticatorPrivate::None) {
@@ -503,7 +530,7 @@ void QHttpSocketEngine::slotSocketConnected()
     data += "\r\n";
 //     qDebug() << ">>>>>>>> sending request" << this;
 //     qDebug() << data;
-//     qDebug() << ">>>>>>>";
+//     qDebug(">>>>>>>");
     d->socket->write(data);
     d->state = ConnectSent;
 }
@@ -569,6 +596,7 @@ void QHttpSocketEngine::slotSocketReadNotification()
         d->state = Connected;
         setLocalAddress(d->socket->localAddress());
         setLocalPort(d->socket->localPort());
+        d->inboundStreamCount = d->outboundStreamCount = 1;
         setState(QAbstractSocket::ConnectedState);
         d->authenticator.detach();
         priv = QAuthenticatorPrivate::getPrivate(d->authenticator);
@@ -770,7 +798,6 @@ void QHttpSocketEngine::emitPendingConnectionNotification()
 void QHttpSocketEngine::emitReadNotification()
 {
     Q_D(QHttpSocketEngine);
-    d->readNotificationActivated = true;
     // if there is a connection notification pending we have to emit the readNotification
     // incase there is connection error. This is only needed for Windows, but it does not
     // hurt in other cases.
@@ -783,7 +810,6 @@ void QHttpSocketEngine::emitReadNotification()
 void QHttpSocketEngine::emitWriteNotification()
 {
     Q_D(QHttpSocketEngine);
-    d->writeNotificationActivated = true;
     if (d->writeNotificationEnabled && !d->writeNotificationPending) {
         d->writeNotificationPending = true;
         QMetaObject::invokeMethod(this, "emitPendingWriteNotification", Qt::QueuedConnection);
@@ -803,8 +829,6 @@ QHttpSocketEnginePrivate::QHttpSocketEnginePrivate()
     : readNotificationEnabled(false)
     , writeNotificationEnabled(false)
     , exceptNotificationEnabled(false)
-    , readNotificationActivated(false)
-    , writeNotificationActivated(false)
     , readNotificationPending(false)
     , writeNotificationPending(false)
     , connectionNotificationPending(false)
