@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -252,18 +258,19 @@ bool QNativeSocketEngine::initialize(qintptr socketDescriptor, QAbstractSocket::
 
     // Start processing incoming data
     if (d->socketType == QAbstractSocket::TcpSocket) {
-        HRESULT hr = QEventDispatcherWinRT::runOnXamlThread([d, socket, this]() {
+        HRESULT hr;
+        QEventDispatcherWinRT::runOnXamlThread([d, &hr, socket, this]() {
             ComPtr<IBuffer> buffer;
             HRESULT hr = g->bufferFactory->Create(READ_BUFFER_SIZE, &buffer);
-            RETURN_HR_IF_FAILED("initialize(): Could not create buffer");
+            RETURN_OK_IF_FAILED("initialize(): Could not create buffer");
             ComPtr<IInputStream> stream;
             hr = socket->get_InputStream(&stream);
-            RETURN_HR_IF_FAILED("initialize(): Could not obtain input stream");
+            RETURN_OK_IF_FAILED("initialize(): Could not obtain input stream");
             hr = stream->ReadAsync(buffer.Get(), READ_BUFFER_SIZE, InputStreamOptions_Partial, d->readOp.GetAddressOf());
-            RETURN_HR_IF_FAILED_WITH_ARGS("initialize(): Failed to read from the socket buffer (%s).",
+            RETURN_OK_IF_FAILED_WITH_ARGS("initialize(): Failed to read from the socket buffer (%s).",
                               socketDescription(this).constData());
             hr = d->readOp->put_Completed(Callback<SocketReadCompletedHandler>(d, &QNativeSocketEnginePrivate::handleReadyRead).Get());
-            RETURN_HR_IF_FAILED_WITH_ARGS("initialize(): Failed to set socket read callback (%s).",
+            RETURN_OK_IF_FAILED_WITH_ARGS("initialize(): Failed to set socket read callback (%s).",
                               socketDescription(this).constData());
             return S_OK;
         });
@@ -491,12 +498,10 @@ void QNativeSocketEngine::close()
             ComPtr<IAsyncAction> action;
             hr = socket3->CancelIOAsync(&action);
             Q_ASSERT_SUCCEEDED(hr);
-            hr = QWinRTFunctions::await(action, QWinRTFunctions::YieldThread, 5000);
+            hr = QWinRTFunctions::await(action);
             // If there is no pending IO (no read established before) the function will fail with
             // "function was called at an unexpected time" which is fine.
-            // Timeout is fine as well. The result will be the socket being hard reset instead of
-            // being closed gracefully
-            if (hr != E_ILLEGAL_METHOD_CALL && hr != ERROR_TIMEOUT)
+            if (hr != E_ILLEGAL_METHOD_CALL)
                 Q_ASSERT_SUCCEEDED(hr);
             return S_OK;
         });
@@ -542,6 +547,7 @@ void QNativeSocketEngine::close()
     d->localAddress.clear();
     d->peerPort = 0;
     d->peerAddress.clear();
+    d->inboundStreamCount = d->outboundStreamCount = 0;
 }
 
 bool QNativeSocketEngine::joinMulticastGroup(const QHostAddress &groupAddress, const QNetworkInterface &iface)
@@ -579,7 +585,6 @@ qint64 QNativeSocketEngine::bytesAvailable() const
     if (d->socketType != QAbstractSocket::TcpSocket)
         return -1;
 
-    QMutexLocker locker(&d->readMutex);
     return d->readBytes.size() - d->readBytes.pos();
 }
 
@@ -592,12 +597,12 @@ qint64 QNativeSocketEngine::read(char *data, qint64 maxlen)
     // There will be a read notification when the socket was closed by the remote host. If that
     // happens and there isn't anything left in the buffer, we have to return -1 in order to signal
     // the closing of the socket.
-    QMutexLocker mutexLocker(&d->readMutex);
     if (d->readBytes.pos() == d->readBytes.size() && d->socketState != QAbstractSocket::ConnectedState) {
         close();
         return -1;
     }
 
+    QMutexLocker mutexLocker(&d->readMutex);
     return d->readBytes.read(data, maxlen);
 }
 
@@ -628,7 +633,6 @@ qint64 QNativeSocketEngine::readDatagram(char *data, qint64 maxlen, QIpPacketHea
                                          PacketHeaderOptions)
 {
     Q_D(QNativeSocketEngine);
-    QMutexLocker locker(&d->readMutex);
     if (d->socketType != QAbstractSocket::UdpSocket || d->pendingDatagrams.isEmpty()) {
         if (header)
             header->clear();
@@ -642,13 +646,12 @@ qint64 QNativeSocketEngine::readDatagram(char *data, qint64 maxlen, QIpPacketHea
     QByteArray readOrigin;
     // Do not read the whole datagram. Put the rest of it back into the "queue"
     if (maxlen < datagram.data.length()) {
-        readOrigin = datagram.data.left(maxlen);
+        QByteArray readOrigin = datagram.data.left(maxlen);
         datagram.data = datagram.data.remove(0, maxlen);
         d->pendingDatagrams.prepend(datagram);
     } else {
         readOrigin = datagram.data;
     }
-    locker.unlock();
     memcpy(data, readOrigin, qMin(maxlen, qint64(datagram.data.length())));
     return readOrigin.length();
 }
@@ -686,14 +689,12 @@ qint64 QNativeSocketEngine::writeDatagram(const char *data, qint64 len, const QI
 bool QNativeSocketEngine::hasPendingDatagrams() const
 {
     Q_D(const QNativeSocketEngine);
-    QMutexLocker locker(&d->readMutex);
     return d->pendingDatagrams.length() > 0;
 }
 
 qint64 QNativeSocketEngine::pendingDatagramSize() const
 {
     Q_D(const QNativeSocketEngine);
-    QMutexLocker locker(&d->readMutex);
     if (d->pendingDatagrams.isEmpty())
         return -1;
 
@@ -1157,6 +1158,7 @@ bool QNativeSocketEnginePrivate::fetchConnectionParameters()
     localAddress.clear();
     peerPort = 0;
     peerAddress.clear();
+    inboundStreamCount = outboundStreamCount = 0;
 
     HRESULT hr;
     if (socketType == QAbstractSocket::TcpSocket) {
@@ -1192,6 +1194,7 @@ bool QNativeSocketEnginePrivate::fetchConnectionParameters()
             hr = info->get_RemotePort(tmpHString.GetAddressOf());
             Q_ASSERT_SUCCEEDED(hr);
             peerPort = qt_QStringFromHString(tmpHString).toInt();
+            inboundStreamCount = outboundStreamCount = 1;
         }
     } else if (socketType == QAbstractSocket::UdpSocket) {
         ComPtr<IHostName> hostName;
@@ -1219,6 +1222,7 @@ bool QNativeSocketEnginePrivate::fetchConnectionParameters()
             hr = info->get_RemotePort(tmpHString.GetAddressOf());
             Q_ASSERT_SUCCEEDED(hr);
             peerPort = qt_QStringFromHString(tmpHString).toInt();
+            inboundStreamCount = outboundStreamCount = 1;
         }
     }
     return true;
@@ -1340,7 +1344,7 @@ HRESULT QNativeSocketEnginePrivate::handleReadyRead(IAsyncBufferOperation *async
     hr = byteArrayAccess->Buffer(&data);
     Q_ASSERT_SUCCEEDED(hr);
 
-    QMutexLocker locker(&readMutex);
+    readMutex.lock();
     if (readBytes.atEnd()) // Everything has been read; the buffer is safe to reset
         readBytes.close();
     if (!readBytes.isOpen())
@@ -1350,7 +1354,7 @@ HRESULT QNativeSocketEnginePrivate::handleReadyRead(IAsyncBufferOperation *async
     Q_ASSERT(readBytes.atEnd());
     readBytes.write(reinterpret_cast<const char*>(data), qint64(bufferLength));
     readBytes.seek(readPos);
-    locker.unlock();
+    readMutex.unlock();
 
     if (notifyOnRead)
         emit q->readReady();
@@ -1414,9 +1418,7 @@ HRESULT QNativeSocketEnginePrivate::handleNewDatagram(IDatagramSocket *socket, I
     datagram.data.resize(length);
     hr = reader->ReadBytes(length, reinterpret_cast<BYTE *>(datagram.data.data()));
     RETURN_OK_IF_FAILED("Could not read datagram");
-    QMutexLocker locker(&readMutex);
     pendingDatagrams.append(datagram);
-    locker.unlock();
     if (notifyOnRead)
         emit q->readReady();
 
