@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -147,7 +153,7 @@ QNetworkProxy QTcpServerPrivate::resolveProxy(const QHostAddress &address, quint
     }
 
     // return the first that we can use
-    foreach (const QNetworkProxy &p, proxies) {
+    for (const QNetworkProxy &p : qAsConst(proxies)) {
         if (p.capabilities() & QNetworkProxy::ListeningCapability)
             return p;
     }
@@ -158,6 +164,23 @@ QNetworkProxy QTcpServerPrivate::resolveProxy(const QHostAddress &address, quint
 }
 #endif
 
+/*! \internal
+*/
+void QTcpServerPrivate::configureCreatedSocket()
+{
+#if defined(Q_OS_UNIX)
+    // Under Unix, we want to be able to bind to the port, even if a socket on
+    // the same address-port is in TIME_WAIT. Under Windows this is possible
+    // anyway -- furthermore, the meaning of reusable on Windows is different:
+    // it means that you can use the same address-port for multiple listening
+    // sockets.
+    // Don't abort though if we can't set that option. For example the socks
+    // engine doesn't support that option, but that shouldn't prevent us from
+    // trying to bind/listen.
+    socketEngine->setOption(QAbstractSocketEngine::AddressReusable, 1);
+#endif
+}
+
 /*! \internal
 */
 void QTcpServerPrivate::readNotification()
@@ -205,6 +228,9 @@ void QTcpServerPrivate::readNotification()
 QTcpServer::QTcpServer(QObject *parent)
     : QObject(*new QTcpServerPrivate, parent)
 {
+#if defined(QTCPSERVER_DEBUG)
+    qDebug("QTcpServer::QTcpServer(%p)", parent);
+#endif
 }
 
 /*!
@@ -218,6 +244,9 @@ QTcpServer::QTcpServer(QObject *parent)
 */
 QTcpServer::~QTcpServer()
 {
+#if defined(QTCPSERVER_DEBUG)
+    qDebug("QTcpServer::~QTcpServer()");
+#endif
     close();
 }
 
@@ -226,6 +255,9 @@ QTcpServer::~QTcpServer()
 QTcpServer::QTcpServer(QTcpServerPrivate &dd, QObject *parent)
     : QObject(dd, parent)
 {
+#if defined(QTCPSERVER_DEBUG)
+    qDebug("QTcpServer::QTcpServer(QTcpServerPrivate == %p, parent == %p)", &dd, parent);
+#endif
 }
 
 /*!
@@ -275,17 +307,7 @@ bool QTcpServer::listen(const QHostAddress &address, quint16 port)
     if (addr.protocol() == QAbstractSocket::AnyIPProtocol && proto == QAbstractSocket::IPv4Protocol)
         addr = QHostAddress::AnyIPv4;
 
-#if defined(Q_OS_UNIX)
-    // Under Unix, we want to be able to bind to the port, even if a socket on
-    // the same address-port is in TIME_WAIT. Under Windows this is possible
-    // anyway -- furthermore, the meaning of reusable on Windows is different:
-    // it means that you can use the same address-port for multiple listening
-    // sockets.
-    // Don't abort though if we can't set that option. For example the socks
-    // engine doesn't support that option, but that shouldn't prevent us from
-    // trying to bind/listen.
-    d->socketEngine->setOption(QAbstractSocketEngine::AddressReusable, 1);
-#endif
+    d->configureCreatedSocket();
 
     if (!d->socketEngine->bind(addr, port)) {
         d->serverSocketError = d->socketEngine->error();
@@ -543,6 +565,10 @@ QTcpSocket *QTcpServer::nextPendingConnection()
     may not be usable with native socket functions, and should only be
     used with QTcpSocket::setSocketDescriptor().
 
+    \note If another socket is created in the reimplementation
+    of this method, it needs to be added to the Pending Connections mechanism
+    by calling addPendingConnection().
+
     \note If you want to handle an incoming connection as a new QTcpSocket
     object in another thread you have to pass the socketDescriptor
     to the other thread and create the QTcpSocket object there and
