@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -99,7 +105,7 @@ struct SslSocketGlobal
     void syncCaCertificates(const QSet<QSslCertificate> &add, const QSet<QSslCertificate> &remove)
     {
         QMutexLocker locker(&certificateMutex);
-        foreach (const QSslCertificate &certificate, add) {
+        for (const QSslCertificate &certificate : add) {
             QHash<QSslCertificate, QAtomicInt>::iterator it = additionalCertificates.find(certificate);
             if (it != additionalCertificates.end()) {
                 it.value().ref(); // Add a reference
@@ -111,7 +117,7 @@ struct SslSocketGlobal
                 additionalCertificates.insert(certificate, 1);
             }
         }
-        foreach (const QSslCertificate &certificate, remove) {
+        for (const QSslCertificate &certificate : remove) {
             QHash<QSslCertificate, QAtomicInt>::iterator it = additionalCertificates.find(certificate);
             if (it != additionalCertificates.end() && !it.value().deref()) {
                 // no more references, remove certificate
@@ -300,6 +306,7 @@ void QSslSocketBackendPrivate::transmit()
                 emit q->bytesWritten(totalBytesWritten);
                 emittedBytesWritten = false;
             }
+            emit q->channelBytesWritten(0, totalBytesWritten);
         }
     }
 
@@ -317,6 +324,7 @@ void QSslSocketBackendPrivate::transmit()
         if (readyReadEmittedPointer)
             *readyReadEmittedPointer = true;
         emit q->readyRead();
+        emit q->channelReadyRead(0);
     }
 
     if (pendingClose) {
@@ -607,7 +615,7 @@ HRESULT QSslSocketBackendPrivate::onSslUpgrade(IAsyncAction *action, AsyncStatus
     }
 
     // Peer chain validation
-    foreach (const QSslCertificate &certificate, peerCertificateChain) {
+    for (const QSslCertificate &certificate : qAsConst(peerCertificateChain)) {
         if (!QSslCertificatePrivate::isBlacklisted(certificate))
             continue;
 
@@ -618,10 +626,10 @@ HRESULT QSslSocketBackendPrivate::onSslUpgrade(IAsyncAction *action, AsyncStatus
 
     if (!sslErrors.isEmpty()) {
         emit q->sslErrors(sslErrors);
-        setErrorAndEmit(QAbstractSocket::SslHandshakeFailedError, sslErrors.first().errorString());
+        setErrorAndEmit(QAbstractSocket::SslHandshakeFailedError, sslErrors.constFirst().errorString());
 
         // Disconnect if there are any non-ignorable errors
-        foreach (const QSslError &error, sslErrors) {
+        for (const QSslError &error : qAsConst(sslErrors)) {
             if (ignoreErrorsList.contains(error))
                 continue;
             q->disconnectFromHost();
