@@ -1,31 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2012 BogDan Vatra <bogdan@kde.org>
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -51,6 +57,7 @@ namespace QtAndroidInput
 
     static bool m_ignoreMouseEvents = false;
     static bool m_softwareKeyboardVisible = false;
+    static QRect m_softwareKeyboardRect;
 
     static QList<QWindowSystemInterface::TouchPoint> m_touchPoints;
 
@@ -70,18 +77,20 @@ namespace QtAndroidInput
                                                   candidatesEnd);
     }
 
-    void showSoftwareKeyboard(int left, int top, int width, int height, int inputHints)
+    void showSoftwareKeyboard(int left, int top, int width, int height, int inputHints, int enterKeyType)
     {
         QJNIObjectPrivate::callStaticMethod<void>(applicationClass(),
                                                   "showSoftwareKeyboard",
-                                                  "(IIIII)V",
+                                                  "(IIIIII)V",
                                                   left,
                                                   top,
                                                   width,
                                                   height,
-                                                  inputHints);
+                                                  inputHints,
+                                                  enterKeyType
+                                                 );
 #ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ SHOWSOFTWAREKEYBOARD" << left << top << width << height << inputHints;
+        qDebug() << "@@@ SHOWSOFTWAREKEYBOARD" << left << top << width << height << inputHints << enterKeyType;
 #endif
     }
 
@@ -89,7 +98,7 @@ namespace QtAndroidInput
     {
         QJNIObjectPrivate::callStaticMethod<void>(applicationClass(), "resetSoftwareKeyboard");
 #ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ RESETSOFTWAREKEYBOARD";
+        qDebug("@@@ RESETSOFTWAREKEYBOARD");
 #endif
     }
 
@@ -97,7 +106,7 @@ namespace QtAndroidInput
     {
         QJNIObjectPrivate::callStaticMethod<void>(applicationClass(), "hideSoftwareKeyboard");
 #ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ HIDESOFTWAREKEYBOARD";
+        qDebug("@@@ HIDESOFTWAREKEYBOARD");
 #endif
     }
 
@@ -106,6 +115,11 @@ namespace QtAndroidInput
         return m_softwareKeyboardVisible;
     }
 
+    QRect softwareKeyboardRect()
+    {
+        return m_softwareKeyboardRect;
+    }
+
 
     static void mouseDown(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y)
     {
@@ -155,7 +169,7 @@ namespace QtAndroidInput
     static void longPress(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y)
     {
         //### TODO: add proper API for Qt 5.2
-        static bool rightMouseFromLongPress = qgetenv("QT_NECESSITAS_COMPATIBILITY_LONG_PRESS").toInt();
+        static bool rightMouseFromLongPress = qEnvironmentVariableIntValue("QT_NECESSITAS_COMPATIBILITY_LONG_PRESS");
         if (!rightMouseFromLongPress)
             return;
         m_ignoreMouseEvents = true;
@@ -238,6 +252,63 @@ namespace QtAndroidInput
         QWindowSystemInterface::handleTouchEvent(window, touchDevice, m_touchPoints);
     }
 
+    static bool isTabletEventSupported(JNIEnv */*env*/, jobject /*thiz*/)
+    {
+#ifdef QT_NO_TABLETEVENT
+        return false;
+#else
+        return true;
+#endif // QT_NO_TABLETEVENT
+    }
+
+    static void tabletEvent(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint deviceId, jlong time, jint action,
+        jint pointerType, jint buttonState, jfloat x, jfloat y, jfloat pressure)
+    {
+#ifndef QT_NO_TABLETEVENT
+        QPointF globalPosF(x, y);
+        QPoint globalPos((int)x, (int)y);
+        QWindow *tlw = topLevelWindowAt(globalPos);
+        QPointF localPos = tlw ? (globalPosF - tlw->position()) : globalPosF;
+
+        // Galaxy Note with plain Android:
+        // 0 1 0    stylus press
+        // 2 1 0    stylus drag
+        // 1 1 0    stylus release
+        // 0 1 2    stylus press with side-button held
+        // 2 1 2    stylus drag with side-button held
+        // 1 1 2    stylus release with side-button held
+        // Galaxy Note 4 with Samsung firmware:
+        // 0 1 0    stylus press
+        // 2 1 0    stylus drag
+        // 1 1 0    stylus release
+        // 211 1 2  stylus press with side-button held
+        // 213 1 2  stylus drag with side-button held
+        // 212 1 2  stylus release with side-button held
+        // when action == ACTION_UP (1) it's a release; otherwise we say which button is pressed
+        Qt::MouseButtons buttons = Qt::NoButton;
+        switch (action) {
+        case 1:     // ACTION_UP
+        case 212:   // stylus release while side-button held on Galaxy Note 4
+            buttons = Qt::NoButton;
+            break;
+        default:    // action is press or drag
+            if (buttonState == 0)
+                buttons = Qt::LeftButton;
+            else // 2 means RightButton
+                buttons = Qt::MouseButtons(buttonState);
+            break;
+        }
+
+#ifdef QT_DEBUG_ANDROID_STYLUS
+        qDebug() << action << pointerType << buttonState << '@' << x << y << "pressure" << pressure << ": buttons" << buttons;
+#endif
+
+        QWindowSystemInterface::handleTabletEvent(tlw, ulong(time),
+            localPos, globalPosF, QTabletEvent::Stylus, pointerType,
+            buttons, pressure, 0, 0, 0., 0., 0, deviceId, Qt::NoModifier);
+#endif // QT_NO_TABLETEVENT
+    }
+
     static int mapAndroidKey(int key)
     {
         // 0--9        0x00000007 -- 0x00000010
@@ -633,7 +704,7 @@ namespace QtAndroidInput
             return Qt::Key_AudioCycleTrack;
 
         default:
-            qWarning() << "Unhandled key code " << key << "!";
+            qWarning() << "Unhandled key code " << key << '!';
             return 0;
         }
     }
@@ -663,37 +734,58 @@ namespace QtAndroidInput
         return unicode ? QString(QChar(unicode)) : QString();
     }
 
-    static void keyDown(JNIEnv */*env*/, jobject /*thiz*/, jint key, jint unicode, jint modifier)
+    static void keyDown(JNIEnv */*env*/, jobject /*thiz*/, jint key, jint unicode, jint modifier, jboolean autoRepeat)
     {
         QWindowSystemInterface::handleKeyEvent(0,
                                                QEvent::KeyPress,
                                                mapAndroidKey(key),
                                                mapAndroidModifiers(modifier),
                                                toString(unicode),
-                                               false);
+                                               autoRepeat);
     }
 
-    static void keyUp(JNIEnv */*env*/, jobject /*thiz*/, jint key, jint unicode, jint modifier)
+    static void keyUp(JNIEnv */*env*/, jobject /*thiz*/, jint key, jint unicode, jint modifier, jboolean autoRepeat)
     {
         QWindowSystemInterface::handleKeyEvent(0,
                                                QEvent::KeyRelease,
                                                mapAndroidKey(key),
                                                mapAndroidModifiers(modifier),
                                                toString(unicode),
-                                               false);
+                                               autoRepeat);
     }
 
     static void keyboardVisibilityChanged(JNIEnv */*env*/, jobject /*thiz*/, jboolean visibility)
     {
         m_softwareKeyboardVisible = visibility;
+        if (!visibility)
+            m_softwareKeyboardRect = QRect();
+
         QAndroidInputContext *inputContext = QAndroidInputContext::androidInputContext();
-        if (inputContext && qGuiApp)
+        if (inputContext && qGuiApp) {
             inputContext->emitInputPanelVisibleChanged();
+            if (!visibility)
+                inputContext->emitKeyboardRectChanged();
+        }
 #ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
         qDebug() << "@@@ KEYBOARDVISIBILITYCHANGED" << inputContext;
 #endif
     }
 
+    static void keyboardGeometryChanged(JNIEnv */*env*/, jobject /*thiz*/, jint x, jint y, jint w, jint h)
+    {
+        QRect r = QRect(x, y, w, h);
+        if (r == m_softwareKeyboardRect)
+            return;
+        m_softwareKeyboardRect = r;
+        QAndroidInputContext *inputContext = QAndroidInputContext::androidInputContext();
+        if (inputContext && qGuiApp)
+            inputContext->emitKeyboardRectChanged();
+
+#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
+        qDebug() << "@@@ KEYBOARDRECTCHANGED" << m_softwareKeyboardRect;
+#endif
+    }
+
     static JNINativeMethod methods[] = {
         {"touchBegin","(I)V",(void*)touchBegin},
         {"touchAdd","(IIIZIIFF)V",(void*)touchAdd},
@@ -702,9 +794,12 @@ namespace QtAndroidInput
         {"mouseUp", "(III)V", (void *)mouseUp},
         {"mouseMove", "(III)V", (void *)mouseMove},
         {"longPress", "(III)V", (void *)longPress},
-        {"keyDown", "(III)V", (void *)keyDown},
-        {"keyUp", "(III)V", (void *)keyUp},
-        {"keyboardVisibilityChanged", "(Z)V", (void *)keyboardVisibilityChanged}
+        {"isTabletEventSupported", "()Z", (void *)isTabletEventSupported},
+        {"tabletEvent", "(IIJIIIFFF)V", (void *)tabletEvent},
+        {"keyDown", "(IIIZ)V", (void *)keyDown},
+        {"keyUp", "(IIIZ)V", (void *)keyUp},
+        {"keyboardVisibilityChanged", "(Z)V", (void *)keyboardVisibilityChanged},
+        {"keyboardGeometryChanged", "(IIII)V", (void *)keyboardGeometryChanged}
     };
 
     bool registerNatives(JNIEnv *env)
