@@ -1,31 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2012 BogDan Vatra <bogdan@kde.org>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,7 +63,7 @@ struct AndroidStyle;
 class QAndroidPlatformNativeInterface: public QPlatformNativeInterface
 {
 public:
-    void *nativeResourceForIntegration(const QByteArray &resource) override;
+    void *nativeResourceForIntegration(const QByteArray &resource);
     std::shared_ptr<AndroidStyle> m_androidStyle;
 };
 
@@ -69,39 +75,39 @@ public:
     QAndroidPlatformIntegration(const QStringList &paramList);
     ~QAndroidPlatformIntegration();
 
-    bool hasCapability(QPlatformIntegration::Capability cap) const override;
+    bool hasCapability(QPlatformIntegration::Capability cap) const;
 
-    QPlatformWindow *createPlatformWindow(QWindow *window) const override;
-    QPlatformBackingStore *createPlatformBackingStore(QWindow *window) const override;
-    QPlatformOpenGLContext *createPlatformOpenGLContext(QOpenGLContext *context) const override;
-    QAbstractEventDispatcher *createEventDispatcher() const override;
+    QPlatformWindow *createPlatformWindow(QWindow *window) const;
+    QPlatformBackingStore *createPlatformBackingStore(QWindow *window) const;
+    QPlatformOpenGLContext *createPlatformOpenGLContext(QOpenGLContext *context) const;
+    QAbstractEventDispatcher *createEventDispatcher() const;
     QAndroidPlatformScreen *screen() { return m_primaryScreen; }
-    QPlatformOffscreenSurface *createPlatformOffscreenSurface(QOffscreenSurface *surface) const override;
+    QPlatformOffscreenSurface *createPlatformOffscreenSurface(QOffscreenSurface *surface) const;
 
     virtual void setDesktopSize(int width, int height);
     virtual void setDisplayMetrics(int width, int height);
     void setScreenSize(int width, int height);
     bool isVirtualDesktop() { return true; }
 
-    QPlatformFontDatabase *fontDatabase() const override;
+    QPlatformFontDatabase *fontDatabase() const;
 
 #ifndef QT_NO_CLIPBOARD
-    QPlatformClipboard *clipboard() const override;
+    QPlatformClipboard *clipboard() const;
 #endif
 
-    QPlatformInputContext *inputContext() const override;
-    QPlatformNativeInterface *nativeInterface() const override;
-    QPlatformServices *services() const override;
+    QPlatformInputContext *inputContext() const;
+    QPlatformNativeInterface *nativeInterface() const;
+    QPlatformServices *services() const;
 
 #ifndef QT_NO_ACCESSIBILITY
-    virtual QPlatformAccessibility *accessibility() const override;
+    virtual QPlatformAccessibility *accessibility() const;
 #endif
 
-    QVariant styleHint(StyleHint hint) const override;
-    Qt::WindowState defaultWindowState(Qt::WindowFlags flags) const override;
+    QVariant styleHint(StyleHint hint) const;
+    Qt::WindowState defaultWindowState(Qt::WindowFlags flags) const;
 
-    QStringList themeNames() const override;
-    QPlatformTheme *createPlatformTheme(const QString &name) const override;
+    QStringList themeNames() const;
+    QPlatformTheme *createPlatformTheme(const QString &name) const;
 
     static void setDefaultDisplayMetrics(int gw, int gh, int sw, int sh, int width, int height);
     static void setDefaultDesktopSize(int gw, int gh);
