@@ -1,32 +1,38 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 BogDan Vatra <bogdan@kde.org>
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,14 +63,14 @@ public:
     QAndroidPlatformScreen();
     ~QAndroidPlatformScreen();
 
-    QRect geometry() const override { return QRect(QPoint(), m_size); }
-    QRect availableGeometry() const override { return m_availableGeometry; }
-    int depth() const override { return m_depth; }
-    QImage::Format format() const override { return m_format; }
-    QSizeF physicalSize() const override { return m_physicalSize; }
+    QRect geometry() const { return QRect(QPoint(), m_size); }
+    QRect availableGeometry() const { return m_availableGeometry; }
+    int depth() const { return m_depth; }
+    QImage::Format format() const { return m_format; }
+    QSizeF physicalSize() const { return m_physicalSize; }
 
     inline QWindow *topWindow() const;
-    QWindow *topLevelAt(const QPoint & p) const override;
+    QWindow *topLevelAt(const QPoint & p) const;
 
     // compositor api
     void addWindow(QAndroidPlatformWindow *window);
@@ -94,11 +100,11 @@ protected:
     QSizeF m_physicalSize;
 
 private:
-    QDpi logicalDpi() const override;
-    qreal pixelDensity()  const override;
-    Qt::ScreenOrientation orientation() const override;
-    Qt::ScreenOrientation nativeOrientation() const override;
-    void surfaceChanged(JNIEnv *env, jobject surface, int w, int h) override;
+    QDpi logicalDpi() const;
+    qreal pixelDensity()  const;
+    Qt::ScreenOrientation orientation() const;
+    Qt::ScreenOrientation nativeOrientation() const;
+    void surfaceChanged(JNIEnv *env, jobject surface, int w, int h);
     void releaseSurface();
     void applicationStateChanged(Qt::ApplicationState);
 
