@@ -1,32 +1,38 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 BogDan Vatra <bogdan@kde.org>
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -48,21 +54,21 @@ class QAndroidPlatformWindow: public QPlatformWindow
 public:
     explicit QAndroidPlatformWindow(QWindow *window);
 
-    void lower() override;
-    void raise() override;
+    void lower();
+    void raise();
 
-    void setVisible(bool visible) override;
+    void setVisible(bool visible);
 
-    void setWindowState(Qt::WindowState state) override;
-    void setWindowFlags(Qt::WindowFlags flags) override;
+    void setWindowState(Qt::WindowState state);
+    void setWindowFlags(Qt::WindowFlags flags);
     Qt::WindowFlags windowFlags() const;
-    void setParent(const QPlatformWindow *window) override;
-    WId winId() const override { return m_windowId; }
+    void setParent(const QPlatformWindow *window);
+    WId winId() const { return m_windowId; }
 
     QAndroidPlatformScreen *platformScreen() const;
 
-    void propagateSizeHints() override;
-    void requestActivateWindow() override;
+    void propagateSizeHints();
+    void requestActivateWindow();
     void updateStatusBarVisibility();
     inline bool isRaster() const {
         if ((window()->flags() & Qt::ForeignWindow) == Qt::ForeignWindow)
@@ -71,7 +77,7 @@ public:
         return window()->surfaceType() == QSurface::RasterSurface
             || window()->surfaceType() == QSurface::RasterGLSurface;
     }
-    bool isExposed() const override;
+    bool isExposed() const;
 
     virtual void applicationStateChanged(Qt::ApplicationState);
 
@@ -81,7 +87,7 @@ public:
     virtual void repaint(const QRegion &) { }
 
 protected:
-    void setGeometry(const QRect &rect) override;
+    void setGeometry(const QRect &rect);
 
 protected:
     Qt::WindowFlags m_windowFlags;
