@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -35,8 +41,12 @@
 #include <QtGui/qaccessible.h>
 #include <private/qcore_mac_p.h>
 
+#include <Carbon/Carbon.h>
+
 QT_BEGIN_NAMESPACE
 
+#ifndef QT_NO_ACCESSIBILITY
+
 QCocoaAccessibility::QCocoaAccessibility()
 {
 
@@ -49,9 +59,11 @@ QCocoaAccessibility::~QCocoaAccessibility()
 
 void QCocoaAccessibility::notifyAccessibilityUpdate(QAccessibleEvent *event)
 {
+    if (!isActive() || !event->accessibleInterface() || !event->accessibleInterface()->isValid())
+        return;
     QMacAccessibilityElement *element = [QMacAccessibilityElement elementWithId: event->uniqueId()];
     if (!element) {
-        qWarning() << "QCocoaAccessibility::notifyAccessibilityUpdate: invalid element";
+        qWarning("QCocoaAccessibility::notifyAccessibilityUpdate: invalid element");
         return;
     }
 
@@ -191,6 +203,8 @@ NSString *macSubrole(QAccessibleInterface *interface)
     QAccessible::State s = interface->state();
     if (s.searchEdit)
         return NSAccessibilitySearchFieldSubrole;
+    if (s.passwordEdit)
+        return NSAccessibilitySecureTextFieldSubrole;
     return nil;
 }
 
@@ -263,7 +277,7 @@ NSArray *unignoredChildren(QAccessibleInterface *interface)
         if (element)
             [kids addObject: element];
         else
-            qWarning() << "QCocoaAccessibility: invalid child";
+            qWarning("QCocoaAccessibility: invalid child");
     }
     return NSAccessibilityUnignoredChildren(kids);
 }
@@ -332,6 +346,7 @@ bool hasValueAttribute(QAccessibleInterface *interface)
     Q_ASSERT(interface);
     const QAccessible::Role qtrole = interface->role();
     if (qtrole == QAccessible::EditableText
+            || qtrole == QAccessible::StaticText
             || interface->valueInterface()
             || interface->state().checkable) {
         return true;
@@ -343,20 +358,28 @@ bool hasValueAttribute(QAccessibleInterface *interface)
 id getValueAttribute(QAccessibleInterface *interface)
 {
     const QAccessible::Role qtrole = interface->role();
+    if (qtrole == QAccessible::StaticText) {
+        return QCFString::toNSString(interface->text(QAccessible::Name));
+    }
     if (qtrole == QAccessible::EditableText) {
         if (QAccessibleTextInterface *textInterface = interface->textInterface()) {
-            // VoiceOver will read out the entire text string at once when returning
-            // text as a value. For large text edits the size of the returned string
-            // needs to be limited and text range attributes need to be used instead.
-            // NSTextEdit returns the first sentence as the value, Do the same here:
+
             int begin = 0;
             int end = textInterface->characterCount();
-            // ### call to textAfterOffset hangs. Booo!
-            //if (textInterface->characterCount() > 0)
-            //    textInterface->textAfterOffset(0, QAccessible2::SentenceBoundary, &begin, &end);
-
-            QString text = textInterface->text(begin, end);
-            //qDebug() << "text" << begin << end << text;
+            QString text;
+            if (interface->state().passwordEdit) {
+                // return round password replacement chars
+                text = QString(end, QChar(kBulletUnicode));
+            } else {
+                // VoiceOver will read out the entire text string at once when returning
+                // text as a value. For large text edits the size of the returned string
+                // needs to be limited and text range attributes need to be used instead.
+                // NSTextEdit returns the first sentence as the value, Do the same here:
+                // ### call to textAfterOffset hangs. Booo!
+                //if (textInterface->characterCount() > 0)
+                //    textInterface->textAfterOffset(0, QAccessible2::SentenceBoundary, &begin, &end);
+                text = textInterface->text(begin, end);
+            }
             return QCFString::toNSString(text);
         }
     }
@@ -374,5 +397,7 @@ id getValueAttribute(QAccessibleInterface *interface)
 
 } // namespace QCocoaAccessible
 
+#endif // QT_NO_ACCESSIBILITY
+
 QT_END_NAMESPACE
 
