@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -47,14 +45,13 @@ QT_BEGIN_NAMESPACE
 
 QCocoaBackingStore::QCocoaBackingStore(QWindow *window)
     : QPlatformBackingStore(window)
-    , m_cgImage(0)
 {
 }
 
 QCocoaBackingStore::~QCocoaBackingStore()
 {
-    CGImageRelease(m_cgImage);
-    m_cgImage = 0;
+    if (QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(window()->handle()))
+        [cocoaWindow->m_qtView clearBackingStore:this];
 }
 
 QPaintDevice *QCocoaBackingStore::paintDevice()
@@ -66,9 +63,6 @@ QPaintDevice *QCocoaBackingStore::paintDevice()
     // either due to a window resize or devicePixelRatio change.
     QSize effectiveBufferSize = m_requestedSize * windowDevicePixelRatio;
     if (m_qImage.size() != effectiveBufferSize) {
-        CGImageRelease(m_cgImage);
-        m_cgImage = 0;
-
         QImage::Format format = (window()->format().hasAlpha() || cocoaWindow->m_drawContentBorderGradient)
                 ? QImage::Format_ARGB32_Premultiplied : QImage::Format_RGB32;
         m_qImage = QImage(effectiveBufferSize, format);
@@ -81,15 +75,6 @@ QPaintDevice *QCocoaBackingStore::paintDevice()
 
 void QCocoaBackingStore::flush(QWindow *win, const QRegion &region, const QPoint &offset)
 {
-    // A flush means that qImage has changed. Since CGImages are seen as
-    // immutable, CoreImage fails to pick up this change for m_cgImage
-    // (since it usually cached), so we must recreate it. We await doing this
-    // until one of the views needs it, since, together with calling
-    // "setNeedsDisplayInRect" instead of "displayRect" we will, in most
-    // cases, get away with doing this once for every repaint. Also note that
-    // m_cgImage is only a reference to the data inside m_qImage, it is not a copy.
-    CGImageRelease(m_cgImage);
-    m_cgImage = 0;
     if (!m_qImage.isNull()) {
         if (QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(win->handle()))
             [cocoaWindow->m_qtView flushBackingStore:this region:region offset:offset];
@@ -119,15 +104,16 @@ bool QCocoaBackingStore::scroll(const QRegion &area, int dx, int dy)
     return true;
 }
 
-CGImageRef QCocoaBackingStore::getBackingStoreCGImage()
+void QCocoaBackingStore::beginPaint(const QRegion &region)
 {
-    if (!m_cgImage)
-        m_cgImage = qt_mac_toCGImage(m_qImage);
-
-    // Warning: do not retain/release/cache the returned image from
-    // outside the backingstore since it shares data with a QImage and
-    // needs special memory considerations.
-    return m_cgImage;
+    if (m_qImage.hasAlphaChannel()) {
+        QPainter p(&m_qImage);
+        p.setCompositionMode(QPainter::CompositionMode_Source);
+        const QVector<QRect> rects = region.rects();
+        const QColor blank = Qt::transparent;
+        for (QVector<QRect>::const_iterator it = rects.begin(), end = rects.end(); it != end; ++it)
+            p.fillRect(*it, blank);
+    }
 }
 
 qreal QCocoaBackingStore::getBackingStoreDevicePixelRatio()
