@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -42,9 +40,14 @@
 #include "qcocoadrag.h"
 #include "qmacclipboard.h"
 #include "qcocoahelpers.h"
+#ifndef QT_NO_WIDGETS
+#include <QtWidgets/qwidget.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
+static const int dragImageMaxChars = 26;
+
 QCocoaDrag::QCocoaDrag() :
     m_drag(0)
 {
@@ -124,27 +127,28 @@ Qt::DropAction QCocoaDrag::drag(QDrag *o)
     m_drag = o;
     m_executed_drop_action = Qt::IgnoreAction;
 
-    QPixmap pm = m_drag->pixmap();
-    if (pm.isNull())
-        pm = defaultPixmap();
-
+    QPoint hotSpot = m_drag->hotSpot();
+    QPixmap pm = dragPixmap(m_drag, hotSpot);
+    QSize pmDeviceIndependentSize = pm.size() / pm.devicePixelRatio();
     NSImage *nsimage = qt_mac_create_nsimage(pm);
+    [nsimage setSize : qt_mac_toNSSize(pmDeviceIndependentSize)];
 
     QMacPasteboard dragBoard((CFStringRef) NSDragPboard, QMacInternalPasteboardMime::MIME_DND);
     m_drag->mimeData()->setData(QLatin1String("application/x-qt-mime-type-name"), QByteArray("dummy"));
-    dragBoard.setMimeData(m_drag->mimeData());
+    dragBoard.setMimeData(m_drag->mimeData(), QMacPasteboard::LazyRequest);
 
     NSPoint event_location = [m_lastEvent locationInWindow];
-    NSPoint local_point = [m_lastView convertPoint:event_location fromView:nil];
-    local_point.x -= m_drag->hotSpot().x();
-    CGFloat flippedY = m_drag->pixmap().height() - m_drag->hotSpot().y();
-    local_point.y += flippedY;
-    NSSize mouseOffset = NSMakeSize(0.0, 0.0);
+    NSWindow *theWindow = [m_lastEvent window];
+    Q_ASSERT(theWindow != nil);
+    event_location.x -= hotSpot.x();
+    CGFloat flippedY = pmDeviceIndependentSize.height() - hotSpot.y();
+    event_location.y -= flippedY;
+    NSSize mouseOffset_unused = NSMakeSize(0.0, 0.0);
     NSPasteboard *pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
 
-    [m_lastView dragImage:nsimage
-        at:local_point
-        offset:mouseOffset
+    [theWindow dragImage:nsimage
+        at:event_location
+        offset:mouseOffset_unused
         event:m_lastEvent
         pasteboard:pboard
         source:m_lastView
@@ -161,6 +165,61 @@ void QCocoaDrag::setAcceptedAction(Qt::DropAction act)
     m_executed_drop_action = act;
 }
 
+QPixmap QCocoaDrag::dragPixmap(QDrag *drag, QPoint &hotSpot) const
+{
+    const QMimeData* data = drag->mimeData();
+    QPixmap pm = drag->pixmap();
+
+    if (pm.isNull()) {
+        QFont f(qApp->font());
+        f.setPointSize(12);
+        QFontMetrics fm(f);
+
+        if (data->hasImage()) {
+            const QImage img = data->imageData().value<QImage>();
+            if (!img.isNull()) {
+                pm = QPixmap::fromImage(img).scaledToWidth(dragImageMaxChars *fm.averageCharWidth());
+            }
+        }
+
+        if (pm.isNull() && (data->hasText() || data->hasUrls()) ) {
+            QString s = data->hasText() ? data->text() : data->urls().first().toString();
+            if (s.length() > dragImageMaxChars)
+                s = s.left(dragImageMaxChars -3) + QChar(0x2026);
+            if (!s.isEmpty()) {
+                const int width = fm.width(s);
+                const int height = fm.height();
+                if (width > 0 && height > 0) {
+                    qreal dpr = 1.0;
+                    if (const QWindow *sourceWindow = qobject_cast<QWindow *>(drag->source())) {
+                        dpr = sourceWindow->devicePixelRatio();
+                    }
+#ifndef QT_NO_WIDGETS
+                    else if (const QWidget *sourceWidget = qobject_cast<QWidget *>(drag->source())) {
+                        if (const QWindow *sourceWindow = sourceWidget->window()->windowHandle())
+                            dpr = sourceWindow->devicePixelRatio();
+                    }
+#endif
+                    pm = QPixmap(width * dpr, height * dpr);
+                    pm.setDevicePixelRatio(dpr);
+                    QPainter p(&pm);
+                    p.fillRect(0, 0, pm.width(), pm.height(), Qt::color0);
+                    p.setPen(Qt::color1);
+                    p.setFont(f);
+                    p.drawText(0, fm.ascent(), s);
+                    p.end();
+                    hotSpot = QPoint(pm.width() / 2, pm.height() / 2);
+                }
+            }
+        }
+    }
+
+    if (pm.isNull())
+        pm = defaultPixmap();
+
+    return pm;
+}
+
 QCocoaDropData::QCocoaDropData(NSPasteboard *pasteboard)
 {
     dropPasteboard = reinterpret_cast<CFStringRef>(const_cast<const NSString *>([pasteboard name]));
