@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -294,8 +300,6 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
     initResources();
     QMacAutoReleasePool pool;
 
-    qApp->setAttribute(Qt::AA_DontUseNativeMenuBar, false);
-
     NSApplication *cocoaApplication = [QNSApplication sharedApplication];
     qt_redirectNSApplicationSendEvent();
 
@@ -307,21 +311,17 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
         // see the function implementation for exceptions.)
         qt_mac_transformProccessToForegroundApplication();
 
-        // Move the application window to front to make it take focus, also when launching
-        // from the terminal. On 10.12+ this call has been moved to applicationDidFinishLauching
-        // to work around issues with loss of focus at startup.
-        if (QSysInfo::macVersion() < QSysInfo::MV_10_12) {
-            // Ignoring other apps is necessary (we must ignore the terminal), but makes
-            // Qt apps play slightly less nice with other apps when lanching from Finder
-            // (See the activateIgnoringOtherApps docs.)
-            [cocoaApplication activateIgnoringOtherApps : YES];
-        }
+        // Move the application window to front to avoid launching behind the terminal.
+        // Ignoring other apps is neccessary (we must ignore the terminal), but makes
+        // Qt apps play slightly less nice with other apps when lanching from Finder
+        // (See the activateIgnoringOtherApps docs.)
+        [cocoaApplication activateIgnoringOtherApps : YES];
     }
 
     // ### For AA_MacPluginApplication we don't want to load the menu nib.
     // Qt 4 also does not set the application delegate, so that behavior
     // is matched here.
-    if (!QCoreApplication::testAttribute(Qt::AA_MacPluginApplication)) {
+    if (!QCoreApplication::testAttribute(Qt::AA_PluginApplication)) {
 
         // Set app delegate, link to the current delegate (if any)
         QCocoaApplicationDelegate *newDelegate = [QCocoaApplicationDelegate sharedDelegate];
@@ -348,6 +348,7 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
 
     QMacInternalPasteboardMime::initializeMimeTypes();
     QCocoaMimeTypes::initializeMimeTypes();
+    QWindowSystemInterfacePrivate::TabletEvent::setPlatformSynthesizesMouse(false);
 }
 
 QCocoaIntegration::~QCocoaIntegration()
@@ -357,7 +358,7 @@ QCocoaIntegration::~QCocoaIntegration()
     qt_resetNSApplicationSendEvent();
 
     QMacAutoReleasePool pool;
-    if (!QCoreApplication::testAttribute(Qt::AA_MacPluginApplication)) {
+    if (!QCoreApplication::testAttribute(Qt::AA_PluginApplication)) {
         // remove the apple event handlers installed by QCocoaApplicationDelegate
         QCocoaApplicationDelegate *delegate = [QCocoaApplicationDelegate sharedDelegate];
         [delegate removeAppleEventHandlers];
@@ -635,4 +636,9 @@ void QCocoaIntegration::setApplicationIcon(const QIcon &icon) const
     [image release];
 }
 
+void QCocoaIntegration::beep() const
+{
+    NSBeep();
+}
+
 QT_END_NAMESPACE
