@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,7 +49,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include "qcocoahelpers.h"
-#include "qcocoaautoreleasepool.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -82,6 +87,7 @@ QMacPasteboard::QMacPasteboard(PasteboardRef p, uchar mt)
     mime_type = mt ? mt : uchar(QMacInternalPasteboardMime::MIME_ALL);
     paste = p;
     CFRetain(paste);
+    resolvingBeforeDestruction = false;
 }
 
 QMacPasteboard::QMacPasteboard(uchar mt)
@@ -95,6 +101,7 @@ QMacPasteboard::QMacPasteboard(uchar mt)
     } else {
         qDebug("PasteBoard: Error creating pasteboard: [%d]", (int)err);
     }
+    resolvingBeforeDestruction = false;
 }
 
 QMacPasteboard::QMacPasteboard(CFStringRef name, uchar mt)
@@ -108,23 +115,14 @@ QMacPasteboard::QMacPasteboard(CFStringRef name, uchar mt)
     } else {
         qDebug("PasteBoard: Error creating pasteboard: %s [%d]", QCFString::toQString(name).toLatin1().constData(), (int)err);
     }
+    resolvingBeforeDestruction = false;
 }
 
 QMacPasteboard::~QMacPasteboard()
 {
     // commit all promises for paste after exit close
-    for (int i = 0; i < promises.count(); ++i) {
-        const Promise &promise = promises.at(i);
-        // At this point app teardown has started and control is somewhere in the Q[Core]Application
-        // destructor. Skip "lazy" promises where the application has not provided data;
-        // the application will generally not be in a state to provide it.
-        if (promise.dataRequestType == LazyRequest)
-            continue;
-        QCFString flavor = QCFString(promise.convertor->flavorFor(promise.mime));
-        NSInteger pbItemId = promise.itemId;
-        promiseKeeper(paste, reinterpret_cast<PasteboardItemID>(pbItemId), flavor, this);
-    }
-
+    resolvingBeforeDestruction = true;
+    PasteboardResolvePromises(paste);
     if (paste)
         CFRelease(paste);
 }
@@ -176,7 +174,7 @@ OSStatus QMacPasteboard::promiseKeeper(PasteboardRef paste, PasteboardItemID id,
     // to request the data from the application.
     QVariant promiseData;
     if (promise.dataRequestType == LazyRequest) {
-        if (!promise.mimeData.isNull())
+        if (!qpaste->resolvingBeforeDestruction && !promise.mimeData.isNull())
             promiseData = promise.mimeData->variantData(promise.mime);
     } else {
         promiseData = promise.variantData;
@@ -555,7 +553,7 @@ QMacPasteboard::sync() const
 
 QString qt_mac_get_pasteboardString(PasteboardRef paste)
 {
-    QCocoaAutoReleasePool pool;
+    QMacAutoReleasePool pool;
     NSPasteboard *pb = nil;
     CFStringRef pbname;
     if (PasteboardCopyName(paste, &pbname) == noErr) {
