@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,13 +55,12 @@ QT_BEGIN_NAMESPACE
 class QMinimalEglContext : public QEGLPlatformContext
 {
 public:
-    QMinimalEglContext(const QSurfaceFormat &format, QPlatformOpenGLContext *share, EGLDisplay display,
-                  EGLenum eglApi = EGL_OPENGL_ES_API)
-        : QEGLPlatformContext(format, share, display, eglApi)
+    QMinimalEglContext(const QSurfaceFormat &format, QPlatformOpenGLContext *share, EGLDisplay display)
+        : QEGLPlatformContext(format, share, display)
     {
     }
 
-    EGLSurface eglSurfaceForPlatformSurface(QPlatformSurface *surface)
+    EGLSurface eglSurfaceForPlatformSurface(QPlatformSurface *surface) Q_DECL_OVERRIDE
     {
         QMinimalEglWindow *window = static_cast<QMinimalEglWindow *>(surface);
         QMinimalEglScreen *screen = static_cast<QMinimalEglScreen *>(window->screen());
@@ -83,34 +80,24 @@ QMinimalEglScreen::QMinimalEglScreen(EGLNativeDisplayType display)
 
     EGLint major, minor;
 
-    if (!eglBindAPI(EGL_OPENGL_ES_API)) {
+    if (Q_UNLIKELY(!eglBindAPI(EGL_OPENGL_ES_API))) {
         qWarning("Could not bind GL_ES API\n");
         qFatal("EGL error");
     }
 
     m_dpy = eglGetDisplay(display);
-    if (m_dpy == EGL_NO_DISPLAY) {
+    if (Q_UNLIKELY(m_dpy == EGL_NO_DISPLAY)) {
         qWarning("Could not open egl display\n");
         qFatal("EGL error");
     }
     qWarning("Opened display %p\n", m_dpy);
 
-    if (!eglInitialize(m_dpy, &major, &minor)) {
+    if (Q_UNLIKELY(!eglInitialize(m_dpy, &major, &minor))) {
         qWarning("Could not initialize egl display\n");
         qFatal("EGL error");
     }
 
     qWarning("Initialized display %d %d\n", major, minor);
-
-    int swapInterval = 1;
-    QByteArray swapIntervalString = qgetenv("QT_QPA_EGLFS_SWAPINTERVAL");
-    if (!swapIntervalString.isEmpty()) {
-        bool ok;
-        swapInterval = swapIntervalString.toInt(&ok);
-        if (!ok)
-            swapInterval = 1;
-    }
-    eglSwapInterval(m_dpy, swapInterval);
 }
 
 QMinimalEglScreen::~QMinimalEglScreen()
@@ -154,9 +141,9 @@ void QMinimalEglScreen::createAndSetPlatformContext()
 
     EGLNativeWindowType eglWindow = 0;
 #ifdef Q_OPENKODE
-    if (kdInitializeNV() == KD_ENOTINITIALIZED) {
+    if (Q_UNLIKELY(kdInitializeNV() == KD_ENOTINITIALIZED))
         qFatal("Did not manage to initialize openkode");
-    }
+
     KDWindow *window = kdCreateWindow(m_dpy,config,0);
 
     kdRealizeWindow(window,&eglWindow);
@@ -167,7 +154,7 @@ void QMinimalEglScreen::createAndSetPlatformContext()
 #endif
 
     m_surface = eglCreateWindowSurface(m_dpy, config, eglWindow, NULL);
-    if (m_surface == EGL_NO_SURFACE) {
+    if (Q_UNLIKELY(m_surface == EGL_NO_SURFACE)) {
         qWarning("Could not create the egl surface: error = 0x%x\n", eglGetError());
         eglTerminate(m_dpy);
         qFatal("EGL error");
