@@ -1,31 +1,37 @@
 /***************************************************************************
 **
 ** Copyright (C) 2011 - 2012 Research In Motion
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -94,13 +100,13 @@ public:
 
     void addFormatToCheck(const QString &format) {
         m_formatsToCheck << format;
-        qClipboardDebug() << Q_FUNC_INFO << "formats=" << m_formatsToCheck;
+        qClipboardDebug() << "formats=" << m_formatsToCheck;
     }
 
     bool hasFormat(const QString &mimetype) const
     {
         const bool result = is_clipboard_format_present(mimetype.toUtf8().constData()) == 0;
-        qClipboardDebug() << Q_FUNC_INFO << "mimetype=" << mimetype << "result=" << result;
+        qClipboardDebug() << "mimetype=" << mimetype << "result=" << result;
         return result;
     }
 
@@ -113,7 +119,7 @@ public:
                 result << format;
         }
 
-        qClipboardDebug() << Q_FUNC_INFO << "result=" << result;
+        qClipboardDebug() << "result=" << result;
         return result;
     }
 
@@ -137,7 +143,7 @@ public:
 protected:
     QVariant retrieveData(const QString &mimetype, QVariant::Type preferredType) const
     {
-        qClipboardDebug() << Q_FUNC_INFO << "mimetype=" << mimetype << "preferredType=" << preferredType;
+        qClipboardDebug() << "mimetype=" << mimetype << "preferredType=" << preferredType;
         if (is_clipboard_format_present(mimetype.toUtf8().constData()) != 0)
             return QMimeData::retrieveData(mimetype, preferredType);
 
@@ -149,7 +155,7 @@ private Q_SLOTS:
     void releaseOwnership()
     {
         if (m_userMimeData) {
-            qClipboardDebug() << Q_FUNC_INFO << "user data formats=" << m_userMimeData->formats() << "system formats=" << formats();
+            qClipboardDebug() << "user data formats=" << m_userMimeData->formats() << "system formats=" << formats();
             delete m_userMimeData;
             m_userMimeData = 0;
             m_clipboard->emitChanged(QClipboard::Clipboard);
@@ -195,7 +201,7 @@ void QQnxClipboard::setMimeData(QMimeData *data, QClipboard::Mode mode)
     }
 
     const QStringList formats = data->formats();
-    qClipboardDebug() << Q_FUNC_INFO << "formats=" << formats;
+    qClipboardDebug() << "formats=" << formats;
 
     Q_FOREACH (const QString &format, formats) {
         const QByteArray buf = data->data(format);
@@ -204,7 +210,7 @@ void QQnxClipboard::setMimeData(QMimeData *data, QClipboard::Mode mode)
             continue;
 
         int ret = set_clipboard_data(format.toUtf8().data(), buf.size(), buf.data());
-        qClipboardDebug() << Q_FUNC_INFO << "set " << format << "to clipboard, size=" << buf.size() << ";ret=" << ret;
+        qClipboardDebug() << "set " << format << "to clipboard, size=" << buf.size() << ";ret=" << ret;
         if (ret)
             m_mimeData->addFormatToCheck(format);
     }
