@@ -1,7 +1,7 @@
 /***************************************************************************
 **
 ** Copyright (C) 2013 BlackBerry Limited. All rights reserved.
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -199,7 +197,7 @@ static int32_t ic_begin_batch_edit(input_session_t *ic)
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_commit_text(input_session_t *ic, spannable_string_t *text, int32_t new_cursor_position)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfCommitText);
     event.ct.text = text;
@@ -214,7 +212,7 @@ static int32_t ic_commit_text(input_session_t *ic, spannable_string_t *text, int
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_delete_surrounding_text(input_session_t *ic, int32_t left_length, int32_t right_length)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfDeleteSurroundingText);
     event.dst.left_length = left_length;
@@ -238,7 +236,7 @@ static int32_t ic_end_batch_edit(input_session_t *ic)
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_finish_composing_text(input_session_t *ic)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfFinishComposingText);
     event.fct.result = -1;
@@ -251,7 +249,7 @@ static int32_t ic_finish_composing_text(input_session_t *ic)
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_get_cursor_position(input_session_t *ic)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfGetCursorPosition);
     event.gcp.result = -1;
@@ -264,7 +262,7 @@ static int32_t ic_get_cursor_position(input_session_t *ic)
 // See comment at beginning of namespace declaration for general information
 static spannable_string_t *ic_get_text_after_cursor(input_session_t *ic, int32_t n, int32_t flags)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfGetTextAfterCursor);
     event.gtac.n = n;
@@ -279,7 +277,7 @@ static spannable_string_t *ic_get_text_after_cursor(input_session_t *ic, int32_t
 // See comment at beginning of namespace declaration for general information
 static spannable_string_t *ic_get_text_before_cursor(input_session_t *ic, int32_t n, int32_t flags)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfGetTextBeforeCursor);
     event.gtac.n = n;
@@ -294,7 +292,7 @@ static spannable_string_t *ic_get_text_before_cursor(input_session_t *ic, int32_
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_send_event(input_session_t *ic, event_t *event)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest imfEvent(ic, ImfSendEvent);
     imfEvent.sae.event = event;
@@ -308,7 +306,7 @@ static int32_t ic_send_event(input_session_t *ic, event_t *event)
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_send_async_event(input_session_t *ic, event_t *event)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     // There's no difference from our point of view between ic_send_event & ic_send_async_event
     QQnxImfRequest imfEvent(ic, ImfSendEvent);
@@ -323,7 +321,7 @@ static int32_t ic_send_async_event(input_session_t *ic, event_t *event)
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_set_composing_region(input_session_t *ic, int32_t start, int32_t end)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfSetComposingRegion);
     event.scr.start = start;
@@ -339,7 +337,7 @@ static int32_t ic_set_composing_region(input_session_t *ic, int32_t start, int32
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_set_composing_text(input_session_t *ic, spannable_string_t *text, int32_t new_cursor_position)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfSetComposingText);
     event.sct.text = text;
@@ -354,7 +352,7 @@ static int32_t ic_set_composing_text(input_session_t *ic, spannable_string_t *te
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_is_text_selected(input_session_t* ic, int32_t* pIsSelected)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfIsTextSelected);
     event.its.pIsSelected = pIsSelected;
@@ -368,7 +366,7 @@ static int32_t ic_is_text_selected(input_session_t* ic, int32_t* pIsSelected)
 // See comment at beginning of namespace declaration for general information
 static int32_t ic_is_all_text_selected(input_session_t* ic, int32_t* pIsSelected)
 {
-    qInputContextIMFRequestDebug() << Q_FUNC_INFO;
+    qInputContextIMFRequestDebug();
 
     QQnxImfRequest event(ic, ImfIsAllTextSelected);
     event.its.pIsSelected = pIsSelected;
@@ -387,7 +385,7 @@ static int32_t ic_perform_editor_action(input_session_t *ic, int32_t editor_acti
     Q_UNUSED(ic);
     Q_UNUSED(editor_action);
 
-    qCritical() << "ic_perform_editor_action not implemented";
+    qCritical("ic_perform_editor_action not implemented");
     return 0;
 }
 
@@ -397,7 +395,7 @@ static int32_t ic_report_fullscreen_mode(input_session_t *ic, int32_t enabled)
     Q_UNUSED(ic);
     Q_UNUSED(enabled);
 
-    qCritical() << "ic_report_fullscreen_mode not implemented";
+    qCritical("ic_report_fullscreen_mode not implemented");
     return 0;
 }
 
@@ -408,7 +406,7 @@ static extracted_text_t *ic_get_extracted_text(input_session_t *ic, extracted_te
     Q_UNUSED(request);
     Q_UNUSED(flags);
 
-    qCritical() << "ic_get_extracted_text not implemented";
+    qCritical("ic_get_extracted_text not implemented");
     return 0;
 }
 
@@ -418,7 +416,7 @@ static spannable_string_t *ic_get_selected_text(input_session_t *ic, int32_t fla
     Q_UNUSED(ic);
     Q_UNUSED(flags);
 
-    qCritical() << "ic_get_selected_text not implemented";
+    qCritical("ic_get_selected_text not implemented");
     return 0;
 }
 
@@ -428,7 +426,7 @@ static int32_t ic_get_cursor_caps_mode(input_session_t *ic, int32_t req_modes)
     Q_UNUSED(ic);
     Q_UNUSED(req_modes);
 
-    qCritical() << "ic_get_cursor_caps_mode not implemented";
+    qCritical("ic_get_cursor_caps_mode not implemented");
     return 0;
 }
 
@@ -438,7 +436,7 @@ static int32_t ic_clear_meta_key_states(input_session_t *ic, int32_t states)
     Q_UNUSED(ic);
     Q_UNUSED(states);
 
-    qCritical() << "ic_clear_meta_key_states not implemented";
+    qCritical("ic_clear_meta_key_states not implemented");
     return 0;
 }
 
@@ -449,7 +447,7 @@ static int32_t ic_set_selection(input_session_t *ic, int32_t start, int32_t end)
     Q_UNUSED(start);
     Q_UNUSED(end);
 
-    qCritical() << "ic_set_selection not implemented";
+    qCritical("ic_set_selection not implemented");
     return 0;
 }
 
@@ -504,7 +502,7 @@ initEvent(event_t *pEvent, const input_session_t *pSession, EventType eventType,
 
 static spannable_string_t *toSpannableString(const QString &text)
 {
-    qInputContextDebug() << Q_FUNC_INFO << text;
+    qInputContextDebug() << text;
 
     spannable_string_t *pString = static_cast<spannable_string_t *>(malloc(sizeof(spannable_string_t)));
     pString->str =  static_cast<wchar_t *>(malloc(sizeof(wchar_t) * text.length() + 1));
@@ -538,29 +536,28 @@ static bool imfAvailable()
 
     if ( p_imf_client_init == 0 ) {
         void *handle = dlopen("libinput_client.so.1", 0);
-        if ( handle ) {
-            p_imf_client_init = (int32_t (*)()) dlsym(handle, "imf_client_init");
-            p_imf_client_disconnect = (void (*)()) dlsym(handle, "imf_client_disconnect");
-            p_ictrl_open_session = (const input_session_t *(*)(connection_interface_t *))dlsym(handle, "ictrl_open_session");
-            p_ictrl_close_session = (void (*)(input_session_t *))dlsym(handle, "ictrl_close_session");
-            p_ictrl_dispatch_event = (int32_t (*)(event_t *))dlsym(handle, "ictrl_dispatch_event");
-            p_vkb_init_selection_service = (int32_t (*)())dlsym(handle, "vkb_init_selection_service");
-            p_ictrl_get_num_active_sessions = (int32_t (*)())dlsym(handle, "ictrl_get_num_active_sessions");
-        } else {
-            qCritical() << Q_FUNC_INFO << "libinput_client.so.1 is not present - IMF services are disabled.";
+        if (Q_UNLIKELY(!handle)) {
+            qCritical("libinput_client.so.1 is not present - IMF services are disabled.");
             s_imfDisabled = true;
             return false;
         }
+        p_imf_client_init = (int32_t (*)()) dlsym(handle, "imf_client_init");
+        p_imf_client_disconnect = (void (*)()) dlsym(handle, "imf_client_disconnect");
+        p_ictrl_open_session = (const input_session_t *(*)(connection_interface_t *))dlsym(handle, "ictrl_open_session");
+        p_ictrl_close_session = (void (*)(input_session_t *))dlsym(handle, "ictrl_close_session");
+        p_ictrl_dispatch_event = (int32_t (*)(event_t *))dlsym(handle, "ictrl_dispatch_event");
+        p_vkb_init_selection_service = (int32_t (*)())dlsym(handle, "vkb_init_selection_service");
+        p_ictrl_get_num_active_sessions = (int32_t (*)())dlsym(handle, "ictrl_get_num_active_sessions");
 
-        if ( p_imf_client_init && p_ictrl_open_session && p_ictrl_dispatch_event ) {
-            s_imfReady = true;
-        } else {
+        if (Q_UNLIKELY(!p_imf_client_init || !p_ictrl_open_session || !p_ictrl_dispatch_event)) {
             p_ictrl_open_session = 0;
             p_ictrl_dispatch_event = 0;
             s_imfDisabled = true;
-            qCritical() << Q_FUNC_INFO << "libinput_client.so.1 did not contain the correct symbols, library mismatch? IMF services are disabled.";
+            qCritical("libinput_client.so.1 did not contain the correct symbols, library mismatch? IMF services are disabled.");
             return false;
         }
+
+        s_imfReady = true;
     }
 
     return s_imfReady;
@@ -579,7 +576,7 @@ QQnxInputContext::QQnxInputContext(QQnxIntegration *integration, QQnxAbstractVir
          m_integration(integration),
          m_virtualKeyboard(keyboard)
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     if (!imfAvailable())
         return;
@@ -589,7 +586,7 @@ QQnxInputContext::QQnxInputContext(QQnxIntegration *integration, QQnxAbstractVir
     Q_ASSERT(sInputContextInstance == 0);
     sInputContextInstance = this;
 
-    if (p_imf_client_init() != 0) {
+    if (Q_UNLIKELY(p_imf_client_init() != 0)) {
         s_imfInitFailed = true;
         qCritical("imf_client_init failed - IMF services will be unavailable");
     }
@@ -602,7 +599,7 @@ QQnxInputContext::QQnxInputContext(QQnxIntegration *integration, QQnxAbstractVir
 
 QQnxInputContext::~QQnxInputContext()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     Q_ASSERT(sInputContextInstance == this);
     sInputContextInstance = 0;
@@ -677,7 +674,7 @@ void QQnxInputContext::processImfEvent(QQnxImfRequest *imfEvent)
 
 bool QQnxInputContext::filterEvent( const QEvent *event )
 {
-    qInputContextDebug() << Q_FUNC_INFO << event;
+    qInputContextDebug() << event;
 
     switch (event->type()) {
     case QEvent::CloseSoftwareInputPanel:
@@ -700,19 +697,19 @@ QRectF QQnxInputContext::keyboardRect() const
 
 void QQnxInputContext::reset()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
     endComposition();
 }
 
 void QQnxInputContext::commit()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
     endComposition();
 }
 
 void QQnxInputContext::update(Qt::InputMethodQueries queries)
 {
-    qInputContextDebug() << Q_FUNC_INFO << queries;
+    qInputContextDebug() << queries;
 
     if (queries & Qt::ImCursorPosition) {
         int lastCaret = m_caretPosition;
@@ -724,7 +721,7 @@ void QQnxInputContext::update(Qt::InputMethodQueries queries)
             initEvent(&caretEvent.event, sInputSession, EVENT_CARET, CARET_POS_CHANGED, sizeof(caretEvent));
             caretEvent.old_pos = lastCaret;
             caretEvent.new_pos = m_caretPosition;
-            qInputContextDebug() << Q_FUNC_INFO << "ictrl_dispatch_event caret changed" << lastCaret << m_caretPosition;
+            qInputContextDebug() << "ictrl_dispatch_event caret changed" << lastCaret << m_caretPosition;
             p_ictrl_dispatch_event(&caretEvent.event);
         }
     }
@@ -732,7 +729,7 @@ void QQnxInputContext::update(Qt::InputMethodQueries queries)
 
 void QQnxInputContext::closeSession()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
     if (!imfAvailable())
         return;
 
@@ -754,7 +751,7 @@ bool QQnxInputContext::openSession()
     closeSession();
     sInputSession = p_ictrl_open_session(&ic_funcs);
 
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     return sInputSession != 0;
 }
@@ -778,7 +775,7 @@ bool QQnxInputContext::hasSelectedText()
 
 bool QQnxInputContext::dispatchRequestSoftwareInputPanel()
 {
-    qInputContextDebug() << Q_FUNC_INFO << "requesting keyboard" << m_inputPanelVisible;
+    qInputContextDebug() << "requesting keyboard" << m_inputPanelVisible;
     m_virtualKeyboard.showKeyboard();
 
     return true;
@@ -786,7 +783,7 @@ bool QQnxInputContext::dispatchRequestSoftwareInputPanel()
 
 bool QQnxInputContext::dispatchCloseSoftwareInputPanel()
 {
-    qInputContextDebug() << Q_FUNC_INFO << "hiding keyboard" << m_inputPanelVisible;
+    qInputContextDebug() << "hiding keyboard" << m_inputPanelVisible;
     m_virtualKeyboard.hideKeyboard();
 
     return true;
@@ -832,7 +829,7 @@ bool QQnxInputContext::dispatchFocusGainEvent(int inputHints)
         focusEvent.style |= IMF_EMAIL_TYPE;
     }
 
-    qInputContextDebug() << Q_FUNC_INFO << "ictrl_dispatch_event focus gain style:" << focusEvent.style;
+    qInputContextDebug() << "ictrl_dispatch_event focus gain style:" << focusEvent.style;
 
     p_ictrl_dispatch_event((event_t *)&focusEvent);
 
@@ -842,7 +839,7 @@ bool QQnxInputContext::dispatchFocusGainEvent(int inputHints)
 void QQnxInputContext::dispatchFocusLossEvent()
 {
     if (hasSession()) {
-        qInputContextDebug() << Q_FUNC_INFO << "ictrl_dispatch_event focus lost";
+        qInputContextDebug("ictrl_dispatch_event focus lost");
 
         focus_event_t focusEvent;
         initEvent(&focusEvent.event, sInputSession, EVENT_FOCUS, FOCUS_LOST, sizeof(focusEvent));
@@ -917,7 +914,7 @@ bool QQnxInputContext::handleKeyboardEvent(int flags, int sym, int mod, int scan
             navigation_event_t navEvent;
             initEvent(&navEvent.event, sInputSession, EVENT_NAVIGATION, key, sizeof(navEvent));
             navEvent.magnitude = 1;
-            qInputContextDebug() << Q_FUNC_INFO << "ictrl_dispatch_even navigation" << key;
+            qInputContextDebug() << "ictrl_dispatch_even navigation" << key;
             p_ictrl_dispatch_event(&navEvent.event);
         }
     } else {
@@ -930,7 +927,7 @@ bool QQnxInputContext::handleKeyboardEvent(int flags, int sym, int mod, int scan
         keyEvent.sequence_id = sequenceId;
 
         p_ictrl_dispatch_event(&keyEvent.event);
-        qInputContextDebug() << Q_FUNC_INFO << "ictrl_dispatch_even key" << key;
+        qInputContextDebug() << "ictrl_dispatch_even key" << key;
     }
 
     return true;
@@ -946,7 +943,7 @@ void QQnxInputContext::updateCursorPosition()
     QCoreApplication::sendEvent(input, &query);
     m_caretPosition = query.value(Qt::ImCursorPosition).toInt();
 
-    qInputContextDebug() << Q_FUNC_INFO << m_caretPosition;
+    qInputContextDebug() << m_caretPosition;
 }
 
 void QQnxInputContext::endComposition()
@@ -959,7 +956,7 @@ void QQnxInputContext::endComposition()
     if (hasSession()) {
         action_event_t actionEvent;
         initEvent(&actionEvent.event, sInputSession, EVENT_ACTION, ACTION_END_COMPOSITION, sizeof(actionEvent));
-        qInputContextDebug() << Q_FUNC_INFO << "ictrl_dispatch_even end composition";
+        qInputContextDebug("ictrl_dispatch_even end composition");
         p_ictrl_dispatch_event(&actionEvent.event);
     }
 }
@@ -976,7 +973,7 @@ void QQnxInputContext::updateComposition(spannable_string_t *text, int32_t new_c
     m_composingText = QString::fromWCharArray(text->str, text->length);
     m_isComposing = true;
 
-    qInputContextDebug() << Q_FUNC_INFO << m_composingText << new_cursor_position;
+    qInputContextDebug() << m_composingText << new_cursor_position;
 
     QList<QInputMethodEvent::Attribute> attributes;
     attributes.append(QInputMethodEvent::Attribute(QInputMethodEvent::Cursor,
@@ -1025,7 +1022,7 @@ void QQnxInputContext::finishComposingText()
     QObject *input = qGuiApp->focusObject();
 
     if (input) {
-        qInputContextDebug() << Q_FUNC_INFO << m_composingText;
+        qInputContextDebug() << m_composingText;
 
         QInputMethodEvent event;
         event.setCommitString(m_composingText);
@@ -1090,13 +1087,13 @@ int32_t QQnxInputContext::processEvent(event_t *event)
     int32_t result = -1;
     switch (event->event_type) {
     case EVENT_SPELL_CHECK: {
-        qInputContextDebug() << Q_FUNC_INFO << "EVENT_SPELL_CHECK";
+        qInputContextDebug("EVENT_SPELL_CHECK");
         result = handleSpellCheck(reinterpret_cast<spell_check_event_t *>(event));
         break;
     }
 
     case EVENT_NAVIGATION: {
-        qInputContextDebug() << Q_FUNC_INFO << "EVENT_NAVIGATION";
+        qInputContextDebug("EVENT_NAVIGATION");
 
         int key = event->event_id == NAVIGATE_UP ? KEYCODE_UP :
             event->event_id == NAVIGATE_DOWN ? KEYCODE_DOWN :
@@ -1119,7 +1116,7 @@ int32_t QQnxInputContext::processEvent(event_t *event)
         int flags = KEY_SYM_VALID | KEY_CAP_VALID;
         if (event->event_id == IMF_KEY_DOWN)
             flags |= KEY_DOWN;
-        qInputContextDebug() << Q_FUNC_INFO << "EVENT_KEY" << flags << keySym;
+        qInputContextDebug() << "EVENT_KEY" << flags << keySym;
         QQnxScreenEventHandler::injectKeyboardEvent(flags, keySym, modifiers, 0, keyCap);
         result = 0;
         break;
@@ -1135,10 +1132,10 @@ int32_t QQnxInputContext::processEvent(event_t *event)
     case EVENT_USER_ACTION:
     case EVENT_STROKE:
     case EVENT_INVOKE_LATER:
-        qCritical() << Q_FUNC_INFO << "Unsupported event type: " << event->event_type;
+        qCritical() << "Unsupported event type: " << event->event_type;
         break;
     default:
-        qCritical() << Q_FUNC_INFO << "Unknown event type: " << event->event_type;
+        qCritical() << "Unknown event type: " << event->event_type;
     }
     return result;
 }
@@ -1159,7 +1156,7 @@ int32_t QQnxInputContext::onCommitText(spannable_string_t *text, int32_t new_cur
 
 int32_t QQnxInputContext::onDeleteSurroundingText(int32_t left_length, int32_t right_length)
 {
-    qInputContextDebug() << Q_FUNC_INFO << "L:" << left_length << " R:" << right_length;
+    qInputContextDebug() << "L:" << left_length << " R:" << right_length;
 
     QObject *input = qGuiApp->focusObject();
     if (!input)
@@ -1190,7 +1187,7 @@ int32_t QQnxInputContext::onFinishComposingText()
 
 int32_t QQnxInputContext::onGetCursorPosition()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     QObject *input = qGuiApp->focusObject();
     if (!input)
@@ -1204,7 +1201,7 @@ int32_t QQnxInputContext::onGetCursorPosition()
 spannable_string_t *QQnxInputContext::onGetTextAfterCursor(int32_t n, int32_t flags)
 {
     Q_UNUSED(flags);
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     QObject *input = qGuiApp->focusObject();
     if (!input)
@@ -1221,7 +1218,7 @@ spannable_string_t *QQnxInputContext::onGetTextAfterCursor(int32_t n, int32_t fl
 spannable_string_t *QQnxInputContext::onGetTextBeforeCursor(int32_t n, int32_t flags)
 {
     Q_UNUSED(flags);
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     QObject *input = qGuiApp->focusObject();
     if (!input)
@@ -1240,7 +1237,7 @@ spannable_string_t *QQnxInputContext::onGetTextBeforeCursor(int32_t n, int32_t f
 
 int32_t QQnxInputContext::onSendEvent(event_t *event)
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
 
     return processEvent(event);
 }
@@ -1256,7 +1253,7 @@ int32_t QQnxInputContext::onSetComposingRegion(int32_t start, int32_t end)
     QString text = query.value(Qt::ImSurroundingText).toString();
     m_caretPosition = query.value(Qt::ImCursorPosition).toInt();
 
-    qInputContextDebug() << Q_FUNC_INFO << text;
+    qInputContextDebug() << text;
 
     m_isUpdatingText = true;
 
@@ -1299,7 +1296,7 @@ int32_t QQnxInputContext::onIsTextSelected(int32_t* pIsSelected)
 {
     *pIsSelected = hasSelectedText();
 
-    qInputContextDebug() << Q_FUNC_INFO << *pIsSelected;
+    qInputContextDebug() << *pIsSelected;
 
     return 0;
 }
@@ -1315,20 +1312,20 @@ int32_t QQnxInputContext::onIsAllTextSelected(int32_t* pIsSelected)
 
     *pIsSelected = query.value(Qt::ImSurroundingText).toString().length() == query.value(Qt::ImCurrentSelection).toString().length();
 
-    qInputContextDebug() << Q_FUNC_INFO << *pIsSelected;
+    qInputContextDebug() << *pIsSelected;
 
     return 0;
 }
 
 void QQnxInputContext::showInputPanel()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
     dispatchRequestSoftwareInputPanel();
 }
 
 void QQnxInputContext::hideInputPanel()
 {
-    qInputContextDebug() << Q_FUNC_INFO;
+    qInputContextDebug();
     dispatchCloseSoftwareInputPanel();
 }
 
@@ -1344,7 +1341,7 @@ QLocale QQnxInputContext::locale() const
 
 void QQnxInputContext::keyboardVisibilityChanged(bool visible)
 {
-    qInputContextDebug() << Q_FUNC_INFO << "visible=" << visible;
+    qInputContextDebug() << "visible=" << visible;
     if (m_inputPanelVisible != visible) {
         m_inputPanelVisible = visible;
         emitInputPanelVisibleChanged();
@@ -1353,7 +1350,7 @@ void QQnxInputContext::keyboardVisibilityChanged(bool visible)
 
 void QQnxInputContext::keyboardLocaleChanged(const QLocale &locale)
 {
-    qInputContextDebug() << Q_FUNC_INFO << "locale=" << locale;
+    qInputContextDebug() << "locale=" << locale;
     if (m_inputPanelLocale != locale) {
         m_inputPanelLocale = locale;
         emitLocaleChanged();
@@ -1362,7 +1359,7 @@ void QQnxInputContext::keyboardLocaleChanged(const QLocale &locale)
 
 void QQnxInputContext::setHighlightColor(int index, const QColor &color)
 {
-    qInputContextDebug() << Q_FUNC_INFO << "setHighlightColor" << index << color << qGuiApp->focusObject();
+    qInputContextDebug() << "setHighlightColor" << index << color << qGuiApp->focusObject();
 
     if (!sInputContextInstance)
         return;
@@ -1381,7 +1378,7 @@ void QQnxInputContext::setHighlightColor(int index, const QColor &color)
 
 void QQnxInputContext::setFocusObject(QObject *object)
 {
-    qInputContextDebug() << Q_FUNC_INFO << "input item=" << object;
+    qInputContextDebug() << "input item=" << object;
 
     // Ensure the colors are reset if we've a change in focus object
     setHighlightColor(-1, QColor());
@@ -1392,13 +1389,17 @@ void QQnxInputContext::setFocusObject(QObject *object)
         if (hasSession())
             dispatchFocusLossEvent();
     } else {
-        QInputMethodQueryEvent query(Qt::ImHints);
+        QInputMethodQueryEvent query(Qt::ImHints | Qt::ImEnterKeyType);
         QCoreApplication::sendEvent(object, &query);
         int inputHints = query.value(Qt::ImHints).toInt();
+        Qt::EnterKeyType qtEnterKeyType = Qt::EnterKeyType(query.value(Qt::ImEnterKeyType).toInt());
 
         dispatchFocusGainEvent(inputHints);
 
         m_virtualKeyboard.setInputHints(inputHints);
+        m_virtualKeyboard.setEnterKeyType(
+            QQnxAbstractVirtualKeyboard::qtEnterKeyTypeToQnx(qtEnterKeyType)
+        );
 
         if (!m_inputPanelVisible)
             showInputPanel();
@@ -1407,7 +1408,7 @@ void QQnxInputContext::setFocusObject(QObject *object)
 
 bool QQnxInputContext::checkSpelling(const QString &text, void *context, void (*spellCheckDone)(void *context, const QString &text, const QList<int> &indices))
 {
-    qInputContextDebug() << Q_FUNC_INFO << "text" << text;
+    qInputContextDebug() << "text" << text;
 
     if (!imfAvailable())
         return false;
