@@ -1,31 +1,37 @@
 /***************************************************************************
 **
 ** Copyright (C) 2011 - 2012 Research In Motion
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -74,18 +80,18 @@ QQnxNavigatorEventNotifier::~QQnxNavigatorEventNotifier()
     if (m_fd != -1)
         close(m_fd);
 
-    qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "navigator event notifier stopped";
+    qNavigatorEventNotifierDebug("navigator event notifier stopped");
 }
 
 void QQnxNavigatorEventNotifier::start()
 {
-    qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "navigator event notifier started";
+    qNavigatorEventNotifierDebug("navigator event notifier started");
 
     // open connection to navigator
     errno = 0;
     m_fd = open(navigatorControlPath, O_RDWR);
     if (m_fd == -1) {
-        qNavigatorEventNotifierDebug() << Q_FUNC_INFO << ": failed to open navigator pps:"
+        qNavigatorEventNotifierDebug() << "failed to open navigator pps:"
                                                       << strerror(errno);
         return;
     }
@@ -96,13 +102,13 @@ void QQnxNavigatorEventNotifier::start()
 
 void QQnxNavigatorEventNotifier::parsePPS(const QByteArray &ppsData, QByteArray &msg, QByteArray &dat, QByteArray &id)
 {
-    qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "data=" << ppsData;
+    qNavigatorEventNotifierDebug() << "data=" << ppsData;
 
     // tokenize pps data into lines
     QList<QByteArray> lines = ppsData.split('\n');
 
     // validate pps object
-    if (lines.size() == 0 || lines.at(0) != "@control")
+    if (Q_UNLIKELY(lines.empty() || lines.at(0) != "@control"))
         qFatal("QQNX: unrecognized pps object, data=%s", ppsData.constData());
 
     // parse pps object attributes and extract values
@@ -110,7 +116,7 @@ void QQnxNavigatorEventNotifier::parsePPS(const QByteArray &ppsData, QByteArray
 
         // tokenize current attribute
         const QByteArray &attr = lines.at(i);
-        qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "attr=" << attr;
+        qNavigatorEventNotifierDebug() << "attr=" << attr;
 
         int firstColon = attr.indexOf(':');
         if (firstColon == -1) {
@@ -127,8 +133,8 @@ void QQnxNavigatorEventNotifier::parsePPS(const QByteArray &ppsData, QByteArray
         QByteArray key = attr.left(firstColon);
         QByteArray value = attr.mid(secondColon + 1);
 
-        qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "key=" << key;
-        qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "val=" << value;
+        qNavigatorEventNotifierDebug() << "key=" << key;
+        qNavigatorEventNotifierDebug() << "val=" << value;
 
         // save attribute value
         if (key == "msg")
@@ -155,18 +161,18 @@ void QQnxNavigatorEventNotifier::replyPPS(const QByteArray &res, const QByteArra
     }
     ppsData += "\n";
 
-    qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "reply=" << ppsData;
+    qNavigatorEventNotifierDebug() << "reply=" << ppsData;
 
     // send pps message to navigator
     errno = 0;
     int bytes = write(m_fd, ppsData.constData(), ppsData.size());
-    if (bytes == -1)
+    if (Q_UNLIKELY(bytes == -1))
         qFatal("QQNX: failed to write navigator pps, errno=%d", errno);
 }
 
 void QQnxNavigatorEventNotifier::handleMessage(const QByteArray &msg, const QByteArray &dat, const QByteArray &id)
 {
-    qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "msg=" << msg << ", dat=" << dat << ", id=" << id;
+    qNavigatorEventNotifierDebug() << "msg=" << msg << ", dat=" << dat << ", id=" << id;
 
     // check message type
     if (msg == "orientationCheck") {
@@ -190,7 +196,7 @@ void QQnxNavigatorEventNotifier::handleMessage(const QByteArray &msg, const QByt
 
 void QQnxNavigatorEventNotifier::readData()
 {
-    qNavigatorEventNotifierDebug() << Q_FUNC_INFO << "reading navigator data";
+    qNavigatorEventNotifierDebug("reading navigator data");
 
     // allocate buffer for pps data
     char buffer[ppsBufferSize];
@@ -198,7 +204,7 @@ void QQnxNavigatorEventNotifier::readData()
     // attempt to read pps data
     errno = 0;
     int bytes = qt_safe_read(m_fd, buffer, ppsBufferSize - 1);
-    if (bytes == -1)
+    if (Q_UNLIKELY(bytes == -1))
         qFatal("QQNX: failed to read navigator pps, errno=%d", errno);
 
     // check if pps data was received
