@@ -1,31 +1,37 @@
 /***************************************************************************
 **
 ** Copyright (C) 2011 - 2013 BlackBerry Limited. All rights reserved.
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -112,7 +118,7 @@ static QSize determineScreenSize(screen_display_t display, bool primaryScreen) {
 #endif
 }
 
-static QQnxWindow *findMultimediaWindow(const QList<QQnxWindow*> windows,
+static QQnxWindow *findMultimediaWindow(const QList<QQnxWindow*> &windows,
                                                     const QByteArray &mmWindowId)
 {
     Q_FOREACH (QQnxWindow *sibling, windows) {
@@ -128,7 +134,7 @@ static QQnxWindow *findMultimediaWindow(const QList<QQnxWindow*> windows,
     return 0;
 }
 
-static QQnxWindow *findMultimediaWindow(const QList<QQnxWindow*> windows,
+static QQnxWindow *findMultimediaWindow(const QList<QQnxWindow*> &windows,
                                                     screen_window_t mmWindowId)
 {
     Q_FOREACH (QQnxWindow *sibling, windows) {
@@ -154,7 +160,7 @@ QQnxScreen::QQnxScreen(screen_context_t screenContext, screen_display_t display,
       m_coverWindow(0),
       m_cursor(new QQnxCursor())
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
     // Cache initial orientation of this display
     int result = screen_get_display_property_iv(m_display, SCREEN_PROPERTY_ROTATION,
                                                 &m_initialRotation);
@@ -191,7 +197,7 @@ QQnxScreen::QQnxScreen(screen_context_t screenContext, screen_display_t display,
 
 QQnxScreen::~QQnxScreen()
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
     Q_FOREACH (QQnxWindow *childWindow, m_childWindows)
         childWindow->setScreen(0);
 
@@ -300,12 +306,12 @@ QPixmap QQnxScreen::grabWindow(WId window, int x, int y, int width, int height)
 
 static int defaultDepth()
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
     static int defaultDepth = 0;
     if (defaultDepth == 0) {
         // check if display depth was specified in environment variable;
         // use default value if no valid value found
-        defaultDepth = qgetenv("QQNX_DISPLAY_DEPTH").toInt();
+        defaultDepth = qEnvironmentVariableIntValue("QQNX_DISPLAY_DEPTH");
         if (defaultDepth != 16 && defaultDepth != 32)
             defaultDepth = 32;
     }
@@ -314,7 +320,7 @@ static int defaultDepth()
 
 QRect QQnxScreen::availableGeometry() const
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
     // available geometry = total geometry - keyboard
     return QRect(m_currentGeometry.x(), m_currentGeometry.y(),
                  m_currentGeometry.width(), m_currentGeometry.height() - m_keyboardHeight);
@@ -334,7 +340,7 @@ qreal QQnxScreen::refreshRate() const
         qWarning("QQnxScreen: Failed to query screen mode. Using default value of 60Hz");
         return 60.0;
     }
-    qScreenDebug() << Q_FUNC_INFO << "screen mode:" << endl
+    qScreenDebug() << "screen mode:" << endl
                    << "      width =" << displayMode.width << endl
                    << "     height =" << displayMode.height << endl
                    << "    refresh =" << displayMode.refresh << endl
@@ -372,7 +378,7 @@ Qt::ScreenOrientation QQnxScreen::orientation() const
         else
             orient = Qt::InvertedLandscapeOrientation;
     }
-    qScreenDebug() << Q_FUNC_INFO << "orientation =" << orient;
+    qScreenDebug() << "orientation =" << orient;
     return orient;
 }
 
@@ -398,7 +404,7 @@ static bool isOrthogonal(int angle1, int angle2)
 
 void QQnxScreen::setRotation(int rotation)
 {
-    qScreenDebug() << Q_FUNC_INFO << "orientation =" << rotation;
+    qScreenDebug() << "orientation =" << rotation;
     // Check if rotation changed
     // We only want to rotate if we are the primary screen
     if (m_currentRotation != rotation && isPrimaryScreen()) {
@@ -419,7 +425,7 @@ void QQnxScreen::setRotation(int rotation)
 
         // Resize root window if we've rotated 90 or 270 from previous orientation
         if (isOrthogonal(m_currentRotation, rotation)) {
-            qScreenDebug() << Q_FUNC_INFO << "resize, size =" << m_currentGeometry.size();
+            qScreenDebug() << "resize, size =" << m_currentGeometry.size();
             if (rootWindow())
                 rootWindow()->setGeometry(QRect(QPoint(0,0), m_currentGeometry.size()));
 
@@ -566,7 +572,7 @@ QQnxWindow *QQnxScreen::findWindow(screen_window_t windowHandle) const
 
 void QQnxScreen::addWindow(QQnxWindow *window)
 {
-    qScreenDebug() << Q_FUNC_INFO << "window =" << window;
+    qScreenDebug() << "window =" << window;
 
     if (m_childWindows.contains(window))
         return;
@@ -584,16 +590,12 @@ void QQnxScreen::addWindow(QQnxWindow *window)
         else
             m_childWindows.push_back(window);
         updateHierarchy();
-    } else {
-#if defined(Q_OS_BLACKBERRY) && !defined(Q_OS_BLACKBERRY_TABLET)
-        m_coverWindow = window;
-#endif
     }
 }
 
 void QQnxScreen::removeWindow(QQnxWindow *window)
 {
-    qScreenDebug() << Q_FUNC_INFO << "window =" << window;
+    qScreenDebug() << "window =" << window;
 
     if (window != m_coverWindow) {
         const int numWindowsRemoved = m_childWindows.removeAll(window);
@@ -608,7 +610,7 @@ void QQnxScreen::removeWindow(QQnxWindow *window)
 
 void QQnxScreen::raiseWindow(QQnxWindow *window)
 {
-    qScreenDebug() << Q_FUNC_INFO << "window =" << window;
+    qScreenDebug() << "window =" << window;
 
     if (window != m_coverWindow) {
         removeWindow(window);
@@ -618,7 +620,7 @@ void QQnxScreen::raiseWindow(QQnxWindow *window)
 
 void QQnxScreen::lowerWindow(QQnxWindow *window)
 {
-    qScreenDebug() << Q_FUNC_INFO << "window =" << window;
+    qScreenDebug() << "window =" << window;
 
     if (window != m_coverWindow) {
         removeWindow(window);
@@ -628,7 +630,7 @@ void QQnxScreen::lowerWindow(QQnxWindow *window)
 
 void QQnxScreen::updateHierarchy()
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
 
     QList<QQnxWindow*>::const_iterator it;
     int result;
@@ -680,7 +682,7 @@ void QQnxScreen::adjustOrientation()
         return;
 
     bool ok = false;
-    const int rotation = qgetenv("ORIENTATION").toInt(&ok);
+    const int rotation = qEnvironmentVariableIntValue("ORIENTATION", &ok);
 
     if (ok)
         setRotation(rotation);
@@ -804,7 +806,7 @@ void QQnxScreen::windowClosed(void *window)
 
 void QQnxScreen::windowGroupStateChanged(const QByteArray &id, Qt::WindowState state)
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
 
     if (!rootWindow() || id != rootWindow()->groupName())
         return;
@@ -819,7 +821,7 @@ void QQnxScreen::windowGroupStateChanged(const QByteArray &id, Qt::WindowState s
 
 void QQnxScreen::activateWindowGroup(const QByteArray &id)
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
 
     if (!rootWindow() || id != rootWindow()->groupName())
         return;
@@ -838,7 +840,7 @@ void QQnxScreen::activateWindowGroup(const QByteArray &id)
 
 void QQnxScreen::deactivateWindowGroup(const QByteArray &id)
 {
-    qScreenDebug() << Q_FUNC_INFO;
+    qScreenDebug();
 
     if (!rootWindow() || id != rootWindow()->groupName())
         return;
@@ -859,7 +861,7 @@ void QQnxScreen::setRootWindow(QQnxWindow *window)
 {
     // Optionally disable the screen power save
     bool ok = false;
-    const int disablePowerSave = qgetenv("QQNX_DISABLE_POWER_SAVE").toInt(&ok);
+    const int disablePowerSave = qEnvironmentVariableIntValue("QQNX_DISABLE_POWER_SAVE", &ok);
     if (ok && disablePowerSave) {
         const int mode = SCREEN_IDLE_MODE_KEEP_AWAKE;
         int result = screen_set_window_property_iv(window->nativeHandle(), SCREEN_PROPERTY_IDLE_MODE, &mode);
