@@ -1,7 +1,7 @@
 /***************************************************************************
 **
-** Copyright (C) 2011 - 2012 Research In Motion
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2013 BlackBerry Limited. All rights reserved.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -49,6 +47,7 @@
 QT_BEGIN_NAMESPACE
 
 class QQnxIntegration;
+class QQnxScreenEventFilter;
 #if defined(QQNX_SCREENEVENTTHREAD)
 class QQnxScreenEventThread;
 #endif
@@ -59,6 +58,9 @@ class QQnxScreenEventHandler : public QObject
 public:
     explicit QQnxScreenEventHandler(QQnxIntegration *integration);
 
+    void addScreenEventFilter(QQnxScreenEventFilter *filter);
+    void removeScreenEventFilter(QQnxScreenEventFilter *filter);
+
     bool handleEvent(screen_event_t event);
     bool handleEvent(screen_event_t event, int qnxType);
 
@@ -72,6 +74,9 @@ Q_SIGNALS:
     void newWindowCreated(void *window);
     void windowClosed(void *window);
 
+protected:
+    void timerEvent(QTimerEvent *event);
+
 #if defined(QQNX_SCREENEVENTTHREAD)
 private Q_SLOTS:
     void processEventsFromScreenThread();
@@ -99,9 +104,11 @@ private:
     screen_window_t m_lastMouseWindow;
     QTouchDevice *m_touchDevice;
     QWindowSystemInterface::TouchPoint m_touchPoints[MaximumTouchPoints];
+    QList<QQnxScreenEventFilter*> m_eventFilters;
 #if defined(QQNX_SCREENEVENTTHREAD)
     QQnxScreenEventThread *m_eventThread;
 #endif
+    int m_focusLostTimer;
 };
 
 QT_END_NAMESPACE
