@@ -1,8 +1,8 @@
 /****************************************************************************
 **
 ** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -11,30 +11,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -65,7 +63,8 @@ enum
     TouchEventFlag = 0x400000,
     ClipboardEventFlag = 0x800000,
     ApplicationEventFlag = 0x1000000,
-    ThemingEventFlag = 0x2000000
+    ThemingEventFlag = 0x2000000,
+    GenericEventFlag = 0x4000000, // Misc
 };
 
 enum WindowsEventType // Simplify event types
@@ -73,12 +72,15 @@ enum WindowsEventType // Simplify event types
     ExposeEvent = WindowEventFlag + 1,
     ActivateWindowEvent = WindowEventFlag + 2,
     DeactivateWindowEvent = WindowEventFlag + 3,
+    MouseActivateWindowEvent = WindowEventFlag + 4,
     LeaveEvent = WindowEventFlag + 5,
     CloseEvent = WindowEventFlag + 6,
     ShowEvent = WindowEventFlag + 7,
+    ShowEventOnParentRestoring = WindowEventFlag + 20,
     HideEvent = WindowEventFlag + 8,
     DestroyEvent = WindowEventFlag + 9,
-    MoveEvent = WindowEventFlag + 10,
+    GeometryChangingEvent = WindowEventFlag + 10,
+    MoveEvent = WindowEventFlag + 11,
     ResizeEvent = WindowEventFlag + 12,
     QuerySizeHints = WindowEventFlag + 15,
     CalculateSize = WindowEventFlag + 16,
@@ -102,6 +104,7 @@ enum WindowsEventType // Simplify event types
     AccessibleObjectFromWindowRequest = ApplicationEventFlag + 3,
     QueryEndSessionApplicationEvent = ApplicationEventFlag + 4,
     EndSessionApplicationEvent = ApplicationEventFlag + 5,
+    AppCommandEvent = ApplicationEventFlag + 6,
     InputMethodStartCompositionEvent = InputMethodEventFlag + 1,
     InputMethodCompositionEvent = InputMethodEventFlag + 2,
     InputMethodEndCompositionEvent = InputMethodEventFlag + 3,
@@ -112,13 +115,23 @@ enum WindowsEventType // Simplify event types
     CompositionSettingsChanged = ThemingEventFlag + 2,
     DisplayChangedEvent = 437,
     SettingChangedEvent = DisplayChangedEvent + 1,
+    ScrollEvent = GenericEventFlag + 1,
     ContextMenu = 123,
+    GestureEvent = 124,
     UnknownEvent = 542
 };
 
+// Matches Process_DPI_Awareness (Windows 8.1 onwards), used for SetProcessDpiAwareness()
+enum ProcessDpiAwareness
+{
+    ProcessDpiUnaware,
+    ProcessSystemDpiAware,
+    ProcessPerMonitorDpiAware
+};
+
 } // namespace QtWindows
 
-inline QtWindows::WindowsEventType windowsEventType(UINT message, WPARAM wParamIn)
+inline QtWindows::WindowsEventType windowsEventType(UINT message, WPARAM wParamIn, LPARAM lParamIn)
 {
     switch (message) {
     case WM_PAINT:
@@ -131,6 +144,8 @@ inline QtWindows::WindowsEventType windowsEventType(UINT message, WPARAM wParamI
     case WM_ACTIVATEAPP:
         return (int)wParamIn ?
         QtWindows::ActivateApplicationEvent : QtWindows::DeactivateApplicationEvent;
+    case WM_MOUSEACTIVATE:
+        return QtWindows::MouseActivateWindowEvent;
     case WM_ACTIVATE:
         return  LOWORD(wParamIn) == WA_INACTIVE ?
             QtWindows::DeactivateWindowEvent : QtWindows::ActivateWindowEvent;
@@ -138,13 +153,21 @@ inline QtWindows::WindowsEventType windowsEventType(UINT message, WPARAM wParamI
         return QtWindows::CursorEvent;
     case WM_MOUSELEAVE:
         return QtWindows::MouseEvent;
+    case WM_HSCROLL:
+        return QtWindows::ScrollEvent;
     case WM_MOUSEWHEEL:
     case WM_MOUSEHWHEEL:
         return QtWindows::MouseWheelEvent;
+#ifndef Q_OS_WINCE
+    case WM_WINDOWPOSCHANGING:
+        return QtWindows::GeometryChangingEvent;
+#endif
     case WM_MOVE:
         return QtWindows::MoveEvent;
     case WM_SHOWWINDOW:
-        return wParamIn ? QtWindows::ShowEvent : QtWindows::HideEvent;
+        if (wParamIn)
+            return lParamIn == SW_PARENTOPENING ? QtWindows::ShowEventOnParentRestoring : QtWindows::ShowEvent;
+        return QtWindows::HideEvent;
     case WM_SIZE:
         return QtWindows::ResizeEvent;
     case WM_NCCALCSIZE:
@@ -231,6 +254,12 @@ inline QtWindows::WindowsEventType windowsEventType(UINT message, WPARAM wParamI
     case WM_ENDSESSION:
         return QtWindows::EndSessionApplicationEvent;
 #endif
+#if defined(WM_APPCOMMAND)
+    case WM_APPCOMMAND:
+        return QtWindows::AppCommandEvent;
+#endif
+    case WM_GESTURE:
+        return QtWindows::GestureEvent;
     default:
         break;
     }
