@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -35,10 +41,11 @@
 #include "qwindowswindow.h"
 #include "qwindowsnativeimage.h"
 #include "qwindowscontext.h"
-#include "qwindowsscaling.h"
 
 #include <QtGui/QWindow>
 #include <QtGui/QPainter>
+#include <private/qhighdpiscaling_p.h>
+#include <private/qimage_p.h>
 
 #include <QtCore/QDebug>
 
@@ -52,7 +59,8 @@ QT_BEGIN_NAMESPACE
 */
 
 QWindowsBackingStore::QWindowsBackingStore(QWindow *window) :
-    QPlatformBackingStore(window)
+    QPlatformBackingStore(window),
+    m_alphaNeedsFill(false)
 {
     qCDebug(lcQpaBackingStore) << __FUNCTION__ << this << window;
 }
@@ -68,33 +76,41 @@ QPaintDevice *QWindowsBackingStore::paintDevice()
     return &m_image->image();
 }
 
-void QWindowsBackingStore::flushDp(QWindow *window, const QRect &br, const QPoint &offset)
+void QWindowsBackingStore::flush(QWindow *window, const QRegion &region,
+                                 const QPoint &offset)
 {
     Q_ASSERT(window);
 
+    const QRect br = region.boundingRect();
     if (QWindowsContext::verbose > 1)
         qCDebug(lcQpaBackingStore) << __FUNCTION__ << this << window << offset << br;
-    QWindowsWindow *rw = QWindowsWindow::baseWindowOf(window);
+    QWindowsWindow *rw = QWindowsWindow::windowsWindowOf(window);
+    Q_ASSERT(rw);
 
 #ifndef Q_OS_WINCE
     const bool hasAlpha = rw->format().hasAlpha();
     const Qt::WindowFlags flags = window->flags();
     if ((flags & Qt::FramelessWindowHint) && QWindowsWindow::setWindowLayered(rw->handle(), flags, hasAlpha, rw->opacity()) && hasAlpha) {
         // Windows with alpha: Use blend function to update.
-        const QMargins marginsDP = rw->frameMarginsDp();
-        const QRect r = rw->geometryDp() + marginsDP;
-        const QPoint frameOffset(marginsDP.left(), marginsDP.top());
+        QRect r = QHighDpi::toNativePixels(window->frameGeometry(), window);
+        QPoint frameOffset(QHighDpi::toNativePixels(QPoint(window->frameMargins().left(), window->frameMargins().top()),
+                                                    static_cast<const QWindow *>(Q_NULLPTR)));
         QRect dirtyRect = br.translated(offset + frameOffset);
 
         SIZE size = {r.width(), r.height()};
         POINT ptDst = {r.x(), r.y()};
         POINT ptSrc = {0, 0};
-        BLENDFUNCTION blend = {AC_SRC_OVER, 0, (BYTE)(255.0 * rw->opacity()), AC_SRC_ALPHA};
+        BLENDFUNCTION blend = {AC_SRC_OVER, 0, BYTE(qRound(255.0 * rw->opacity())), AC_SRC_ALPHA};
         if (QWindowsContext::user32dll.updateLayeredWindowIndirect) {
             RECT dirty = {dirtyRect.x(), dirtyRect.y(),
                 dirtyRect.x() + dirtyRect.width(), dirtyRect.y() + dirtyRect.height()};
             UPDATELAYEREDWINDOWINFO info = {sizeof(info), NULL, &ptDst, &size, m_image->hdc(), &ptSrc, 0, &blend, ULW_ALPHA, &dirty};
-            QWindowsContext::user32dll.updateLayeredWindowIndirect(rw->handle(), &info);
+            const BOOL result = QWindowsContext::user32dll.updateLayeredWindowIndirect(rw->handle(), &info);
+            if (!result)
+                qErrnoWarning("UpdateLayeredWindowIndirect failed for ptDst=(%d, %d),"
+                              " size=(%dx%d), dirty=(%dx%d %d, %d)", r.x(), r.y(),
+                              r.width(), r.height(), dirtyRect.width(), dirtyRect.height(),
+                              dirtyRect.x(), dirtyRect.y());
         } else {
             QWindowsContext::user32dll.updateLayeredWindow(rw->handle(), NULL, &ptDst, &size, m_image->hdc(), &ptSrc, 0, &blend, ULW_ALPHA);
         }
@@ -110,7 +126,7 @@ void QWindowsBackingStore::flushDp(QWindow *window, const QRect &br, const QPoin
                     m_image->hdc(), br.x() + offset.x(), br.y() + offset.y(), SRCCOPY)) {
             const DWORD lastError = GetLastError(); // QTBUG-35926, QTBUG-29716: may fail after lock screen.
             if (lastError != ERROR_SUCCESS && lastError != ERROR_INVALID_HANDLE)
-                qErrnoWarning(lastError, "%s: BitBlt failed", __FUNCTION__);
+                qErrnoWarning(int(lastError), "%s: BitBlt failed", __FUNCTION__);
         }
         rw->releaseDC();
 #ifndef Q_OS_WINCE
@@ -127,27 +143,34 @@ void QWindowsBackingStore::flushDp(QWindow *window, const QRect &br, const QPoin
     }
 }
 
-void QWindowsBackingStore::resize(const QSize &sizeDip, const QRegion &regionDip)
+void QWindowsBackingStore::resize(const QSize &size, const QRegion &region)
 {
-    const QSize size = sizeDip * QWindowsScaling::factor();
     if (m_image.isNull() || m_image->image().size() != size) {
 #ifndef QT_NO_DEBUG_OUTPUT
         if (QWindowsContext::verbose && lcQpaBackingStore().isDebugEnabled()) {
             qCDebug(lcQpaBackingStore)
-                << __FUNCTION__ << ' ' << window() << ' ' << size << ' ' << sizeDip << ' '
-                << regionDip << " from: " << (m_image.isNull() ? QSize() : m_image->image().size());
+                << __FUNCTION__ << ' ' << window() << ' ' << size << ' ' << region
+                << " from: " << (m_image.isNull() ? QSize() : m_image->image().size());
         }
 #endif
-        const QImage::Format format = window()->format().hasAlpha() ?
-            QImage::Format_ARGB32_Premultiplied : QWindowsNativeImage::systemFormat();
+        QImage::Format format = window()->format().hasAlpha() ?
+                    QImage::Format_ARGB32_Premultiplied : QWindowsNativeImage::systemFormat();
+
+        // The backingstore composition (enabling render-to-texture widgets)
+        // punches holes in the backingstores using the alpha channel. Hence
+        // the need for a true alpha format.
+        if (QImage::toPixelFormat(format).alphaUsage() == QPixelFormat::UsesAlpha)
+            m_alphaNeedsFill = true;
+        else // upgrade but here we know app painting does not rely on alpha hence no need to fill
+            format = qt_maybeAlphaVersionWithSameDepth(format);
 
         QWindowsNativeImage *oldwni = m_image.data();
         QWindowsNativeImage *newwni = new QWindowsNativeImage(size.width(), size.height(), format);
 
-        if (oldwni && !regionDip.isEmpty()) {
+        if (oldwni && !region.isEmpty()) {
             const QImage &oldimg(oldwni->image());
             QImage &newimg(newwni->image());
-            QRegion staticRegion = QWindowsScaling::mapToNative(regionDip);
+            QRegion staticRegion(region);
             staticRegion &= QRect(0, 0, oldimg.width(), oldimg.height());
             staticRegion &= QRect(0, 0, newimg.width(), newimg.height());
             QPainter painter(&newimg);
@@ -156,38 +179,36 @@ void QWindowsBackingStore::resize(const QSize &sizeDip, const QRegion &regionDip
                 painter.drawImage(rect, oldimg, rect);
         }
 
-        if (QWindowsScaling::isActive())
-            newwni->setDevicePixelRatio(QWindowsScaling::factor());
         m_image.reset(newwni);
     }
 }
 
 Q_GUI_EXPORT void qt_scrollRectInImage(QImage &img, const QRect &rect, const QPoint &offset);
 
-bool QWindowsBackingStore::scroll(const QRegion &areaDip, int dxDip, int dyDip)
+bool QWindowsBackingStore::scroll(const QRegion &area, int dx, int dy)
 {
     if (m_image.isNull() || m_image->image().isNull())
         return false;
 
-    const QPoint dp = QPoint(dxDip, dyDip) * QWindowsScaling::factor();
-    const QVector<QRect> rects = areaDip.rects();
+    const QVector<QRect> rects = area.rects();
+    const QPoint offset(dx, dy);
     for (int i = 0; i < rects.size(); ++i)
-        qt_scrollRectInImage(m_image->image(), QWindowsScaling::mapToNative(rects.at(i)), dp);
+        qt_scrollRectInImage(m_image->image(), rects.at(i), offset);
 
     return true;
 }
 
-void QWindowsBackingStore::beginPaint(const QRegion &regionDip)
+void QWindowsBackingStore::beginPaint(const QRegion &region)
 {
     if (QWindowsContext::verbose > 1)
-        qCDebug(lcQpaBackingStore) <<__FUNCTION__ << regionDip;
+        qCDebug(lcQpaBackingStore) <<__FUNCTION__ << region;
 
-    if (m_image->image().hasAlphaChannel()) {
+    if (m_alphaNeedsFill) {
         QPainter p(&m_image->image());
         p.setCompositionMode(QPainter::CompositionMode_Source);
         const QColor blank = Qt::transparent;
-        foreach (const QRect &r, regionDip.rects())
-            p.fillRect(QWindowsScaling::mapToNative(r), blank);
+        foreach (const QRect &r, region.rects())
+            p.fillRect(r, blank);
     }
 }
 
