@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -36,7 +42,6 @@
 #include "qwindowscontext.h"
 #include "qwindowswindow.h"
 #include "qwindowsscreen.h"
-#include "qwindowsscaling.h"
 
 #include <QtGui/QBitmap>
 #include <QtGui/QImage>
@@ -44,15 +49,17 @@
 #include <QtGui/QGuiApplication>
 #include <QtGui/QScreen>
 #include <QtGui/private/qguiapplication_p.h> // getPixmapCursor()
+#include <QtGui/private/qhighdpiscaling_p.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QScopedArrayPointer>
 
-static void initResources()
+static bool initResources()
 {
 #if !defined (Q_OS_WINCE) && !defined (QT_NO_IMAGEFORMAT_PNG)
     Q_INIT_RESOURCE(cursors);
 #endif
+    return true;
 }
 
 QT_BEGIN_NAMESPACE
@@ -68,19 +75,14 @@ Q_GUI_EXPORT HBITMAP qt_createIconMask(const QBitmap &bitmap);
     \ingroup qt-lighthouse-win
 */
 
-QWindowsCursorCacheKey::QWindowsCursorCacheKey(const QCursor &c)
-    : shape(c.shape()), bitmapCacheKey(0), maskCacheKey(0)
+QWindowsPixmapCursorCacheKey::QWindowsPixmapCursorCacheKey(const QCursor &c)
+    : bitmapCacheKey(c.pixmap().cacheKey()), maskCacheKey(0)
 {
-    if (shape == Qt::BitmapCursor) {
-        const qint64 pixmapCacheKey = c.pixmap().cacheKey();
-        if (pixmapCacheKey) {
-            bitmapCacheKey = pixmapCacheKey;
-        } else {
-            Q_ASSERT(c.bitmap());
-            Q_ASSERT(c.mask());
-            bitmapCacheKey = c.bitmap()->cacheKey();
-            maskCacheKey = c.mask()->cacheKey();
-        }
+    if (!bitmapCacheKey) {
+        Q_ASSERT(c.bitmap());
+        Q_ASSERT(c.mask());
+        bitmapCacheKey = c.bitmap()->cacheKey();
+        maskCacheKey = c.mask()->cacheKey();
     }
 }
 
@@ -98,9 +100,14 @@ QWindowsCursorCacheKey::QWindowsCursorCacheKey(const QCursor &c)
     \sa QWindowsWindowCursor
 */
 
-HCURSOR QWindowsCursor::createPixmapCursor(const QPixmap &pixmap, const QPoint &hotSpot)
+HCURSOR QWindowsCursor::createPixmapCursor(QPixmap pixmap, const QPoint &hotSpot, qreal scaleFactor)
 {
     HCURSOR cur = 0;
+    const qreal pixmapScaleFactor = scaleFactor / pixmap.devicePixelRatioF();
+    if (!qFuzzyCompare(pixmapScaleFactor, 1)) {
+        pixmap = pixmap.scaled((pixmapScaleFactor * QSizeF(pixmap.size())).toSize(),
+                               Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    }
     QBitmap mask = pixmap.mask();
     if (mask.isNull()) {
         mask = QBitmap(pixmap.size());
@@ -112,8 +119,8 @@ HCURSOR QWindowsCursor::createPixmapCursor(const QPixmap &pixmap, const QPoint &
 
     ICONINFO ii;
     ii.fIcon     = 0;
-    ii.xHotspot  = hotSpot.x();
-    ii.yHotspot  = hotSpot.y();
+    ii.xHotspot  = DWORD(qRound(hotSpot.x() * scaleFactor));
+    ii.yHotspot  = DWORD(qRound(hotSpot.y() * scaleFactor));
     ii.hbmMask   = im;
     ii.hbmColor  = ic;
 
@@ -141,8 +148,8 @@ static HCURSOR createBitmapCursor(const QImage &bbits, const QImage &mbits,
     QScopedArrayPointer<uchar> xMask(new uchar[height * n]);
     int x = 0;
     for (int i = 0; i < height; ++i) {
-        const uchar *bits = bbits.scanLine(i);
-        const uchar *mask = mbits.scanLine(i);
+        const uchar *bits = bbits.constScanLine(i);
+        const uchar *mask = mbits.constScanLine(i);
         for (int j = 0; j < n; ++j) {
             uchar b = bits[j];
             uchar m = mask[j];
@@ -172,8 +179,8 @@ static HCURSOR createBitmapCursor(const QImage &bbits, const QImage &mbits,
             x += sysN;
         } else {
             int fillWidth = n > sysN ? sysN : n;
-            const uchar *bits = bbits.scanLine(i);
-            const uchar *mask = mbits.scanLine(i);
+            const uchar *bits = bbits.constScanLine(i);
+            const uchar *mask = mbits.constScanLine(i);
             for (int j = 0; j < fillWidth; ++j) {
                 uchar b = bits[j];
                 uchar m = mask[j];
@@ -207,16 +214,54 @@ static HCURSOR createBitmapCursor(const QImage &bbits, const QImage &mbits,
 #endif
 }
 
-static inline QSize systemCursorSize() { return QSize(GetSystemMetrics(SM_CXCURSOR), GetSystemMetrics(SM_CYCURSOR)); }
-static inline QSize standardCursorSize() { return QSize(32, 32); }
+// Create a cursor from image and mask of the format QImage::Format_Mono.
+static HCURSOR createBitmapCursor(const QCursor &cursor, qreal scaleFactor = 1)
+{
+    Q_ASSERT(cursor.shape() == Qt::BitmapCursor && cursor.bitmap());
+    QImage bbits = cursor.bitmap()->toImage();
+    QImage mbits = cursor.mask()->toImage();
+    scaleFactor /= bbits.devicePixelRatioF();
+    if (!qFuzzyCompare(scaleFactor, 1)) {
+        const QSize scaledSize = (QSizeF(bbits.size()) * scaleFactor).toSize();
+        bbits = bbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+        mbits = mbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    }
+    bbits = bbits.convertToFormat(QImage::Format_Mono);
+    mbits = mbits.convertToFormat(QImage::Format_Mono);
+    const bool invb = bbits.colorCount() > 1 && qGray(bbits.color(0)) < qGray(bbits.color(1));
+    const bool invm = mbits.colorCount() > 1 && qGray(mbits.color(0)) < qGray(mbits.color(1));
+    return createBitmapCursor(bbits, mbits, cursor.hotSpot(), invb, invm);
+}
+
+static QSize systemCursorSize(const QPlatformScreen *screen = Q_NULLPTR)
+{
+    const QSize primaryScreenCursorSize(GetSystemMetrics(SM_CXCURSOR), GetSystemMetrics(SM_CYCURSOR));
+    if (screen) {
+        // Correct the size if the DPI value of the screen differs from
+        // that of the primary screen.
+        if (const QScreen *primaryQScreen = QGuiApplication::primaryScreen()) {
+            const QPlatformScreen *primaryScreen = primaryQScreen->handle();
+            if (screen != primaryScreen) {
+                const qreal logicalDpi = screen->logicalDpi().first;
+                const qreal primaryScreenLogicalDpi = primaryScreen->logicalDpi().first;
+                if (!qFuzzyCompare(logicalDpi, primaryScreenLogicalDpi))
+                    return (QSizeF(primaryScreenCursorSize) * logicalDpi / primaryScreenLogicalDpi).toSize();
+            }
+        }
+    }
+    return primaryScreenCursorSize;
+}
 
 #if defined (Q_OS_WINCE) || defined (QT_NO_IMAGEFORMAT_PNG)
+
+static inline QSize standardCursorSize() { return QSize(32, 32); }
+
 // Create pixmap cursors from data and scale the image if the cursor size is
 // higher than the standard 32. Note that bitmap cursors as produced by
 // createBitmapCursor() only work for standard sizes (32,48,64...), which does
 // not work when scaling the 16x16 openhand cursor bitmaps to 150% (resulting
 // in a non-standard 24x24 size).
-static QCursor createPixmapCursorFromData(const QSize &systemCursorSize,
+static QWindowsCursor::PixmapCursor createPixmapCursorFromData(const QSize &systemCursorSize,
                                           // The cursor size the bitmap is targeted for
                                           const QSize &bitmapTargetCursorSize,
                                           // The actual size of the bitmap data
@@ -234,10 +279,11 @@ static QCursor createPixmapCursorFromData(const QSize &systemCursorSize,
         rawImage = rawImage.transformed(transform, Qt::SmoothTransformation);
     }
     const QPoint hotSpot(rawImage.width() / 2, rawImage.height() / 2);
-    return QCursor(rawImage, hotSpot.x(), hotSpot.y());
+    return QWindowsCursor::PixmapCursor(rawImage, hotSpot);
 }
 
-QCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape)
+QWindowsCursor::PixmapCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape,
+                                                          const QPlatformScreen *screen)
 {
     // Non-standard Windows cursors are created from bitmaps
     static const uchar vsplit_bits[] = {
@@ -405,22 +451,22 @@ QCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape)
 
     switch (cursorShape) {
     case Qt::SplitVCursor:
-        return createPixmapCursorFromData(systemCursorSize(), standardCursorSize(), 32, vsplit_bits, vsplitm_bits);
+        return createPixmapCursorFromData(systemCursorSize(screen), standardCursorSize(), 32, vsplit_bits, vsplitm_bits);
     case Qt::SplitHCursor:
-        return createPixmapCursorFromData(systemCursorSize(), standardCursorSize(), 32, hsplit_bits, hsplitm_bits);
+        return createPixmapCursorFromData(systemCursorSize(screen), standardCursorSize(), 32, hsplit_bits, hsplitm_bits);
     case Qt::OpenHandCursor:
-        return createPixmapCursorFromData(systemCursorSize(), standardCursorSize(), 16, openhand_bits, openhandm_bits);
+        return createPixmapCursorFromData(systemCursorSize(screen), standardCursorSize(), 16, openhand_bits, openhandm_bits);
     case Qt::ClosedHandCursor:
-        return createPixmapCursorFromData(systemCursorSize(), standardCursorSize(), 16, closedhand_bits, closedhandm_bits);
+        return createPixmapCursorFromData(systemCursorSize(screen), standardCursorSize(), 16, closedhand_bits, closedhandm_bits);
     case Qt::DragCopyCursor:
-        return QCursor(QPixmap(copyDragCursorXpmC), 0, 0);
+        return QWindowsCursor::PixmapCursor(QPixmap(copyDragCursorXpmC), QPoint(0, 0));
     case Qt::DragMoveCursor:
-        return QCursor(QPixmap(moveDragCursorXpmC), 0, 0);
+        return QWindowsCursor::PixmapCursor(QPixmap(moveDragCursorXpmC), QPoint(0, 0));
     case Qt::DragLinkCursor:
-        return QCursor(QPixmap(linkDragCursorXpmC), 0, 0);
+        return QWindowsCursor::PixmapCursor(QPixmap(linkDragCursorXpmC), QPoint(0, 0));
     }
 
-    return QCursor();
+    return QWindowsCursor::PixmapCursor();
 }
 #else // Q_OS_WINCE || QT_NO_IMAGEFORMAT_PNG
 struct QWindowsCustomPngCursor {
@@ -431,7 +477,7 @@ struct QWindowsCustomPngCursor {
     int hotSpotY;
 };
 
-QCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape)
+QWindowsCursor::PixmapCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape, const QPlatformScreen *screen)
 {
     static const QWindowsCustomPngCursor pngCursors[] = {
         { Qt::SplitVCursor, 32, "splitvcursor_32.png", 11, 11 },
@@ -457,14 +503,14 @@ QCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape)
         { Qt::DragLinkCursor, 64, "draglinkcursor_64.png", 0, 0 }
     };
 
-    const int cursorSize = GetSystemMetrics(SM_CXCURSOR);
+    const QSize cursorSize = systemCursorSize(screen);
     const QWindowsCustomPngCursor *sEnd = pngCursors + sizeof(pngCursors) / sizeof(pngCursors[0]);
     const QWindowsCustomPngCursor *bestFit = 0;
     int sizeDelta = INT_MAX;
     for (const QWindowsCustomPngCursor *s = pngCursors; s < sEnd; ++s) {
         if (s->shape != cursorShape)
             continue;
-        const int currentSizeDelta = qMax(s->size, cursorSize) - qMin(s->size, cursorSize);
+        const int currentSizeDelta = qMax(s->size, cursorSize.width()) - qMin(s->size, cursorSize.width());
         if (currentSizeDelta < sizeDelta) {
             bestFit = s;
             if (currentSizeDelta == 0)
@@ -474,11 +520,11 @@ QCursor QWindowsCursor::customCursor(Qt::CursorShape cursorShape)
     }
 
     if (!bestFit)
-        return QCursor();
+        return PixmapCursor();
 
     const QPixmap rawImage(QStringLiteral(":/qt-project.org/windows/cursors/images/") +
                            QString::fromLatin1(bestFit->fileName));
-    return QCursor(rawImage, bestFit->hotSpotX, bestFit->hotSpotY);
+    return PixmapCursor(rawImage, QPoint(bestFit->hotSpotX, bestFit->hotSpotY));
 }
 #endif // Q_OS_WINCE || QT_NO_IMAGEFORMAT_PNG
 
@@ -487,8 +533,10 @@ struct QWindowsStandardCursorMapping {
     LPCWSTR resource;
 };
 
-HCURSOR QWindowsCursor::createSystemCursor(const QCursor &c)
+HCURSOR QWindowsCursor::createCursorFromShape(Qt::CursorShape cursorShape, const QPlatformScreen *screen)
 {
+    Q_ASSERT(cursorShape != Qt::BitmapCursor);
+
     static const QWindowsStandardCursorMapping standardCursors[] = {
         { Qt::ArrowCursor, IDC_ARROW},
         { Qt::UpArrowCursor, IDC_UPARROW },
@@ -506,20 +554,9 @@ HCURSOR QWindowsCursor::createSystemCursor(const QCursor &c)
         { Qt::PointingHandCursor, IDC_HAND }
     };
 
-    const Qt::CursorShape cursorShape = c.shape();
     switch (cursorShape) {
-    case Qt::BitmapCursor: {
-        const QPixmap pixmap = c.pixmap();
-        if (!pixmap.isNull())
-            return QWindowsCursor::createPixmapCursor(pixmap, c.hotSpot());
-        const QImage bbits = c.bitmap()->toImage().convertToFormat(QImage::Format_Mono);
-        const QImage mbits = c.mask()->toImage().convertToFormat(QImage::Format_Mono);
-        const bool invb = bbits.colorCount() > 1 && qGray(bbits.color(0)) < qGray(bbits.color(1));
-        const bool invm = mbits.colorCount() > 1 && qGray(mbits.color(0)) < qGray(mbits.color(1));
-        return createBitmapCursor(bbits, mbits, c.hotSpot(), invb, invm);
-    }
     case Qt::BlankCursor: {
-        QImage blank = QImage(systemCursorSize(), QImage::Format_Mono);
+        QImage blank = QImage(systemCursorSize(screen), QImage::Format_Mono);
         blank.fill(0); // ignore color table
         return createBitmapCursor(blank, blank);
     }
@@ -530,7 +567,7 @@ HCURSOR QWindowsCursor::createSystemCursor(const QCursor &c)
     case Qt::DragCopyCursor:
     case Qt::DragMoveCursor:
     case Qt::DragLinkCursor:
-        return createSystemCursor(customCursor(cursorShape));
+        return QWindowsCursor::createPixmapCursor(customCursor(cursorShape, screen));
     default:
         break;
     }
@@ -540,7 +577,7 @@ HCURSOR QWindowsCursor::createSystemCursor(const QCursor &c)
     for (const QWindowsStandardCursorMapping *s = standardCursors; s < sEnd; ++s) {
         if (s->shape == cursorShape) {
 #ifndef Q_OS_WINCE
-            return (HCURSOR)LoadImage(0, s->resource, IMAGE_CURSOR, 0, 0, LR_DEFAULTSIZE | LR_SHARED);
+            return static_cast<HCURSOR>(LoadImage(0, s->resource, IMAGE_CURSOR, 0, 0, LR_DEFAULTSIZE | LR_SHARED));
 #else
             return LoadCursor(0, s->resource);
 #endif
@@ -555,44 +592,52 @@ HCURSOR QWindowsCursor::createSystemCursor(const QCursor &c)
     \brief Return cached standard cursor resources or create new ones.
 */
 
-QWindowsWindowCursor QWindowsCursor::standardWindowCursor(Qt::CursorShape shape)
+CursorHandlePtr QWindowsCursor::standardWindowCursor(Qt::CursorShape shape)
 {
-    const QWindowsCursorCacheKey key(shape);
-    CursorCache::iterator it = m_cursorCache.find(key);
-    if (it == m_cursorCache.end())
-        it = m_cursorCache.insert(key, QWindowsWindowCursor(QCursor(shape)));
-    return it.value();
+    StandardCursorCache::Iterator it = m_standardCursorCache.find(shape);
+    if (it == m_standardCursorCache.end()) {
+        if (const HCURSOR hc = QWindowsCursor::createCursorFromShape(shape, m_screen))
+            it = m_standardCursorCache.insert(shape, CursorHandlePtr(new CursorHandle(hc)));
+    }
+    return it != m_standardCursorCache.end() ? it.value() : CursorHandlePtr(new CursorHandle);
 }
 
 /*!
     \brief Return cached pixmap cursor or create new one.
 */
 
-QWindowsWindowCursor QWindowsCursor::pixmapWindowCursor(const QCursor &c)
+CursorHandlePtr QWindowsCursor::pixmapWindowCursor(const QCursor &c)
 {
-    const  QWindowsCursorCacheKey cacheKey(c);
-    CursorCache::iterator it = m_cursorCache.find(cacheKey);
-    if (it == m_cursorCache.end()) {
-        if (m_cursorCache.size() > 50) {
+    const QWindowsPixmapCursorCacheKey cacheKey(c);
+    PixmapCursorCache::iterator it = m_pixmapCursorCache.find(cacheKey);
+    if (it == m_pixmapCursorCache.end()) {
+        if (m_pixmapCursorCache.size() > 50) {
             // Prevent the cursor cache from growing indefinitely hitting GDI resource
             // limits if new pixmap cursors are created repetitively by purging out
             // all-noncurrent pixmap cursors (QTBUG-43515)
             const HCURSOR currentCursor = GetCursor();
-            for (it = m_cursorCache.begin(); it != m_cursorCache.end() ; ) {
-                if (it.key().bitmapCacheKey && it.value().handle() != currentCursor)
-                    it = m_cursorCache.erase(it);
+            for (it = m_pixmapCursorCache.begin(); it != m_pixmapCursorCache.end() ; ) {
+                if (it.value()->handle() != currentCursor)
+                    it = m_pixmapCursorCache.erase(it);
                 else
                     ++it;
             }
         }
-        it = m_cursorCache.insert(cacheKey, QWindowsWindowCursor(c));
+        const qreal scaleFactor = QHighDpiScaling::factor(m_screen);
+        const QPixmap pixmap = c.pixmap();
+        const HCURSOR hc = pixmap.isNull()
+            ? createBitmapCursor(c, scaleFactor)
+            : QWindowsCursor::createPixmapCursor(pixmap, c.hotSpot(), scaleFactor);
+        it = m_pixmapCursorCache.insert(cacheKey, CursorHandlePtr(new CursorHandle(hc)));
     }
     return it.value();
 }
 
-QWindowsCursor::QWindowsCursor()
+QWindowsCursor::QWindowsCursor(const QPlatformScreen *screen)
+    : m_screen(screen)
 {
-    initResources();
+    static const bool dummy = initResources();
+    Q_UNUSED(dummy)
 }
 
 /*!
@@ -604,17 +649,19 @@ QWindowsCursor::QWindowsCursor()
 
 void QWindowsCursor::changeCursor(QCursor *cursorIn, QWindow *window)
 {
-    if (!window)
+    QWindowsWindow *platformWindow = QWindowsWindow::windowsWindowOf(window);
+    if (!platformWindow) // Desktop/Foreign window.
         return;
+
     if (!cursorIn) {
-        QWindowsWindow::baseWindowOf(window)->setCursor(QWindowsWindowCursor());
+        platformWindow->setCursor(CursorHandlePtr(new CursorHandle));
         return;
     }
-    const QWindowsWindowCursor wcursor =
+    const CursorHandlePtr wcursor =
         cursorIn->shape() == Qt::BitmapCursor ?
         pixmapWindowCursor(*cursorIn) : standardWindowCursor(cursorIn->shape());
-    if (wcursor.handle()) {
-        QWindowsWindow::baseWindowOf(window)->setCursor(wcursor);
+    if (wcursor->handle()) {
+        platformWindow->setCursor(wcursor);
     } else {
         qWarning("%s: Unable to obtain system cursor for %d",
                  __FUNCTION__, cursorIn->shape());
@@ -646,13 +693,115 @@ QWindowsCursor::CursorState QWindowsCursor::cursorState()
 
 QPoint QWindowsCursor::pos() const
 {
-    return mousePosition() / QWindowsScaling::factor();
+    return mousePosition();
 }
 
 void QWindowsCursor::setPos(const QPoint &pos)
 {
-    const QPoint posDp = pos * QWindowsScaling::factor();
-    SetCursorPos(posDp.x() , posDp.y());
+    SetCursorPos(pos.x() , pos.y());
+}
+
+QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
+{
+    switch (action) {
+    case Qt::CopyAction:
+        if (m_copyDragCursor.isNull())
+            m_copyDragCursor = QWindowsCursor::customCursor(Qt::DragCopyCursor, m_screen).pixmap;
+        return m_copyDragCursor;
+    case Qt::TargetMoveAction:
+    case Qt::MoveAction:
+        if (m_moveDragCursor.isNull())
+            m_moveDragCursor = QWindowsCursor::customCursor(Qt::DragMoveCursor, m_screen).pixmap;
+        return m_moveDragCursor;
+    case Qt::LinkAction:
+        if (m_linkDragCursor.isNull())
+            m_linkDragCursor = QWindowsCursor::customCursor(Qt::DragLinkCursor, m_screen).pixmap;
+        return m_linkDragCursor;
+    default:
+        break;
+    }
+
+    static const char * const ignoreDragCursorXpmC[] = {
+    "24 30 3 1",
+    ".        c None",
+    "a        c #000000",
+    "X        c #FFFFFF",
+    "aa......................",
+    "aXa.....................",
+    "aXXa....................",
+    "aXXXa...................",
+    "aXXXXa..................",
+    "aXXXXXa.................",
+    "aXXXXXXa................",
+    "aXXXXXXXa...............",
+    "aXXXXXXXXa..............",
+    "aXXXXXXXXXa.............",
+    "aXXXXXXaaaa.............",
+    "aXXXaXXa................",
+    "aXXaaXXa................",
+    "aXa..aXXa...............",
+    "aa...aXXa...............",
+    "a.....aXXa..............",
+    "......aXXa.....XXXX.....",
+    ".......aXXa..XXaaaaXX...",
+    ".......aXXa.XaaaaaaaaX..",
+    "........aa.XaaaXXXXaaaX.",
+    "...........XaaaaX..XaaX.",
+    "..........XaaXaaaX..XaaX",
+    "..........XaaXXaaaX.XaaX",
+    "..........XaaX.XaaaXXaaX",
+    "..........XaaX..XaaaXaaX",
+    "...........XaaX..XaaaaX.",
+    "...........XaaaXXXXaaaX.",
+    "............XaaaaaaaaX..",
+    ".............XXaaaaXX...",
+    "...............XXXX....."};
+
+    if (m_ignoreDragCursor.isNull()) {
+#if !defined (Q_OS_WINCE)
+        HCURSOR cursor = LoadCursor(NULL, IDC_NO);
+        ICONINFO iconInfo = {0, 0, 0, 0, 0};
+        GetIconInfo(cursor, &iconInfo);
+        BITMAP bmColor = {0, 0, 0, 0, 0, 0, 0};
+
+        if (iconInfo.hbmColor
+            && GetObject(iconInfo.hbmColor, sizeof(BITMAP), &bmColor)
+            && bmColor.bmWidth == bmColor.bmWidthBytes / 4) {
+            const int colorBitsLength = bmColor.bmHeight * bmColor.bmWidthBytes;
+            uchar *colorBits = new uchar[colorBitsLength];
+            GetBitmapBits(iconInfo.hbmColor, colorBitsLength, colorBits);
+            const QImage colorImage(colorBits, bmColor.bmWidth, bmColor.bmHeight,
+                                    bmColor.bmWidthBytes, QImage::Format_ARGB32);
+
+            m_ignoreDragCursor = QPixmap::fromImage(colorImage);
+            delete [] colorBits;
+        } else {
+            m_ignoreDragCursor = QPixmap(ignoreDragCursorXpmC);
+        }
+
+        DeleteObject(iconInfo.hbmMask);
+        DeleteObject(iconInfo.hbmColor);
+        DestroyCursor(cursor);
+#else // !Q_OS_WINCE
+        m_ignoreDragCursor = QPixmap(ignoreDragCursorXpmC);
+#endif // !Q_OS_WINCE
+    }
+    return m_ignoreDragCursor;
+}
+
+HCURSOR QWindowsCursor::hCursor(const QCursor &c) const
+{
+    const Qt::CursorShape shape = c.shape();
+    if (shape == Qt::BitmapCursor) {
+        const auto pit = m_pixmapCursorCache.constFind(QWindowsPixmapCursorCacheKey(c));
+        if (pit != m_pixmapCursorCache.constEnd())
+            return pit.value()->handle();
+    } else {
+        const auto sit = m_standardCursorCache.constFind(shape);
+        if (sit != m_standardCursorCache.constEnd())
+            return sit.value()->handle();
+    }
+    return HCURSOR(0);
 }
 
 /*!
@@ -660,78 +809,11 @@ void QWindowsCursor::setPos(const QPoint &pos)
     \brief Per-Window cursor. Contains a QCursor and manages its associated system
      cursor handle resource.
 
-    Based on QSharedDataPointer, so that it can be passed around and
-    used as a property of QWindowsBaseWindow.
-
     \internal
     \ingroup qt-lighthouse-win
     \sa QWindowsCursor
 */
 
-class QWindowsWindowCursorData : public QSharedData
-{
-public:
-    QWindowsWindowCursorData() : m_cursor(Qt::ArrowCursor), m_handle(0) {}
-    explicit QWindowsWindowCursorData(const QCursor &c);
-    ~QWindowsWindowCursorData();
-
-    const QCursor m_cursor;
-    const HCURSOR m_handle;
-};
-
-QWindowsWindowCursorData::QWindowsWindowCursorData(const QCursor &c) :
-    m_cursor(c),
-    m_handle(QWindowsCursor::createSystemCursor(c))
-{
-}
-
-QWindowsWindowCursorData::~QWindowsWindowCursorData()
-{
-    if (m_handle)
-        DestroyCursor(m_handle);
-}
-
-QWindowsWindowCursor::QWindowsWindowCursor() :
-    m_data(new QWindowsWindowCursorData)
-{
-}
-
-QWindowsWindowCursor::QWindowsWindowCursor(const QCursor &c) :
-    m_data(new QWindowsWindowCursorData(c))
-{
-}
-
-QWindowsWindowCursor::~QWindowsWindowCursor()
-{
-}
-
-QWindowsWindowCursor::QWindowsWindowCursor(const QWindowsWindowCursor &rhs) :
-    m_data(rhs.m_data)
-{
-}
-
-QWindowsWindowCursor & QWindowsWindowCursor::operator =(const QWindowsWindowCursor &rhs)
-{
-    if (this != &rhs)
-        m_data.operator =(rhs.m_data);
-    return *this;
-}
-
-bool QWindowsWindowCursor::isNull() const
-{
-    return m_data->m_handle == 0;
-}
-
-QCursor QWindowsWindowCursor::cursor() const
-{
-    return m_data->m_cursor;
-}
-
-HCURSOR QWindowsWindowCursor::handle() const
-{
-    return m_data->m_handle;
-}
-
 QT_END_NAMESPACE
 
 #endif // !QT_NO_CURSOR
