@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -46,13 +44,13 @@
 #include <QtCore/QSharedPointer>
 #include "qtwindows_additional.h"
 
-QT_BEGIN_NAMESPACE
-
 #if !defined(QT_NO_DIRECTWRITE)
     struct IDWriteFactory;
     struct IDWriteGdiInterop;
 #endif
 
+QT_BEGIN_NAMESPACE
+
 class QWindowsFontEngineData
 {
     Q_DISABLE_COPY(QWindowsFontEngineData)
@@ -77,24 +75,25 @@ public:
     QWindowsFontDatabase();
     ~QWindowsFontDatabase();
 
-    virtual void populateFontDatabase();
-    virtual QFontEngineMulti *fontEngineMulti(QFontEngine *fontEngine, QChar::Script script);
-    virtual QFontEngine *fontEngine(const QFontDef &fontDef, void *handle);
-    virtual QFontEngine *fontEngine(const QByteArray &fontData, qreal pixelSize, QFont::HintingPreference hintingPreference);
-    virtual QStringList fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const;
-    virtual QStringList addApplicationFont(const QByteArray &fontData, const QString &fileName);
-    virtual void releaseHandle(void *handle);
-    virtual QString fontDir() const;
+    void populateFontDatabase() Q_DECL_OVERRIDE;
+    void populateFamily(const QString &familyName) Q_DECL_OVERRIDE;
+    QFontEngineMulti *fontEngineMulti(QFontEngine *fontEngine, QChar::Script script) Q_DECL_OVERRIDE;
+    QFontEngine *fontEngine(const QFontDef &fontDef, void *handle) Q_DECL_OVERRIDE;
+    QFontEngine *fontEngine(const QByteArray &fontData, qreal pixelSize, QFont::HintingPreference hintingPreference) Q_DECL_OVERRIDE;
+    QStringList fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const Q_DECL_OVERRIDE;
+    QStringList addApplicationFont(const QByteArray &fontData, const QString &fileName) Q_DECL_OVERRIDE;
+    void releaseHandle(void *handle) Q_DECL_OVERRIDE;
+    QString fontDir() const Q_DECL_OVERRIDE;
 
-    virtual QFont defaultFont() const { return systemDefaultFont(); }
-    virtual bool fontsAlwaysScalable() const;
+    QFont defaultFont() const  Q_DECL_OVERRIDE { return systemDefaultFont(); }
+    bool fontsAlwaysScalable() const Q_DECL_OVERRIDE;
     void derefUniqueFont(const QString &uniqueFont);
     void refUniqueFont(const QString &uniqueFont);
 
     static QFont systemDefaultFont();
 
     static QFontEngine *createEngine(const QFontDef &request,
-                                     HDC fontHdc, int dpi, bool rawMode,
+                                     int dpi,
                                      const QSharedPointer<QWindowsFontEngineData> &data);
 
     static HFONT systemFont();
@@ -104,11 +103,11 @@ public:
     static LOGFONT fontDefToLOGFONT(const QFontDef &fontDef);
 
     static QStringList extraTryFontsForFamily(const QString &family);
+    static QString familyForStyleHint(QFont::StyleHint styleHint);
 
 private:
-    void populate(const QString &family = QString());
+    void populateFamily(const QString &familyName, bool registerAlias);
     void removeApplicationFonts();
-    QSet<QString> m_families;
 
     struct WinApplicationFont {
         HANDLE handle;
@@ -125,6 +124,10 @@ private:
     QMap<QString, UniqueFontData> m_uniqueFontData;
 };
 
+#ifndef QT_NO_DEBUG_STREAM
+QDebug operator<<(QDebug, const QFontDef &def);
+#endif
+
 QT_END_NAMESPACE
 
 #endif // QWINDOWSFONTDATABASE_H
