@@ -1,39 +1,44 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
+** Copyright (C) 2016 The Qt Company Ltd.
 ** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qwindowsintegration.h"
-#include "qwindowsscaling.h"
 #include "qwindowswindow.h"
 #include "qwindowscontext.h"
 #include "qwindowsopenglcontext.h"
@@ -45,7 +50,6 @@
 #  include "qwindowsfontdatabase_ft.h"
 #endif
 #include "qwindowsfontdatabase.h"
-#include "qwindowsguieventdispatcher.h"
 #ifndef QT_NO_CLIPBOARD
 #  include "qwindowsclipboard.h"
 #  ifndef QT_NO_DRAGANDDROP
@@ -63,10 +67,13 @@
 #if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
 #  include "qwindowssessionmanager.h"
 #endif
+#include <QtGui/qtouchdevice.h>
 #include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/private/qhighdpiscaling_p.h>
 #include <QtGui/qpa/qplatforminputcontextfactory_p.h>
 
-#include <QtCore/private/qeventdispatcher_win_p.h>
+#include <QtPlatformSupport/private/qwindowsguieventdispatcher_p.h>
+
 #include <QtCore/QDebug>
 #include <QtCore/QVariant>
 
@@ -139,7 +146,8 @@ struct QWindowsIntegrationPrivate
 #  endif
 #endif
 #ifndef QT_NO_OPENGL
-    QSharedPointer<QWindowsStaticOpenGLContext> m_staticOpenGLContext;
+    QMutex m_staticContextLock;
+    QScopedPointer<QWindowsStaticOpenGLContext> m_staticOpenGLContext;
 #endif // QT_NO_OPENGL
     QScopedPointer<QPlatformInputContext> m_inputContext;
 #ifndef QT_NO_ACCESSIBILITY
@@ -191,6 +199,10 @@ static inline unsigned parseOptions(const QStringList &paramList,
             }
         } else if (param == QLatin1String("gl=gdi")) {
             options |= QWindowsIntegration::DisableArb;
+        } else if (param == QLatin1String("nodirectwrite")) {
+            options |= QWindowsIntegration::DontUseDirectWriteFonts;
+        } else if (param == QLatin1String("nocolorfonts")) {
+            options |= QWindowsIntegration::DontUseColorFonts;
         } else if (param == QLatin1String("nomousefromtouch")) {
             options |= QWindowsIntegration::DontPassOsMouseEventsSynthesizedFromTouch;
         } else if (parseIntOption(param, QLatin1String("verbose"), 0, INT_MAX, &QWindowsContext::verbose)
@@ -220,27 +232,16 @@ QWindowsIntegrationPrivate::QWindowsIntegrationPrivate(const QStringList &paramL
     if (tabletAbsoluteRange >= 0)
         m_context.setTabletAbsoluteRange(tabletAbsoluteRange);
     if (!dpiAwarenessSet) { // Set only once in case of repeated instantiations of QGuiApplication.
-        m_context.setProcessDpiAwareness(dpiAwareness);
+        if (!QCoreApplication::testAttribute(Qt::AA_PluginApplication)) {
+            m_context.setProcessDpiAwareness(dpiAwareness);
+            qCDebug(lcQpaWindows)
+                << __FUNCTION__ << "DpiAwareness=" << dpiAwareness
+                << "effective process DPI awareness=" << QWindowsContext::processDpiAwareness();
+        }
         dpiAwarenessSet = true;
     }
-    // Determine suitable scale factor, don't mix Windows and Qt scaling
-    if (dpiAwareness != QtWindows::ProcessDpiUnaware)
-        QWindowsScaling::setFactor(QWindowsScaling::determineUiScaleFactor());
-    qCDebug(lcQpaWindows)
-        << __FUNCTION__ << "DpiAwareness=" << dpiAwareness <<",Scaling="
-        << QWindowsScaling::factor();
 
-    QTouchDevice *touchDevice = m_context.touchDevice();
-    if (touchDevice) {
-#ifdef Q_OS_WINCE
-        touchDevice->setCapabilities(touchDevice->capabilities() | QTouchDevice::MouseEmulation);
-#else
-        if (!(m_options & QWindowsIntegration::DontPassOsMouseEventsSynthesizedFromTouch)) {
-            touchDevice->setCapabilities(touchDevice->capabilities() | QTouchDevice::MouseEmulation);
-        }
-#endif
-        QWindowSystemInterface::registerTouchDevice(touchDevice);
-    }
+    m_context.initTouch(m_options);
 }
 
 QWindowsIntegrationPrivate::~QWindowsIntegrationPrivate()
@@ -268,10 +269,9 @@ QWindowsIntegration::~QWindowsIntegration()
 
 void QWindowsIntegration::initialize()
 {
-    if (QPlatformInputContext *pluginContext = QPlatformInputContextFactory::create())
-        d->m_inputContext.reset(pluginContext);
-    else
-        d->m_inputContext.reset(new QWindowsInputContext);
+    QString icStr = QPlatformInputContextFactory::requested();
+    icStr.isNull() ? d->m_inputContext.reset(new QWindowsInputContext)
+                   : d->m_inputContext.reset(QPlatformInputContextFactory::create(icStr));
 }
 
 bool QWindowsIntegration::hasCapability(QPlatformIntegration::Capability cap) const
@@ -297,23 +297,52 @@ bool QWindowsIntegration::hasCapability(QPlatformIntegration::Capability cap) co
         return true;
     case AllGLFunctionsQueryable:
         return true;
+    case SwitchableWidgetComposition:
+        return true;
     default:
         return QPlatformIntegration::hasCapability(cap);
     }
     return false;
 }
 
-QWindowsWindowData QWindowsIntegration::createWindowData(QWindow *window) const
+QPlatformWindow *QWindowsIntegration::createPlatformWindow(QWindow *window) const
 {
+    if (window->type() == Qt::Desktop) {
+        QWindowsDesktopWindow *result = new QWindowsDesktopWindow(window);
+        qCDebug(lcQpaWindows) << "Desktop window:" << window
+            << showbase << hex << result->winId() << noshowbase << dec << result->geometry();
+        return result;
+    }
+
+    if (window->type() == Qt::ForeignWindow) {
+        const HWND hwnd = reinterpret_cast<HWND>(window->winId());
+        if (!IsWindow(hwnd)) {
+           qWarning("Windows QPA: Invalid foreign window ID %p.", hwnd);
+           return nullptr;
+        }
+        QWindowsForeignWindow *result = new QWindowsForeignWindow(window, hwnd);
+        const QRect obtainedGeometry = result->geometry();
+        QScreen *screen = Q_NULLPTR;
+        if (const QPlatformScreen *pScreen = result->screenForGeometry(obtainedGeometry))
+            screen = pScreen->screen();
+        if (screen && screen != window->screen())
+            window->setScreen(screen);
+        qCDebug(lcQpaWindows) << "Foreign window:" << window << showbase << hex
+            << result->winId() << noshowbase << dec << obtainedGeometry << screen;
+        return result;
+    }
+
     QWindowsWindowData requested;
     requested.flags = window->flags();
-    requested.geometry = QWindowsScaling::mapToNative(window->geometry());
+    requested.geometry = QHighDpi::toNativePixels(window->geometry(), window);
     // Apply custom margins (see  QWindowsWindow::setCustomMargins())).
     const QVariant customMarginsV = window->property("_q_windowsCustomMargins");
     if (customMarginsV.isValid())
         requested.customMargins = qvariant_cast<QMargins>(customMarginsV);
 
-    QWindowsWindowData obtained = QWindowsWindowData::create(window, requested, window->title());
+    QWindowsWindowData obtained =
+        QWindowsWindowData::create(window, requested,
+                                   QWindowsWindow::formatWindowTitle(window->title()));
     qCDebug(lcQpaWindows).nospace()
         << __FUNCTION__ << ' ' << window
         << "\n    Requested: " << requested.geometry << " frame incl.="
@@ -322,22 +351,34 @@ QWindowsWindowData QWindowsIntegration::createWindowData(QWindow *window) const
         << "\n    Obtained : " << obtained.geometry << " margins=" << obtained.frame
         << " handle=" << obtained.hwnd << ' ' << obtained.flags << '\n';
 
-    if (obtained.hwnd) {
-        if (requested.flags != obtained.flags)
-            window->setFlags(obtained.flags);
-        // Trigger geometry change signals of QWindow.
-        if ((obtained.flags & Qt::Desktop) != Qt::Desktop && requested.geometry != obtained.geometry)
-            QWindowSystemInterface::handleGeometryChange(window, QWindowsScaling::mapFromNative(obtained.geometry));
+    if (Q_UNLIKELY(!obtained.hwnd))
+        return Q_NULLPTR;
+
+    QWindowsWindow *result = createPlatformWindowHelper(window, obtained);
+    Q_ASSERT(result);
+
+    if (requested.flags != obtained.flags)
+        window->setFlags(obtained.flags);
+    // Trigger geometry change (unless it has a special state in which case setWindowState()
+    // will send the message) and screen change signals of QWindow.
+    if ((obtained.flags & Qt::Desktop) != Qt::Desktop) {
+        const Qt::WindowState state = window->windowState();
+        if (state != Qt::WindowMaximized && state != Qt::WindowFullScreen
+            && requested.geometry != obtained.geometry) {
+            QWindowSystemInterface::handleGeometryChange(window, obtained.geometry);
+        }
+        QPlatformScreen *screen = result->screenForGeometry(obtained.geometry);
+        if (screen && result->screen() != screen)
+            QWindowSystemInterface::handleWindowScreenChanged(window, screen->screen());
     }
 
-    return obtained;
+    return result;
 }
 
-QPlatformWindow *QWindowsIntegration::createPlatformWindow(QWindow *window) const
+// Overridden to return a QWindowsDirect2DWindow in Direct2D plugin.
+QWindowsWindow *QWindowsIntegration::createPlatformWindowHelper(QWindow *window, const QWindowsWindowData &data) const
 {
-    QWindowsWindowData data = createWindowData(window);
-    return data.hwnd ? new QWindowsWindow(window, data)
-                     : Q_NULLPTR;
+    return new QWindowsWindow(window, data);
 }
 
 #ifndef QT_NO_OPENGL
@@ -348,8 +389,13 @@ QWindowsStaticOpenGLContext *QWindowsStaticOpenGLContext::doCreate()
     QWindowsOpenGLTester::Renderer requestedRenderer = QWindowsOpenGLTester::requestedRenderer();
     switch (requestedRenderer) {
     case QWindowsOpenGLTester::DesktopGl:
-        if (QWindowsStaticOpenGLContext *glCtx = QOpenGLStaticContext::create())
+        if (QWindowsStaticOpenGLContext *glCtx = QOpenGLStaticContext::create()) {
+            if ((QWindowsOpenGLTester::supportedRenderers() & QWindowsOpenGLTester::DisableRotationFlag)
+                && !QWindowsScreen::setOrientationPreference(Qt::LandscapeOrientation)) {
+                qCWarning(lcQpaGl, "Unable to disable rotation.");
+            }
             return glCtx;
+        }
         qCWarning(lcQpaGl, "System OpenGL failed. Falling back to Software OpenGL.");
         return QOpenGLStaticContext::create(true);
     // If ANGLE is requested, use it, don't try anything else.
@@ -372,8 +418,13 @@ QWindowsStaticOpenGLContext *QWindowsStaticOpenGLContext::doCreate()
 
     const QWindowsOpenGLTester::Renderers supportedRenderers = QWindowsOpenGLTester::supportedRenderers();
     if (supportedRenderers & QWindowsOpenGLTester::DesktopGl) {
-        if (QWindowsStaticOpenGLContext *glCtx = QOpenGLStaticContext::create())
+        if (QWindowsStaticOpenGLContext *glCtx = QOpenGLStaticContext::create()) {
+            if ((supportedRenderers & QWindowsOpenGLTester::DisableRotationFlag)
+                && !QWindowsScreen::setOrientationPreference(Qt::LandscapeOrientation)) {
+                qCWarning(lcQpaGl, "Unable to disable rotation.");
+            }
             return glCtx;
+        }
     }
     if (QWindowsOpenGLTester::Renderers glesRenderers = supportedRenderers & QWindowsOpenGLTester::GlesMask) {
         if (QWindowsEGLStaticContext *eglCtx = QWindowsEGLStaticContext::create(glesRenderers))
@@ -425,8 +476,9 @@ QWindowsStaticOpenGLContext *QWindowsIntegration::staticOpenGLContext()
     if (!integration)
         return 0;
     QWindowsIntegrationPrivate *d = integration->d.data();
+    QMutexLocker lock(&d->m_staticContextLock);
     if (d->m_staticOpenGLContext.isNull())
-        d->m_staticOpenGLContext = QSharedPointer<QWindowsStaticOpenGLContext>(QWindowsStaticOpenGLContext::create());
+        d->m_staticOpenGLContext.reset(QWindowsStaticOpenGLContext::create());
     return d->m_staticOpenGLContext.data();
 }
 #endif // !QT_NO_OPENGL
@@ -507,8 +559,8 @@ QVariant QWindowsIntegration::styleHint(QPlatformIntegration::StyleHint hint) co
     case QPlatformIntegration::FontSmoothingGamma:
         return QVariant(QWindowsFontDatabase::fontSmoothingGamma());
     case QPlatformIntegration::MouseDoubleClickInterval:
-        if (const int ms = GetDoubleClickTime())
-            return QVariant(ms);
+        if (const UINT ms = GetDoubleClickTime())
+            return QVariant(int(ms));
         break;
     case QPlatformIntegration::UseRtlExtensions:
         return QVariant(d->m_context.useRTLExtensions());
@@ -587,4 +639,9 @@ QPlatformServices *QWindowsIntegration::services() const
     return &d->m_services;
 }
 
+void QWindowsIntegration::beep() const
+{
+    MessageBeep(MB_OK);  // For QApplication
+}
+
 QT_END_NAMESPACE
