@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -59,7 +65,7 @@
 #include <QtGui/private/qguiapplication_p.h>
 
 #ifdef XCB_USE_EGL
-#include <EGL/egl.h>
+# include <QtPlatformSupport/private/qt_egl_p.h>
 #endif
 
 #ifdef XCB_USE_XLIB
@@ -119,6 +125,7 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
     , m_defaultVisualId(UINT_MAX)
 {
     m_instance = this;
+    qApp->setAttribute(Qt::AA_CompressHighFrequencyEvents, true);
 
     qRegisterMetaType<QXcbWindow*>();
 #ifdef XCB_USE_XLIB
@@ -158,7 +165,7 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
 
     bool underDebugger = runningUnderDebugger();
     if (noGrabArg && doGrabArg && underDebugger) {
-        qWarning() << "Both -nograb and -dograb command line arguments specified. Please pick one. -nograb takes prcedence";
+        qWarning("Both -nograb and -dograb command line arguments specified. Please pick one. -nograb takes prcedence");
         doGrabArg = false;
     }
 
@@ -197,7 +204,7 @@ QPlatformWindow *QXcbIntegration::createPlatformWindow(QWindow *window) const
 {
     QXcbScreen *screen = static_cast<QXcbScreen *>(window->screen()->handle());
     QXcbGlIntegration *glIntegration = screen->connection()->glIntegration();
-    if (window->type() != Qt::Desktop) {
+    if (window->type() != Qt::Desktop && window->supportsOpenGL()) {
         if (glIntegration) {
             QXcbWindow *xcbWindow = glIntegration->createWindow(window);
             xcbWindow->create();
@@ -244,17 +251,30 @@ QPlatformOffscreenSurface *QXcbIntegration::createPlatformOffscreenSurface(QOffs
 bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
     switch (cap) {
-    case ThreadedPixmaps: return true;
-    case OpenGL: return m_connections.first()->glIntegration();
-    case ThreadedOpenGL: return m_connections.at(0)->threadedEventHandling()
-                         && m_connections.at(0)->glIntegration()
-                             && m_connections.at(0)->glIntegration()->supportsThreadedOpenGL();
-    case WindowMasks: return true;
-    case MultipleWindows: return true;
-    case ForeignWindows: return true;
-    case SyncState: return true;
-    case RasterGLSurface: return true;
-    case SwitchableWidgetComposition: return true;
+    case OpenGL:
+    case ThreadedOpenGL:
+    {
+        const auto *connection = qAsConst(m_connections).first();
+        if (const auto *integration = connection->glIntegration())
+            return cap != ThreadedOpenGL
+                || (connection->threadedEventHandling() && integration->supportsThreadedOpenGL());
+        return false;
+    }
+
+    case ThreadedPixmaps:
+    case WindowMasks:
+    case MultipleWindows:
+    case ForeignWindows:
+    case SyncState:
+    case RasterGLSurface:
+        return true;
+
+    case SwitchableWidgetComposition:
+    {
+        return m_connections.at(0)->glIntegration()
+            && m_connections.at(0)->glIntegration()->supportsSwitchableWidgetComposition();
+    }
+
     default: return QPlatformIntegration::hasCapability(cap);
     }
 }
@@ -456,4 +476,17 @@ void QXcbIntegration::sync()
     }
 }
 
+// For QApplication::beep()
+void QXcbIntegration::beep() const
+{
+    QScreen *priScreen = QGuiApplication::primaryScreen();
+    if (!priScreen)
+        return;
+    QPlatformScreen *screen = priScreen->handle();
+    if (!screen)
+        return;
+    xcb_connection_t *connection = static_cast<QXcbScreen *>(screen)->xcb_connection();
+    xcb_bell(connection, 0);
+}
+
 QT_END_NAMESPACE
