@@ -1,31 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 John Layt <jlayt@kde.org>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -42,8 +48,6 @@
 
 QT_BEGIN_NAMESPACE
 
-#ifndef QT_NO_PRINTER
-
 QPpdPrintDevice::QPpdPrintDevice()
     : QPlatformPrintDevice(),
       m_cupsDest(0),
@@ -269,6 +273,7 @@ void QPpdPrintDevice::loadInputSlots() const
     if (m_ppd) {
         ppd_option_t *inputSlots = ppdFindOption(m_ppd, "InputSlot");
         if (inputSlots) {
+            m_inputSlots.reserve(inputSlots->num_choices);
             for (int i = 0; i < inputSlots->num_choices; ++i)
                 m_inputSlots.append(QPrintUtils::ppdChoiceToInputSlot(inputSlots->choices[i]));
         }
@@ -309,6 +314,7 @@ void QPpdPrintDevice::loadOutputBins() const
     if (m_ppd) {
         ppd_option_t *outputBins = ppdFindOption(m_ppd, "OutputBin");
         if (outputBins) {
+            m_outputBins.reserve(outputBins->num_choices);
             for (int i = 0; i < outputBins->num_choices; ++i)
                 m_outputBins.append(QPrintUtils::ppdChoiceToOutputBin(outputBins->choices[i]));
         }
@@ -350,6 +356,7 @@ void QPpdPrintDevice::loadDuplexModes() const
     if (m_ppd) {
         ppd_option_t *duplexModes = ppdFindOption(m_ppd, "Duplex");
         if (duplexModes) {
+            m_duplexModes.reserve(duplexModes->num_choices);
             for (int i = 0; i < duplexModes->num_choices; ++i)
                 m_duplexModes.append(QPrintUtils::ppdChoiceToDuplexMode(duplexModes->choices[i].choice));
         }
@@ -472,6 +479,4 @@ cups_ptype_e QPpdPrintDevice::printerTypeFlags() const
     return static_cast<cups_ptype_e>(printerOption("printer-type").toUInt());
 }
 
-#endif // QT_NO_PRINTER
-
 QT_END_NAMESPACE
