@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -157,7 +163,8 @@ QSqlDriver::~QSqlDriver()
 
 bool QSqlDriver::isOpen() const
 {
-    return d_func()->isOpen;
+    Q_D(const QSqlDriver);
+    return d->isOpen;
 }
 
 /*!
@@ -167,7 +174,8 @@ bool QSqlDriver::isOpen() const
 
 bool QSqlDriver::isOpenError() const
 {
-    return d_func()->isOpenError;
+    Q_D(const QSqlDriver);
+    return d->isOpenError;
 }
 
 /*!
@@ -274,7 +282,8 @@ bool QSqlDriver::isOpenError() const
 
 void QSqlDriver::setOpen(bool open)
 {
-    d_func()->isOpen = open;
+    Q_D(QSqlDriver);
+    d->isOpen = open;
 }
 
 /*!
@@ -288,9 +297,10 @@ void QSqlDriver::setOpen(bool open)
 
 void QSqlDriver::setOpenError(bool error)
 {
-    d_func()->isOpenError = error;
+    Q_D(QSqlDriver);
+    d->isOpenError = error;
     if (error)
-        d_func()->isOpen = false;
+        d->isOpen = false;
 }
 
 /*!
@@ -341,7 +351,8 @@ bool QSqlDriver::rollbackTransaction()
 
 void QSqlDriver::setLastError(const QSqlError &error)
 {
-    d_func()->error = error;
+    Q_D(QSqlDriver);
+    d->error = error;
 }
 
 /*!
@@ -351,7 +362,8 @@ void QSqlDriver::setLastError(const QSqlError &error)
 
 QSqlError QSqlDriver::lastError() const
 {
-    return d_func()->error;
+    Q_D(const QSqlDriver);
+    return d->error;
 }
 
 /*!
@@ -454,6 +466,9 @@ QString QSqlDriver::stripDelimiters(const QString &identifier, IdentifierType ty
     with the values from \a rec. If \a preparedStatement is true, the
     string will contain placeholders instead of values.
 
+    The generated flag in each field of \a rec determines whether the
+    field is included in the generated statement.
+
     This method can be used to manipulate tables without having to worry
     about database-dependent SQL dialects. For non-prepared statements,
     the values will be properly escaped.
@@ -492,7 +507,9 @@ QString QSqlDriver::sqlStatement(StatementType type, const QString &tableName,
             ? QString()
             : prepareIdentifier(tableName, QSqlDriver::TableName, this) + QLatin1Char('.');
         for (int i = 0; i < rec.count(); ++i) {
-            s.append(QLatin1String(i? " AND " : "WHERE "));
+            if (!rec.isGenerated(i))
+                continue;
+            s.append(s.isEmpty() ? QLatin1String("WHERE ") : QLatin1String(" AND "));
             s.append(tableNamePrefix);
             s.append(prepareIdentifier(rec.fieldName(i), QSqlDriver::FieldName, this));
             if (rec.isNull(i))
@@ -654,7 +671,6 @@ QString QSqlDriver::formatValue(const QSqlField &field, bool trimStrings) const
                 break;
             }
         }
-            // fall through
         default:
             r = field.value().toString();
             break;
@@ -770,7 +786,8 @@ QStringList QSqlDriver::subscribedToNotifications() const
 */
 void QSqlDriver::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy precisionPolicy)
 {
-    d_func()->precisionPolicy = precisionPolicy;
+    Q_D(QSqlDriver);
+    d->precisionPolicy = precisionPolicy;
 }
 
 /*!
@@ -783,7 +800,8 @@ void QSqlDriver::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy prec
 */
 QSql::NumericalPrecisionPolicy QSqlDriver::numericalPrecisionPolicy() const
 {
-    return d_func()->precisionPolicy;
+    Q_D(const QSqlDriver);
+    return d->precisionPolicy;
 }
 
 /*!
@@ -793,7 +811,8 @@ QSql::NumericalPrecisionPolicy QSqlDriver::numericalPrecisionPolicy() const
 */
 QSqlDriver::DbmsType QSqlDriver::dbmsType() const
 {
-    return d_func()->dbmsType;
+    Q_D(const QSqlDriver);
+    return d->dbmsType;
 }
 
 /*!
