@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -75,7 +62,7 @@ static const char docTypeHeader[] =
 
 #define PROGRAMNAME     "qdbuscpp2xml"
 #define PROGRAMVERSION  "0.2"
-#define PROGRAMCOPYRIGHT "Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies)."
+#define PROGRAMCOPYRIGHT "Copyright (C) 2016 The Qt Company Ltd."
 
 static QString outputFile;
 static int flags;
@@ -99,8 +86,9 @@ static const char help[] =
 int qDBusParametersForMethod(const FunctionDef &mm, QVector<int>& metaTypes, QString &errorMsg)
 {
     QList<QByteArray> parameterTypes;
+    parameterTypes.reserve(mm.arguments.size());
 
-    foreach (const ArgumentDef &arg, mm.arguments)
+    for (const ArgumentDef &arg : mm.arguments)
         parameterTypes.append(arg.normalizedType);
 
     return qDBusParametersForMethod(parameterTypes, metaTypes, errorMsg);
@@ -139,7 +127,7 @@ static QString addFunction(const FunctionDef &mm, bool isSignal = false) {
             return QString();           // wasn't a valid type
         }
     }
-    QList<ArgumentDef> names = mm.arguments;
+    QVector<ArgumentDef> names = mm.arguments;
     QVector<int> types;
     QString errorMsg;
     int inputCount = qDBusParametersForMethod(mm, types, errorMsg);
@@ -213,7 +201,7 @@ static QString generateInterfaceXml(const ClassDef *mo)
     if (flags & (QDBusConnection::ExportScriptableProperties |
                  QDBusConnection::ExportNonScriptableProperties)) {
         static const char *accessvalues[] = {0, "read", "write", "readwrite"};
-        foreach (const PropertyDef &mp, mo->propertyList) {
+        for (const PropertyDef &mp : mo->propertyList) {
             if (!((!mp.scriptable.isEmpty() && (flags & QDBusConnection::ExportScriptableProperties)) ||
                   (!mp.scriptable.isEmpty() && (flags & QDBusConnection::ExportNonScriptableProperties))))
                 continue;
@@ -248,20 +236,26 @@ static QString generateInterfaceXml(const ClassDef *mo)
     // now add methods:
 
     if (flags & (QDBusConnection::ExportScriptableSignals | QDBusConnection::ExportNonScriptableSignals)) {
-        foreach (const FunctionDef &mm, mo->signalList) {
+        for (const FunctionDef &mm : mo->signalList) {
             if (mm.wasCloned)
                 continue;
+            if (!mm.isScriptable && !(flags & QDBusConnection::ExportNonScriptableSignals))
+                continue;
 
             retval += addFunction(mm, true);
         }
     }
 
     if (flags & (QDBusConnection::ExportScriptableSlots | QDBusConnection::ExportNonScriptableSlots)) {
-        foreach (const FunctionDef &slot, mo->slotList) {
+        for (const FunctionDef &slot : mo->slotList) {
+            if (!slot.isScriptable && !(flags & QDBusConnection::ExportNonScriptableSlots))
+                continue;
             if (slot.access == FunctionDef::Public)
               retval += addFunction(slot);
         }
-        foreach (const FunctionDef &method, mo->methodList) {
+        for (const FunctionDef &method : mo->methodList) {
+            if (!method.isScriptable && !(flags & QDBusConnection::ExportNonScriptableSlots))
+                continue;
             if (method.access == FunctionDef::Public)
               retval += addFunction(method);
         }
@@ -273,7 +267,7 @@ QString qDBusInterfaceFromClassDef(const ClassDef *mo)
 {
     QString interface;
 
-    foreach (const ClassInfoDef &cid, mo->classInfoList) {
+    for (const ClassInfoDef &cid : mo->classInfoList) {
         if (cid.name == QCLASSINFO_DBUS_INTERFACE)
             return QString::fromUtf8(cid.value);
     }
@@ -296,7 +290,7 @@ QString qDBusInterfaceFromClassDef(const ClassDef *mo)
 
 QString qDBusGenerateClassDefXml(const ClassDef *cdef)
 {
-    foreach (const ClassInfoDef &cid, cdef->classInfoList) {
+    for (const ClassInfoDef &cid : cdef->classInfoList) {
         if (cid.name == QCLASSINFO_DBUS_INTROSPECTION)
             return QString::fromUtf8(cid.value);
     }
@@ -398,11 +392,12 @@ static void parseCmdLine(QStringList &arguments)
 int main(int argc, char **argv)
 {
     QStringList args;
+    args.reserve(argc - 1);
     for (int n = 1; n < argc; ++n)
         args.append(QString::fromLocal8Bit(argv[n]));
     parseCmdLine(args);
 
-    QList<ClassDef> classes;
+    QVector<ClassDef> classes;
 
     for (int i = 0; i < args.count(); ++i) {
         const QString arg = args.at(i);
@@ -422,7 +417,7 @@ int main(int argc, char **argv)
         pp.macros["Q_MOC_RUN"];
         pp.macros["__cplusplus"];
 
-        const QByteArray filename = QFile::decodeName(argv[i]).toLatin1();
+        const QByteArray filename = arg.toLocal8Bit();
 
         moc.filename = filename;
         moc.currentFilenames.push(filename);
@@ -451,7 +446,7 @@ int main(int argc, char **argv)
 
     output.write(docTypeHeader);
     output.write("<node>\n");
-    foreach (const ClassDef &cdef, classes) {
+    for (const ClassDef &cdef : qAsConst(classes)) {
         QString xml = qDBusGenerateClassDefXml(&cdef);
         output.write(xml.toLocal8Bit());
     }
