@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -132,7 +138,7 @@ QAccessibleInterface *QAccessibleTable::cellAt(int row, int column) const
         return 0;
     Q_ASSERT(role() != QAccessible::Tree);
     QModelIndex index = view()->model()->index(row, column, view()->rootIndex());
-    if (!index.isValid()) {
+    if (Q_UNLIKELY(!index.isValid())) {
         qWarning() << "QAccessibleTable::cellAt: invalid index: " << index << " for " << view();
         return 0;
     }
@@ -505,7 +511,7 @@ QAccessibleInterface *QAccessibleTable::child(int logicalIndex) const
 
     if (!iface) {
         QModelIndex index = view()->model()->index(row, column, view()->rootIndex());
-        if (!index.isValid()) {
+        if (Q_UNLIKELY(!index.isValid())) {
             qWarning() << "QAccessibleTable::child: Invalid index at: " << row << column;
             return 0;
         }
@@ -550,8 +556,20 @@ void QAccessibleTable::modelChange(QAccessibleTableModelChangeEvent *event)
             QAccessible::Id id = iter.value();
             QAccessibleInterface *iface = QAccessible::accessibleInterface(id);
             Q_ASSERT(iface);
-            if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsInserted
-                && iface->role() == QAccessible::RowHeader) {
+            if (iface->role() == QAccessible::Cell || iface->role() == QAccessible::ListItem) {
+                Q_ASSERT(iface->tableCellInterface());
+                QAccessibleTableCell *cell = static_cast<QAccessibleTableCell*>(iface->tableCellInterface());
+                if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsInserted
+                        && cell->m_index.row() >= event->firstRow()) {
+                    int newRow = cell->m_index.row() + newRows;
+                    cell->m_index = cell->m_index.sibling(newRow, cell->m_index.column());
+                } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::ColumnsInserted
+                        && cell->m_index.column() >= event->firstColumn()) {
+                    int newColumn = cell->m_index.column() + newColumns;
+                    cell->m_index = cell->m_index.sibling(cell->m_index.row(), newColumn);
+                }
+            } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsInserted
+                       && iface->role() == QAccessible::RowHeader) {
                 QAccessibleTableHeaderCell *cell = static_cast<QAccessibleTableHeaderCell*>(iface);
                 if (cell->index >= event->firstRow()) {
                     cell->index += newRows;
@@ -590,11 +608,27 @@ void QAccessibleTable::modelChange(QAccessibleTableModelChangeEvent *event)
             if (iface->role() == QAccessible::Cell || iface->role() == QAccessible::ListItem) {
                 Q_ASSERT(iface->tableCellInterface());
                 QAccessibleTableCell *cell = static_cast<QAccessibleTableCell*>(iface->tableCellInterface());
-                // Since it is a QPersistentModelIndex, we only need to check if it is valid
-                if (cell->m_index.isValid())
-                    newCache.insert(indexOfChild(cell), id);
-                else
-                    QAccessible::deleteAccessibleInterface(id);
+                if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsRemoved) {
+                    if (cell->m_index.row() < event->firstRow()) {
+                        newCache.insert(indexOfChild(cell), id);
+                    } else if (cell->m_index.row() > event->lastRow()) {
+                        int newRow = cell->m_index.row() - deletedRows;
+                        cell->m_index = cell->m_index.sibling(newRow, cell->m_index.column());
+                        newCache.insert(indexOfChild(cell), id);
+                    } else {
+                        QAccessible::deleteAccessibleInterface(id);
+                    }
+                } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::ColumnsRemoved) {
+                    if (cell->m_index.column() < event->firstColumn()) {
+                        newCache.insert(indexOfChild(cell), id);
+                    } else if (cell->m_index.column() > event->lastColumn()) {
+                        int newColumn = cell->m_index.column() - deletedColumns;
+                        cell->m_index = cell->m_index.sibling(cell->m_index.row(), newColumn);
+                        newCache.insert(indexOfChild(cell), id);
+                    } else {
+                        QAccessible::deleteAccessibleInterface(id);
+                    }
+                }
             } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsRemoved
                        && iface->role() == QAccessible::RowHeader) {
                 QAccessibleTableHeaderCell *cell = static_cast<QAccessibleTableHeaderCell*>(iface);
@@ -638,7 +672,7 @@ QModelIndex QAccessibleTree::indexFromLogical(int row, int column) const
         return QModelIndex();
 
     const QTreeView *treeView = qobject_cast<const QTreeView*>(view());
-    if ((row < 0) || (column < 0) || (treeView->d_func()->viewItems.count() <= row)) {
+    if (Q_UNLIKELY(row < 0 || column < 0 || treeView->d_func()->viewItems.count() <= row)) {
         qWarning() << "QAccessibleTree::indexFromLogical: invalid index: " << row << column << " for " << treeView;
         return QModelIndex();
     }
@@ -748,7 +782,7 @@ int QAccessibleTree::indexOfChild(const QAccessibleInterface *iface) const
 QAccessibleInterface *QAccessibleTree::cellAt(int row, int column) const
 {
     QModelIndex index = indexFromLogical(row, column);
-    if (!index.isValid()) {
+    if (Q_UNLIKELY(!index.isValid())) {
         qWarning() << "Requested invalid tree cell: " << row << column;
         return 0;
     }
@@ -807,7 +841,7 @@ bool QAccessibleTree::selectRow(int row)
 QAccessibleTableCell::QAccessibleTableCell(QAbstractItemView *view_, const QModelIndex &index_, QAccessible::Role role_)
     : /* QAccessibleSimpleEditableTextInterface(this), */ view(view_), m_index(index_), m_role(role_)
 {
-    if (!index_.isValid())
+    if (Q_UNLIKELY(!index_.isValid()))
         qWarning() << "QAccessibleTableCell::QAccessibleTableCell with invalid index: " << index_;
 }
 
