@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -1188,6 +1194,10 @@ void QInputDialog::done(int result)
 
     \snippet dialogs/standarddialogs/dialog.cpp 3
 
+    \warning Do not delete \a parent during the execution of the dialog. If you
+    want to do this, you should create the dialog yourself using one of the
+    QInputDialog constructors.
+
     \sa getInt(), getDouble(), getItem(), getMultiLineText()
 */
 
@@ -1195,18 +1205,18 @@ QString QInputDialog::getText(QWidget *parent, const QString &title, const QStri
                               QLineEdit::EchoMode mode, const QString &text, bool *ok,
                               Qt::WindowFlags flags, Qt::InputMethodHints inputMethodHints)
 {
-    QAutoPointer<QInputDialog> dialog(new QInputDialog(parent, flags));
-    dialog->setWindowTitle(title);
-    dialog->setLabelText(label);
-    dialog->setTextValue(text);
-    dialog->setTextEchoMode(mode);
-    dialog->setInputMethodHints(inputMethodHints);
+    QInputDialog dialog(parent, flags);
+    dialog.setWindowTitle(title);
+    dialog.setLabelText(label);
+    dialog.setTextValue(text);
+    dialog.setTextEchoMode(mode);
+    dialog.setInputMethodHints(inputMethodHints);
 
-    const int ret = dialog->exec();
+    int ret = dialog.exec();
     if (ok)
         *ok = !!ret;
     if (ret) {
-        return dialog->textValue();
+        return dialog.textValue();
     } else {
         return QString();
     }
@@ -1236,6 +1246,10 @@ QString QInputDialog::getText(QWidget *parent, const QString &title, const QStri
 
     \snippet dialogs/standarddialogs/dialog.cpp 4
 
+    \warning Do not delete \a parent during the execution of the dialog. If you
+    want to do this, you should create the dialog yourself using one of the
+    QInputDialog constructors.
+
     \sa getInt(), getDouble(), getItem(), getText()
 */
 
@@ -1243,18 +1257,18 @@ QString QInputDialog::getMultiLineText(QWidget *parent, const QString &title, co
                                        const QString &text, bool *ok, Qt::WindowFlags flags,
                                        Qt::InputMethodHints inputMethodHints)
 {
-    QAutoPointer<QInputDialog> dialog(new QInputDialog(parent, flags));
-    dialog->setOptions(QInputDialog::UsePlainTextEditForTextInput);
-    dialog->setWindowTitle(title);
-    dialog->setLabelText(label);
-    dialog->setTextValue(text);
-    dialog->setInputMethodHints(inputMethodHints);
+    QInputDialog dialog(parent, flags);
+    dialog.setOptions(QInputDialog::UsePlainTextEditForTextInput);
+    dialog.setWindowTitle(title);
+    dialog.setLabelText(label);
+    dialog.setTextValue(text);
+    dialog.setInputMethodHints(inputMethodHints);
 
-    const int ret = dialog->exec();
+    int ret = dialog.exec();
     if (ok)
         *ok = !!ret;
     if (ret) {
-        return dialog->textValue();
+        return dialog.textValue();
     } else {
         return QString();
     }
@@ -1284,24 +1298,28 @@ QString QInputDialog::getMultiLineText(QWidget *parent, const QString &title, co
 
     \snippet dialogs/standarddialogs/dialog.cpp 0
 
+    \warning Do not delete \a parent during the execution of the dialog. If you
+    want to do this, you should create the dialog yourself using one of the
+    QInputDialog constructors.
+
     \sa getText(), getDouble(), getItem(), getMultiLineText()
 */
 
 int QInputDialog::getInt(QWidget *parent, const QString &title, const QString &label, int value,
                          int min, int max, int step, bool *ok, Qt::WindowFlags flags)
 {
-    QAutoPointer<QInputDialog> dialog(new QInputDialog(parent, flags));
-    dialog->setWindowTitle(title);
-    dialog->setLabelText(label);
-    dialog->setIntRange(min, max);
-    dialog->setIntValue(value);
-    dialog->setIntStep(step);
+    QInputDialog dialog(parent, flags);
+    dialog.setWindowTitle(title);
+    dialog.setLabelText(label);
+    dialog.setIntRange(min, max);
+    dialog.setIntValue(value);
+    dialog.setIntStep(step);
 
-    const int ret = dialog->exec();
+    int ret = dialog.exec();
     if (ok)
         *ok = !!ret;
     if (ret) {
-        return dialog->intValue();
+        return dialog.intValue();
     } else {
         return value;
     }
@@ -1332,6 +1350,10 @@ int QInputDialog::getInt(QWidget *parent, const QString &title, const QString &l
 
     \snippet dialogs/standarddialogs/dialog.cpp 0
 
+    \warning Do not delete \a parent during the execution of the dialog. If you
+    want to do this, you should create the dialog yourself using one of the
+    QInputDialog constructors.
+
     \sa getText(), getDouble(), getItem(), getMultiLineText()
 */
 
@@ -1357,6 +1379,10 @@ int QInputDialog::getInt(QWidget *parent, const QString &title, const QString &l
 
     \snippet dialogs/standarddialogs/dialog.cpp 1
 
+    \warning Do not delete \a parent during the execution of the dialog. If you
+    want to do this, you should create the dialog yourself using one of the
+    QInputDialog constructors.
+
     \sa getText(), getInt(), getItem(), getMultiLineText()
 */
 
@@ -1364,18 +1390,18 @@ double QInputDialog::getDouble(QWidget *parent, const QString &title, const QStr
                                double value, double min, double max, int decimals, bool *ok,
                                Qt::WindowFlags flags)
 {
-    QAutoPointer<QInputDialog> dialog(new QInputDialog(parent, flags));
-    dialog->setWindowTitle(title);
-    dialog->setLabelText(label);
-    dialog->setDoubleDecimals(decimals);
-    dialog->setDoubleRange(min, max);
-    dialog->setDoubleValue(value);
+    QInputDialog dialog(parent, flags);
+    dialog.setWindowTitle(title);
+    dialog.setLabelText(label);
+    dialog.setDoubleDecimals(decimals);
+    dialog.setDoubleRange(min, max);
+    dialog.setDoubleValue(value);
 
-    const int ret = dialog->exec();
+    int ret = dialog.exec();
     if (ok)
         *ok = !!ret;
     if (ret) {
-        return dialog->doubleValue();
+        return dialog.doubleValue();
     } else {
         return value;
     }
@@ -1407,6 +1433,10 @@ double QInputDialog::getDouble(QWidget *parent, const QString &title, const QStr
 
     \snippet dialogs/standarddialogs/dialog.cpp 2
 
+    \warning Do not delete \a parent during the execution of the dialog. If you
+    want to do this, you should create the dialog yourself using one of the
+    QInputDialog constructors.
+
     \sa getText(), getInt(), getDouble(), getMultiLineText()
 */
 
@@ -1416,19 +1446,19 @@ QString QInputDialog::getItem(QWidget *parent, const QString &title, const QStri
 {
     QString text(items.value(current));
 
-    QAutoPointer<QInputDialog> dialog(new QInputDialog(parent, flags));
-    dialog->setWindowTitle(title);
-    dialog->setLabelText(label);
-    dialog->setComboBoxItems(items);
-    dialog->setTextValue(text);
-    dialog->setComboBoxEditable(editable);
-    dialog->setInputMethodHints(inputMethodHints);
+    QInputDialog dialog(parent, flags);
+    dialog.setWindowTitle(title);
+    dialog.setLabelText(label);
+    dialog.setComboBoxItems(items);
+    dialog.setTextValue(text);
+    dialog.setComboBoxEditable(editable);
+    dialog.setInputMethodHints(inputMethodHints);
 
-    const int ret = dialog->exec();
+    int ret = dialog.exec();
     if (ok)
         *ok = !!ret;
     if (ret) {
-        return dialog->textValue();
+        return dialog.textValue();
     } else {
         return text;
     }
