@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -116,7 +122,6 @@ public:
     qreal step;
 
     struct Pair {
-        Pair(qreal a, qreal b) : step(a), value(b) {}
         bool operator <(const Pair &other) const
         { return step < other.step; }
         bool operator==(const Pair &other) const
@@ -124,64 +129,63 @@ public:
         qreal step;
         qreal value;
     };
-    QList<Pair> xPosition;
-    QList<Pair> yPosition;
-    QList<Pair> rotation;
-    QList<Pair> verticalScale;
-    QList<Pair> horizontalScale;
-    QList<Pair> verticalShear;
-    QList<Pair> horizontalShear;
-    QList<Pair> xTranslation;
-    QList<Pair> yTranslation;
+    QVector<Pair> xPosition;
+    QVector<Pair> yPosition;
+    QVector<Pair> rotation;
+    QVector<Pair> verticalScale;
+    QVector<Pair> horizontalScale;
+    QVector<Pair> verticalShear;
+    QVector<Pair> horizontalShear;
+    QVector<Pair> xTranslation;
+    QVector<Pair> yTranslation;
 
-    qreal linearValueForStep(qreal step, QList<Pair> *source, qreal defaultValue = 0);
-    void insertUniquePair(qreal step, qreal value, QList<Pair> *binList, const char* method);
+    qreal linearValueForStep(qreal step, const QVector<Pair> &source, qreal defaultValue = 0);
+    void insertUniquePair(qreal step, qreal value, QVector<Pair> *binList, const char* method);
 };
+Q_DECLARE_TYPEINFO(QGraphicsItemAnimationPrivate::Pair, Q_PRIMITIVE_TYPE);
 
-qreal QGraphicsItemAnimationPrivate::linearValueForStep(qreal step, QList<Pair> *source, qreal defaultValue)
+qreal QGraphicsItemAnimationPrivate::linearValueForStep(qreal step, const QVector<Pair> &source, qreal defaultValue)
 {
-    if (source->isEmpty())
+    if (source.isEmpty())
         return defaultValue;
     step = qMin<qreal>(qMax<qreal>(step, 0), 1);
 
     if (step == 1)
-        return source->last().value;
+        return source.back().value;
 
     qreal stepBefore = 0;
     qreal stepAfter = 1;
-    qreal valueBefore = source->first().step == 0 ? source->first().value : defaultValue;
-    qreal valueAfter = source->last().value;
+    qreal valueBefore = source.front().step == 0 ? source.front().value : defaultValue;
+    qreal valueAfter = source.back().value;
 
     // Find the closest step and value before the given step.
-    for (int i = 0; i < source->size() && step >= source->at(i).step; ++i) {
-        stepBefore = source->at(i).step;
-        valueBefore = source->at(i).value;
+    for (int i = 0; i < source.size() && step >= source[i].step; ++i) {
+        stepBefore = source[i].step;
+        valueBefore = source[i].value;
     }
 
     // Find the closest step and value after the given step.
-    for (int j = source->size() - 1; j >= 0 && step < source->at(j).step; --j) {
-        stepAfter = source->at(j).step;
-        valueAfter = source->at(j).value;
+    for (int i = source.size() - 1; i >= 0 && step < source[i].step; --i) {
+        stepAfter = source[i].step;
+        valueAfter = source[i].value;
     }
 
     // Do a simple linear interpolation.
     return valueBefore + (valueAfter - valueBefore) * ((step - stepBefore) / (stepAfter - stepBefore));
 }
 
-void QGraphicsItemAnimationPrivate::insertUniquePair(qreal step, qreal value, QList<Pair> *binList, const char* method)
+void QGraphicsItemAnimationPrivate::insertUniquePair(qreal step, qreal value, QVector<Pair> *binList, const char* method)
 {
     if (!check_step_valid(step, method))
         return;
 
-    Pair pair(step, value);
+    const Pair pair = { step, value };
 
-    QList<Pair>::iterator result = std::lower_bound(binList->begin(), binList->end(), pair);
-    if ((result != binList->end()) && !(pair < *result))
+    const QVector<Pair>::iterator result = std::lower_bound(binList->begin(), binList->end(), pair);
+    if (result == binList->end() || pair < *result)
+        binList->insert(result, pair);
+    else
         result->value = value;
-    else {
-        *binList << pair;
-        std::sort(binList->begin(), binList->end());
-    }
 }
 
 /*!
@@ -259,8 +263,8 @@ void QGraphicsItemAnimation::setTimeLine(QTimeLine *timeLine)
 QPointF QGraphicsItemAnimation::posAt(qreal step) const
 {
     check_step_valid(step, "posAt");
-    return QPointF(d->linearValueForStep(step, &d->xPosition, d->startPos.x()),
-                   d->linearValueForStep(step, &d->yPosition, d->startPos.y()));
+    return QPointF(d->linearValueForStep(step, d->xPosition, d->startPos.x()),
+                   d->linearValueForStep(step, d->yPosition, d->startPos.y()));
 }
 
 /*!
@@ -284,7 +288,9 @@ void QGraphicsItemAnimation::setPosAt(qreal step, const QPointF &pos)
 QList<QPair<qreal, QPointF> > QGraphicsItemAnimation::posList() const
 {
     QList<QPair<qreal, QPointF> > list;
-    for (int i = 0; i < d->xPosition.size(); ++i)
+    const int xPosCount = d->xPosition.size();
+    list.reserve(xPosCount);
+    for (int i = 0; i < xPosCount; ++i)
         list << QPair<qreal, QPointF>(d->xPosition.at(i).step, QPointF(d->xPosition.at(i).value, d->yPosition.at(i).value));
 
     return list;
@@ -317,7 +323,7 @@ QMatrix QGraphicsItemAnimation::matrixAt(qreal step) const
 qreal QGraphicsItemAnimation::rotationAt(qreal step) const
 {
     check_step_valid(step, "rotationAt");
-    return d->linearValueForStep(step, &d->rotation);
+    return d->linearValueForStep(step, d->rotation);
 }
 
 /*!
@@ -338,7 +344,9 @@ void QGraphicsItemAnimation::setRotationAt(qreal step, qreal angle)
 QList<QPair<qreal, qreal> > QGraphicsItemAnimation::rotationList() const
 {
     QList<QPair<qreal, qreal> > list;
-    for (int i = 0; i < d->rotation.size(); ++i)
+    const int numRotations = d->rotation.size();
+    list.reserve(numRotations);
+    for (int i = 0; i < numRotations; ++i)
         list << QPair<qreal, qreal>(d->rotation.at(i).step, d->rotation.at(i).value);
 
     return list;
@@ -352,7 +360,7 @@ QList<QPair<qreal, qreal> > QGraphicsItemAnimation::rotationList() const
 qreal QGraphicsItemAnimation::xTranslationAt(qreal step) const
 {
     check_step_valid(step, "xTranslationAt");
-    return d->linearValueForStep(step, &d->xTranslation);
+    return d->linearValueForStep(step, d->xTranslation);
 }
 
 /*!
@@ -363,7 +371,7 @@ qreal QGraphicsItemAnimation::xTranslationAt(qreal step) const
 qreal QGraphicsItemAnimation::yTranslationAt(qreal step) const
 {
     check_step_valid(step, "yTranslationAt");
-    return d->linearValueForStep(step, &d->yTranslation);
+    return d->linearValueForStep(step, d->yTranslation);
 }
 
 /*!
@@ -386,7 +394,9 @@ void QGraphicsItemAnimation::setTranslationAt(qreal step, qreal dx, qreal dy)
 QList<QPair<qreal, QPointF> > QGraphicsItemAnimation::translationList() const
 {
     QList<QPair<qreal, QPointF> > list;
-    for (int i = 0; i < d->xTranslation.size(); ++i)
+    const int numTranslations = d->xTranslation.size();
+    list.reserve(numTranslations);
+    for (int i = 0; i < numTranslations; ++i)
         list << QPair<qreal, QPointF>(d->xTranslation.at(i).step, QPointF(d->xTranslation.at(i).value, d->yTranslation.at(i).value));
 
     return list;
@@ -401,7 +411,7 @@ qreal QGraphicsItemAnimation::verticalScaleAt(qreal step) const
 {
     check_step_valid(step, "verticalScaleAt");
 
-    return d->linearValueForStep(step, &d->verticalScale, 1);
+    return d->linearValueForStep(step, d->verticalScale, 1);
 }
 
 /*!
@@ -412,7 +422,7 @@ qreal QGraphicsItemAnimation::verticalScaleAt(qreal step) const
 qreal QGraphicsItemAnimation::horizontalScaleAt(qreal step) const
 {
     check_step_valid(step, "horizontalScaleAt");
-    return d->linearValueForStep(step, &d->horizontalScale, 1);
+    return d->linearValueForStep(step, d->horizontalScale, 1);
 }
 
 /*!
@@ -435,7 +445,9 @@ void QGraphicsItemAnimation::setScaleAt(qreal step, qreal sx, qreal sy)
 QList<QPair<qreal, QPointF> > QGraphicsItemAnimation::scaleList() const
 {
     QList<QPair<qreal, QPointF> > list;
-    for (int i = 0; i < d->horizontalScale.size(); ++i)
+    const int numScales = d->horizontalScale.size();
+    list.reserve(numScales);
+    for (int i = 0; i < numScales; ++i)
         list << QPair<qreal, QPointF>(d->horizontalScale.at(i).step, QPointF(d->horizontalScale.at(i).value, d->verticalScale.at(i).value));
 
     return list;
@@ -449,7 +461,7 @@ QList<QPair<qreal, QPointF> > QGraphicsItemAnimation::scaleList() const
 qreal QGraphicsItemAnimation::verticalShearAt(qreal step) const
 {
     check_step_valid(step, "verticalShearAt");
-    return d->linearValueForStep(step, &d->verticalShear, 0);
+    return d->linearValueForStep(step, d->verticalShear, 0);
 }
 
 /*!
@@ -460,7 +472,7 @@ qreal QGraphicsItemAnimation::verticalShearAt(qreal step) const
 qreal QGraphicsItemAnimation::horizontalShearAt(qreal step) const
 {
     check_step_valid(step, "horizontalShearAt");
-    return d->linearValueForStep(step, &d->horizontalShear, 0);
+    return d->linearValueForStep(step, d->horizontalShear, 0);
 }
 
 /*!
@@ -483,7 +495,9 @@ void QGraphicsItemAnimation::setShearAt(qreal step, qreal sh, qreal sv)
 QList<QPair<qreal, QPointF> > QGraphicsItemAnimation::shearList() const
 {
     QList<QPair<qreal, QPointF> > list;
-    for (int i = 0; i < d->horizontalShear.size(); ++i)
+    const int numShears = d->horizontalShear.size();
+    list.reserve(numShears);
+    for (int i = 0; i < numShears; ++i)
         list << QPair<qreal, QPointF>(d->horizontalShear.at(i).step, QPointF(d->horizontalShear.at(i).value, d->verticalShear.at(i).value));
 
     return list;
@@ -580,4 +594,6 @@ void QGraphicsItemAnimation::afterAnimationStep(qreal step)
 
 QT_END_NAMESPACE
 
+#include "moc_qgraphicsitemanimation.cpp"
+
 #endif // QT_NO_GRAPHICSVIEW
