@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -77,6 +83,8 @@
 #include <QtCore/qmath.h>
 #include <QtCore/qdebug.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 static inline int intmaxlog(int n)
@@ -545,23 +553,18 @@ QList<QGraphicsItem *> QGraphicsSceneBspTreeIndex::items(Qt::SortOrder order) co
     Q_D(const QGraphicsSceneBspTreeIndex);
     const_cast<QGraphicsSceneBspTreeIndexPrivate*>(d)->purgeRemovedItems();
     QList<QGraphicsItem *> itemList;
+    itemList.reserve(d->indexedItems.size() + d->unindexedItems.size());
+
+    // Rebuild the list of items to avoid holes. ### We could also just
+    // compress the item lists at this point.
+    QGraphicsItem *null = nullptr; // work-around for (at least) MSVC 2012 emitting
+                                   // warning C4100 for its own header <algorithm>
+                                   // when passing nullptr directly to remove_copy:
+    std::remove_copy(d->indexedItems.cbegin(), d->indexedItems.cend(),
+                     std::back_inserter(itemList), null);
+    std::remove_copy(d->unindexedItems.cbegin(), d->unindexedItems.cend(),
+                     std::back_inserter(itemList), null);
 
-    // If freeItemIndexes is empty, we know there are no holes in indexedItems and
-    // unindexedItems.
-    if (d->freeItemIndexes.isEmpty()) {
-        if (d->unindexedItems.isEmpty()) {
-            itemList = d->indexedItems;
-        } else {
-            itemList = d->indexedItems + d->unindexedItems;
-        }
-    } else {
-        // Rebuild the list of items to avoid holes. ### We could also just
-        // compress the item lists at this point.
-        foreach (QGraphicsItem *item, d->indexedItems + d->unindexedItems) {
-            if (item)
-                itemList << item;
-        }
-    }
     d->sortItems(&itemList, order, d->sortCacheEnabled);
     return itemList;
 }
@@ -691,7 +694,8 @@ void QGraphicsSceneBspTreeIndex::itemChange(const QGraphicsItem *item, QGraphics
 bool QGraphicsSceneBspTreeIndex::event(QEvent *event)
 {
     Q_D(QGraphicsSceneBspTreeIndex);
-    if (event->type() == QEvent::Timer) {
+    switch (event->type()) {
+    case QEvent::Timer:
             if (d->indexTimerId && static_cast<QTimerEvent *>(event)->timerId() == d->indexTimerId) {
                 if (d->restartIndexTimer) {
                     d->restartIndexTimer = false;
@@ -700,8 +704,11 @@ bool QGraphicsSceneBspTreeIndex::event(QEvent *event)
                     d->_q_updateIndex();
                 }
             }
+     // Fallthrough intended - support timers in subclasses.
+    default:
+        return QObject::event(event);
     }
-    return QObject::event(event);
+    return true;
 }
 
 QT_END_NAMESPACE
