@@ -1,39 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -332,11 +330,14 @@ void QColumnView::scrollTo(const QModelIndex &index, ScrollHint hint)
     }
 
 #ifndef QT_NO_ANIMATION
-    d->currentAnimation.setEndValue(newScrollbarValue);
-    d->currentAnimation.start();
-#else
-    horizontalScrollBar()->setValue(newScrollbarValue);
+    if (style()->styleHint(QStyle::SH_Widget_Animate, 0, this)) {
+        d->currentAnimation.setEndValue(newScrollbarValue);
+        d->currentAnimation.start();
+    } else
 #endif //QT_NO_ANIMATION
+    {
+        horizontalScrollBar()->setValue(newScrollbarValue);
+    }
 }
 
 /*!
@@ -411,9 +412,9 @@ void QColumnViewPrivate::updateScrollbars()
     // find the total horizontal length of the laid out columns
     int horizontalLength = 0;
     if (!columns.isEmpty()) {
-        horizontalLength = (columns.last()->x() + columns.last()->width()) - columns.first()->x();
+        horizontalLength = (columns.constLast()->x() + columns.constLast()->width()) - columns.constFirst()->x();
         if (horizontalLength <= 0) // reverse mode
-            horizontalLength = (columns.first()->x() + columns.first()->width()) - columns.last()->x();
+            horizontalLength = (columns.constFirst()->x() + columns.constFirst()->width()) - columns.constLast()->x();
     }
 
     QSize viewportSize = viewport->size();
@@ -567,7 +568,7 @@ void QColumnViewPrivate::closeColumns(const QModelIndex &parent, bool build)
     bool clearAll = !parent.isValid();
     bool passThroughRoot = false;
 
-    QList<QModelIndex> dirsToAppend;
+    QVector<QModelIndex> dirsToAppend;
 
     // Find the last column that matches the parent's tree
     int currentColumn = -1;
@@ -628,7 +629,7 @@ void QColumnViewPrivate::closeColumns(const QModelIndex &parent, bool build)
     while (!dirsToAppend.isEmpty()) {
         QAbstractItemView *newView = createColumn(dirsToAppend.takeLast(), true);
         if (!dirsToAppend.isEmpty())
-            newView->setCurrentIndex(dirsToAppend.last());
+            newView->setCurrentIndex(dirsToAppend.constLast());
     }
 
     if (build && !alreadyExists)
@@ -712,8 +713,8 @@ QAbstractItemView *QColumnViewPrivate::createColumn(const QModelIndex &index, bo
         columnSizes.resize(qMax(columnSizes.count(), columns.count() + 1));
         columnSizes[columns.count()] = initialWidth;
     }
-    if (!columns.isEmpty() && columns.last()->isHidden())
-        columns.last()->setVisible(true);
+    if (!columns.isEmpty() && columns.constLast()->isHidden())
+        columns.constLast()->setVisible(true);
 
     columns.append(view);
     doLayout();
@@ -792,11 +793,8 @@ void QColumnView::initializeColumn(QAbstractItemView *column) const
     column->setModel(model());
 
     // Copy the custom delegate per row
-    QMapIterator<int, QPointer<QAbstractItemDelegate> > i(d->rowDelegates);
-    while (i.hasNext()) {
-        i.next();
+    for (auto i = d->rowDelegates.cbegin(), end = d->rowDelegates.cend(); i != end; ++i)
         column->setItemDelegateForRow(i.key(), i.value());
-    }
 
     // set the delegate to be the columnview delegate
     QAbstractItemDelegate *delegate = column->itemDelegate();
@@ -837,7 +835,7 @@ void QColumnViewPrivate::setPreviewWidget(QWidget *widget)
 {
     Q_Q(QColumnView);
     if (previewColumn) {
-        if (!columns.isEmpty() && columns.last() == previewColumn)
+        if (!columns.isEmpty() && columns.constLast() == previewColumn)
             columns.removeLast();
         previewColumn->deleteLater();
     }
@@ -866,11 +864,15 @@ void QColumnView::setColumnWidths(const QList<int> &list)
 {
     Q_D(QColumnView);
     int i = 0;
-    for (; (i < list.count() && i < d->columns.count()); ++i) {
+    const int listCount = list.count();
+    const int count = qMin(listCount, d->columns.count());
+    for (; i < count; ++i) {
         d->columns.at(i)->resize(list.at(i), d->columns.at(i)->height());
         d->columnSizes[i] = list.at(i);
     }
-    for (; i < list.count(); ++i)
+
+    d->columnSizes.reserve(listCount);
+    for (; i < listCount; ++i)
         d->columnSizes.append(list.at(i));
 }
 
@@ -883,7 +885,9 @@ QList<int> QColumnView::columnWidths() const
 {
     Q_D(const QColumnView);
     QList<int> list;
-    for (int i = 0; i < d->columns.count(); ++i)
+    const int columnCount = d->columns.count();
+    list.reserve(columnCount);
+    for (int i = 0; i < columnCount; ++i)
         list.append(d->columnSizes.at(i));
     return list;
 }
@@ -981,8 +985,11 @@ void QColumnViewPrivate::_q_changeCurrentColumn()
             QAbstractItemView *view = columns.at(i);
             view->setSelectionModel(replacementSelectionModel);
             view->setFocusPolicy(Qt::NoFocus);
-            if (columns.size() > i + 1)
-                view->setCurrentIndex(columns.at(i+1)->rootIndex());
+            if (columns.size() > i + 1) {
+                const QModelIndex newRootIndex = columns.at(i + 1)->rootIndex();
+                if (newRootIndex.isValid())
+                    view->setCurrentIndex(newRootIndex);
+            }
             break;
         }
     }
@@ -996,11 +1003,11 @@ void QColumnViewPrivate::_q_changeCurrentColumn()
             parentColumn->setCurrentIndex(current.parent());
     }
 
-    if (columns.last()->isHidden()) {
-        columns.last()->setVisible(true);
+    if (columns.constLast()->isHidden()) {
+        columns.constLast()->setVisible(true);
     }
-    if (columns.last()->selectionModel())
-        columns.last()->selectionModel()->clear();
+    if (columns.constLast()->selectionModel())
+        columns.constLast()->selectionModel()->clear();
     updateScrollbars();
 }
 
