@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -66,7 +72,7 @@ static inline int panTouchPoints()
     static const char panTouchPointVariable[] = "QT_PAN_TOUCHPOINTS";
     if (qEnvironmentVariableIsSet(panTouchPointVariable)) {
         bool ok;
-        const int result = qgetenv(panTouchPointVariable).toInt(&ok);
+        const int result = qEnvironmentVariableIntValue(panTouchPointVariable, &ok);
         if (ok && result >= 1)
             return result;
         qWarning() << "Ignoring invalid value of " << panTouchPointVariable;
@@ -106,7 +112,7 @@ QGestureManager::QGestureManager(QObject *parent)
 
 QGestureManager::~QGestureManager()
 {
-    qDeleteAll(m_recognizers.values());
+    qDeleteAll(m_recognizers);
     foreach (QGestureRecognizer *recognizer, m_obsoleteGestures.keys()) {
         qDeleteAll(m_obsoleteGestures.value(recognizer));
         delete recognizer;
@@ -117,7 +123,7 @@ QGestureManager::~QGestureManager()
 Qt::GestureType QGestureManager::registerGestureRecognizer(QGestureRecognizer *recognizer)
 {
     QGesture *dummy = recognizer->create(0);
-    if (!dummy) {
+    if (Q_UNLIKELY(!dummy)) {
         qWarning("QGestureManager::registerGestureRecognizer: "
                  "the recognizer fails to create a gesture object, skipping registration.");
         return Qt::GestureType(0);
@@ -213,7 +219,8 @@ QGesture *QGestureManager::getState(QObject *object, QGestureRecognizer *recogni
     }
 
     // check if the QGesture for this recognizer has already been created
-    foreach (QGesture *state, m_objectGestures.value(QGestureManager::ObjectGesture(object, type))) {
+    const auto states = m_objectGestures.value(QGestureManager::ObjectGesture(object, type));
+    for (QGesture *state : states) {
         if (m_gestureToRecognizer.value(state) == recognizer)
             return state;
     }
@@ -545,12 +552,12 @@ bool QGestureManager::filterEvent(QObject *receiver, QEvent *event)
     // filter method.
     QWidgetWindow *widgetWindow = qobject_cast<QWidgetWindow *>(receiver);
 
-    if (widgetWindow)
+    if (widgetWindow && widgetWindow->widget())
         return filterEvent(widgetWindow->widget(), event);
 
-    if (!m_gestureToRecognizer.contains(static_cast<QGesture *>(receiver)))
+    QGesture *state = qobject_cast<QGesture *>(receiver);
+    if (!state || !m_gestureToRecognizer.contains(state))
         return false;
-    QGesture *state = static_cast<QGesture *>(receiver);
     QMultiMap<QObject *, Qt::GestureType> contexts;
     contexts.insert(state, state->gestureType());
     return filterEventThroughContexts(contexts, event);
@@ -572,18 +579,19 @@ void QGestureManager::getGestureTargets(const QSet<QGesture*> &gestures,
     }
 
     // for each gesture type
-    foreach (Qt::GestureType type, gestureByTypes.keys()) {
-        QHash<QWidget *, QGesture *> gestures = gestureByTypes.value(type);
-        foreach (QWidget *widget, gestures.keys()) {
+    for (GestureByTypes::const_iterator git = gestureByTypes.cbegin(), gend = gestureByTypes.cend(); git != gend; ++git) {
+        const QHash<QWidget *, QGesture *> &gestures = git.value();
+        for (QHash<QWidget *, QGesture *>::const_iterator wit = gestures.cbegin(), wend = gestures.cend(); wit != wend; ++wit) {
+            QWidget *widget = wit.key();
             QWidget *w = widget->parentWidget();
             while (w) {
                 QMap<Qt::GestureType, Qt::GestureFlags>::const_iterator it
-                        = w->d_func()->gestureContext.constFind(type);
+                        = w->d_func()->gestureContext.constFind(git.key());
                 if (it != w->d_func()->gestureContext.constEnd()) {
                     // i.e. 'w' listens to gesture 'type'
                     if (!(it.value() & Qt::DontStartGestureOnChildren) && w != widget) {
                         // conflicting gesture!
-                        (*conflicts)[widget].append(gestures[widget]);
+                        (*conflicts)[widget].append(wit.value());
                         break;
                     }
                 }
@@ -594,7 +602,7 @@ void QGestureManager::getGestureTargets(const QSet<QGesture*> &gestures,
                 w = w->parentWidget();
             }
             if (!w)
-                (*normal)[widget].append(gestures[widget]);
+                (*normal)[widget].append(wit.value());
         }
     }
 }
@@ -639,17 +647,17 @@ void QGestureManager::deliverEvents(const QSet<QGesture *> &gestures,
         Q_ASSERT(gestureType != Qt::CustomGesture);
         Q_UNUSED(gestureType);
 
-        if (target) {
+        if (Q_UNLIKELY(!target)) {
+            qCDebug(lcGestureManager) << "QGestureManager::deliverEvent: could not find the target for gesture"
+                    << gesture->gestureType();
+            qWarning("QGestureManager::deliverEvent: could not find the target for gesture");
+            undeliveredGestures->insert(gesture);
+        } else {
             if (gesture->state() == Qt::GestureStarted) {
                 startedGestures.insert(gesture);
             } else {
                 normalStartedGestures[target].append(gesture);
             }
-        } else {
-            qCDebug(lcGestureManager) << "QGestureManager::deliverEvent: could not find the target for gesture"
-                    << gesture->gestureType();
-            qWarning("QGestureManager::deliverEvent: could not find the target for gesture");
-            undeliveredGestures->insert(gesture);
         }
     }
 
@@ -677,7 +685,8 @@ void QGestureManager::deliverEvents(const QSet<QGesture *> &gestures,
 
         QApplication::sendEvent(receiver, &event);
         bool eventAccepted = event.isAccepted();
-        foreach(QGesture *gesture, event.gestures()) {
+        const auto eventGestures = event.gestures();
+        for (QGesture *gesture : eventGestures) {
             if (eventAccepted || event.isAccepted(gesture)) {
                 QWidget *w = event.m_targetWidgets.value(gesture->gestureType(), 0);
                 Q_ASSERT(w);
@@ -703,7 +712,8 @@ void QGestureManager::deliverEvents(const QSet<QGesture *> &gestures,
             QGestureEvent event(it.value());
             QApplication::sendEvent(it.key(), &event);
             bool eventAccepted = event.isAccepted();
-            foreach (QGesture *gesture, event.gestures()) {
+            const auto eventGestures = event.gestures();
+            for (QGesture *gesture : eventGestures) {
                 if (gesture->state() == Qt::GestureStarted &&
                     (eventAccepted || event.isAccepted(gesture))) {
                     QWidget *w = event.m_targetWidgets.value(gesture->gestureType(), 0);
