@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -52,9 +50,6 @@
 #include "qandroidstyle_p.h"
 #endif
 #endif
-#ifndef QT_NO_STYLE_GTK
-#include "qgtkstyle_p.h"
-#endif
 #ifndef QT_NO_STYLE_WINDOWSXP
 #include "qwindowsxpstyle_p.h"
 #endif
@@ -74,10 +69,8 @@
 
 QT_BEGIN_NAMESPACE
 
-#ifndef QT_NO_LIBRARY
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
     (QStyleFactoryInterface_iid, QLatin1String("/styles"), Qt::CaseInsensitive))
-#endif
 
 /*!
     \class QStyleFactory
@@ -94,7 +87,7 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
 
     The valid keys can be retrieved using the keys()
     function. Typically they include "windows" and "fusion".
-    Depending on the platform, "windowsxp", "windowsvista", "gtk"
+    Depending on the platform, "windowsxp", "windowsvista"
     and "macintosh" may be available.
     Note that keys are case insensitive.
 
@@ -151,25 +144,18 @@ QStyle *QStyleFactory::create(const QString& key)
         ret = new QAndroidStyle;
     else
 #endif
-#ifndef QT_NO_STYLE_GTK
-    if (style == QLatin1String("gtk") || style == QLatin1String("gtk+"))
-        ret = new QGtkStyle;
-    else
-#endif
 #ifndef QT_NO_STYLE_MAC
     if (style.startsWith(QLatin1String("macintosh"))) {
         ret = new QMacStyle;
-#  ifdef Q_WS_MAC
+#  ifdef Q_DEAD_CODE_FROM_QT4_MAC
         if (style == QLatin1String("macintosh"))
             style += QLatin1String(" (aqua)");
 #  endif
     } else
 #endif
     { } // Keep these here - they make the #ifdefery above work
-#ifndef QT_NO_LIBRARY
     if (!ret)
         ret = qLoadPlugin<QStyle, QStylePlugin>(loader(), style);
-#endif
     if(ret)
         ret->setObjectName(style);
     return ret;
@@ -184,14 +170,12 @@ QStyle *QStyleFactory::create(const QString& key)
 QStringList QStyleFactory::keys()
 {
     QStringList list;
-#ifndef QT_NO_LIBRARY
     typedef QMultiMap<int, QString> PluginKeyMap;
 
     const PluginKeyMap keyMap = loader()->keyMap();
     const PluginKeyMap::const_iterator cend = keyMap.constEnd();
     for (PluginKeyMap::const_iterator it = keyMap.constBegin(); it != cend; ++it)
         list.append(it.value());
-#endif
 #ifndef QT_NO_STYLE_WINDOWS
     if (!list.contains(QLatin1String("Windows")))
         list << QLatin1String("Windows");
@@ -218,17 +202,13 @@ QStringList QStyleFactory::keys()
     if (!list.contains(QLatin1String("Android")))
         list << QLatin1String("Android");
 #endif
-#ifndef QT_NO_STYLE_GTK
-    if (!list.contains(QLatin1String("GTK+")))
-        list << QLatin1String("GTK+");
-#endif
 #ifndef QT_NO_STYLE_FUSION
     if (!list.contains(QLatin1String("Fusion")))
         list << QLatin1String("Fusion");
 #endif
 #ifndef QT_NO_STYLE_MAC
     QString mstyle = QLatin1String("Macintosh");
-# ifdef Q_WS_MAC
+# ifdef Q_DEAD_CODE_FROM_QT4_MAC
     mstyle += QLatin1String(" (aqua)");
 # endif
     if (!list.contains(mstyle))
