@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -50,18 +56,6 @@
 #include <qmap.h>
 #include <qt_windows.h>
 
-// Note, these tests are duplicated in qwizard_win.cpp.
-#ifdef Q_CC_GNU
-#  include <w32api.h>
-#  if (__W32API_MAJOR_VERSION >= 3 || (__W32API_MAJOR_VERSION == 2 && __W32API_MINOR_VERSION >= 5))
-#    ifdef _WIN32_WINNT
-#      undef _WIN32_WINNT
-#    endif
-#    define _WIN32_WINNT 0x0501
-#    include <commctrl.h>
-#  endif
-#endif
-
 #include <uxtheme.h>
 
 #if WINVER >= 0x0600
@@ -214,15 +208,15 @@ public:
     static RECT toRECT(const QRect &qr);
     bool isValid();
 
-    QSize size();
-    QMargins margins(const QRect &rect, int propId = TMT_CONTENTMARGINS);
-    QMargins margins(int propId = TMT_CONTENTMARGINS);
+    QSizeF size();
+    QMarginsF margins(const QRect &rect, int propId = TMT_CONTENTMARGINS);
+    QMarginsF margins(int propId = TMT_CONTENTMARGINS);
 
-    static QSize themeSize(const QWidget *w = 0, QPainter *p = 0, int themeIn = -1, int part = 0, int state = 0);
-    static QMargins themeMargins(const QRect &rect, const QWidget *w = 0, QPainter *p = 0, int themeIn = -1,
-                                 int part = 0, int state = 0, int propId = TMT_CONTENTMARGINS);
-    static QMargins themeMargins(const QWidget *w = 0, QPainter *p = 0, int themeIn = -1,
-                                 int part = 0, int state = 0, int propId = TMT_CONTENTMARGINS);
+    static QSizeF themeSize(const QWidget *w = 0, QPainter *p = 0, int themeIn = -1, int part = 0, int state = 0);
+    static QMarginsF themeMargins(const QRect &rect, const QWidget *w = 0, QPainter *p = 0, int themeIn = -1,
+                                  int part = 0, int state = 0, int propId = TMT_CONTENTMARGINS);
+    static QMarginsF themeMargins(const QWidget *w = 0, QPainter *p = 0, int themeIn = -1,
+                                  int part = 0, int state = 0, int propId = TMT_CONTENTMARGINS);
 
     const QWidget *widget;
     QPainter *painter;
@@ -362,9 +356,10 @@ public:
         ToolBarTheme,
         ToolTipTheme,
         TrackBarTheme,
-        TreeViewTheme,
+        XpTreeViewTheme, // '+'/'-' shape treeview indicators (XP)
         WindowTheme,
         StatusTheme,
+        VistaTreeViewTheme, // arrow shape treeview indicators (Vista) obtained from "explorer" theme.
         NThemes
     };
 
@@ -399,8 +394,8 @@ public:
 
     void setTransparency(QWidget *widget, XPThemeData &themeData);
     bool drawBackground(XPThemeData &themeData);
-    bool drawBackgroundThruNativeBuffer(XPThemeData &themeData);
-    bool drawBackgroundDirectly(XPThemeData &themeData);
+    bool drawBackgroundThruNativeBuffer(XPThemeData &themeData, int aditionalDevicePixelRatio);
+    bool drawBackgroundDirectly(HDC dc, XPThemeData &themeData, int aditionalDevicePixelRatio);
 
     bool hasAlphaChannel(const QRect &rect);
     bool fixAlphaChannel(const QRect &rect);
@@ -415,6 +410,7 @@ public:
     static QString themeName(int theme);
     static inline bool hasTheme(int theme) { return theme >= 0 && theme < NThemes && m_themes[theme]; }
     static bool isItemViewDelegateLineEdit(const QWidget *widget);
+    static bool isLineEditBaseColorSet(const QStyleOption *option, const QWidget *widget);
 
     QIcon dockFloat, dockClose;
 
@@ -424,6 +420,9 @@ private:
     void showProperties(XPThemeData &themeData);
 #endif
 
+    static bool initVistaTreeViewTheming();
+    static void cleanupVistaTreeViewTheming();
+
     static QBasicAtomicInt ref;
     static bool use_xp;
     static QPixmap *tabbody;
@@ -435,12 +434,13 @@ private:
     uchar *bufferPixels;
     int bufferW, bufferH;
 
+    static HWND m_vistaTreeViewHelper;
     static HTHEME m_themes[NThemes];
 };
 
-inline QSize XPThemeData::size()
+inline QSizeF XPThemeData::size()
 {
-    QSize result(0, 0);
+    QSizeF result(0, 0);
     if (isValid()) {
         SIZE size;
         if (SUCCEEDED(QWindowsXPStylePrivate::pGetThemePartSize(handle(), 0, partId, stateId, 0, TS_TRUE, &size)))
@@ -449,9 +449,9 @@ inline QSize XPThemeData::size()
     return result;
 }
 
-inline QMargins XPThemeData::margins(const QRect &qRect, int propId)
+inline QMarginsF XPThemeData::margins(const QRect &qRect, int propId)
 {
-    QMargins result(0, 0, 0 ,0);
+    QMarginsF result(0, 0, 0 ,0);
     if (isValid()) {
         MARGINS margins;
         RECT rect = XPThemeData::toRECT(qRect);
@@ -461,9 +461,9 @@ inline QMargins XPThemeData::margins(const QRect &qRect, int propId)
     return result;
 }
 
-inline QMargins XPThemeData::margins(int propId)
+inline QMarginsF XPThemeData::margins(int propId)
 {
-    QMargins result(0, 0, 0 ,0);
+    QMarginsF result(0, 0, 0 ,0);
     if (isValid()) {
         MARGINS margins;
         if (SUCCEEDED(QWindowsXPStylePrivate::pGetThemeMargins(handle(), 0, partId, stateId, propId, NULL, &margins)))
@@ -472,21 +472,21 @@ inline QMargins XPThemeData::margins(int propId)
     return result;
 }
 
-inline QSize XPThemeData::themeSize(const QWidget *w, QPainter *p, int themeIn, int part, int state)
+inline QSizeF XPThemeData::themeSize(const QWidget *w, QPainter *p, int themeIn, int part, int state)
 {
     XPThemeData theme(w, p, themeIn, part, state);
     return theme.size();
 }
 
-inline QMargins XPThemeData::themeMargins(const QRect &rect, const QWidget *w, QPainter *p, int themeIn,
-                                          int part, int state, int propId)
+inline QMarginsF XPThemeData::themeMargins(const QRect &rect, const QWidget *w, QPainter *p, int themeIn,
+                                           int part, int state, int propId)
 {
     XPThemeData theme(w, p, themeIn, part, state);
     return theme.margins(rect, propId);
 }
 
-inline QMargins XPThemeData::themeMargins(const QWidget *w, QPainter *p, int themeIn,
-                                          int part, int state, int propId)
+inline QMarginsF XPThemeData::themeMargins(const QWidget *w, QPainter *p, int themeIn,
+                                           int part, int state, int propId)
 {
     XPThemeData theme(w, p, themeIn, part, state);
     return theme.margins(propId);
