@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,8 +53,13 @@
 #include <qscreen.h>
 #include <qbackingstore.h>
 #include <qpa/qplatformnativeinterface.h>
+#include <qpa/qplatformsystemtrayicon.h>
+#include <qpa/qplatformtheme.h>
+#include <private/qguiapplication_p.h>
 #include <qdebug.h>
 
+#include <QtPlatformHeaders/qxcbwindowfunctions.h>
+#include <QtPlatformHeaders/qxcbintegrationfunctions.h>
 #ifndef QT_NO_SYSTEMTRAYICON
 QT_BEGIN_NAMESPACE
 
@@ -79,11 +82,12 @@ public:
     QRect globalGeometry() const;
 
 protected:
-    virtual void mousePressEvent(QMouseEvent *ev);
-    virtual void mouseDoubleClickEvent(QMouseEvent *ev);
-    virtual bool event(QEvent *);
-    virtual void paintEvent(QPaintEvent *);
-    virtual void resizeEvent(QResizeEvent *);
+    virtual void mousePressEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
+    virtual void mouseDoubleClickEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
+    virtual bool event(QEvent *) Q_DECL_OVERRIDE;
+    virtual void paintEvent(QPaintEvent *) Q_DECL_OVERRIDE;
+    virtual void resizeEvent(QResizeEvent *) Q_DECL_OVERRIDE;
+    virtual void moveEvent(QMoveEvent *) Q_DECL_OVERRIDE;
 
 private slots:
     void systemTrayWindowChanged(QScreen *screen);
@@ -92,6 +96,7 @@ private:
     bool addToTray();
 
     QSystemTrayIcon *q;
+    QPixmap background;
 };
 
 QSystemTrayIconSys::QSystemTrayIconSys(QSystemTrayIcon *qIn)
@@ -101,11 +106,34 @@ QSystemTrayIconSys::QSystemTrayIconSys(QSystemTrayIcon *qIn)
     setObjectName(QStringLiteral("QSystemTrayIconSys"));
     setToolTip(q->toolTip());
     setAttribute(Qt::WA_AlwaysShowToolTips, true);
-    setAttribute(Qt::WA_TranslucentBackground, true);
     setAttribute(Qt::WA_QuitOnClose, false);
     const QSize size(22, 22); // Gnome, standard size
     setGeometry(QRect(QPoint(0, 0), size));
     setMinimumSize(size);
+
+    // We need two different behaviors depending on whether the X11 visual for the system tray
+    // (a) exists and (b) supports an alpha channel, i.e. is 32 bits.
+    // If we have a visual that has an alpha channel, we can paint this widget with a transparent
+    // background and it will work.
+    // However, if there's no alpha channel visual, in order for transparent tray icons to work,
+    // we do not have a transparent background on the widget, but set the BackPixmap property of our
+    // window to ParentRelative (so that it inherits the background of its X11 parent window), call
+    // xcb_clear_region before painting (so that the inherited background is visible) and then grab
+    // the just-drawn background from the X11 server.
+    bool hasAlphaChannel = QXcbIntegrationFunctions::xEmbedSystemTrayVisualHasAlphaChannel();
+    setAttribute(Qt::WA_TranslucentBackground, hasAlphaChannel);
+    if (!hasAlphaChannel) {
+        createWinId();
+        QXcbWindowFunctions::setParentRelativeBackPixmap(windowHandle());
+
+        // XXX: This is actually required, but breaks things ("QWidget::paintEngine: Should no
+        // longer be called"). Why is this needed? When the widget is drawn, we use tricks to grab
+        // the tray icon's background from the server. If the tray icon isn't visible (because
+        // another window is on top of it), the trick fails and instead uses the content of that
+        // other window as the background.
+        // setAttribute(Qt::WA_PaintOnScreen);
+    }
+
     addToTray();
 }
 
@@ -117,15 +145,11 @@ bool QSystemTrayIconSys::addToTray()
     createWinId();
     setMouseTracking(true);
 
-    bool requestResult = false;
-    if (!QMetaObject::invokeMethod(QGuiApplication::platformNativeInterface(),
-                                   "requestSystemTrayWindowDock", Qt::DirectConnection,
-                                   Q_RETURN_ARG(bool, requestResult),
-                                   Q_ARG(const QWindow *, windowHandle()))
-            || !requestResult) {
-        qWarning("requestSystemTrayWindowDock failed.");
+    if (!QXcbWindowFunctions::requestSystemTrayWindowDock(windowHandle()))
         return false;
-    }
+
+    if (!background.isNull())
+        background = QPixmap();
     show();
     return true;
 }
@@ -143,15 +167,7 @@ void QSystemTrayIconSys::systemTrayWindowChanged(QScreen *)
 
 QRect QSystemTrayIconSys::globalGeometry() const
 {
-    QRect result;
-    if (!QMetaObject::invokeMethod(QGuiApplication::platformNativeInterface(),
-                                   "systemTrayWindowGlobalGeometry", Qt::DirectConnection,
-                                   Q_RETURN_ARG(QRect, result),
-                                   Q_ARG(const QWindow *, windowHandle()))
-        || !result.isValid()) {
-        qWarning("systemTrayWindowGlobalGeometry failed.");
-    }
-    return result;
+    return QXcbWindowFunctions::systemTrayWindowGlobalGeometry(windowHandle());
 }
 
 void QSystemTrayIconSys::mousePressEvent(QMouseEvent *ev)
@@ -160,7 +176,9 @@ void QSystemTrayIconSys::mousePressEvent(QMouseEvent *ev)
 #ifndef QT_NO_CONTEXTMENU
     if (ev->button() == Qt::RightButton && q->contextMenu())
         q->contextMenu()->popup(globalPos);
-#endif
+#else
+    Q_UNUSED(globalPos)
+#endif // QT_NO_CONTEXTMENU
 
     if (QBalloonTip::isBalloonVisible()) {
         emit q->messageClicked();
@@ -184,6 +202,9 @@ void QSystemTrayIconSys::mouseDoubleClickEvent(QMouseEvent *ev)
 bool QSystemTrayIconSys::event(QEvent *e)
 {
     switch (e->type()) {
+    case QEvent::ToolTip:
+        QApplication::sendEvent(q, e);
+        break;
 #ifndef QT_NO_WHEELEVENT
     case QEvent::Wheel:
         return QApplication::sendEvent(q, e);
@@ -196,35 +217,62 @@ bool QSystemTrayIconSys::event(QEvent *e)
 
 void QSystemTrayIconSys::paintEvent(QPaintEvent *)
 {
-    // Note: Transparent pixels require a particular Visual which XCB
-    // currently does not support yet.
     const QRect rect(QPoint(0, 0), geometry().size());
     QPainter painter(this);
-    painter.setCompositionMode(QPainter::CompositionMode_Source);
-    painter.fillRect(rect, Qt::transparent);
+
+    // If we have Qt::WA_TranslucentBackground set, during widget creation
+    // we detected the systray visual supported an alpha channel
+    if (testAttribute(Qt::WA_TranslucentBackground)) {
+        painter.setCompositionMode(QPainter::CompositionMode_Source);
+        painter.fillRect(rect, Qt::transparent);
+    } else {
+        // clearRegion() was called on XEMBED_EMBEDDED_NOTIFY, so we hope that got done by now.
+        // Grab the tray background pixmap, before rendering the icon for the first time.
+        if (background.isNull()) {
+            background = QGuiApplication::primaryScreen()->grabWindow(winId(),
+                                0, 0, rect.size().width(), rect.size().height());
+        }
+        // Then paint over the icon area with the background before compositing the icon on top.
+        painter.drawPixmap(QPoint(0, 0), background);
+    }
     painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
     q->icon().paint(&painter, rect);
 }
 
-void QSystemTrayIconSys::resizeEvent(QResizeEvent *)
+void QSystemTrayIconSys::moveEvent(QMoveEvent *event)
 {
-    update();
+    QWidget::moveEvent(event);
+    if (QBalloonTip::isBalloonVisible())
+        QBalloonTip::updateBalloonPosition(globalGeometry().center());
 }
 
+void QSystemTrayIconSys::resizeEvent(QResizeEvent *event)
+{
+    update();
+    QWidget::resizeEvent(event);
+    if (QBalloonTip::isBalloonVisible())
+        QBalloonTip::updateBalloonPosition(globalGeometry().center());
+}
 ////////////////////////////////////////////////////////////////////////////
 
 QSystemTrayIconPrivate::QSystemTrayIconPrivate()
     : sys(0),
+      qpa_sys(QGuiApplicationPrivate::platformTheme()->createPlatformSystemTrayIcon()),
       visible(false)
 {
 }
 
 QSystemTrayIconPrivate::~QSystemTrayIconPrivate()
 {
+    delete qpa_sys;
 }
 
 void QSystemTrayIconPrivate::install_sys()
 {
+    if (qpa_sys) {
+        install_sys_qpa();
+        return;
+    }
     Q_Q(QSystemTrayIcon);
     if (!sys && locateSystemTray()) {
         sys = new QSystemTrayIconSys(q);
@@ -235,6 +283,8 @@ void QSystemTrayIconPrivate::install_sys()
 
 QRect QSystemTrayIconPrivate::geometry_sys() const
 {
+    if (qpa_sys)
+        return geometry_sys_qpa();
     if (!sys)
         return QRect();
     return sys->globalGeometry();
@@ -242,6 +292,10 @@ QRect QSystemTrayIconPrivate::geometry_sys() const
 
 void QSystemTrayIconPrivate::remove_sys()
 {
+    if (qpa_sys) {
+        remove_sys_qpa();
+        return;
+    }
     if (!sys)
         return;
     QBalloonTip::hideBalloon();
@@ -252,17 +306,26 @@ void QSystemTrayIconPrivate::remove_sys()
 
 void QSystemTrayIconPrivate::updateIcon_sys()
 {
+    if (qpa_sys) {
+        updateIcon_sys_qpa();
+        return;
+    }
     if (sys)
         sys->updateIcon();
 }
 
 void QSystemTrayIconPrivate::updateMenu_sys()
 {
-
+    if (qpa_sys)
+        updateMenu_sys_qpa();
 }
 
 void QSystemTrayIconPrivate::updateToolTip_sys()
 {
+    if (qpa_sys) {
+        updateToolTip_sys_qpa();
+        return;
+    }
     if (!sys)
         return;
 #ifndef QT_NO_TOOLTIP
@@ -272,25 +335,38 @@ void QSystemTrayIconPrivate::updateToolTip_sys()
 
 bool QSystemTrayIconPrivate::isSystemTrayAvailable_sys()
 {
+    QScopedPointer<QPlatformSystemTrayIcon> sys(QGuiApplicationPrivate::platformTheme()->createPlatformSystemTrayIcon());
+    if (sys && sys->isSystemTrayAvailable())
+        return true;
+
+    // no QPlatformSystemTrayIcon so fall back to default xcb platform behavior
     const QString platform = QGuiApplication::platformName();
-    if (platform.compare(QStringLiteral("xcb"), Qt::CaseInsensitive) == 0)
+    if (platform.compare(QLatin1String("xcb"), Qt::CaseInsensitive) == 0)
        return locateSystemTray();
     return false;
 }
 
 bool QSystemTrayIconPrivate::supportsMessages_sys()
 {
+    QScopedPointer<QPlatformSystemTrayIcon> sys(QGuiApplicationPrivate::platformTheme()->createPlatformSystemTrayIcon());
+    if (sys)
+        return sys->supportsMessages();
+
+    // no QPlatformSystemTrayIcon so fall back to default xcb platform behavior
     return true;
 }
 
-void QSystemTrayIconPrivate::showMessage_sys(const QString &message, const QString &title,
+void QSystemTrayIconPrivate::showMessage_sys(const QString &title, const QString &message,
                                    QSystemTrayIcon::MessageIcon icon, int msecs)
 {
+    if (qpa_sys) {
+        showMessage_sys_qpa(title, message, icon, msecs);
+        return;
+    }
     if (!sys)
         return;
-    const QPoint g = sys->globalGeometry().topLeft();
-    QBalloonTip::showBalloon(icon, message, title, sys->systemTrayIcon(),
-                             QPoint(g.x() + sys->width()/2, g.y() + sys->height()/2),
+    QBalloonTip::showBalloon(icon, title, message, sys->systemTrayIcon(),
+                             sys->globalGeometry().center(),
                              msecs);
 }
 
