@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -58,6 +64,10 @@ QT_BEGIN_NAMESPACE
 
 //#define QSPLITTER_DEBUG
 
+QSplitterPrivate::~QSplitterPrivate()
+{
+}
+
 /*!
     \class QSplitterHandle
     \brief The QSplitterHandle class provides handle functionality for the splitter.
@@ -1054,7 +1064,7 @@ void QSplitter::setCollapsible(int index, bool collapse)
 {
     Q_D(QSplitter);
 
-    if (index < 0 || index >= d->list.size()) {
+    if (Q_UNLIKELY(index < 0 || index >= d->list.size())) {
         qWarning("QSplitter::setCollapsible: Index %d out of range", index);
         return;
     }
@@ -1067,7 +1077,7 @@ void QSplitter::setCollapsible(int index, bool collapse)
 bool QSplitter::isCollapsible(int index) const
 {
     Q_D(const QSplitter);
-    if (index < 0 || index >= d->list.size()) {
+    if (Q_UNLIKELY(index < 0 || index >= d->list.size())) {
         qWarning("QSplitter::isCollapsible: Index %d out of range", index);
         return false;
     }
@@ -1089,6 +1099,8 @@ void QSplitter::resizeEvent(QResizeEvent *)
 
     If \a widget is already in the splitter, it will be moved to the new position.
 
+    \note The splitter takes ownership of the widget.
+
     \sa insertWidget(), widget(), indexOf()
 */
 void QSplitter::addWidget(QWidget *widget)
@@ -1103,7 +1115,9 @@ void QSplitter::addWidget(QWidget *widget)
 
     If \a widget is already in the splitter, it will be moved to the new position.
 
-    if \a index is an invalid index, then the widget will be inserted at the end.
+    If \a index is an invalid index, then the widget will be inserted at the end.
+
+    \note The splitter takes ownership of the widget.
 
     \sa addWidget(), indexOf(), widget()
 */
@@ -1211,7 +1225,7 @@ void QSplitter::childEvent(QChildEvent *c)
 {
     Q_D(QSplitter);
     if (!c->child()->isWidgetType()) {
-        if (c->type() == QEvent::ChildAdded && qobject_cast<QLayout *>(c->child()))
+        if (Q_UNLIKELY(c->type() == QEvent::ChildAdded && qobject_cast<QLayout *>(c->child())))
             qWarning("Adding a QLayout to a QSplitter is not supported.");
         return;
     }
@@ -1498,8 +1512,11 @@ QList<int> QSplitter::sizes() const
     Q_D(const QSplitter);
     ensurePolished();
 
+    const int numSizes = d->list.size();
     QList<int> list;
-    for (int i = 0; i < d->list.size(); ++i) {
+    list.reserve(numSizes);
+
+    for (int i = 0; i < numSizes; ++i) {
         QSplitterLayoutStruct *s = d->list.at(i);
         list.append(d->pick(s->rect.size()));
     }
@@ -1594,8 +1611,10 @@ QByteArray QSplitter::saveState() const
 
     stream << qint32(SplitterMagic);
     stream << qint32(version);
+    const int numSizes = d->list.size();
     QList<int> list;
-    for (int i = 0; i < d->list.size(); ++i) {
+    list.reserve(numSizes);
+    for (int i = 0; i < numSizes; ++i) {
         QSplitterLayoutStruct *s = d->list.at(i);
         list.append(s->sizer);
     }
@@ -1724,4 +1743,6 @@ QTextStream& operator>>(QTextStream& ts, QSplitter& splitter)
 
 QT_END_NAMESPACE
 
+#include "moc_qsplitter.cpp"
+
 #endif // QT_NO_SPLITTER
