@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -341,7 +347,7 @@ void QToolBarAreaLayoutInfo::removeToolBar(QToolBar *toolBar)
 void QToolBarAreaLayoutInfo::insertToolBarBreak(QToolBar *before)
 {
     if (before == 0) {
-        if (!lines.isEmpty() && lines.last().toolBarItems.isEmpty())
+        if (!lines.isEmpty() && lines.constLast().toolBarItems.isEmpty())
             return;
         lines.append(QToolBarAreaLayoutLine(o));
         return;
@@ -598,21 +604,16 @@ int QToolBarAreaLayoutInfo::distance(const QPoint &pos) const
         case QInternal::LeftDock:
             if (pos.y() < rect.bottom())
                 return pos.x() - rect.right();
-            break;
         case QInternal::RightDock:
             if (pos.y() < rect.bottom())
                 return rect.left() - pos.x();
-            break;
         case QInternal::TopDock:
             if (pos.x() < rect.right())
                 return pos.y() - rect.bottom();
-            break;
         case QInternal::BottomDock:
             if (pos.x() < rect.right())
                 return rect.top() - pos.y();
-            break;
-
-        case QInternal::DockCount:
+        default:
             break;
     }
     return -1;
@@ -1133,7 +1134,7 @@ QRect QToolBarAreaLayout::itemRect(const QList<int> &path) const
 QLayoutItem *QToolBarAreaLayout::plug(const QList<int> &path)
 {
     QToolBarAreaLayoutItem *item = this->item(path);
-    if (!item) {
+    if (Q_UNLIKELY(!item)) {
         qWarning() << "No item at" << path;
         return 0;
     }
@@ -1265,7 +1266,7 @@ void QToolBarAreaLayout::saveState(QDataStream &stream) const
                 const QToolBarAreaLayoutItem &item = line.toolBarItems.at(k);
                 QWidget *widget = const_cast<QLayoutItem*>(item.widgetItem)->widget();
                 QString objectName = widget->objectName();
-                if (objectName.isEmpty()) {
+                if (Q_UNLIKELY(objectName.isEmpty())) {
                     qWarning("QMainWindow::saveState(): 'objectName' not set for QToolBar %p '%s'",
                                 widget, widget->windowTitle().toLocal8Bit().constData());
                 }
