@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -111,7 +117,7 @@ public:
     void _q_buttonClicked();
     void _q_widgetDestroyed(QObject*);
 
-    const Page *page(const QObject *widget) const;
+    const Page *page(QWidget *widget) const;
     const Page *page(int index) const;
     Page *page(int index);
 
@@ -123,7 +129,7 @@ public:
     Page *currentPage;
 };
 
-const QToolBoxPrivate::Page *QToolBoxPrivate::page(const QObject *widget) const
+const QToolBoxPrivate::Page *QToolBoxPrivate::page(QWidget *widget) const
 {
     if (!widget)
         return 0;
@@ -443,9 +449,11 @@ void QToolBoxPrivate::relayout()
 void QToolBoxPrivate::_q_widgetDestroyed(QObject *object)
 {
     Q_Q(QToolBox);
+    // no verification - vtbl corrupted already
+    QWidget *p = (QWidget*)object;
 
-    const QToolBoxPrivate::Page * const c = page(object);
-    if (!c)
+    const QToolBoxPrivate::Page *c = page(p);
+    if (!p || !c)
         return;
 
     layout->removeWidget(c->sv);
@@ -519,10 +527,10 @@ QWidget * QToolBox::currentWidget() const
 void QToolBox::setCurrentWidget(QWidget *widget)
 {
     int i = indexOf(widget);
-    if (i >= 0)
-        setCurrentIndex(i);
-    else
+    if (Q_UNLIKELY(i < 0))
         qWarning("QToolBox::setCurrentWidget: widget not contained in tool box");
+    else
+        setCurrentIndex(i);
 }
 
 /*!
