@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -78,8 +73,6 @@ private slots:
 
     void signalsEmittedAfterFileMoved();
 
-    void watchUnicodeCharacters();
-
 private:
     QString m_tempDirPattern;
 #endif // QT_NO_FILESYSTEMWATCHER
@@ -94,7 +87,7 @@ tst_QFileSystemWatcher::tst_QFileSystemWatcher()
     m_tempDirPattern += QStringLiteral("tst_qfilesystemwatcherXXXXXX");
 #endif // QT_NO_FILESYSTEMWATCHER
 
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
+#if defined(Q_OS_ANDROID)
     QDir::setCurrent(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
 #endif
 }
@@ -733,9 +726,10 @@ void tst_QFileSystemWatcher::signalsEmittedAfterFileMoved()
     QString movePath = testDir.filePath("movehere");
 
     for (int i = 0; i < fileCount; ++i) {
-        QFile f(testDir.filePath(QString("test%1.txt").arg(i)));
+        const QByteArray iB = QByteArray::number(i);
+        QFile f(testDir.filePath(QLatin1String("test") + QString::fromLatin1(iB) + QLatin1String(".txt")));
         QVERIFY(f.open(QIODevice::WriteOnly));
-        f.write(QByteArray("i am ") + QByteArray::number(i));
+        f.write(QByteArray("i am ") + iB);
         f.close();
     }
 
@@ -765,25 +759,6 @@ void tst_QFileSystemWatcher::signalsEmittedAfterFileMoved()
     QVERIFY2(changedSpy.count() <= fileCount, changedSpy.receivedFilesMessage());
     QTRY_COMPARE(changedSpy.count(), fileCount);
 }
-
-void tst_QFileSystemWatcher::watchUnicodeCharacters()
-{
-    QTemporaryDir temporaryDirectory(m_tempDirPattern);
-    QVERIFY2(temporaryDirectory.isValid(), qPrintable(temporaryDirectory.errorString()));
-
-    QDir testDir(temporaryDirectory.path());
-    const QString subDir(QString::fromLatin1("caf\xe9"));
-    QVERIFY(testDir.mkdir(subDir));
-    testDir = QDir(temporaryDirectory.path() + QDir::separator() + subDir);
-
-    QFileSystemWatcher watcher;
-    QVERIFY(watcher.addPath(testDir.path()));
-
-    FileSystemWatcherSpy changedSpy(&watcher, FileSystemWatcherSpy::SpyOnDirectoryChanged);
-    QCOMPARE(changedSpy.count(), 0);
-    QVERIFY(testDir.mkdir("creme"));
-    QTRY_COMPARE(changedSpy.count(), 1);
-}
 #endif // QT_NO_FILESYSTEMWATCHER
 
 QTEST_MAIN(tst_QFileSystemWatcher)
