@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,20 +52,28 @@ private slots:
 void tst_QProcessEnvironment::operator_eq()
 {
     QProcessEnvironment e1;
-    QVERIFY(e1 == e1);
+    QCOMPARE(e1, e1);
     e1.clear();
-    QVERIFY(e1 == e1);
+    QCOMPARE(e1, e1);
 
     e1 = QProcessEnvironment();
     QProcessEnvironment e2;
-    QVERIFY(e1 == e2);
+    QCOMPARE(e1, e2);
 
     e1.clear();
-    QVERIFY(e1 != e2);
+    QCOMPARE(e1, e2);
 
     e2.clear();
+    QCOMPARE(e1, e2);
 
-    QVERIFY(e1 == e2);
+    e1.insert("FOO", "bar");
+    QVERIFY(e1 != e2);
+
+    e2.insert("FOO", "bar");
+    QCOMPARE(e1, e2);
+
+    e2.insert("FOO", "baz");
+    QVERIFY(e1 != e2);
 }
 
 void tst_QProcessEnvironment::clearAndIsEmpty()
@@ -196,6 +199,15 @@ void tst_QProcessEnvironment::insertEnv()
     QCOMPARE(e.value("Hello"), QString("Another World"));
     QCOMPARE(e.value("FOO2"), QString("bar2"));
     QCOMPARE(e.value("A2"), QString("bc2"));
+
+    QProcessEnvironment e3;
+    e3.insert("FOO2", "bar2");
+    e3.insert("A2", "bc2");
+    e3.insert("Hello", "Another World");
+
+    e3.insert(e3); // mustn't deadlock
+
+    QCOMPARE(e3, e2);
 }
 
 void tst_QProcessEnvironment::caseSensitivity()
