@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -47,6 +34,15 @@ class tst_QResourceEngine: public QObject
 {
     Q_OBJECT
 
+public:
+    tst_QResourceEngine()
+#if defined(Q_OS_ANDROID)
+        : m_runtimeResourceRcc(QFileInfo(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/runtime_resource.rcc")).absoluteFilePath())
+#else
+        : m_runtimeResourceRcc(QFINDTESTDATA("runtime_resource.rcc"))
+#endif
+    {}
+
 private slots:
     void initTestCase();
     void cleanupTestCase();
@@ -59,20 +55,47 @@ private slots:
     void searchPath();
     void doubleSlashInRoot();
     void setLocale();
+
+private:
+    const QString m_runtimeResourceRcc;
 };
 
 
 void tst_QResourceEngine::initTestCase()
 {
-    QVERIFY(QResource::registerResource(QFINDTESTDATA("runtime_resource.rcc")));
-    QVERIFY(QResource::registerResource(QFINDTESTDATA("runtime_resource.rcc"), "/secondary_root/"));
+#if defined(Q_OS_ANDROID)
+    QString sourcePath(QStringLiteral(":/android_testdata/"));
+    QString dataPath(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+
+    QDirIterator it(sourcePath, QDirIterator::Subdirectories);
+    while (it.hasNext()) {
+        it.next();
+
+        QFileInfo fileInfo = it.fileInfo();
+        if (!fileInfo.isDir()) {
+            QString destination(dataPath + QLatin1Char('/') + fileInfo.filePath().mid(sourcePath.length()));
+            QFileInfo destinationFileInfo(destination);
+            if (!destinationFileInfo.exists()) {
+                QVERIFY(QDir().mkpath(destinationFileInfo.path()));
+                QVERIFY(QFile::copy(fileInfo.filePath(), destination));
+                QVERIFY(QFileInfo(destination).exists());
+            }
+        }
+    }
+
+    QVERIFY(QDir::setCurrent(dataPath));
+#endif
+
+    QVERIFY(!m_runtimeResourceRcc.isEmpty());
+    QVERIFY(QResource::registerResource(m_runtimeResourceRcc));
+    QVERIFY(QResource::registerResource(m_runtimeResourceRcc, "/secondary_root/"));
 }
 
 void tst_QResourceEngine::cleanupTestCase()
 {
     // make sure we don't leak memory
-    QVERIFY(QResource::unregisterResource(QFINDTESTDATA("runtime_resource.rcc")));
-    QVERIFY(QResource::unregisterResource(QFINDTESTDATA("runtime_resource.rcc"), "/secondary_root/"));
+    QVERIFY(QResource::unregisterResource(m_runtimeResourceRcc));
+    QVERIFY(QResource::unregisterResource(m_runtimeResourceRcc, "/secondary_root/"));
 }
 
 void tst_QResourceEngine::checkStructure_data()
@@ -86,16 +109,25 @@ void tst_QResourceEngine::checkStructure_data()
 
     QFileInfo info;
 
+    QStringList rootContents;
+    rootContents << QLatin1String("aliasdir")
+                 << QLatin1String("otherdir")
+                 << QLatin1String("qt-project.org")
+                 << QLatin1String("runtime_resource")
+                 << QLatin1String("searchpath1")
+                 << QLatin1String("searchpath2")
+                 << QLatin1String("secondary_root")
+                 << QLatin1String("test")
+                 << QLatin1String("withoutslashes");
+
+#if defined(Q_OS_ANDROID)
+    rootContents.insert(1, QLatin1String("android_testdata"));
+#endif
+
     QTest::newRow("root dir")          << QString(":/")
                                        << QString()
                                        << (QStringList() << "search_file.txt")
-                                       << (QStringList() << QLatin1String("aliasdir") << QLatin1String("otherdir")
-                                           << QLatin1String("qt-project.org")
-                                           << QLatin1String("runtime_resource")
-                                           << QLatin1String("searchpath1") << QLatin1String("searchpath2")
-                                           << QLatin1String("secondary_root")
-                                           << QLatin1String("test")
-                                           << QLatin1String("withoutslashes"))
+                                       << rootContents
                                        << QLocale::c()
                                        << qlonglong(0);
 
