@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -42,10 +29,12 @@
 #include <QtTest/QtTest>
 #include <qcoreapplication.h>
 #include <qstring.h>
+#include <qtemporarydir.h>
 #include <qtemporaryfile.h>
 #include <qfile.h>
 #include <qdir.h>
 #include <qset.h>
+#include <qtextcodec.h>
 
 #if defined(Q_OS_WIN)
 # include <windows.h>
@@ -93,19 +82,43 @@ private slots:
     void QTBUG_4796_data();
     void QTBUG_4796();
     void guaranteeUnique();
+private:
+    QTemporaryDir m_temporaryDir;
+    QString m_previousCurrent;
 };
 
 void tst_QTemporaryFile::initTestCase()
 {
+    QVERIFY2(m_temporaryDir.isValid(), qPrintable(m_temporaryDir.errorString()));
+    m_previousCurrent = QDir::currentPath();
+    QVERIFY(QDir::setCurrent(m_temporaryDir.path()));
+
     // For QTBUG_4796
     QVERIFY(QDir("test-XXXXXX").exists() || QDir().mkdir("test-XXXXXX"));
     QCoreApplication::setApplicationName("tst_qtemporaryfile");
+
+#if defined(Q_OS_ANDROID)
+    QString sourceDir(":/android_testdata/");
+    QDirIterator it(sourceDir, QDirIterator::Subdirectories);
+    while (it.hasNext()) {
+        it.next();
+
+        QFileInfo sourceFileInfo = it.fileInfo();
+        if (!sourceFileInfo.isDir()) {
+            QFileInfo destinationFileInfo(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QLatin1Char('/') + sourceFileInfo.filePath().mid(sourceDir.length()));
+
+            if (!destinationFileInfo.exists()) {
+                QVERIFY(QDir().mkpath(destinationFileInfo.path()));
+                QVERIFY(QFile::copy(sourceFileInfo.filePath(), destinationFileInfo.filePath()));
+            }
+        }
+    }
+#endif
 }
 
 void tst_QTemporaryFile::cleanupTestCase()
 {
-    // From QTBUG_4796
-    QVERIFY(QDir().rmdir("test-XXXXXX"));
+    QDir::setCurrent(m_previousCurrent);
 }
 
 void tst_QTemporaryFile::construction()
@@ -128,6 +141,38 @@ void tst_QTemporaryFile::getSetCheck()
     QCOMPARE(true, obj1.autoRemove());
 }
 
+static inline bool canHandleUnicodeFileNames()
+{
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
+    return true;
+#else
+    // Check for UTF-8 by converting the Euro symbol (see tst_utf8)
+    return QFile::encodeName(QString(QChar(0x20AC))) == QByteArrayLiteral("\342\202\254");
+#endif
+}
+
+static QString hanTestText()
+{
+    QString text;
+    text += QChar(0x65B0);
+    text += QChar(0x5E10);
+    text += QChar(0x6237);
+    return text;
+}
+
+static QString umlautTestText()
+{
+    QString text;
+    text += QChar(0xc4);
+    text += QChar(0xe4);
+    text += QChar(0xd6);
+    text += QChar(0xf6);
+    text += QChar(0xdc);
+    text += QChar(0xfc);
+    text += QChar(0xdf);
+    return text;
+}
+
 void tst_QTemporaryFile::fileTemplate_data()
 {
     QTest::addColumn<QString>("constructorTemplate");
@@ -154,6 +199,14 @@ void tst_QTemporaryFile::fileTemplate_data()
     QTest::newRow("set template, with xxx") << "" << "qt_" << ".xxx" << "qt_XXXXXX.xxx";
     QTest::newRow("set template, with >6 X's") << "" << "qt_" << ".xxx" << "qt_XXXXXXXXXXXXXX.xxx";
     QTest::newRow("set template, with >6 X's, no suffix") << "" << "qt_" << "" << "qt_XXXXXXXXXXXXXX";
+    if (canHandleUnicodeFileNames()) {
+        // Test Umlauts (contained in Latin1)
+        QString prefix = "qt_" + umlautTestText();
+        QTest::newRow("Umlauts") << (prefix + "XXXXXX") << prefix << QString() << QString();
+        // Test Chinese
+        prefix = "qt_" + hanTestText();
+        QTest::newRow("Chinese characters") << (prefix + "XXXXXX") << prefix << QString() << QString();
+    }
 }
 
 void tst_QTemporaryFile::fileTemplate()
@@ -251,28 +304,39 @@ void tst_QTemporaryFile::autoRemove()
         QTemporaryFile file("tempXXXXXX");
         QVERIFY(file.open());
         fileName = file.fileName();
+        // QTBUG-39976, file mappings should be cleared as well.
+        QVERIFY(file.write("test"));
+        QVERIFY(file.flush());
+        uchar *mapped = file.map(0, file.size());
+        QVERIFY(mapped);
         file.close();
     }
     QVERIFY(!QFile::exists(fileName));
 }
 
+struct ChdirOnReturn
+{
+    ChdirOnReturn(const QString& d) : dir(d) {}
+    ~ChdirOnReturn() {
+        QDir::setCurrent(dir);
+    }
+    QString dir;
+};
+
 void tst_QTemporaryFile::nonWritableCurrentDir()
 {
 #ifdef Q_OS_UNIX
     if (::geteuid() == 0)
         QSKIP("not valid running this test as root");
 
-    struct ChdirOnReturn
-    {
-        ChdirOnReturn(const QString& d) : dir(d) {}
-        ~ChdirOnReturn() {
-            QDir::setCurrent(dir);
-        }
-        QString dir;
-    };
     ChdirOnReturn cor(QDir::currentPath());
 
+#if defined(Q_OS_ANDROID)
+    QDir::setCurrent("/data");
+#else
     QDir::setCurrent("/home");
+#endif
+
     // QTemporaryFile("tempXXXXXX") is probably a bad idea in any app
     // where the current dir could anything...
     QTemporaryFile file("tempXXXXXX");
@@ -332,6 +396,7 @@ void tst_QTemporaryFile::removeAndReOpen()
         QVERIFY(!QFile::exists(fileName));
 
         QVERIFY(file.open());
+        QCOMPARE(QFileInfo(file.fileName()).path(), QFileInfo(fileName).path());
         fileName = file.fileName();
         QVERIFY(QFile::exists(fileName));
     }
@@ -371,7 +436,7 @@ void tst_QTemporaryFile::resize()
 
 void tst_QTemporaryFile::openOnRootDrives()
 {
-#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
     unsigned int lastErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
 #endif
     // If it's possible to create a file in the root directory, it
@@ -385,7 +450,7 @@ void tst_QTemporaryFile::openOnRootDrives()
             QVERIFY(file.open());
         }
     }
-#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
     SetErrorMode(lastErrorMode);
 #endif
 }
@@ -441,13 +506,21 @@ void tst_QTemporaryFile::rename()
 void tst_QTemporaryFile::renameFdLeak()
 {
 #ifdef Q_OS_UNIX
+
+#  if defined(Q_OS_ANDROID)
+    ChdirOnReturn cor(QDir::currentPath());
+    QDir::setCurrent(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+#  endif
+
+    const QByteArray sourceFile = QFile::encodeName(QFINDTESTDATA(__FILE__));
+    QVERIFY(!sourceFile.isEmpty());
     // Test this on Unix only
 
     // Open a bunch of files to force the fd count to go up
     static const int count = 10;
     int bunch_of_files[count];
     for (int i = 0; i < count; ++i) {
-        bunch_of_files[i] = ::open(qPrintable(QFINDTESTDATA("tst_qtemporaryfile.cpp")), O_RDONLY);
+        bunch_of_files[i] = ::open(sourceFile.constData(), O_RDONLY);
         QVERIFY(bunch_of_files[i] != -1);
     }
 
@@ -508,15 +581,16 @@ void tst_QTemporaryFile::keepOpenMode()
     {
         QTemporaryFile file;
         QVERIFY(file.open());
+        QCOMPARE(file.openMode(), QIODevice::ReadWrite);
         QCOMPARE(file.write(data), (qint64)data.size());
         QVERIFY(file.rename("temporary-file.txt"));
 
         QVERIFY(((QFile &)file).open(QIODevice::ReadOnly));
-        QVERIFY(QIODevice::ReadOnly == file.openMode());
+        QCOMPARE(file.openMode(), QIODevice::ReadOnly);
         QCOMPARE(file.readAll(), data);
 
         QVERIFY(((QFile &)file).open(QIODevice::WriteOnly));
-        QVERIFY(QIODevice::WriteOnly == file.openMode());
+        QCOMPARE(file.openMode(), QIODevice::WriteOnly);
     }
 }
 
@@ -642,8 +716,17 @@ void tst_QTemporaryFile::createNativeFile_data()
     QTest::addColumn<bool>("valid");
     QTest::addColumn<QByteArray>("content");
 
-    QTest::newRow("nativeFile") << QFINDTESTDATA("resources/test.txt") << (qint64)-1 << false << QByteArray();
-    QTest::newRow("nativeFileWithPos") << QFINDTESTDATA("resources/test.txt") << (qint64)5 << false << QByteArray();
+#if defined(Q_OS_ANDROID)
+    const QString nativeFilePath = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/resources/test.txt");
+#else
+    const QString nativeFilePath = QFINDTESTDATA("resources/test.txt");
+#endif
+
+    // File might not exist locally in case of sandboxing or remote testing
+    if (!nativeFilePath.startsWith(QLatin1String(":/"))) {
+        QTest::newRow("nativeFile") << nativeFilePath << (qint64)-1 << false << QByteArray();
+        QTest::newRow("nativeFileWithPos") << nativeFilePath << (qint64)5 << false << QByteArray();
+    }
     QTest::newRow("resourceFile") << ":/resources/test.txt" << (qint64)-1 << true << QByteArray("This is a test");
     QTest::newRow("resourceFileWithPos") << ":/resources/test.txt" << (qint64)5 << true << QByteArray("This is a test");
 }
@@ -661,7 +744,7 @@ void tst_QTemporaryFile::createNativeFile()
         f.seek(currentPos);
     }
     QTemporaryFile *tempFile = QTemporaryFile::createNativeFile(f);
-    QVERIFY(valid == (bool)tempFile);
+    QCOMPARE(valid, (bool)tempFile);
     if (currentPos != -1)
         QCOMPARE(currentPos, f.pos());
     if (valid) {
