@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -84,9 +79,9 @@ void tst_QFuture::resultStore()
 
     {
         QtPrivate::ResultStore<int> store;
-        QVERIFY(store.begin() == store.end());
-        QVERIFY(store.resultAt(0) == store.end());
-        QVERIFY(store.resultAt(1) == store.end());
+        QCOMPARE(store.begin(), store.end());
+        QCOMPARE(store.resultAt(0), store.end());
+        QCOMPARE(store.resultAt(1), store.end());
     }
 
 
@@ -118,7 +113,7 @@ void tst_QFuture::resultStore()
         store.addResults(-1, &vec1, 2, 2);
         QtPrivate::ResultIteratorBase it = store.begin();
         QCOMPARE(it.resultIndex(), 0);
-        QVERIFY(it == store.begin());
+        QCOMPARE(it, store.begin());
         QVERIFY(it != store.end());
 
         ++it;
@@ -133,7 +128,7 @@ void tst_QFuture::resultStore()
         QCOMPARE(it.resultIndex(), 3);
 
         ++it;
-        QVERIFY(it == store.end());
+        QCOMPARE(it, store.end());
     }
     {
         QtPrivate::ResultStoreBase store;
@@ -1012,14 +1007,14 @@ void tst_QFuture::iterators()
         QFuture<int>::const_iterator i1 = f.begin(), i2 = i1 + 1;
         QFuture<int>::const_iterator c1 = i1, c2 = c1 + 1;
 
-        QVERIFY(i1 == i1);
-        QVERIFY(i1 == c1);
-        QVERIFY(c1 == i1);
-        QVERIFY(c1 == c1);
-        QVERIFY(i2 == i2);
-        QVERIFY(i2 == c2);
-        QVERIFY(c2 == i2);
-        QVERIFY(c2 == c2);
+        QCOMPARE(i1, i1);
+        QCOMPARE(i1, c1);
+        QCOMPARE(c1, i1);
+        QCOMPARE(c1, c1);
+        QCOMPARE(i2, i2);
+        QCOMPARE(i2, c2);
+        QCOMPARE(c2, i2);
+        QCOMPARE(c2, c2);
 
         QVERIFY(i1 != i2);
         QVERIFY(i1 != c2);
@@ -1061,14 +1056,14 @@ void tst_QFuture::iterators()
         QFuture<QString>::const_iterator i1 = f.begin(), i2 = i1 + 1;
         QFuture<QString>::const_iterator c1 = i1, c2 = c1 + 1;
 
-        QVERIFY(i1 == i1);
-        QVERIFY(i1 == c1);
-        QVERIFY(c1 == i1);
-        QVERIFY(c1 == c1);
-        QVERIFY(i2 == i2);
-        QVERIFY(i2 == c2);
-        QVERIFY(c2 == i2);
-        QVERIFY(c2 == c2);
+        QCOMPARE(i1, i1);
+        QCOMPARE(i1, c1);
+        QCOMPARE(c1, i1);
+        QCOMPARE(c1, c1);
+        QCOMPARE(i2, i2);
+        QCOMPARE(i2, c2);
+        QCOMPARE(c2, i2);
+        QCOMPARE(c2, c2);
 
         QVERIFY(i1 != i2);
         QVERIFY(i1 != c2);
@@ -1438,6 +1433,8 @@ void tst_QFuture::nestedExceptions()
     QVERIFY(MyClass::caught);
 }
 
+#endif // QT_NO_EXCEPTIONS
+
 void tst_QFuture::nonGlobalThreadPool()
 {
     static Q_CONSTEXPR int Answer = 42;
@@ -1485,7 +1482,5 @@ void tst_QFuture::nonGlobalThreadPool()
     }
 }
 
-#endif // QT_NO_EXCEPTIONS
-
 QTEST_MAIN(tst_QFuture)
 #include "tst_qfuture.moc"
