@@ -1,31 +1,27 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -962,88 +958,51 @@ void tst_QThreadPool::clear()
 void tst_QThreadPool::cancel()
 {
     QSemaphore sem(0);
-    QSemaphore startedThreads(0);
-
-    class SemaphoreReleaser
-    {
-        QSemaphore &sem;
-        int n;
-        Q_DISABLE_COPY(SemaphoreReleaser)
-    public:
-        explicit SemaphoreReleaser(QSemaphore &sem, int n)
-            : sem(sem), n(n) {}
-
-        ~SemaphoreReleaser()
-        {
-            sem.release(n);
-        }
-    };
-
     class BlockingRunnable : public QRunnable
     {
     public:
         QSemaphore & sem;
-        QSemaphore &startedThreads;
-        QAtomicInt &dtorCounter;
-        QAtomicInt &runCounter;
+        int & dtorCounter;
+        int & runCounter;
         int dummy;
-
-        explicit BlockingRunnable(QSemaphore &s, QSemaphore &started, QAtomicInt &c, QAtomicInt &r)
-            : sem(s), startedThreads(started), dtorCounter(c), runCounter(r){}
-
-        ~BlockingRunnable()
-        {
-            dtorCounter.fetchAndAddRelaxed(1);
-        }
-
+        BlockingRunnable(QSemaphore & s, int & c, int & r) : sem(s), dtorCounter(c), runCounter(r){}
+        ~BlockingRunnable(){dtorCounter++;}
         void run()
         {
-            startedThreads.release();
-            runCounter.fetchAndAddRelaxed(1);
+            runCounter++;
             sem.acquire();
             count.ref();
         }
     };
-
-    enum {
-        MaxThreadCount = 3,
-        OverProvisioning = 2,
-        runs = MaxThreadCount * OverProvisioning
-    };
+    typedef BlockingRunnable* BlockingRunnablePtr;
 
     QThreadPool threadPool;
-    threadPool.setMaxThreadCount(MaxThreadCount);
-    BlockingRunnable *runnables[runs];
-
-    // ensure that the QThreadPool doesn't deadlock if any of the checks fail
-    // and cause an early return:
-    const SemaphoreReleaser semReleaser(sem, runs);
-
+    threadPool.setMaxThreadCount(3);
+    int runs = 2 * threadPool.maxThreadCount();
+    BlockingRunnablePtr* runnables = new BlockingRunnablePtr[runs];
     count.store(0);
-    QAtomicInt dtorCounter = 0;
-    QAtomicInt runCounter = 0;
+    int dtorCounter = 0;
+    int runCounter = 0;
     for (int i = 0; i < runs; i++) {
-        runnables[i] = new BlockingRunnable(sem, startedThreads, dtorCounter, runCounter);
+        runnables[i] = new BlockingRunnable(sem, dtorCounter, runCounter);
         runnables[i]->setAutoDelete(i != 0 && i != (runs-1)); //one which will run and one which will not
         threadPool.cancel(runnables[i]); //verify NOOP for jobs not in the queue
         threadPool.start(runnables[i]);
     }
-    // wait for all worker threads to have started up:
-    QVERIFY(startedThreads.tryAcquire(MaxThreadCount, 60*1000 /* 1min */));
-
     for (int i = 0; i < runs; i++) {
         threadPool.cancel(runnables[i]);
     }
     runnables[0]->dummy = 0; //valgrind will catch this if cancel() is crazy enough to delete currently running jobs
     runnables[runs-1]->dummy = 0;
-    QCOMPARE(dtorCounter.load(), runs - threadPool.maxThreadCount() - 1);
+    QCOMPARE(dtorCounter, runs-threadPool.maxThreadCount()-1);
     sem.release(threadPool.maxThreadCount());
     threadPool.waitForDone();
-    QCOMPARE(runCounter.load(), threadPool.maxThreadCount());
+    QCOMPARE(runCounter, threadPool.maxThreadCount());
     QCOMPARE(count.load(), threadPool.maxThreadCount());
-    QCOMPARE(dtorCounter.load(), runs - 2);
+    QCOMPARE(dtorCounter, runs-2);
     delete runnables[0]; //if the pool deletes them then we'll get double-free crash
     delete runnables[runs-1];
+    delete[] runnables;
 }
 
 void tst_QThreadPool::destroyingWaitsForTasksToFinish()
