@@ -2,31 +2,26 @@
 **
 ** Copyright (C) 2015 Giuseppe D'Angelo <dangelog@gmail.com>.
 ** Copyright (C) 2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Giuseppe D'Angelo <giuseppe.dangelo@kdab.com>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -656,7 +651,7 @@ void tst_QRegularExpression::normalMatch_data()
     m.captured << "c123def" << "c12" << "3" << "def";
     offset = 2;
     for (int i = 0; i <= offset; ++i) {
-        QTest::newRow(QStringLiteral("match06-offset%1").arg(i).toUtf8().constData())
+        QTest::newRow(("match06-offset" + QByteArray::number(i)).constData())
                 << QRegularExpression("(\\w*)(\\d+)(\\w*)")
                 << QStringLiteral("abc123def").mid(offset - i)
                 << i
@@ -669,7 +664,7 @@ void tst_QRegularExpression::normalMatch_data()
     m.captured << QString("");
     offset = 9;
     for (int i = 0; i <= offset; ++i) {
-        QTest::newRow(QStringLiteral("match07-offset%1").arg(i).toUtf8().constData())
+        QTest::newRow(("match07-offset" + QByteArray::number(i)).constData())
                 << QRegularExpression("\\w*")
                 << QStringLiteral("abc123def").mid(offset - i)
                 << i
@@ -747,7 +742,7 @@ void tst_QRegularExpression::normalMatch_data()
     m.isValid = true;
     offset = 1;
     for (int i = 0; i <= offset; ++i) {
-        QTest::newRow(QStringLiteral("nomatch02-offset%1").arg(i).toUtf8().constData())
+        QTest::newRow(("nomatch02-offset" + QByteArray::number(i)).constData())
             << QRegularExpression("(\\w+) (\\w+)")
             << QStringLiteral("a string").mid(offset - i)
             << i
@@ -759,7 +754,7 @@ void tst_QRegularExpression::normalMatch_data()
     m.isValid = true;
     offset = 9;
     for (int i = 0; i <= offset; ++i) {
-        QTest::newRow(QStringLiteral("nomatch03-offset%1").arg(i).toUtf8().constData())
+        QTest::newRow(("nomatch03-offset" + QByteArray::number(i)).constData())
                 << QRegularExpression("\\w+")
                 << QStringLiteral("abc123def").mid(offset - i)
                 << i
@@ -930,7 +925,7 @@ void tst_QRegularExpression::partialMatch_data()
     m.captured << "def";
     offset = 1;
     for (int i = 0; i <= offset; ++i) {
-        QTest::newRow(QStringLiteral("softmatch08-offset%1").arg(i).toUtf8().constData())
+        QTest::newRow(("softmatch08-offset" + QByteArray::number(i)).constData())
                 << QRegularExpression("abc\\w+X|defY")
                 << QStringLiteral("abcdef").mid(offset - i)
                 << i
@@ -1016,7 +1011,7 @@ void tst_QRegularExpression::partialMatch_data()
     m.captured << "def";
     offset = 1;
     for (int i = 0; i <= offset; ++i) {
-        QTest::newRow(QStringLiteral("hardmatch08-offset%1").arg(i).toUtf8().constData())
+        QTest::newRow(("hardmatch08-offset" + QByteArray::number(i)).constData())
                 << QRegularExpression("abc\\w+X|defY")
                 << QStringLiteral("abcdef").mid(offset - i)
                 << i
@@ -1423,6 +1418,7 @@ static void verifyEquality(const QRegularExpression &re1, const QRegularExpressi
 {
     QVERIFY(re1 == re2);
     QVERIFY(re2 == re1);
+    QCOMPARE(qHash(re1), qHash(re2));
     QVERIFY(!(re1 != re2));
     QVERIFY(!(re2 != re1));
 
@@ -1430,22 +1426,26 @@ static void verifyEquality(const QRegularExpression &re1, const QRegularExpressi
 
     QVERIFY(re1 == re3);
     QVERIFY(re3 == re1);
+    QCOMPARE(qHash(re1), qHash(re3));
     QVERIFY(!(re1 != re3));
     QVERIFY(!(re3 != re1));
 
     QVERIFY(re2 == re3);
     QVERIFY(re3 == re2);
+    QCOMPARE(qHash(re2), qHash(re3));
     QVERIFY(!(re2 != re3));
     QVERIFY(!(re3 != re2));
 
     re3 = re2;
     QVERIFY(re1 == re3);
     QVERIFY(re3 == re1);
+    QCOMPARE(qHash(re1), qHash(re3));
     QVERIFY(!(re1 != re3));
     QVERIFY(!(re3 != re1));
 
     QVERIFY(re2 == re3);
     QVERIFY(re3 == re2);
+    QCOMPARE(qHash(re2), qHash(re3));
     QVERIFY(!(re2 != re3));
     QVERIFY(!(re3 != re2));
 }
