@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -34,33 +29,64 @@
 #include <QtTest/QtTest>
 
 #include <private/qringbuffer_p.h>
+#include <qvector.h>
 
 class tst_QRingBuffer : public QObject
 {
     Q_OBJECT
 private slots:
+    void constructing();
+    void usingInVector();
     void readPointerAtPositionWriteRead();
     void readPointerAtPositionEmptyRead();
     void readPointerAtPositionWithHead();
     void readPointerAtPositionReadTooMuch();
-    void sizeWhenEmpty();
     void sizeWhenReservedAndChopped();
     void sizeWhenReserved();
     void free();
     void reserveAndRead();
+    void reserveFrontAndRead();
     void chop();
     void ungetChar();
     void indexOf();
     void appendAndRead();
+    void peek();
     void readLine();
 };
 
+void tst_QRingBuffer::constructing()
+{
+    QRingBuffer ringBuffer;
+
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(0));
+    QVERIFY(ringBuffer.isEmpty());
+    QCOMPARE(ringBuffer.nextDataBlockSize(), Q_INT64_C(0));
+    QVERIFY(ringBuffer.readPointer() == Q_NULLPTR);
+    QCOMPARE(ringBuffer.skip(5), Q_INT64_C(0));
+    QCOMPARE(ringBuffer.read(), QByteArray());
+    QCOMPARE(ringBuffer.getChar(), -1);
+    QVERIFY(!ringBuffer.canReadLine());
+
+    char buf[5];
+    QCOMPARE(ringBuffer.peek(buf, sizeof(buf)), Q_INT64_C(0));
+}
+
+void tst_QRingBuffer::usingInVector()
+{
+    QRingBuffer ringBuffer;
+    QVector<QRingBuffer> buffers;
+
+    ringBuffer.reserve(5);
+    buffers.append(ringBuffer);
+    QCOMPARE(buffers[0].size(), Q_INT64_C(5));
+}
+
 void tst_QRingBuffer::sizeWhenReserved()
 {
     QRingBuffer ringBuffer;
     ringBuffer.reserve(5);
 
-    QCOMPARE(ringBuffer.size(), 5);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(5));
 }
 
 void tst_QRingBuffer::sizeWhenReservedAndChopped()
@@ -69,14 +95,7 @@ void tst_QRingBuffer::sizeWhenReservedAndChopped()
     ringBuffer.reserve(31337);
     ringBuffer.chop(31337);
 
-    QCOMPARE(ringBuffer.size(), 0);
-}
-
-void tst_QRingBuffer::sizeWhenEmpty()
-{
-    QRingBuffer ringBuffer;
-
-    QCOMPARE(ringBuffer.size(), 0);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(0));
 }
 
 void tst_QRingBuffer::readPointerAtPositionReadTooMuch()
@@ -86,7 +105,7 @@ void tst_QRingBuffer::readPointerAtPositionReadTooMuch()
     qint64 length;
     const char *buf = ringBuffer.readPointerAtPosition(42, length);
     QVERIFY(buf == 0);
-    QVERIFY(length == 0);
+    QCOMPARE(length, Q_INT64_C(0));
 }
 
 void tst_QRingBuffer::readPointerAtPositionWithHead()
@@ -101,22 +120,22 @@ void tst_QRingBuffer::readPointerAtPositionWithHead()
     qint64 length;
     const char* buf2 = ringBuffer.readPointerAtPosition(0, length);
 
-    QCOMPARE(length, qint64(2));
-    QVERIFY(*buf2 == '2');
-    QVERIFY(*(buf2+1) == '3');
+    QCOMPARE(length, Q_INT64_C(2));
+    QCOMPARE(*buf2, '2');
+    QCOMPARE(*(buf2 + 1), '3');
 
     // advance 2 more, ringBuffer should be empty then
     ringBuffer.free(2);
     buf2 = ringBuffer.readPointerAtPosition(0, length);
-    QCOMPARE(length, qint64(0));
+    QCOMPARE(length, Q_INT64_C(0));
     QVERIFY(buf2 == 0);
 
     // check buffer with 2 blocks
     memcpy(ringBuffer.reserve(4), "0123", 4);
     ringBuffer.append(QByteArray("45678", 5));
     ringBuffer.free(3);
-    buf2 = ringBuffer.readPointerAtPosition(1, length);
-    QCOMPARE(length, qint64(5));
+    buf2 = ringBuffer.readPointerAtPosition(Q_INT64_C(1), length);
+    QCOMPARE(length, Q_INT64_C(5));
 }
 
 void tst_QRingBuffer::readPointerAtPositionEmptyRead()
@@ -126,7 +145,7 @@ void tst_QRingBuffer::readPointerAtPositionEmptyRead()
     qint64 length;
     const char *buf = ringBuffer.readPointerAtPosition(0, length);
     QVERIFY(buf == 0);
-    QVERIFY(length == 0);
+    QCOMPARE(length, Q_INT64_C(0));
 }
 
 void tst_QRingBuffer::readPointerAtPositionWriteRead()
@@ -138,7 +157,7 @@ void tst_QRingBuffer::readPointerAtPositionWriteRead()
     inData.putChar(0x23);
     inData.write("Qt rocks!");
     for (int i = 0; i < 5000; i++)
-        inData.write(QString("Number %1").arg(i).toUtf8());
+        inData.write("Number " + QByteArray::number(i));
     inData.reset();
     QVERIFY(inData.size() > 0);
 
@@ -149,14 +168,14 @@ void tst_QRingBuffer::readPointerAtPositionWriteRead()
         // write in chunks of 50 bytes
         // this ensures there will be multiple QByteArrays inside the QRingBuffer
         // since QRingBuffer is then only using individual arrays of around 4000 bytes
-        qint64 thisWrite = qMin(remaining, qint64(50));
+        qint64 thisWrite = qMin(remaining, Q_INT64_C(50));
         char *pos = ringBuffer.reserve(thisWrite);
         inData.read(pos, thisWrite);
         remaining -= thisWrite;
     }
     // was data put into it?
     QVERIFY(ringBuffer.size() > 0);
-    QCOMPARE(qint64(ringBuffer.size()), inData.size());
+    QCOMPARE(ringBuffer.size(), inData.size());
 
     //read from the QRingBuffer in loop, put back into another QBuffer
     QBuffer outData;
@@ -187,12 +206,12 @@ void tst_QRingBuffer::free()
     ringBuffer.append(QByteArray("01234", 5));
 
     ringBuffer.free(1);
-    QCOMPARE(ringBuffer.size(), 4095 + 2048 + 5);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(4095) + 2048 + 5);
     ringBuffer.free(4096);
-    QCOMPARE(ringBuffer.size(), 2047 + 5);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(2047) + 5);
     ringBuffer.free(48);
     ringBuffer.free(2000);
-    QCOMPARE(ringBuffer.size(), 4);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(4));
     QVERIFY(memcmp(ringBuffer.readPointer(), "1234", 4) == 0);
 }
 
@@ -211,11 +230,33 @@ void tst_QRingBuffer::reserveAndRead()
     for (int i = 1; i < 256; ++i) {
         QByteArray ba;
         ba.resize(i);
-        int thisRead = ringBuffer.read(ba.data(), i);
-        QCOMPARE(thisRead, i);
-        QVERIFY(ba.count(char(i)) == i);
+        qint64 thisRead = ringBuffer.read(ba.data(), i);
+        QCOMPARE(thisRead, qint64(i));
+        QCOMPARE(ba.count(char(i)), i);
     }
-    QVERIFY(ringBuffer.size() == 0);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(0));
+}
+
+void tst_QRingBuffer::reserveFrontAndRead()
+{
+    QRingBuffer ringBuffer;
+    // fill buffer with an arithmetic progression
+    for (int i = 1; i < 256; ++i) {
+        QByteArray ba(i, char(i));
+        char *ringPos = ringBuffer.reserveFront(i);
+        QVERIFY(ringPos);
+        memcpy(ringPos, ba.constData(), i);
+    }
+
+    // readback and check stored data
+    for (int i = 255; i > 0; --i) {
+        QByteArray ba;
+        ba.resize(i);
+        qint64 thisRead = ringBuffer.read(ba.data(), i);
+        QCOMPARE(thisRead, qint64(i));
+        QCOMPARE(ba.count(char(i)), i);
+    }
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(0));
 }
 
 void tst_QRingBuffer::chop()
@@ -227,12 +268,12 @@ void tst_QRingBuffer::chop()
     ringBuffer.reserve(4096);
 
     ringBuffer.chop(1);
-    QCOMPARE(ringBuffer.size(), 5 + 2048 + 4095);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(5) + 2048 + 4095);
     ringBuffer.chop(4096);
-    QCOMPARE(ringBuffer.size(), 5 + 2047);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(5) + 2047);
     ringBuffer.chop(48);
     ringBuffer.chop(2000);
-    QCOMPARE(ringBuffer.size(), 4);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(4));
     QVERIFY(memcmp(ringBuffer.readPointer(), "0123", 4) == 0);
 }
 
@@ -244,11 +285,11 @@ void tst_QRingBuffer::ungetChar()
 
     for (int i = 1; i < 31; ++i) {
         int c = ringBuffer.getChar();
-        QVERIFY(c == 1);
+        QCOMPARE(c, 1);
         ringBuffer.getChar();
         ringBuffer.ungetChar(char(c)); // unget first char
     }
-    QCOMPARE(ringBuffer.size(), 1);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(1));
 }
 
 void tst_QRingBuffer::indexOf()
@@ -258,10 +299,11 @@ void tst_QRingBuffer::indexOf()
         ringBuffer.putChar(char(i));
 
     for (int i = 1; i < 256; ++i) {
-        int index = ringBuffer.indexOf(char(i));
-        QCOMPARE(i - 1, index);
-        QCOMPARE(index, ringBuffer.indexOf(char(i), i));
-        QVERIFY(ringBuffer.indexOf(char(i), i - 1) == -1); // test for absent char
+        qint64 index = ringBuffer.indexOf(char(i));
+        QCOMPARE(index, qint64(i - 1));
+        QCOMPARE(ringBuffer.indexOf(char(i), i, i >> 1), index);
+        QCOMPARE(ringBuffer.indexOf(char(i), 256, i), Q_INT64_C(-1));
+        QCOMPARE(ringBuffer.indexOf(char(i), i - 1), -1); // test for absent char
     }
 }
 
@@ -275,9 +317,34 @@ void tst_QRingBuffer::appendAndRead()
     ringBuffer.append(ba2);
     ringBuffer.append(ba3);
 
-    QVERIFY(ringBuffer.read() == ba1);
-    QVERIFY(ringBuffer.read() == ba2);
-    QVERIFY(ringBuffer.read() == ba3);
+    QCOMPARE(ringBuffer.read(), ba1);
+    QCOMPARE(ringBuffer.read(), ba2);
+    QCOMPARE(ringBuffer.read(), ba3);
+}
+
+void tst_QRingBuffer::peek()
+{
+    QRingBuffer ringBuffer;
+    QByteArray testBuffer;
+    // fill buffer with an arithmetic progression
+    for (int i = 1; i < 256; ++i) {
+        char *ringPos = ringBuffer.reserve(i);
+        QVERIFY(ringPos);
+        memset(ringPos, i, i);
+        testBuffer.append(ringPos, i);
+    }
+
+    // check stored data
+    QByteArray resultBuffer;
+    int peekPosition = testBuffer.size();
+    for (int i = 1; i < 256; ++i) {
+        QByteArray ba(i, 0);
+        peekPosition -= i;
+        qint64 thisPeek = ringBuffer.peek(ba.data(), i, peekPosition);
+        QCOMPARE(thisPeek, qint64(i));
+        resultBuffer.prepend(ba);
+    }
+    QCOMPARE(resultBuffer, testBuffer);
 }
 
 void tst_QRingBuffer::readLine()
@@ -293,18 +360,18 @@ void tst_QRingBuffer::readLine()
 
     char stringBuf[102];
     stringBuf[101] = 0; // non-crash terminator
-    QVERIFY(ringBuffer.readLine(stringBuf, sizeof(stringBuf) - 2) == ba1.size());
-    QVERIFY(QByteArray(stringBuf, strlen(stringBuf)) == ba1);
+    QCOMPARE(ringBuffer.readLine(stringBuf, sizeof(stringBuf) - 2), qint64(ba1.size()));
+    QCOMPARE(QByteArray(stringBuf, int(strlen(stringBuf))), ba1);
 
     // check first empty string reading
-    stringBuf[0] = 0xFF;
-    QCOMPARE(ringBuffer.readLine(stringBuf, sizeof(stringBuf) - 2), ba2.size());
-    QVERIFY(stringBuf[0] == ba2[0]);
+    stringBuf[0] = char(0xFF);
+    QCOMPARE(ringBuffer.readLine(stringBuf, int(sizeof(stringBuf)) - 2), qint64(ba2.size()));
+    QCOMPARE(stringBuf[0], ba2.at(0));
 
-    QVERIFY(ringBuffer.readLine(stringBuf, sizeof(stringBuf) - 2) == (ba3.size() + ba4.size()
-        + ba2.size()));
-    QVERIFY(QByteArray(stringBuf, strlen(stringBuf)) == (ba3 + ba4 + ba2));
-    QVERIFY(ringBuffer.size() == 0);
+    QCOMPARE(ringBuffer.readLine(stringBuf, int(sizeof(stringBuf)) - 2),
+             qint64(ba3.size() + ba4.size() + ba2.size()));
+    QCOMPARE(QByteArray(stringBuf, int(strlen(stringBuf))), ba3 + ba4 + ba2);
+    QCOMPARE(ringBuffer.size(), Q_INT64_C(0));
 }
 
 QTEST_APPLESS_MAIN(tst_QRingBuffer)
