@@ -1,32 +1,27 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Copyright (C) 2012 Intel Corporation.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -63,6 +58,7 @@ private slots:
     void basics();
     void operators();
     void swap();
+    void moveSemantics();
     void useOfForwardDeclared();
     void memoryManagement();
     void dropLastReferenceOfForwardDeclared();
@@ -421,6 +417,66 @@ void tst_QSharedPointer::swap()
     QVERIFY(w2.isNull());
 }
 
+void tst_QSharedPointer::moveSemantics()
+{
+#ifdef Q_COMPILER_RVALUE_REFS
+    QSharedPointer<int> p1, p2(new int(42)), control = p2;
+    QVERIFY(p1 != control);
+    QVERIFY(p1.isNull());
+    QVERIFY(p2 == control);
+    QVERIFY(!p2.isNull());
+    QVERIFY(*p2 == 42);
+
+    // move assignment
+    p1 = std::move(p2);
+    QVERIFY(p1 == control);
+    QVERIFY(!p1.isNull());
+    QVERIFY(p2 != control);
+    QVERIFY(p2.isNull());
+    QVERIFY(*p1 == 42);
+
+    // move construction
+    QSharedPointer<int> p3 = std::move(p1);
+    QVERIFY(p1 != control);
+    QVERIFY(p1.isNull());
+    QVERIFY(p3 == control);
+    QVERIFY(!p3.isNull());
+    QVERIFY(*p3 == 42);
+
+    QWeakPointer<int> w1, w2 = control;
+
+    QVERIFY(w1.isNull());
+    QVERIFY(!w2.isNull());
+    QVERIFY(w2.lock() == control);
+    QVERIFY(!w1.lock());
+
+    // move assignment
+    w1 = std::move(w2);
+    QVERIFY(w2.isNull());
+    QVERIFY(!w1.isNull());
+    QVERIFY(w1.lock() == control);
+    QVERIFY(!w2.lock());
+
+    // move construction
+    QWeakPointer<int> w3 = std::move(w1);
+    QVERIFY(w1.isNull());
+    QVERIFY(!w3.isNull());
+    QVERIFY(w3.lock() == control);
+    QVERIFY(!w1.lock());
+
+    p1.reset();
+    p2.reset();
+    p3.reset();
+    control.reset();
+
+    QVERIFY(w1.isNull());
+    QVERIFY(w2.isNull());
+    QVERIFY(w3.isNull());
+#else
+    QSKIP("This test requires C++11 rvalue/move semantics support in the compiler.");
+#endif
+}
+
 void tst_QSharedPointer::useOfForwardDeclared()
 {
     // this just a compile test: use the forward-declared class
