@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -82,21 +77,8 @@ struct Foo
 
 void tst_QVarLengthArray::append()
 {
-    QVarLengthArray<QString, 2> v;
-    v.append(QString("1"));
-    v.append(v.front());
-    QCOMPARE(v.capacity(), 2);
-    // transition from prealloc to heap:
-    v.append(v.front());
-    QVERIFY(v.capacity() > 2);
-    QCOMPARE(v.front(), v.back());
-    while (v.size() < v.capacity())
-        v.push_back(v[0]);
-    QCOMPARE(v.back(), v.front());
-    QCOMPARE(v.size(), v.capacity());
-    // transition from heap to larger heap:
-    v.push_back(v.front());
-    QCOMPARE(v.back(), v.front());
+    QVarLengthArray<QString> v;
+    v.append(QString("hello"));
 
     QVarLengthArray<int> v2; // rocket!
     v2.append(5);
@@ -161,14 +143,14 @@ void tst_QVarLengthArray::oldTests()
         QVarLengthArray<QString> sa(10);
         sa[0] = "Hello";
         sa[9] = "World";
-        QVERIFY(*sa.data() == "Hello");
-        QVERIFY(sa[9] == "World");
+        QCOMPARE(*sa.data(), QLatin1String("Hello"));
+        QCOMPARE(sa[9], QLatin1String("World"));
         sa.reserve(512);
-        QVERIFY(*sa.data() == "Hello");
-        QVERIFY(sa[9] == "World");
+        QCOMPARE(*sa.data(), QLatin1String("Hello"));
+        QCOMPARE(sa[9], QLatin1String("World"));
         sa.resize(512);
-        QVERIFY(*sa.data() == "Hello");
-        QVERIFY(sa[9] == "World");
+        QCOMPARE(*sa.data(), QLatin1String("Hello"));
+        QCOMPARE(sa[9], QLatin1String("World"));
     }
     {
         int arr[2] = {1, 2};
