@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -44,6 +31,7 @@
 #include <QImageReader>
 #include <qicon.h>
 #include <qiconengine.h>
+#include <QtCore/QStandardPaths>
 
 #include <algorithm>
 
@@ -61,6 +49,7 @@ private slots:
     void actualSize2_data(); // test with 2 pixmaps with different aspect ratio
     void actualSize2();
     void isNull();
+    void isMask();
     void swap();
     void bestMatch();
     void cacheKey();
@@ -71,6 +60,7 @@ private slots:
     void streamAvailableSizes_data();
     void streamAvailableSizes();
     void fromTheme();
+    void fromThemeCache();
 
 #ifndef QT_NO_WIDGETS
     void task184901_badCache();
@@ -83,14 +73,8 @@ private:
     const QString m_pngImageFileName;
     const QString m_pngRectFileName;
     const QString m_sourceFileName;
-
-    const static QIcon staticIcon;
 };
 
-// Creating an icon statically should not cause a crash.
-// But we do not officially support this. See QTBUG-8666
-const QIcon tst_QIcon::staticIcon = QIcon::fromTheme("edit-find");
-
 bool tst_QIcon::haveImageFormat(QByteArray const& desiredFormat)
 {
     return QImageReader::supportedImageFormats().contains(desiredFormat);
@@ -232,6 +216,20 @@ void tst_QIcon::isNull() {
     QVERIFY(iconSupportedFormat.actualSize(QSize(32, 32)).isValid());
 }
 
+void tst_QIcon::isMask()
+{
+    QIcon icon;
+    icon.setIsMask(true);
+    icon.addPixmap(QPixmap());
+    QVERIFY(icon.isMask());
+
+    QIcon icon2;
+    icon2.setIsMask(true);
+    QVERIFY(icon2.isMask());
+    icon2.setIsMask(false);
+    QVERIFY(!icon2.isMask());
+}
+
 void tst_QIcon::swap()
 {
     QPixmap p1(1, 1), p2(2, 2);
@@ -362,10 +360,10 @@ void tst_QIcon::cacheKey()
     qint64 icon1_key = icon1.cacheKey();
     QIcon icon2 = icon1;
 
-    QVERIFY(icon2.cacheKey() == icon1.cacheKey());
+    QCOMPARE(icon2.cacheKey(), icon1.cacheKey());
     icon2.detach();
     QVERIFY(icon2.cacheKey() != icon1.cacheKey());
-    QVERIFY(icon1.cacheKey() == icon1_key);
+    QCOMPARE(icon1.cacheKey(), icon1_key);
 }
 
 void tst_QIcon::detach()
@@ -382,7 +380,7 @@ void tst_QIcon::detach()
 
     img1 = icon1.pixmap(32, 32).toImage();
     img2 = icon2.pixmap(32, 32).toImage();
-    QVERIFY(img1 == img2);
+    QCOMPARE(img1, img2);
 }
 
 void tst_QIcon::addFile()
@@ -567,10 +565,12 @@ void tst_QIcon::task184901_badCache()
 
 void tst_QIcon::fromTheme()
 {
-    QString searchPath = QLatin1String(":/icons");
-    QIcon::setThemeSearchPaths(QStringList() << searchPath);
-    QVERIFY(QIcon::themeSearchPaths().size() == 1);
-    QCOMPARE(searchPath, QIcon::themeSearchPaths()[0]);
+    QString firstSearchPath = QLatin1String(":/icons");
+    QString secondSearchPath = QLatin1String(":/second_icons");
+    QIcon::setThemeSearchPaths(QStringList() << firstSearchPath << secondSearchPath);
+    QCOMPARE(QIcon::themeSearchPaths().size(), 2);
+    QCOMPARE(firstSearchPath, QIcon::themeSearchPaths()[0]);
+    QCOMPARE(secondSearchPath, QIcon::themeSearchPaths()[1]);
 
     QString themeName("testtheme");
     QIcon::setThemeName(themeName);
@@ -584,6 +584,14 @@ void tst_QIcon::fromTheme()
     QVERIFY(appointmentIcon.availableSizes().contains(QSize(32, 32)));
     QVERIFY(appointmentIcon.availableSizes().contains(QSize(22, 22)));
 
+    // Test fallback to less specific icon
+    QIcon specificAppointmentIcon = QIcon::fromTheme("appointment-new-specific");
+    QVERIFY(!QIcon::hasThemeIcon("appointment-new-specific"));
+    QVERIFY(QIcon::hasThemeIcon("appointment-new"));
+    QCOMPARE(specificAppointmentIcon.name(), QString::fromLatin1("appointment-new"));
+    QCOMPARE(specificAppointmentIcon.availableSizes(), appointmentIcon.availableSizes());
+    QCOMPARE(specificAppointmentIcon.pixmap(32).cacheKey(), appointmentIcon.pixmap(32).cacheKey());
+
     // Test icon from parent theme
     QIcon abIcon = QIcon::fromTheme("address-book-new");
     QVERIFY(!abIcon.isNull());
@@ -597,7 +605,7 @@ void tst_QIcon::fromTheme()
 
     // Test non existing icon with fallback
     noIcon = QIcon::fromTheme("broken-icon", abIcon);
-    QVERIFY(noIcon.cacheKey() == abIcon.cacheKey());
+    QCOMPARE(noIcon.cacheKey(), abIcon.cacheKey());
 
     // Test svg-only icon
     noIcon = QIcon::fromTheme("svg-icon", abIcon);
@@ -635,8 +643,97 @@ void tst_QIcon::fromTheme()
     QIcon::setThemeName("");
     abIcon = QIcon::fromTheme("address-book-new");
     QVERIFY(abIcon.isNull());
+
+    // Passing a full path to fromTheme is not very useful, but should work anyway
+    QIcon fullPathIcon = QIcon::fromTheme(m_pngImageFileName);
+    QVERIFY(!fullPathIcon.isNull());
 }
 
+static inline QString findGtkUpdateIconCache()
+{
+    QString binary = QLatin1String("gtk-update-icon-cache");
+#ifdef Q_OS_WIN
+    binary += QLatin1String(".exe");
+#endif
+    return QStandardPaths::findExecutable(binary);
+}
+
+void tst_QIcon::fromThemeCache()
+{
+    QTemporaryDir dir;
+    QVERIFY2(dir.isValid(), qPrintable(dir.errorString()));
+
+    QVERIFY(QDir().mkpath(dir.path() + QLatin1String("/testcache/16x16/actions")));
+    QVERIFY(QFile(QStringLiteral(":/styles/commonstyle/images/standardbutton-open-16.png"))
+        .copy( dir.path() + QLatin1String("/testcache/16x16/actions/button-open.png")));
+
+    {
+        QFile index(dir.path() + QLatin1String("/testcache/index.theme"));
+        QVERIFY(index.open(QFile::WriteOnly));
+        index.write("[Icon Theme]\nDirectories=16x16/actions\n[16x16/actions]\nSize=16\nContext=Actions\nType=Fixed\n");
+    }
+    QIcon::setThemeSearchPaths(QStringList() << dir.path());
+    QIcon::setThemeName("testcache");
+
+    // We just created a theme with that icon, it must exist
+    QVERIFY(!QIcon::fromTheme("button-open").isNull());
+
+    QString cacheName = dir.path() + QLatin1String("/testcache/icon-theme.cache");
+
+    // An invalid cache should not prevent lookup
+    {
+        QFile cacheFile(cacheName);
+        QVERIFY(cacheFile.open(QFile::WriteOnly));
+        QDataStream(&cacheFile) << quint16(1) << quint16(0) << "invalid corrupted stuff in there\n";
+    }
+    QIcon::setThemeSearchPaths(QStringList() << dir.path()); // reload themes
+    QVERIFY(!QIcon::fromTheme("button-open").isNull());
+
+    // An empty cache should prevent the lookup
+    {
+        QFile cacheFile(cacheName);
+        QVERIFY(cacheFile.open(QFile::WriteOnly));
+        QDataStream ds(&cacheFile);
+        ds << quint16(1) << quint16(0); // 0: version
+        ds << quint32(12) << quint32(20); // 4: hash offset / dir list offset
+        ds << quint32(1) << quint32(0xffffffff); // 12: one empty bucket
+        ds << quint32(1) << quint32(28); // 20: list with one element
+        ds.writeRawData("16x16/actions", sizeof("16x16/actions")); // 28
+    }
+    QIcon::setThemeSearchPaths(QStringList() << dir.path()); // reload themes
+    QVERIFY(QIcon::fromTheme("button-open").isNull()); // The icon was not in the cache, it should not be found
+
+    // Adding an icon should be changing the modification date of one sub directory which should make the cache ignored
+    QTest::qWait(1000); // wait enough to have a different modification time in seconds
+    QVERIFY(QFile(QStringLiteral(":/styles/commonstyle/images/standardbutton-save-16.png"))
+        .copy(dir.path() + QLatin1String("/testcache/16x16/actions/button-save.png")));
+    QVERIFY(QFileInfo(cacheName).lastModified() < QFileInfo(dir.path() + QLatin1String("/testcache/16x16/actions")).lastModified());
+    QIcon::setThemeSearchPaths(QStringList() << dir.path()); // reload themes
+    QVERIFY(!QIcon::fromTheme("button-open").isNull());
+
+    // Try to run the actual gtk-update-icon-cache and make sure that icons are still found
+    const QString gtkUpdateIconCache = findGtkUpdateIconCache();
+    if (gtkUpdateIconCache.isEmpty()) {
+        QIcon::setThemeSearchPaths(QStringList());
+        QSKIP("gtk-update-icon-cache not run (binary not found)");
+    }
+#ifndef QT_NO_PROCESS
+    QProcess process;
+    process.start(gtkUpdateIconCache,
+                  QStringList() << QStringLiteral("-f") << QStringLiteral("-t") << (dir.path() + QLatin1String("/testcache")));
+    QVERIFY2(process.waitForStarted(), qPrintable(QLatin1String("Unable to start: ")
+                                                  + gtkUpdateIconCache + QLatin1String(": ")
+                                                  + process.errorString()));
+    QVERIFY(process.waitForFinished());
+    QCOMPARE(process.exitStatus(), QProcess::NormalExit);
+    QCOMPARE(process.exitCode(), 0);
+#endif // QT_NO_PROCESS
+    QVERIFY(QFileInfo(cacheName).lastModified() >= QFileInfo(dir.path() + QLatin1String("/testcache/16x16/actions")).lastModified());
+    QIcon::setThemeSearchPaths(QStringList() << dir.path()); // reload themes
+    QVERIFY(!QIcon::fromTheme("button-open").isNull());
+    QVERIFY(!QIcon::fromTheme("button-open-fallback").isNull());
+    QVERIFY(QIcon::fromTheme("notexist-fallback").isNull());
+}
 
 void tst_QIcon::task223279_inconsistentAddFile()
 {
