@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,6 +44,8 @@
 # include <unistd.h>
 #endif
 
+#include <algorithm>
+
 typedef QMap<QString, QString> QStringMap;
 typedef QList<int> QIntList;
 Q_DECLARE_METATYPE(QImageWriter::ImageWriterError)
@@ -66,14 +55,8 @@ class tst_QImageWriter : public QObject
 {
     Q_OBJECT
 
-public:
-    tst_QImageWriter();
-    virtual ~tst_QImageWriter();
-
 public slots:
-    void init();
     void initTestCase();
-    void cleanup();
 
 private slots:
     void getSetCheck();
@@ -95,13 +78,15 @@ private slots:
 
     void saveToTemporaryFile();
 private:
+    QTemporaryDir m_temporaryDir;
     QString prefix;
+    QString writePrefix;
 };
 
 // helper to skip an autotest when the given image format is not supported
 #define SKIP_IF_UNSUPPORTED(format) do {                                                          \
     if (!QByteArray(format).isEmpty() && !QImageReader::supportedImageFormats().contains(format)) \
-        QSKIP("\"" + QByteArray(format) + "\" images are not supported");             \
+        QSKIP('"' + QByteArray(format) + "\" images are not supported");             \
 } while (0)
 
 static void initializePadding(QImage *image)
@@ -117,9 +102,11 @@ static void initializePadding(QImage *image)
 
 void tst_QImageWriter::initTestCase()
 {
+    QVERIFY(m_temporaryDir.isValid());
     prefix = QFINDTESTDATA("images/");
     if (prefix.isEmpty())
         QFAIL("Can't find images directory!");
+    writePrefix = m_temporaryDir.path() + QLatin1Char('/');
 }
 
 // Testing get/set functions
@@ -165,28 +152,6 @@ void tst_QImageWriter::getSetCheck()
     QCOMPARE(1.1f, obj1.gamma());
 }
 
-tst_QImageWriter::tst_QImageWriter()
-{
-}
-
-tst_QImageWriter::~tst_QImageWriter()
-{
-    QDir dir(prefix);
-    QStringList filesToDelete = dir.entryList(QStringList() << "gen-*" , QDir::NoDotAndDotDot | QDir::Files);
-    foreach( QString file, filesToDelete) {
-        QFile::remove(dir.absoluteFilePath(file));
-    }
-
-}
-
-void tst_QImageWriter::init()
-{
-}
-
-void tst_QImageWriter::cleanup()
-{
-}
-
 void tst_QImageWriter::writeImage_data()
 {
     QTest::addColumn<QString>("fileName");
@@ -201,6 +166,7 @@ void tst_QImageWriter::writeImage_data()
     QTest::newRow("PBM: ship63") << QString("ship63.pbm") << true << QByteArray("pbm");
     QTest::newRow("XBM: gnus") << QString("gnus.xbm") << false << QByteArray("xbm");
     QTest::newRow("JPEG: beavis") << QString("beavis.jpg") << true << QByteArray("jpeg");
+    QTest::newRow("ICO: App") << QString("App.ico") << true << QByteArray("ico");
 }
 
 void tst_QImageWriter::writeImage()
@@ -218,7 +184,7 @@ void tst_QImageWriter::writeImage()
         QVERIFY2(!image.isNull(), qPrintable(reader.errorString()));
     }
     {
-        QImageWriter writer(prefix + "gen-" + fileName, format);
+        QImageWriter writer(writePrefix + "gen-" + fileName, format);
         QVERIFY(writer.write(image));
     }
 
@@ -230,11 +196,11 @@ void tst_QImageWriter::writeImage()
 #endif
         if (!skip) {
             // Shouldn't be able to write to read-only file
-            QFile sourceFile(prefix + "gen-" + fileName);
+            QFile sourceFile(writePrefix + "gen-" + fileName);
             QFile::Permissions permissions = sourceFile.permissions();
             QVERIFY(sourceFile.setPermissions(QFile::ReadOwner | QFile::ReadUser | QFile::ReadGroup | QFile::ReadOther));
 
-            QImageWriter writer(prefix + "gen-" + fileName, format);
+            QImageWriter writer(writePrefix + "gen-" + fileName, format);
             QVERIFY(!writer.write(image));
 
             QVERIFY(sourceFile.setPermissions(permissions));
@@ -243,7 +209,7 @@ void tst_QImageWriter::writeImage()
 
     QImage image2;
     {
-        QImageReader reader(prefix + "gen-" + fileName);
+        QImageReader reader(writePrefix + "gen-" + fileName);
         image2 = reader.read();
         QVERIFY(!image2.isNull());
     }
@@ -262,7 +228,7 @@ void tst_QImageWriter::writeImage2_data()
     QTest::addColumn<QImage>("image");
 
     const QStringList formats = QStringList() << "bmp" << "xpm" << "png"
-                                              << "ppm"; //<< "jpeg";
+                                              << "ppm" << "ico"; //<< "jpeg";
     QImage image0(70, 70, QImage::Format_ARGB32);
     image0.fill(QColor(Qt::red).rgb());
 
@@ -271,8 +237,8 @@ void tst_QImageWriter::writeImage2_data()
         QImage image = image0.convertToFormat(imgFormat);
         initializePadding(&image);
         foreach (const QString format, formats) {
-            const QString fileName = QString("solidcolor_%1.%2").arg(imgFormat)
-                                     .arg(format);
+            const QString fileName = QLatin1String("solidcolor_")
+                + QString::number(imgFormat) + QLatin1Char('.') + format;
             QTest::newRow(fileName.toLatin1()) << fileName
                                                << format.toLatin1()
                                                << image;
@@ -342,7 +308,7 @@ void tst_QImageWriter::supportedFormats()
 {
     QList<QByteArray> formats = QImageWriter::supportedImageFormats();
     QList<QByteArray> sortedFormats = formats;
-    qSort(sortedFormats);
+    std::sort(sortedFormats.begin(), sortedFormats.end());
 
     // check that the list is sorted
     QCOMPARE(formats, sortedFormats);
@@ -359,7 +325,7 @@ void tst_QImageWriter::supportedMimeTypes()
 {
     QList<QByteArray> mimeTypes = QImageWriter::supportedMimeTypes();
     QList<QByteArray> sortedMimeTypes = mimeTypes;
-    qSort(sortedMimeTypes);
+    std::sort(sortedMimeTypes.begin(), sortedMimeTypes.end());
 
     // check that the list is sorted
     QCOMPARE(mimeTypes, sortedMimeTypes);
@@ -416,7 +382,7 @@ void tst_QImageWriter::testCanWrite()
     {
         // check if canWrite won't leave an empty file
         QTemporaryDir dir;
-        QVERIFY(dir.isValid());
+        QVERIFY2(dir.isValid(), qPrintable(dir.errorString()));
         QString fileName(dir.path() + QLatin1String("/001.garble"));
         QVERIFY(!QFileInfo(fileName).exists());
         QImageWriter writer(fileName);
@@ -462,7 +428,7 @@ void tst_QImageWriter::supportsOption()
                << QImageIOHandler::Animation
                << QImageIOHandler::BackgroundColor;
 
-    QImageWriter writer(prefix + fileName);
+    QImageWriter writer(writePrefix + fileName);
     for (int i = 0; i < options.size(); ++i) {
         QVERIFY(writer.supportsOption(QImageIOHandler::ImageOption(options.at(i))));
         allOptions.remove(QImageIOHandler::ImageOption(options.at(i)));
@@ -478,13 +444,13 @@ void tst_QImageWriter::saveWithNoFormat_data()
     QTest::addColumn<QByteArray>("format");
     QTest::addColumn<QImageWriter::ImageWriterError>("error");
 
-    QTest::newRow("garble") << prefix + QString("gen-out.garble") << QByteArray("jpeg") << QImageWriter::UnsupportedFormatError;
-    QTest::newRow("bmp") << prefix + QString("gen-out.bmp") << QByteArray("bmp") << QImageWriter::ImageWriterError(0);
-    QTest::newRow("xbm") << prefix + QString("gen-out.xbm") << QByteArray("xbm") << QImageWriter::ImageWriterError(0);
-    QTest::newRow("xpm") << prefix + QString("gen-out.xpm") << QByteArray("xpm") << QImageWriter::ImageWriterError(0);
-    QTest::newRow("png") << prefix + QString("gen-out.png") << QByteArray("png") << QImageWriter::ImageWriterError(0);
-    QTest::newRow("ppm") << prefix + QString("gen-out.ppm") << QByteArray("ppm") << QImageWriter::ImageWriterError(0);
-    QTest::newRow("pbm") << prefix + QString("gen-out.pbm") << QByteArray("pbm") << QImageWriter::ImageWriterError(0);
+    QTest::newRow("garble") << writePrefix + QString("gen-out.garble") << QByteArray("jpeg") << QImageWriter::UnsupportedFormatError;
+    QTest::newRow("bmp") << writePrefix + QString("gen-out.bmp") << QByteArray("bmp") << QImageWriter::ImageWriterError(0);
+    QTest::newRow("xbm") << writePrefix + QString("gen-out.xbm") << QByteArray("xbm") << QImageWriter::ImageWriterError(0);
+    QTest::newRow("xpm") << writePrefix + QString("gen-out.xpm") << QByteArray("xpm") << QImageWriter::ImageWriterError(0);
+    QTest::newRow("png") << writePrefix + QString("gen-out.png") << QByteArray("png") << QImageWriter::ImageWriterError(0);
+    QTest::newRow("ppm") << writePrefix + QString("gen-out.ppm") << QByteArray("ppm") << QImageWriter::ImageWriterError(0);
+    QTest::newRow("pbm") << writePrefix + QString("gen-out.pbm") << QByteArray("pbm") << QImageWriter::ImageWriterError(0);
 }
 
 void tst_QImageWriter::saveWithNoFormat()
@@ -524,7 +490,7 @@ void tst_QImageWriter::saveToTemporaryFile()
     {
         // 1) Via QImageWriter's API, with a standard temp file name
         QTemporaryFile file;
-        QVERIFY(file.open());
+        QVERIFY2(file.open(), qPrintable(file.errorString()));
         QImageWriter writer(&file, "PNG");
         if (writer.canWrite())
             QVERIFY(writer.write(image));
@@ -538,7 +504,7 @@ void tst_QImageWriter::saveToTemporaryFile()
     {
         // 2) Via QImage's API, with a standard temp file name
         QTemporaryFile file;
-        QVERIFY(file.open());
+        QVERIFY2(file.open(), qPrintable(file.errorString()));
         QVERIFY(image.save(&file, "PNG"));
         file.reset();
         QImage tmp;
@@ -548,7 +514,7 @@ void tst_QImageWriter::saveToTemporaryFile()
     {
         // 3) Via QImageWriter's API, with a named temp file
         QTemporaryFile file("tempXXXXXX");
-        QVERIFY(file.open());
+        QVERIFY2(file.open(), qPrintable(file.errorString()));
         QImageWriter writer(&file, "PNG");
         QVERIFY(writer.write(image));
 #if defined(Q_OS_WINCE)
@@ -559,7 +525,7 @@ void tst_QImageWriter::saveToTemporaryFile()
     {
         // 4) Via QImage's API, with a named temp file
         QTemporaryFile file("tempXXXXXX");
-        QVERIFY(file.open());
+        QVERIFY2(file.open(), qPrintable(file.errorString()));
         QVERIFY(image.save(&file, "PNG"));
         file.reset();
         QImage tmp;
