@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -114,7 +109,6 @@ private slots:
     void vaoCreate();
     void bufferCreate();
     void bufferMapRange();
-    void defaultQGLCurrentBuffer();
 };
 
 struct SharedResourceTracker
@@ -1526,33 +1520,6 @@ void tst_QOpenGL::bufferMapRange()
     ctx->doneCurrent();
 }
 
-void tst_QOpenGL::defaultQGLCurrentBuffer()
-{
-    QScopedPointer<QSurface> surface(createSurface(QSurface::Window));
-    QScopedPointer<QOpenGLContext> ctx(new QOpenGLContext);
-    ctx->create();
-    ctx->makeCurrent(surface.data());
-
-    // Bind default FBO on the current context, and record what's the current QGL FBO. It should
-    // be Q_NULLPTR because the default platform OpenGL FBO is not backed by a
-    // QOpenGLFramebufferObject.
-    QOpenGLFramebufferObject::bindDefault();
-    QOpenGLFramebufferObject *defaultQFBO = QOpenGLContextPrivate::get(ctx.data())->qgl_current_fbo;
-
-    // Create new FBO, bind it, and see that the QGL FBO points to the newly created FBO.
-    QScopedPointer<QOpenGLFramebufferObject> obj(new QOpenGLFramebufferObject(128, 128));
-    obj->bind();
-    QOpenGLFramebufferObject *customQFBO = QOpenGLContextPrivate::get(ctx.data())->qgl_current_fbo;
-    QVERIFY(defaultQFBO != customQFBO);
-
-    // Bind the default FBO, and check that the QGL FBO points to the original FBO object.
-    QOpenGLFramebufferObject::bindDefault();
-    QOpenGLFramebufferObject *finalQFBO = QOpenGLContextPrivate::get(ctx.data())->qgl_current_fbo;
-    QCOMPARE(defaultQFBO, finalQFBO);
-
-    ctx->doneCurrent();
-}
-
 void tst_QOpenGL::nullTextureInitializtion()
 {
     QScopedPointer<QSurface> surface(createSurface(QSurface::Window));
