@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -58,13 +45,6 @@ class tst_QFont : public QObject
 {
 Q_OBJECT
 
-public:
-    tst_QFont();
-    virtual ~tst_QFont();
-
-public slots:
-    void init();
-    void cleanup();
 private slots:
     void getSetCheck();
     void exactMatch();
@@ -76,7 +56,8 @@ private slots:
     void isCopyOf();
     void italicOblique();
     void insertAndRemoveSubstitutions();
-    void serializeSpacing();
+    void serialize_data();
+    void serialize();
     void lastResortFont();
     void styleName();
     void defaultFamily_data();
@@ -122,27 +103,6 @@ void tst_QFont::getSetCheck()
     QCOMPARE(QFont::StyleStrategy(QFont::OpenGLCompatible), obj1.styleStrategy());
 }
 
-tst_QFont::tst_QFont()
-{
-}
-
-tst_QFont::~tst_QFont()
-{
-
-}
-
-void tst_QFont::init()
-{
-// TODO: Add initialization code here.
-// This will be executed immediately before each test is run.
-}
-
-void tst_QFont::cleanup()
-{
-// TODO: Add cleanup code here.
-// This will be executed immediately after each test is run.
-}
-
 void tst_QFont::exactMatch()
 {
     QFont font;
@@ -150,158 +110,10 @@ void tst_QFont::exactMatch()
     // Check if a non-existing font hasn't an exact match
     font = QFont( "BogusFont", 33 );
     QVERIFY( !font.exactMatch() );
-
-#ifdef Q_OS_WIN
-    QSKIP("Exact matching on windows misses a lot because of the sample chars");
-#endif
-
-
-    if (QGuiApplication::platformName() == QLatin1String("xcb")) {
-        QVERIFY(QFont("sans").exactMatch());
-        QVERIFY(QFont("sans-serif").exactMatch());
-        QVERIFY(QFont("serif").exactMatch());
-        QVERIFY(QFont("monospace").exactMatch());
-    }
-
-    QSKIP("This test is bogus on Unix with support for font aliases in fontconfig");
-
-    QFontDatabase fdb;
-
-    QList<QFontDatabase::WritingSystem> systems = fdb.writingSystems();
-    for (int system = 0; system < systems.count(); ++system) {
-        QStringList families = fdb.families(systems[system]);
-        if (families.isEmpty())
-            return;
-
-        QStringList::ConstIterator f_it, f_end = families.end();
-        for (f_it = families.begin(); f_it != f_end; ++f_it) {
-            const QString &family = *f_it;
-            if (family.contains('['))
-                continue;
-
-            QStringList styles = fdb.styles(family);
-            QVERIFY(!styles.isEmpty());
-            QStringList::ConstIterator s_it, s_end = styles.end();
-            for (s_it = styles.begin(); s_it != s_end; ++s_it) {
-                const QString &style = *s_it;
-
-                if (fdb.isSmoothlyScalable(family, style)) {
-                    // smoothly scalable font... don't need to load every pointsize
-                    font = fdb.font(family, style, 12);
-                    QFontInfo fontinfo(font);
-
-                    if (! fontinfo.exactMatch()) {
-                        // Unfortunately, this can fail, since
-                        // QFontDatabase does not fill in all font
-                        // properties.  Check to make sure that the
-                        // test didn't fail for obvious reasons
-
-                        if (fontinfo.family().isEmpty()
-                                && fontinfo.pointSize() == 0) {
-                            // this is a box rendering engine... this can happen from
-                            // time to time, especially on X11 with iso10646-1 or
-                            // unknown font encodings
-                            continue;
-                        }
-
-#ifdef Q_OS_WIN
-                        if (font.family().startsWith("MS ") || fontinfo.family().startsWith("MS ")) {
-                            /* qDebug("Family matching skipped for MS-Alias font: %s, fontinfo: %s",
-                               font.family().latin1(), fontinfo.family().latin1());
-                               */
-                        } else
-#endif
-                        {
-                            if (!(font.family() == fontinfo.family()
-                                        || fontinfo.family().contains(font.family())
-                                        || fontinfo.family().isEmpty())) {
-                                qDebug("Test about to fail for font: %s, fontinfo: %s",
-                                        font.family().toLatin1().constData(),
-                                        fontinfo.family().toLatin1().constData());
-                            }
-                            QVERIFY(font.family() == fontinfo.family()
-                                    || fontinfo.family().contains(font.family())
-                                    || fontinfo.family().isEmpty());
-                        }
-                        if (font.pointSize() != -1) {
-                            QVERIFY(font.pointSize() == fontinfo.pointSize());
-                        } else {
-                            QVERIFY(font.pixelSize() == fontinfo.pixelSize());
-                        }
-                        QVERIFY(font.italic() == fontinfo.italic());
-                        if (font.weight() != fontinfo.weight()) {
-                            qDebug("font is %s", font.toString().toLatin1().constData());
-                        }
-                        QVERIFY(font.weight() == fontinfo.weight());
-                    } else {
-                        font.setFixedPitch(!fontinfo.fixedPitch());
-                        QFontInfo fontinfo1(font);
-                        QVERIFY( !fontinfo1.exactMatch() );
-
-                        font.setFixedPitch(fontinfo.fixedPitch());
-                        QFontInfo fontinfo2(font);
-                        QVERIFY( fontinfo2.exactMatch() );
-                    }
-                }
-#if 0
-                // ############## can only work if we have float point sizes in QFD
-                else {
-                    QList<int> sizes = fdb.pointSizes(family, style);
-                    QVERIFY(!sizes.isEmpty());
-                    QList<int>::ConstIterator z_it, z_end = sizes.end();
-                    for (z_it = sizes.begin(); z_it != z_end; ++z_it) {
-                        const int size = *z_it;
-
-                        // Initialize the font, and check if it is an exact match
-                        font = fdb.font(family, style, size);
-                        QFontInfo fontinfo(font, (QFont::Script) script);
-
-                        if (! fontinfo.exactMatch()) {
-                            // Unfortunately, this can fail, since
-                            // QFontDatabase does not fill in all font
-                            // properties.  Check to make sure that the
-                            // test didn't fail for obvious reasons
-
-                            if (fontinfo.family().isEmpty()
-                                    && fontinfo.pointSize() == 0) {
-                                // this is a box rendering engine... this can happen from
-                                // time to time, especially on X11 with iso10646-1 or
-                                // unknown font encodings
-                                continue;
-                            }
-
-                            // no need to skip MS-fonts here it seems
-                            if (!(font.family() == fontinfo.family()
-                                        || fontinfo.family().contains(font.family())
-                                        || fontinfo.family().isEmpty())) {
-                                qDebug("Test about to fail for font: %s, fontinfo: %s",
-                                        font.family().latin1(), fontinfo.family().latin1());
-                            }
-                            QVERIFY(font.family() == fontinfo.family()
-                                    || fontinfo.family().contains(font.family())
-                                    || fontinfo.family().isEmpty());
-                            if (font.pointSize() != -1) {
-                                QVERIFY(font.pointSize() == fontinfo.pointSize());
-                            } else {
-                                QVERIFY(font.pixelSize() == fontinfo.pixelSize());
-                            }
-                            QVERIFY(font.italic() == fontinfo.italic());
-                            QVERIFY(font.weight() == fontinfo.weight());
-                        } else {
-                            font.setFixedPitch(!fontinfo.fixedPitch());
-                            QFontInfo fontinfo1(font, (QFont::Script) script);
-                            QVERIFY( !fontinfo1.exactMatch() );
-
-                            font.setFixedPitch(fontinfo.fixedPitch());
-                            QFontInfo fontinfo2(font, (QFont::Script) script);
-                            QVERIFY( fontinfo2.exactMatch() );
-                        }
-                    }
-                }
-#endif
-            }
-        }
-    }
+    QVERIFY(!QFont("sans").exactMatch());
+    QVERIFY(!QFont("sans-serif").exactMatch());
+    QVERIFY(!QFont("serif").exactMatch());
+    QVERIFY(!QFont("monospace").exactMatch());
 }
 
 void tst_QFont::italicOblique()
@@ -378,42 +190,42 @@ void tst_QFont::compare()
         QVERIFY(font != font2);
         QCOMPARE(font < font2,!(font2 < font));
         font2.setItalic(false);
-        QVERIFY(font == font2);
+        QCOMPARE(font, font2);
         QVERIFY(!(font < font2));
 
         font2.setWeight(QFont::Bold);
         QVERIFY(font != font2);
         QCOMPARE(font < font2,!(font2 < font));
         font2.setWeight(QFont::Normal);
-        QVERIFY(font == font2);
+        QCOMPARE(font, font2);
         QVERIFY(!(font < font2));
 
         font.setUnderline(true);
         QVERIFY(font != font2);
         QCOMPARE(font < font2,!(font2 < font));
         font.setUnderline(false);
-        QVERIFY(font == font2);
+        QCOMPARE(font, font2);
         QVERIFY(!(font < font2));
 
         font.setStrikeOut(true);
         QVERIFY(font != font2);
         QCOMPARE(font < font2,!(font2 < font));
         font.setStrikeOut(false);
-        QVERIFY(font == font2);
+        QCOMPARE(font, font2);
         QVERIFY(!(font < font2));
 
         font.setOverline(true);
         QVERIFY(font != font2);
         QCOMPARE(font < font2,!(font2 < font));
         font.setOverline(false);
-        QVERIFY(font == font2);
+        QCOMPARE(font, font2);
         QVERIFY(!(font < font2));
 
         font.setCapitalization(QFont::SmallCaps);
         QVERIFY(font != font2);
         QCOMPARE(font < font2,!(font2 < font));
         font.setCapitalization(QFont::MixedCase);
-        QVERIFY(font == font2);
+        QCOMPARE(font, font2);
         QVERIFY(!(font < font2));
     }
 }
@@ -433,27 +245,27 @@ void tst_QFont::resolve()
     font1.setWeight(QFont::Bold);
     QFont font2 = font1.resolve(font);
 
-    QVERIFY(font2.weight() == font1.weight());
+    QCOMPARE(font2.weight(), font1.weight());
 
-    QVERIFY(font2.pointSize() == font.pointSize());
-    QVERIFY(font2.italic() == font.italic());
-    QVERIFY(font2.underline() == font.underline());
-    QVERIFY(font2.overline() == font.overline());
-    QVERIFY(font2.strikeOut() == font.strikeOut());
-    QVERIFY(font2.stretch() == font.stretch());
+    QCOMPARE(font2.pointSize(), font.pointSize());
+    QCOMPARE(font2.italic(), font.italic());
+    QCOMPARE(font2.underline(), font.underline());
+    QCOMPARE(font2.overline(), font.overline());
+    QCOMPARE(font2.strikeOut(), font.strikeOut());
+    QCOMPARE(font2.stretch(), font.stretch());
 
     QFont font3;
     font3.setStretch(QFont::UltraCondensed);
     QFont font4 = font3.resolve(font1).resolve(font);
 
-    QVERIFY(font4.stretch() == font3.stretch());
+    QCOMPARE(font4.stretch(), font3.stretch());
 
-    QVERIFY(font4.weight() == font.weight());
-    QVERIFY(font4.pointSize() == font.pointSize());
-    QVERIFY(font4.italic() == font.italic());
-    QVERIFY(font4.underline() == font.underline());
-    QVERIFY(font4.overline() == font.overline());
-    QVERIFY(font4.strikeOut() == font.strikeOut());
+    QCOMPARE(font4.weight(), font.weight());
+    QCOMPARE(font4.pointSize(), font.pointSize());
+    QCOMPARE(font4.italic(), font.italic());
+    QCOMPARE(font4.underline(), font.underline());
+    QCOMPARE(font4.overline(), font.overline());
+    QCOMPARE(font4.strikeOut(), font.strikeOut());
 
 
     QFont f1,f2,f3;
@@ -486,8 +298,8 @@ void tst_QFont::resetFont()
 
     child->setFont(QFont()); // reset font
 
-    QVERIFY(child->font().resolve() == 0);
-    QVERIFY(child->font().pointSize() == parent.font().pointSize());
+    QCOMPARE(child->font().resolve(), uint(0));
+    QCOMPARE(child->font().pointSize(), parent.font().pointSize());
     QVERIFY(parent.font().resolve() != 0);
 }
 #endif
@@ -511,7 +323,7 @@ void tst_QFont::isCopyOf()
 
 void tst_QFont::insertAndRemoveSubstitutions()
 {
-    QFont::removeSubstitution("BogusFontFamily");
+    QFont::removeSubstitutions("BogusFontFamily");
     // make sure it is empty before we start
     QVERIFY(QFont::substitutes("BogusFontFamily").isEmpty());
     QVERIFY(QFont::substitutes("bogusfontfamily").isEmpty());
@@ -528,50 +340,122 @@ void tst_QFont::insertAndRemoveSubstitutions()
     QCOMPARE(QFont::substitutes("BogusFontFamily").count(), 3);
     QCOMPARE(QFont::substitutes("bogusfontfamily").count(), 3);
 
-    QFont::removeSubstitution("BogusFontFamily");
+    QFont::removeSubstitutions("BogusFontFamily");
     // make sure it is empty again
     QVERIFY(QFont::substitutes("BogusFontFamily").isEmpty());
     QVERIFY(QFont::substitutes("bogusfontfamily").isEmpty());
 }
 
+Q_DECLARE_METATYPE(QDataStream::Version)
 
-static QFont copyFont(const QFont &font1) // copy using a QDataStream
+void tst_QFont::serialize_data()
 {
-    QBuffer buffer;
-    buffer.open(QIODevice::WriteOnly);
-    QDataStream ds(&buffer);
-    ds << font1;
-    buffer.close();
+    QTest::addColumn<QFont>("font");
+    // The version in which the tested feature was added.
+    QTest::addColumn<QDataStream::Version>("minimumStreamVersion");
 
-    buffer.open(QIODevice::ReadOnly);
-    QFont font2;
-    ds >> font2;
-    return font2;
-}
+    QFont basicFont;
+    // Versions <= Qt 2.1 had broken point size serialization,
+    // so we set an integer point size.
+    basicFont.setPointSize(9);
 
-void tst_QFont::serializeSpacing()
-{
-    QFont font;
-    QCOMPARE(font.letterSpacing(), 0.);
-    QCOMPARE(font.wordSpacing(), 0.);
+    QFont font = basicFont;
+    QTest::newRow("defaultConstructed") << font << QDataStream::Qt_1_0;
 
     font.setLetterSpacing(QFont::AbsoluteSpacing, 105);
-    QCOMPARE(font.letterSpacing(), 105.);
-    QCOMPARE(font.letterSpacingType(), QFont::AbsoluteSpacing);
-    QCOMPARE(font.wordSpacing(), 0.);
-    QFont font2 = copyFont(font);
-    QCOMPARE(font2.letterSpacing(), 105.);
-    QCOMPARE(font2.letterSpacingType(), QFont::AbsoluteSpacing);
-    QCOMPARE(font2.wordSpacing(), 0.);
+    QTest::newRow("letterSpacing") << font << QDataStream::Qt_4_5;
 
+    font = basicFont;
     font.setWordSpacing(50.0);
-    QCOMPARE(font.letterSpacing(), 105.);
-    QCOMPARE(font.wordSpacing(), 50.);
+    QTest::newRow("wordSpacing") << font << QDataStream::Qt_4_5;
 
-    QFont font3 = copyFont(font);
-    QCOMPARE(font3.letterSpacing(), 105.);
-    QCOMPARE(font3.letterSpacingType(), QFont::AbsoluteSpacing);
-    QCOMPARE(font3.wordSpacing(), 50.);
+    font = basicFont;
+    font.setPointSize(20);
+    QTest::newRow("pointSize") << font << QDataStream::Qt_1_0;
+
+    font = basicFont;
+    font.setPixelSize(32);
+    QTest::newRow("pixelSize") << font << QDataStream::Qt_3_0;
+
+    font = basicFont;
+    font.setStyleHint(QFont::Monospace);
+    QTest::newRow("styleHint") << font << QDataStream::Qt_1_0;
+
+    font = basicFont;
+    font.setStretch(4000);
+    QTest::newRow("stretch") << font << QDataStream::Qt_4_3;
+
+    font = basicFont;
+    font.setWeight(99);
+    QTest::newRow("weight") << font << QDataStream::Qt_1_0;
+
+    font = basicFont;
+    font.setUnderline(true);
+    QTest::newRow("underline") << font << QDataStream::Qt_1_0;
+
+    font = basicFont;
+    font.setStrikeOut(true);
+    QTest::newRow("strikeOut") << font << QDataStream::Qt_1_0;
+
+    font = basicFont;
+    font.setFixedPitch(true);
+    // This fails for versions less than this, as ignorePitch is set to false
+    // whenever setFixedPitch() is called, but ignorePitch is considered an
+    // extended bit, which were apparently not available until 4.4.
+    QTest::newRow("fixedPitch") << font << QDataStream::Qt_4_4;
+
+    font = basicFont;
+    font.setLetterSpacing(QFont::AbsoluteSpacing, 10);
+    // Fails for 4.4 because letterSpacing wasn't read until 4.5.
+    QTest::newRow("letterSpacing") << font << QDataStream::Qt_4_5;
+
+    font = basicFont;
+    font.setKerning(false);
+    QTest::newRow("kerning") << font << QDataStream::Qt_4_0;
+
+    font = basicFont;
+    font.setStyleStrategy(QFont::NoFontMerging);
+    // This wasn't read properly until 5.4.
+    QTest::newRow("styleStrategy") << font << QDataStream::Qt_5_4;
+
+    font = basicFont;
+    font.setHintingPreference(QFont::PreferFullHinting);
+    // This wasn't read until 5.4.
+    QTest::newRow("hintingPreference") << font << QDataStream::Qt_5_4;
+
+    font = basicFont;
+    font.setStyleName("Regular Black Condensed");
+    // This wasn't read until 5.4.
+    QTest::newRow("styleName") << font << QDataStream::Qt_5_4;
+
+    font = basicFont;
+    font.setCapitalization(QFont::AllUppercase);
+    // This wasn't read until 5.6.
+    QTest::newRow("capitalization") << font << QDataStream::Qt_5_6;
+}
+
+void tst_QFont::serialize()
+{
+    QFETCH(QFont, font);
+    QFETCH(QDataStream::Version, minimumStreamVersion);
+
+    QDataStream stream;
+    const int thisVersion = stream.version();
+
+    for (int version = minimumStreamVersion; version <= thisVersion; ++version) {
+        QBuffer buffer;
+        buffer.open(QIODevice::WriteOnly);
+        stream.setDevice(&buffer);
+        stream.setVersion(version);
+        stream << font;
+        buffer.close();
+
+        buffer.open(QIODevice::ReadOnly);
+        QFont readFont;
+        stream >> readFont;
+        QVERIFY2(readFont == font, qPrintable(QString::fromLatin1("Fonts do not compare equal for QDataStream version ") +
+            QString::fromLatin1("%1:\nactual: %2\nexpected: %3").arg(version).arg(readFont.toString()).arg(font.toString())));
+    }
 }
 
 // QFont::lastResortFont() may abort with qFatal() on QWS/QPA
@@ -614,7 +498,7 @@ static inline QByteArray msgNotAcceptableFont(const QString &defaultFamily, cons
 {
     QString res = QString::fromLatin1("Font family '%1' is not one of the following acceptable results: ").arg(defaultFamily);
     Q_FOREACH (const QString &family, acceptableFamilies)
-        res += QString::fromLatin1("\n %1").arg(family);
+        res += QLatin1String("\n ") + family;
     return res.toLocal8Bit();
 }
 
@@ -668,24 +552,24 @@ void tst_QFont::sharing()
     QCOMPARE(QFontPrivate::get(f)->engineData->ref.load(), 1 + refs_by_cache);
 
     QFont f2(f);
-    QVERIFY(QFontPrivate::get(f2) == QFontPrivate::get(f));
+    QCOMPARE(QFontPrivate::get(f2), QFontPrivate::get(f));
     QCOMPARE(QFontPrivate::get(f2)->ref.load(), 2);
     QVERIFY(QFontPrivate::get(f2)->engineData);
-    QVERIFY(QFontPrivate::get(f2)->engineData == QFontPrivate::get(f)->engineData);
+    QCOMPARE(QFontPrivate::get(f2)->engineData, QFontPrivate::get(f)->engineData);
     QCOMPARE(QFontPrivate::get(f2)->engineData->ref.load(), 1 + refs_by_cache);
 
     f2.setKerning(!f.kerning());
     QVERIFY(QFontPrivate::get(f2) != QFontPrivate::get(f));
     QCOMPARE(QFontPrivate::get(f2)->ref.load(), 1);
     QVERIFY(QFontPrivate::get(f2)->engineData);
-    QVERIFY(QFontPrivate::get(f2)->engineData == QFontPrivate::get(f)->engineData);
+    QCOMPARE(QFontPrivate::get(f2)->engineData, QFontPrivate::get(f)->engineData);
     QCOMPARE(QFontPrivate::get(f2)->engineData->ref.load(), 2 + refs_by_cache);
 
     f2 = f;
-    QVERIFY(QFontPrivate::get(f2) == QFontPrivate::get(f));
+    QCOMPARE(QFontPrivate::get(f2), QFontPrivate::get(f));
     QCOMPARE(QFontPrivate::get(f2)->ref.load(), 2);
     QVERIFY(QFontPrivate::get(f2)->engineData);
-    QVERIFY(QFontPrivate::get(f2)->engineData == QFontPrivate::get(f)->engineData);
+    QCOMPARE(QFontPrivate::get(f2)->engineData, QFontPrivate::get(f)->engineData);
     QCOMPARE(QFontPrivate::get(f2)->engineData->ref.load(), 1 + refs_by_cache);
 
     if (f.pointSize() > 0)
