@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -70,6 +65,7 @@ private slots:
     void setRawDataAndGetAsVector();
     void boundingRect();
     void mixedScripts();
+    void multiLineBoundingRect();
 
 private:
     int m_testFontId;
@@ -735,6 +731,34 @@ void tst_QGlyphRun::mixedScripts()
     QCOMPARE(glyphRuns.size(), 2);
 }
 
+void tst_QGlyphRun::multiLineBoundingRect()
+{
+    QTextLayout layout;
+    layout.setText("Foo Bar");
+    layout.beginLayout();
+
+    QTextLine line = layout.createLine();
+    line.setNumColumns(4);
+    line.setPosition(QPointF(0, 0));
+
+    line = layout.createLine();
+    line.setPosition(QPointF(0, 10));
+
+    layout.endLayout();
+
+    QCOMPARE(layout.lineCount(), 2);
+
+    QList<QGlyphRun> firstLineGlyphRuns = layout.lineAt(0).glyphRuns();
+    QList<QGlyphRun> allGlyphRuns = layout.glyphRuns();
+    QCOMPARE(firstLineGlyphRuns.size(), 1);
+    QCOMPARE(allGlyphRuns.size(), 1);
+
+    QGlyphRun firstLineGlyphRun = firstLineGlyphRuns.first();
+    QGlyphRun allGlyphRun = allGlyphRuns.first();
+
+    QVERIFY(firstLineGlyphRun.boundingRect().height() < allGlyphRun.boundingRect().height());
+}
+
 #endif // QT_NO_RAWFONT
 
 QTEST_MAIN(tst_QGlyphRun)
