@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -93,6 +88,8 @@ private slots:
     void multipleRawFontsFromData();
 
     void rawFontFromInvalidData();
+
+    void kernedAdvances();
 private:
     QString testFont;
     QString testFontBoldItalic;
@@ -948,12 +945,44 @@ void tst_QRawFont::rawFontFromInvalidData()
 
     QVERIFY(!font.isValid());
 
-    invalidData.fill(255, 1024);
+    invalidData.fill(char(255), 1024);
     font.loadFromData(invalidData, 10, QFont::PreferDefaultHinting);
 
     QVERIFY(!font.isValid());
 }
 
+#define FUZZY_LTEQ(X, Y) (X < Y || qFuzzyCompare(X, Y))
+
+void tst_QRawFont::kernedAdvances()
+{
+    const int emSquareSize = 1000;
+    const qreal pixelSize = 16.0;
+    const int underScoreAW = 500;
+    const int underscoreTwoKerning = -500;
+    const qreal errorMargin = 1.0 / 16.0; // Fixed point error margin
+
+    QRawFont font(testFont, pixelSize);
+    QVERIFY(font.isValid());
+
+    QVector<quint32> glyphIndexes = font.glyphIndexesForString(QStringLiteral("__"));
+    QCOMPARE(glyphIndexes.size(), 2);
+
+    QVector<QPointF> advances = font.advancesForGlyphIndexes(glyphIndexes, QRawFont::KernedAdvances);
+    QCOMPARE(advances.size(), 2);
+
+    qreal expectedAdvanceWidth = pixelSize * underScoreAW / emSquareSize;
+    QVERIFY(FUZZY_LTEQ(qAbs(advances.at(0).x() - expectedAdvanceWidth), errorMargin));
+
+    glyphIndexes = font.glyphIndexesForString(QStringLiteral("_2"));
+    QCOMPARE(glyphIndexes.size(), 2);
+
+    advances = font.advancesForGlyphIndexes(glyphIndexes, QRawFont::KernedAdvances);
+    QCOMPARE(advances.size(), 2);
+
+    expectedAdvanceWidth = pixelSize * (underScoreAW + underscoreTwoKerning) / emSquareSize;
+    QVERIFY(FUZZY_LTEQ(qAbs(advances.at(0).x() - expectedAdvanceWidth), errorMargin));
+}
+
 #endif // QT_NO_RAWFONT
 
 QTEST_MAIN(tst_QRawFont)
