@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -59,13 +54,10 @@ class tst_QTextLayout : public QObject
 
 public:
     tst_QTextLayout();
-    virtual ~tst_QTextLayout();
 
-
-public slots:
+private slots:
     void init();
     void cleanup();
-private slots:
     void getSetCheck();
     void lineBreaking();
 #ifdef QT_BUILD_INTERNAL
@@ -142,7 +134,6 @@ private slots:
     void xToCursorForLigatures();
     void cursorInNonStopChars();
     void nbsp();
-    void nbspWithFormat();
     void noModificationOfInputString();
     void superscriptCrash_qtbug53911();
 
@@ -202,10 +193,6 @@ tst_QTextLayout::tst_QTextLayout()
 #endif
 }
 
-tst_QTextLayout::~tst_QTextLayout()
-{
-}
-
 void tst_QTextLayout::init()
 {
     testFont = QFont();
@@ -1126,10 +1113,10 @@ void tst_QTextLayout::boundingRectTopLeft()
 void tst_QTextLayout::graphemeBoundaryForSurrogatePairs()
 {
     QString txt;
-    txt.append("a");
+    txt.append(QLatin1Char('a'));
     txt.append(0xd87e);
     txt.append(0xdc25);
-    txt.append("b");
+    txt.append(QLatin1Char('b'));
     QTextLayout layout(txt);
     QTextEngine *engine = layout.engine();
     const QCharAttributes *attrs = engine->attributes();
@@ -2267,41 +2254,5 @@ void tst_QTextLayout::superscriptCrash_qtbug53911()
     qDeleteAll(textLayouts);
 }
 
-void tst_QTextLayout::nbspWithFormat()
-{
-    QString s1 = QLatin1String("ABCDEF ");
-    QString s2 = QLatin1String("GHI");
-    QChar nbsp(QChar::Nbsp);
-    QString s3 = QLatin1String("JKLMNOPQRSTUVWXYZ");
-
-    QTextLayout layout;
-    layout.setText(s1 + s2 + nbsp + s3);
-
-    QTextLayout::FormatRange formatRange;
-    formatRange.start = s1.length() + s2.length();
-    formatRange.length = 1;
-    formatRange.format.setFontUnderline(true);
-
-    QList<QTextLayout::FormatRange> overrides;
-    overrides.append(formatRange);
-
-    layout.setAdditionalFormats(overrides);
-
-    layout.beginLayout();
-    forever {
-        QTextLine line = layout.createLine();
-        if (!line.isValid())
-            break;
-        line.setLineWidth(1);
-    }
-    layout.endLayout();
-
-    QCOMPARE(layout.lineCount(), 2);
-    QCOMPARE(layout.lineAt(0).textStart(), 0);
-    QCOMPARE(layout.lineAt(0).textLength(), s1.length());
-    QCOMPARE(layout.lineAt(1).textStart(), s1.length());
-    QCOMPARE(layout.lineAt(1).textLength(), s2.length() + 1 + s3.length());
-}
-
 QTEST_MAIN(tst_QTextLayout)
 #include "tst_qtextlayout.moc"
