@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -72,16 +67,17 @@ namespace QTest {
     template<>
     char *toString(const QList<QNetworkCookie> &list)
     {
-        QString result = "QList(";
+        QByteArray result = "QList(";
         bool first = true;
         foreach (QNetworkCookie cookie, list) {
             if (!first)
                 result += ", ";
             first = false;
-            result += QString::fromLatin1("QNetworkCookie(%1)").arg(QLatin1String(cookie.toRawForm()));
+            result += "QNetworkCookie(" + cookie.toRawForm() + ')';
         }
 
-        return qstrdup(result.append(')').toLocal8Bit());
+        result.append(')');
+        return qstrdup(result.constData());
     }
 }
 
@@ -192,16 +188,16 @@ void tst_QNetworkCookieJar::setCookiesFromUrl_data()
     result += cookie;
     QTest::newRow("effective-tld1-accepted") << preset << cookie << "http://something.co.uk" << result << true;
 
-    // 2. anything .ck is an effective TLD ('*.ck'), but 'www.ck' is an exception
+    // 2. anything .mz is an effective TLD ('*.mz'), but 'teledata.mz' is an exception
     result.clear();
     preset.clear();
-    cookie.setDomain(".foo.ck");
-    QTest::newRow("effective-tld2-denied") << preset << cookie << "http://foo.ck" << result << false;
-    QTest::newRow("effective-tld2-denied2") << preset << cookie << "http://www.foo.ck" << result << false;
-    QTest::newRow("effective-tld2-denied3") << preset << cookie << "http://www.anything.foo.ck" << result << false;
-    cookie.setDomain(".www.ck");
+    cookie.setDomain(".farmacia.mz");
+    QTest::newRow("effective-tld2-denied") << preset << cookie << "http://farmacia.mz" << result << false;
+    QTest::newRow("effective-tld2-denied2") << preset << cookie << "http://www.farmacia.mz" << result << false;
+    QTest::newRow("effective-tld2-denied3") << preset << cookie << "http://www.anything.farmacia.mz" << result << false;
+    cookie.setDomain(".teledata.mz");
     result += cookie;
-    QTest::newRow("effective-tld2-accepted") << preset << cookie << "http://www.www.ck" << result << true;
+    QTest::newRow("effective-tld2-accepted") << preset << cookie << "http://www.teledata.mz" << result << true;
 
     result.clear();
     preset.clear();
